# - Try to find LibXml2
# Once done this will define
#
#  LIBXML2_FOUND - system has LibXml2
#  LIBXML2_INCLUDE_DIR - the LibXml2 include directory
#  LIBXML2_LIBRARIES - the libraries needed to use LibXml2
#  LIBXML2_DEFINITIONS - Compiler switches required for using LibXml2
#
# Copyright (c) 2006, Alexander Neundorf <neundorf@kde.org>
# This code is available under the BSD license, see licenses/BSD for details.

# Copyright (c) 2006, Alexander Neundorf, <neundorf@kde.org>
#
# Redistribution and use is allowed according to the terms of the BSD license.
# For details see the accompanying COPYING-CMAKE-SCRIPTS file.
#
# NOTE: customized find-module
#       find our libxml2 on windows

IF (LIBXML2_INCLUDE_DIR AND LIBXML2_LIBRARIES)
   # in cache already
   SET(LibXml2_FIND_QUIETLY TRUE)
ENDIF (LIBXML2_INCLUDE_DIR AND LIBXML2_LIBRARIES)

IF (NOT WIN32)
   # use pkg-config to get the directories and then use these values
   # in the FIND_PATH() and FIND_LIBRARY() calls
   INCLUDE(UsePkgConfig)
   PKGCONFIG(libxml-2.0 _LibXml2IncDir _LibXml2LinkDir _LibXml2LinkFlags _LibXml2Cflags)
   SET(LIBXML2_DEFINITIONS ${_LibXml2Cflags})
ELSE (NOT WIN32)
   # on windows use env.var EXTERNLIBS or LIBXML2_HOME
   SET(_LibXml2IncDir $ENV{EXTERNLIBS}/libxml2/include)
   SET(_LibXml2LinkDir $ENV{EXTERNLIBS}/libxml2/lib $ENV{LIBXML2_HOME})
ENDIF (NOT WIN32)

FIND_PATH(LIBXML2_INCLUDE_DIR libxml/xpath.h
   PATHS
   ${_LibXml2IncDir}
   PATH_SUFFIXES libxml2
   )

FIND_LIBRARY(LIBXML2_LIBRARY_RELEASE NAMES xml2 libxml2
   PATHS
   ${_LibXml2LinkDir}
   )
FIND_LIBRARY(LIBXML2_LIBRARY_DEBUG NAMES xml2D libxml2D
   PATHS
   ${_LibXml2LinkDir}
   )

IF (LIBXML2_INCLUDE_DIR)
  BC_ADJUST_LIB_VARS(LIBXML2)
  IF (LIBXML2_LIBRARY)
    SET (LIBXML2_LIBRARIES ${LIBXML2_LIBRARY} CACHE STRING "The libxml2 - Libraries" FORCE)
  ENDIF (LIBXML2_LIBRARY)
ENDIF (LIBXML2_INCLUDE_DIR)

IF (LIBXML2_INCLUDE_DIR AND LIBXML2_LIBRARIES)
   SET(LIBXML2_FOUND TRUE)
ELSE (LIBXML2_INCLUDE_DIR AND LIBXML2_LIBRARIES)
   SET(LIBXML2_FOUND FALSE)
ENDIF (LIBXML2_INCLUDE_DIR AND LIBXML2_LIBRARIES)

IF (LIBXML2_FOUND)
   IF (NOT LibXml2_FIND_QUIETLY)
      MESSAGE(STATUS "Found LibXml2: ${LIBXML2_LIBRARIES}")
   ENDIF (NOT LibXml2_FIND_QUIETLY)
ELSE (LIBXML2_FOUND)
   IF (LibXml2_FIND_REQUIRED)
      MESSAGE(SEND_ERROR "Could NOT find LibXml2")
   ENDIF (LibXml2_FIND_REQUIRED)
ENDIF (LIBXML2_FOUND)

MARK_AS_ADVANCED(LIBXML2_INCLUDE_DIR LIBXML2_LIBRARIES)

