# @file
# 
# Build the following targets:
#  - ${MARMOT_profile_LIB}
#  - ${MARMOT_unimci_LIB}
#
# @author Blasius Czink, Tobias Hilbrich
#

INCLUDE_DIRECTORIES(
  ${MARMOT_GEN_HEADER_PATH}
  ${MARMOT_HEADER_PATH}
  ${MPI_INCLUDE_DIR}
)

IF (MARMOT_BUILD_STATIC)
  ADD_LIBRARY(${MARMOT_profile_LIB}
    STATIC
    ${CMAKE_BINARY_DIR}/generated/c_wrapper.cc
  )
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB} PROPERTIES COMPILE_FLAGS "${OpenMP_CXX_FLAGS}")
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB} PROPERTIES DEBUG_POSTFIX "${CMAKE_DEBUG_POSTFIX}")
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB} PROPERTIES CLEAN_DIRECT_OUTPUT 1)

  ADD_LIBRARY(${MARMOT_unimci_LIB}
      STATIC
      ${MARMOT_HEADER_PATH}/mpi_standard.h
      ${CMAKE_BINARY_DIR}/generated/unimci_wrapper.cc
  )
  SET_TARGET_PROPERTIES(${MARMOT_unimci_LIB} PROPERTIES COMPILE_FLAGS "${OpenMP_CXX_FLAGS}")
  SET_TARGET_PROPERTIES(${MARMOT_unimci_lib} PROPERTIES DEBUG_POSTFIX "${CMAKE_DEBUG_POSTFIX}")
  SET_TARGET_PROPERTIES(${MARMOT_unimci_LIB} PROPERTIES CLEAN_DIRECT_OUTPUT 1)

  # install
  INSTALL(TARGETS
          ${MARMOT_profile_LIB}
          ${MARMOT_unimci_LIB}
          DESTINATION "lib"
  )
  BC_COPY_TARGET_PDB(${MARMOT_profile_LIB})
  BC_COPY_TARGET_PDB(${MARMOT_unimci_LIB})
  
  #Fortran Wrappers (if enabled)
  IF (MARMOT_ENABLE_FORTRAN)
      ADD_LIBRARY(${MARMOT_fortran_LIB}
  	    STATIC
          ${MARMOT_HEADER_PATH}/mpo-fortran-mangling.h
          ${MARMOT_HEADER_PATH}/mpi_standard.h
          ${CMAKE_BINARY_DIR}/generated/fortran_wrapper.cc
      )
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB} PROPERTIES COMPILE_FLAGS "${OpenMP_CXX_FLAGS}")
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB} PROPERTIES DEBUG_POSTFIX "${CMAKE_DEBUG_POSTFIX}")
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB} PROPERTIES CLEAN_DIRECT_OUTPUT 1)
  
      INSTALL(TARGETS ${MARMOT_fortran_LIB} DESTINATION "lib")      
  ENDIF (MARMOT_ENABLE_FORTRAN)
  
ENDIF (MARMOT_BUILD_STATIC)

IF (MARMOT_BUILD_SHARED)
  ADD_LIBRARY(${MARMOT_profile_LIB}-shared
    SHARED
    ${CMAKE_BINARY_DIR}/generated/c_wrapper.cc
  )
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB}-shared PROPERTIES COMPILE_FLAGS "${OpenMP_CXX_FLAGS}")
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB}-shared PROPERTIES LINK_FLAGS "${MARMOT_ADD_SHARED_LFLAGS}")
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB}-shared PROPERTIES OUTPUT_NAME "${MARMOT_profile_LIB}")
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB}-shared PROPERTIES DEBUG_POSTFIX "${CMAKE_DEBUG_POSTFIX}")
  SET_TARGET_PROPERTIES(${MARMOT_profile_LIB}-shared PROPERTIES CLEAN_DIRECT_OUTPUT 1)

  ADD_LIBRARY(${MARMOT_unimci_LIB}-shared
    SHARED
    ${MARMOT_HEADER_PATH}/mpi_standard.h
    ${CMAKE_BINARY_DIR}/generated/unimci_wrapper.cc
  )
  SET_TARGET_PROPERTIES(${MARMOT_unimci_LIB}-shared PROPERTIES COMPILE_FLAGS "${OpenMP_CXX_FLAGS}")
  SET_TARGET_PROPERTIES(${MARMOT_unimci_LIB}-shared PROPERTIES LINK_FLAGS "${MARMOT_ADD_SHARED_LFLAGS}")
  SET_TARGET_PROPERTIES(${MARMOT_unimci_LIB}-shared PROPERTIES OUTPUT_NAME "${MARMOT_unimci_LIB}")
  SET_TARGET_PROPERTIES(${MARMOT_unimci_LIB}-shared PROPERTIES DEBUG_POSTFIX "${CMAKE_DEBUG_POSTFIX}")
  SET_TARGET_PROPERTIES(${MARMOT_unimci_LIB}-shared PROPERTIES CLEAN_DIRECT_OUTPUT 1)

  # install
  INSTALL(TARGETS
          ${MARMOT_profile_LIB}-shared
          ${MARMOT_unimci_LIB}-shared
          DESTINATION "lib"
  )
  
  #Fortran Wrappers (if enabled)
  IF (MARMOT_ENABLE_FORTRAN)
      ADD_LIBRARY(${MARMOT_fortran_LIB}-shared
  	    SHARED
          ${MARMOT_HEADER_PATH}/mpo-fortran-mangling.h
          ${MARMOT_HEADER_PATH}/mpi_standard.h
          ${CMAKE_BINARY_DIR}/generated/fortran_wrapper.cc
      )
      
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB} PROPERTIES COMPILE_FLAGS "${OpenMP_CXX_FLAGS}")
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB}-shared PROPERTIES LINK_FLAGS "${MARMOT_ADD_SHARED_LFLAGS}")
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB}-shared PROPERTIES OUTPUT_NAME "${MARMOT_fortran_LIB}")
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB}-shared PROPERTIES DEBUG_POSTFIX "${CMAKE_DEBUG_POSTFIX}")
      SET_TARGET_PROPERTIES(${MARMOT_fortran_LIB}-shared PROPERTIES CLEAN_DIRECT_OUTPUT 1)
  
      INSTALL(TARGETS ${MARMOT_fortran_LIB}-shared DESTINATION "lib")      
  ENDIF (MARMOT_ENABLE_FORTRAN)
ENDIF (MARMOT_BUILD_SHARED)
