# @file HaveMPI2Helpers.cmake
# 
# Generates the sourceinfompicalls.h file of Marmot. 
#
# @author Tobias Hilbrich

##Needed to parse headers !!!
INCLUDE (generation/GenerateMacros)

#===================================
# Macro: MARMOT_GEN_CREATE_WRAPPER
#===================================
#This is a hook name, as the parser calls
#a macro of that name.
#
# Used to generate sourceinfompicalls.h
MACRO (MARMOT_GEN_CREATE_WRAPPER outfile callname retval p_args_n_types)
    SET (args_n_types ${${p_args_n_types}})

    #Text to add to file
    SET (text "")
    
    #Header for FeatureTest
    STRING(REGEX REPLACE "^MPI[_]" "MPI_SOURCEINFO_" sourceinfoname "${callname}")
    SET (text "${text}${retval} ${sourceinfoname} (")
        
    IF (args_n_types)
        #For calls with arguments, loop over them and print
        FOREACH (t_n_a ${args_n_types})
            MARMOT_GEN_SPLIT_T_N_A ("${t_n_a}" type arg)
                
            #print argument and type
            SET (text "${text}${type} ${arg}, ")
        ENDFOREACH (t_n_a)
    ENDIF (args_n_types)
    
    #Trailer         
    SET (text "${text} int line, const char *scfile);\n")
    
 #   SET (available TRUE)
 #   HaveMPI_GetMPI2CallList (call_list)
 #   IF ("${call_list}" MATCHES "${callname}")
 #       HaveMPI_IsMPI2CallAvailable ("${callname}" available) 
 #   ENDIF ("${call_list}" MATCHES "${callname}")
        
    #Write to output file
 #   IF (available)
        FILE (APPEND "${outfile}" "${text}")
 #   ENDIF (available)         
       
ENDMACRO (MARMOT_GEN_CREATE_WRAPPER outdir callname retval p_args_n_types)

#===================================
# Actual script 
#===================================
#MESSAGE (STATUS "Building sourceinfompicalls.h")

SET (INFILES ${INFILES} "${PROJECT_SOURCE_DIR}/SRC/INCLUDE/mpi_standard.h")
SET (INFILES ${INFILES} "${PROJECT_SOURCE_DIR}/SRC/INCLUDE/mpi2_1sided.h")
SET (INFILES ${INFILES} "${PROJECT_SOURCE_DIR}/SRC/INCLUDE/mpi2_extcoll.h")
SET (INFILES ${INFILES} "${PROJECT_SOURCE_DIR}/SRC/INCLUDE/mpi2_file.h")
SET (INFILES ${INFILES} "${PROJECT_SOURCE_DIR}/SRC/INCLUDE/mpi2_standard.h")
SET (INFILES ${INFILES} "${PROJECT_SOURCE_DIR}/SRC/INCLUDE/mpi2_proc.h")

SET (OUTFILE "${MARMOT_GEN_HEADER_PATH}/sourceinfompicalls.h")

SET (IGNOREFILE "${PROJECT_BINARY_DIR}/generated/ignore-list.txt")
CONFIGURE_FILE("${PROJECT_SOURCE_DIR}/CMAKEMODULES/generation/ignore-list.txt" "${IGNOREFILE}" COPYONLY)
HaveMPI_GetUnsupportedCallList(unsuported_list)
FOREACH (call ${unsuported_list})
    FILE (APPEND "${IGNOREFILE}" "\n${call}")
ENDFOREACH (call)

FILE (WRITE "${OUTFILE}" 
    "/*Generated File, do not generate, part of Marmot.*/\n"
    "#include \"config.h\"\n"
    "#if defined(__cplusplus)\n"
    "extern \"C\" {\n"
    "#endif\n\n")

FOREACH (INFILE ${INFILES})
    MARMOT_GEN_PARSE_HEADER ("${INFILE}" "${OUTFILE}" "${IGNOREFILE}")
ENDFOREACH (INFILE ${INFILES})

FILE (APPEND "${OUTFILE}" 
    "\n\n#if defined(__cplusplus)\n"
    "}\n"
    "#endif\n")
