#include <string>
#include <vector>
#include <iostream>
#include <sstream>
#define CODES_DEFAULTVALUE 1
//#define DEBUG
/*!
 Class ParseErrCodesEnv. Can be used to read (probably from an environmnent variable) a string defining which errors (i.e. error codes) to look for. The syntax is easy. The only characters allowed are 0123456789-()^,. You can define single codes or code ranges, use brackets and negate expresions.
Examples:
<ul>
	<li> (1-3),^4,5,18 <br> This will record codes 1 to 3, 5 and 18. Code 4 won't be recorded</li>
	<li> 13,19,^(20-23) <br> This will record codes 13 and 19. Codes 20 to 23 won't be recorded</li>
</ul>
Comma separated values are evaluated from left to right and overwrite prior definitions. For example
<ul>
	<li> (1-10),^(5-10) <br> will record codes 1 to 4. Codes 5 to 10 won't be recorded</li>
</ul>
*/
class ParseErrCodesEnv
{
	private:
		std::string ClassName;
		/**
		 * Prefix for outputs to stdout
		 */
		std::string StdOutPrefix;
		/**
		 * Prefix for outputs to stderr
		 */
		std::string StdErrPrefix;
		/**
		 * The expression to evaluate
		 */
		std::string expr;
		/**
		 * The resulting code vector
		 */
		std::vector <int> *code_vector;
		/**
		 * The internal evaluation function
		 */
		std::vector <int> *evaluate(std::string substr);
		/**
		 * number of codes (size of the code vector)
		 */
		int numcodes;
		/**
		 * default value to use in the code vector (usually 1)
		 */
		int defaultval;
	public:
		/**
		 * The public evaluation function
		 * \param str The expression to be evaluated
		 * \param numcodes The number of error codes (size of the code vector)
		 * \param defaultval The default value to be used in the code vector (1 or 0)
		 */
 		ParseErrCodesEnv(std::string str, int numcodes, int defaultval);

 		/**
 		 * Destructor.
 		 */
 		~ParseErrCodesEnv(void);

		/**
		 * Print the expression to be evaluated
		 */
		void printExpr();
		/**
		 * Print the code vector
		 */
		void printCodeVector();
		/**
		 * Evaluate the expression and set the appropriate values in the code vector
		 */
		void evaluate();
		/**
		 * \return The code vector
		 */
		std::vector <int> *getCodeVector();
		/**
		 * Updates the code vector with the entries of another vector
		 * \param vec The vector which entries should be inserted into the code vector
		 */
		void record(std::vector <int> *vec);
		/**
		 * Checks, wheter a specific number is a valid number (i.e. in the defined range)
		 * \param number The error code to be checked
		 * \return 1 if ok, 0 otherwise
		 */
		int validCodeNumber(int number);
		/**
		 * \param code A specific error code
		 * \return The value of this error code in the code_vector, i.e. 0 or 1, -1 if out of range
		 */
		int getValue(int code);
		/**
		 *\param the array to be filled
		 */
		void getErrCodes(bool *errcodes);
		/**
		 *\return a string describing the selection
		 */
		std::string *getString();
};

