/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart, (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Macros for enhancing MPI-Calls with linenumber and fileinformation
 *
 *  @author Tobias Hilbrich
 *
 *  $Id: enhancempicalls.h 1057 2010-01-28 11:34:50Z tobias $
 */

#define MPI_Abort(comm,errorcode) MPI_SOURCEINFO_Abort(comm,errorcode,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Address(location, address) MPI_SOURCEINFO_Address(location, address,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm) MPI_SOURCEINFO_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm) MPI_SOURCEINFO_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm) MPI_SOURCEINFO_Allreduce(sendbuf, recvbuf, count, datatype, op, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm) MPI_SOURCEINFO_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm) MPI_SOURCEINFO_Alltoallv(sendbuf, sendcounts, sdispls, sendtype, recvbuf, recvcounts, rdispls, recvtype, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Attr_delete(comm, keyval) MPI_SOURCEINFO_Attr_delete(comm, keyval,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Attr_get(comm, keyval, attribute_val, flag) MPI_SOURCEINFO_Attr_get(comm, keyval, attribute_val, flag,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Attr_put(comm, keyval, attribute_val) MPI_SOURCEINFO_Attr_put(comm, keyval, attribute_val,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Barrier( ) MPI_SOURCEINFO_Barrier( ,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Bcast(buffer, count, datatype, root, comm) MPI_SOURCEINFO_Bcast(buffer, count, datatype, root, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Bsend(buf, count, datatype, dest, tag, comm) MPI_SOURCEINFO_Bsend(buf, count, datatype, dest, tag, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Bsend_init(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Bsend_init(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Buffer_attach( buffer, size) MPI_SOURCEINFO_Buffer_attach( buffer, size,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Buffer_detach( buffer, size) MPI_SOURCEINFO_Buffer_detach( buffer, size,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cancel(request) MPI_SOURCEINFO_Cancel(request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cart_coords(comm, rank, maxdims, coords) MPI_SOURCEINFO_Cart_coords(comm, rank, maxdims, coords,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cart_create(comm_old, ndims, dims, periods, reorder, comm_cart) MPI_SOURCEINFO_Cart_create(comm_old, ndims, dims, periods, reorder, comm_cart,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cart_get(comm, maxdims, dims, periods, coords) MPI_SOURCEINFO_Cart_get(comm, maxdims, dims, periods, coords,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cart_map(comm, ndims, dims, periods, newrank) MPI_SOURCEINFO_Cart_map(comm, ndims, dims, periods, newrank,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cart_rank(comm, coords, rank) MPI_SOURCEINFO_Cart_rank(comm, coords, rank,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cart_shift(comm, direction, disp, rank_source, rank_dest) MPI_SOURCEINFO_Cart_shift(comm, direction, disp, rank_source, rank_dest,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cart_sub(comm, remain_dims, newcomm) MPI_SOURCEINFO_Cart_sub(comm, remain_dims, newcomm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Cartdim_get(comm, ndims) MPI_SOURCEINFO_Cartdim_get(comm, ndims,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_compare(comm1,comm2,result) MPI_SOURCEINFO_Comm_compare(comm1,comm2,result,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_create(comm,group,newcomm) MPI_SOURCEINFO_Comm_create(comm,group,newcomm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_dup(comm,newcomm) MPI_SOURCEINFO_Comm_dup(comm,newcomm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_free(comm) MPI_SOURCEINFO_Comm_free(comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_group(comm,group) MPI_SOURCEINFO_Comm_group(comm,group,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_rank(comm, rank) MPI_SOURCEINFO_Comm_rank(comm, rank,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_remote_group(comm, group) MPI_SOURCEINFO_Comm_remote_group(comm, group,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_remote_size(comm, size) MPI_SOURCEINFO_Comm_remote_size(comm, size,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_size(comm, size) MPI_SOURCEINFO_Comm_size(comm, size,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_split(comm, color, key, newcomm) MPI_SOURCEINFO_Comm_split(comm, color, key, newcomm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Comm_test_inter(comm, flag) MPI_SOURCEINFO_Comm_test_inter(comm, flag,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Dims_create(nnodes,ndims,dims) MPI_SOURCEINFO_Dims_create(nnodes,ndims,dims,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Errhandler_create(function,errhandler) MPI_SOURCEINFO_Errhandler_create(function,errhandler,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Errhandler_free(errhandler) MPI_SOURCEINFO_Errhandler_free(errhandler,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Errhandler_get(comm,errhandler) MPI_SOURCEINFO_Errhandler_get(comm,errhandler,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Errhandler_set(comm,errhandler) MPI_SOURCEINFO_Errhandler_set(comm,errhandler,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Error_class(errorcode,errorclass) MPI_SOURCEINFO_Error_class(errorcode,errorclass,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Error_string(errorcode,string,resultlen) MPI_SOURCEINFO_Error_string(errorcode,string,resultlen,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Finalize() MPI_SOURCEINFO_Finalize(__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Gather(sendbuf,sendcount,sendtype,recvbuf,recvcount,recvtype,root,comm) MPI_SOURCEINFO_Gather(sendbuf,sendcount,sendtype,recvbuf,recvcount,recvtype,root,comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Gatherv(sendbuf,sendcount,sendtype,recvbuf,recvcounts,displs,recvtype,root,comm) MPI_SOURCEINFO_Gatherv(sendbuf,sendcount,sendtype,recvbuf,recvcounts,displs,recvtype,root,comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Get_count(status,datatype,count) MPI_SOURCEINFO_Get_count(status,datatype,count,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Get_elements(status,datatype,count) MPI_SOURCEINFO_Get_elements(status,datatype,count,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Get_processor_name(name,resultlen) MPI_SOURCEINFO_Get_processor_name(name,resultlen,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Get_version(version,subversion) MPI_SOURCEINFO_Get_version(version,subversion,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Graph_create(comm_old,nnodes,index,edges,reorder,comm_graph) MPI_SOURCEINFO_Graph_create(comm_old,nnodes,index,edges,reorder,comm_graph,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Graph_get(comm,maxindex,maxedges,index,edges) MPI_SOURCEINFO_Graph_get(comm,maxindex,maxedges,index,edges,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Graph_map(comm,nnodes,index,edges,newrank) MPI_SOURCEINFO_Graph_map(comm,nnodes,index,edges,newrank,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Graph_neighbors(comm,rank,maxneighbors,neighbors) MPI_SOURCEINFO_Graph_neighbors(comm,rank,maxneighbors,neighbors,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Graph_neighbors_count(comm,rank,nneighbors) MPI_SOURCEINFO_Graph_neighbors_count(comm,rank,nneighbors,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Graphdims_get(comm,nnodes,nedges) MPI_SOURCEINFO_Graphdims_get(comm,nnodes,nedges,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_compare(group1,group2,result) MPI_SOURCEINFO_Group_compare(group1,group2,result,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_difference(group1,group2,newgroup) MPI_SOURCEINFO_Group_difference(group1,group2,newgroup,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_excl(group,n,ranks,newgroup) MPI_SOURCEINFO_Group_excl(group,n,ranks,newgroup,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_free(group) MPI_SOURCEINFO_Group_free(group,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_incl(group,n,ranks,newgroup) MPI_SOURCEINFO_Group_incl(group,n,ranks,newgroup,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_intersection(group1,group2,newgroup) MPI_SOURCEINFO_Group_intersection(group1,group2,newgroup,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_range_excl(group,n,ranges,newgroup) MPI_SOURCEINFO_Group_range_excl(group,n,ranges,newgroup,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_range_incl(group,n,ranges,newgroup) MPI_SOURCEINFO_Group_range_incl(group,n,ranges,newgroup,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_rank(group,rank) MPI_SOURCEINFO_Group_rank(group,rank,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_size(group,size) MPI_SOURCEINFO_Group_size(group,size,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_translate_ranks(group1,n,ranks1,group2,ranks2) MPI_SOURCEINFO_Group_translate_ranks(group1,n,ranks1,group2,ranks2,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Group_union(group1,group2,newgroup) MPI_SOURCEINFO_Group_union(group1,group2,newgroup,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Ibsend(buf,count,datatype,dest,tag,comm,request) MPI_SOURCEINFO_Ibsend(buf,count,datatype,dest,tag,comm,request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Init(argc,argv) MPI_SOURCEINFO_Init(argc,argv,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Init_thread(argc,argv,required,provided) MPI_SOURCEINFO_Init_thread(argc,argv,required,provided,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Initialized(flag) MPI_SOURCEINFO_Initialized(flag,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Intercomm_create(local_comm,local_leader,peer_comm,remote_leader,tag,newintercomm) MPI_SOURCEINFO_Intercomm_create(local_comm,local_leader,peer_comm,remote_leader,tag,newintercomm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Intercomm_merge(intercomm,high,newintracomm) MPI_SOURCEINFO_Intercomm_merge(intercomm,high,newintracomm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Iprobe(source,tag,comm,flag,status) MPI_SOURCEINFO_Iprobe(source,tag,comm,flag,status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Irecv(buf,count,datatype,source,tag,comm,request) MPI_SOURCEINFO_Irecv(buf,count,datatype,source,tag,comm,request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Irsend(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Irsend(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Isend(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Isend(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Issend(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Issend(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Keyval_create(copy_fn, delete_fn, keyval, extra_state) MPI_SOURCEINFO_Keyval_create(copy_fn, delete_fn, keyval, extra_state,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Keyval_free(keyval) MPI_SOURCEINFO_Keyval_free(keyval,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Op_create(function, commute, op) MPI_SOURCEINFO_Op_create(function, commute, op,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Op_free( op) MPI_SOURCEINFO_Op_free( op,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm) MPI_SOURCEINFO_Pack(inbuf, incount, datatype, outbuf, outsize, position, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Pack_size(incount,datatype,comm,size) MPI_SOURCEINFO_Pack_size(incount,datatype,comm,size,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Pcontrol(level, others) MPI_SOURCEINFO_Pcontrol(level,__LINE__,__FILE__, others,__LINE__,__FILE__)
#define MPI_Probe(source,tag,comm, status) MPI_SOURCEINFO_Probe(source,tag,comm, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Recv(buf,count,datatype,source,tag,comm,status) MPI_SOURCEINFO_Recv(buf,count,datatype,source,tag,comm,status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Recv_init(buf,count,datatype,source,tag, comm,request) MPI_SOURCEINFO_Recv_init(buf,count,datatype,source,tag, comm,request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Reduce(sendbuf,recvbuf,count,datatype, op,root,comm) MPI_SOURCEINFO_Reduce(sendbuf,recvbuf,count,datatype, op,root,comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Reduce_scatter(sendbuf,recvbuf,recvcounts,datatype,op,comm) MPI_SOURCEINFO_Reduce_scatter(sendbuf,recvbuf,recvcounts,datatype,op,comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Request_free(request) MPI_SOURCEINFO_Request_free(request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Rsend(buf,count, datatype,dest,tag,comm) MPI_SOURCEINFO_Rsend(buf,count, datatype,dest,tag,comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Rsend_init(buf,count,datatype,dest,tag, comm,request) MPI_SOURCEINFO_Rsend_init(buf,count,datatype,dest,tag, comm,request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Scan(sendbuf,recvbuf,count, datatype, op, comm) MPI_SOURCEINFO_Scan(sendbuf,recvbuf,count, datatype, op, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm) MPI_SOURCEINFO_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Scatterv(sendbuf,sendcounts,displs, sendtype, recvbuf, recvcount, recvtype, root, comm) MPI_SOURCEINFO_Scatterv(sendbuf,sendcounts,displs, sendtype, recvbuf, recvcount, recvtype, root, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Send(buf, count,datatype, dest, tag, comm) MPI_SOURCEINFO_Send(buf, count,datatype, dest, tag, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Send_init(buf, count, datatype, dest, tag, comm, request) MPI_SOURCEINFO_Send_init(buf, count, datatype, dest, tag, comm, request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status) MPI_SOURCEINFO_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Sendrecv_replace(buf,count, datatype, dest, sendtag, source, recvtag, comm, status) MPI_SOURCEINFO_Sendrecv_replace(buf,count, datatype, dest, sendtag, source, recvtag, comm, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Ssend(buf, count, datatype, dest, tag, comm) MPI_SOURCEINFO_Ssend(buf, count, datatype, dest, tag, comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Ssend_init(buf,count,datatype,dest,tag,comm,request) MPI_SOURCEINFO_Ssend_init(buf,count,datatype,dest,tag,comm,request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Start(request) MPI_SOURCEINFO_Start(request,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Startall(count,array_of_requests) MPI_SOURCEINFO_Startall(count,array_of_requests,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Test(request, flag, status) MPI_SOURCEINFO_Test(request, flag, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Test_cancelled(status, flag) MPI_SOURCEINFO_Test_cancelled(status, flag,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Testall(count, array_of_requests, flag, array_of_statuses) MPI_SOURCEINFO_Testall(count, array_of_requests, flag, array_of_statuses,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Testany(count, array_of_requests, index, flag, status) MPI_SOURCEINFO_Testany(count, array_of_requests, index, flag, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses) MPI_SOURCEINFO_Testsome(incount, array_of_requests, outcount, array_of_indices, array_of_statuses,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Topo_test(comm, status) MPI_SOURCEINFO_Topo_test(comm, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_commit(datatype) MPI_SOURCEINFO_Type_commit(datatype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_contiguous(count,oldtype,newtype) MPI_SOURCEINFO_Type_contiguous(count,oldtype,newtype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_extent(datatype,extent) MPI_SOURCEINFO_Type_extent(datatype,extent,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_free(datatype) MPI_SOURCEINFO_Type_free(datatype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype) MPI_SOURCEINFO_Type_hindexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_hvector(count, blocklength, stride, oldtype, newtype) MPI_SOURCEINFO_Type_hvector(count, blocklength, stride, oldtype, newtype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype) MPI_SOURCEINFO_Type_indexed(count, array_of_blocklengths, array_of_displacements, oldtype, newtype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_lb(datatype, displacement) MPI_SOURCEINFO_Type_lb(datatype, displacement,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_size(datatype,size) MPI_SOURCEINFO_Type_size(datatype,size,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_struct(count, array_of_blocklengths,array_of_displacements, array_of_types, newtype) MPI_SOURCEINFO_Type_struct(count, array_of_blocklengths,array_of_displacements, array_of_types, newtype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_ub(datatype,displacement) MPI_SOURCEINFO_Type_ub(datatype,displacement,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Type_vector(count, blocklength, stride, oldtype, newtype) MPI_SOURCEINFO_Type_vector(count, blocklength, stride, oldtype, newtype,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Unpack(inbuf, insize, position, outbuf, outcount, datatype,comm) MPI_SOURCEINFO_Unpack(inbuf, insize, position, outbuf, outcount, datatype,comm,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Wait(request, status) MPI_SOURCEINFO_Wait(request, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Waitall(count, array_of_requests, array_of_statuses) MPI_SOURCEINFO_Waitall(count, array_of_requests, array_of_statuses,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Waitany(count, array_of_requests, index, status) MPI_SOURCEINFO_Waitany(count, array_of_requests, index, status,__LINE__,__FILE__,__LINE__,__FILE__)
#define MPI_Waitsome(incount,array_of_requests, outcount,array_of_indices, array_of_statuses) MPI_SOURCEINFO_Waitsome(incount,array_of_requests, outcount,array_of_indices, array_of_statuses,__LINE__,__FILE__,__LINE__,__FILE__)
/*#define MPI_Wtick() MPI_SOURCEINFO_Wtick(__LINE__,__FILE__)*/
/*#define MPI_Wtime() MPI_SOURCEINFO_Wtime(__LINE__,__FILE__)*/

#define MPI_Accumulate( origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win) MPI_SOURCEINFO_Accumulate( origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, op, win,__LINE__,__FILE__)
#define MPI_Get( origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win) MPI_SOURCEINFO_Get( origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win,__LINE__,__FILE__)
#define MPI_Put( origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win) MPI_SOURCEINFO_Put( origin_addr, origin_count, origin_datatype, target_rank, target_disp, target_count, target_datatype, win,__LINE__,__FILE__)
#define MPI_Win_complete( win) MPI_SOURCEINFO_Win_complete( win,__LINE__,__FILE__)
#define MPI_Win_create( base, size, disp_unit, info, comm, win_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Win_create( base, size, disp_unit, info, comm, win_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Win_fence( assert, win) MPI_SOURCEINFO_Win_fence( assert, win,__LINE__,__FILE__)
#define MPI_Win_free( win_CLASS_SINGLE_IO) MPI_SOURCEINFO_Win_free( win_CLASS_SINGLE_IO,__LINE__,__FILE__)
#define MPI_Win_lock( lock_type, rank, assert, win) MPI_SOURCEINFO_Win_lock( lock_type, rank, assert, win,__LINE__,__FILE__)
#define MPI_Win_post( group, assert, win) MPI_SOURCEINFO_Win_post( group, assert, win,__LINE__,__FILE__)
#define MPI_Win_start( group, assert, win) MPI_SOURCEINFO_Win_start( group, assert, win,__LINE__,__FILE__)
#define MPI_Win_test( win, flag) MPI_SOURCEINFO_Win_test( win, flag,__LINE__,__FILE__)
#define MPI_Win_unlock( rank, win) MPI_SOURCEINFO_Win_unlock( rank, win,__LINE__,__FILE__)
#define MPI_Win_wait( win) MPI_SOURCEINFO_Win_wait( win,__LINE__,__FILE__)

#define MPI_Alltoallw( sendbuf, sendcounts, sdispls, sendtypes_CLASS_ARRAY_IN_sendcounts, recvbuf, recvcounts, rdispls, recvtypes_CLASS_ARRAY_IN_recvcounts, comm) MPI_SOURCEINFO_Alltoallw( sendbuf, sendcounts, sdispls, sendtypes_CLASS_ARRAY_IN_sendcounts, recvbuf, recvcounts, rdispls, recvtypes_CLASS_ARRAY_IN_recvcounts, comm,__LINE__,__FILE__)
#define MPI_Exscan( sendbuf, recvbuf, count, datatype, op, comm) MPI_SOURCEINFO_Exscan( sendbuf, recvbuf, count, datatype, op, comm,__LINE__,__FILE__)

#define MPI_File_open( comm, filename_CLASS_SINGLE_IN, amode, info, fh_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_open( comm, filename_CLASS_SINGLE_IN, amode, info, fh_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_close( fh_CLASS_SINGLE_IO) MPI_SOURCEINFO_File_close( fh_CLASS_SINGLE_IO,__LINE__,__FILE__)
#define MPI_File_delete( filename_CLASS_SINGLE_IN, info) MPI_SOURCEINFO_File_delete( filename_CLASS_SINGLE_IN, info,__LINE__,__FILE__)
#define MPI_File_set_size( fh, size) MPI_SOURCEINFO_File_set_size( fh, size,__LINE__,__FILE__)
#define MPI_File_preallocate( fh, size) MPI_SOURCEINFO_File_preallocate( fh, size,__LINE__,__FILE__)
#define MPI_File_get_size( fh, size_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_size( fh, size_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_get_group( fh, group_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_group( fh, group_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_get_amode( fh, amode) MPI_SOURCEINFO_File_get_amode( fh, amode,__LINE__,__FILE__)
#define MPI_File_set_info( fh, info) MPI_SOURCEINFO_File_set_info( fh, info,__LINE__,__FILE__)
#define MPI_File_get_info( fh, info_used_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_info( fh, info_used_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_set_view( fh, disp, etype, filetype, datarep_CLASS_SINGLE_IN, info) MPI_SOURCEINFO_File_set_view( fh, disp, etype, filetype, datarep_CLASS_SINGLE_IN, info,__LINE__,__FILE__)
#define MPI_File_get_view( fh, disp_CLASS_SINGLE_OUT, etype_CLASS_SINGLE_OUT, filetype_CLASS_SINGLE_OUT, datarep_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_view( fh, disp_CLASS_SINGLE_OUT, etype_CLASS_SINGLE_OUT, filetype_CLASS_SINGLE_OUT, datarep_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_at( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_at( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_at_all( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_at_all( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_at( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_at( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_at_all( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_at_all( fh, offset, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_iread_at( fh, offset, buf, count, datatype, request_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_iread_at( fh, offset, buf, count, datatype, request_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_iwrite_at( fh, offset, buf, count, datatype, request_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_iwrite_at( fh, offset, buf, count, datatype, request_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_all( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_all( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_all( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_all( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_iread( fh, buf, count, datatype, request_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_iread( fh, buf, count, datatype, request_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_iwrite( fh, buf, count, datatype, request_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_iwrite( fh, buf, count, datatype, request_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_seek( fh, offset, whence) MPI_SOURCEINFO_File_seek( fh, offset, whence,__LINE__,__FILE__)
#define MPI_File_get_position( fh, offset_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_position( fh, offset_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_get_byte_offset( fh, offset, disp_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_byte_offset( fh, offset, disp_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_shared( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_shared( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_shared( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_shared( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_iread_shared( fh, buf, count, datatype, request_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_iread_shared( fh, buf, count, datatype, request_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_iwrite_shared( fh, buf, count, datatype, request_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_iwrite_shared( fh, buf, count, datatype, request_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_ordered( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_ordered( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_ordered( fh, buf, count, datatype, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_ordered( fh, buf, count, datatype, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_seek_shared( fh, offset, whence) MPI_SOURCEINFO_File_seek_shared( fh, offset, whence,__LINE__,__FILE__)
#define MPI_File_get_position_shared( fh, offset_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_position_shared( fh, offset_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_at_all_begin( fh, offset, buf, count, datatype) MPI_SOURCEINFO_File_read_at_all_begin( fh, offset, buf, count, datatype,__LINE__,__FILE__)
#define MPI_File_read_at_all_end( fh, buf, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_at_all_end( fh, buf, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_at_all_begin( fh, offset, buf, count, datatype) MPI_SOURCEINFO_File_write_at_all_begin( fh, offset, buf, count, datatype,__LINE__,__FILE__)
#define MPI_File_write_at_all_end( fh, buf, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_at_all_end( fh, buf, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_all_begin( fh, buf, count, datatype) MPI_SOURCEINFO_File_read_all_begin( fh, buf, count, datatype,__LINE__,__FILE__)
#define MPI_File_read_all_end( fh, buf, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_all_end( fh, buf, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_all_begin( fh, buf, count, datatype) MPI_SOURCEINFO_File_write_all_begin( fh, buf, count, datatype,__LINE__,__FILE__)
#define MPI_File_write_all_end( fh, buf, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_all_end( fh, buf, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_read_ordered_begin( fh, buf, count, datatype) MPI_SOURCEINFO_File_read_ordered_begin( fh, buf, count, datatype,__LINE__,__FILE__)
#define MPI_File_read_ordered_end( fh, buf, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_read_ordered_end( fh, buf, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_write_ordered_begin( fh, buf, count, datatype) MPI_SOURCEINFO_File_write_ordered_begin( fh, buf, count, datatype,__LINE__,__FILE__)
#define MPI_File_write_ordered_end( fh, buf, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_write_ordered_end( fh, buf, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_get_type_extent( fh, datatype, extent_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_type_extent( fh, datatype, extent_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_set_atomicity( fh, flag) MPI_SOURCEINFO_File_set_atomicity( fh, flag,__LINE__,__FILE__)
#define MPI_File_get_atomicity( fh, flag) MPI_SOURCEINFO_File_get_atomicity( fh, flag,__LINE__,__FILE__)
#define MPI_File_sync( fh) MPI_SOURCEINFO_File_sync( fh,__LINE__,__FILE__)
#define MPI_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state) MPI_SOURCEINFO_Register_datarep(datarep, read_conversion_fn, write_conversion_fn, dtype_file_extent_fn, extra_state,__LINE__,__FILE__)

#define MPI_Close_port( port_name_CLASS_SINGLE_IN) MPI_SOURCEINFO_Close_port( port_name_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Comm_accept( port_name_CLASS_SINGLE_IN, info, root, comm, newcomm_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Comm_accept( port_name_CLASS_SINGLE_IN, info, root, comm, newcomm_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Comm_connect( port_name_CLASS_SINGLE_IN, info, root, comm, newcomm_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Comm_connect( port_name_CLASS_SINGLE_IN, info, root, comm, newcomm_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Comm_disconnect( comm_CLASS_SINGLE_IO) MPI_SOURCEINFO_Comm_disconnect( comm_CLASS_SINGLE_IO,__LINE__,__FILE__)
#define MPI_Comm_get_parent( parent_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Comm_get_parent( parent_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Comm_join( fd, intercomm_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Comm_join( fd, intercomm_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Comm_spawn( command_CLASS_SINGLE_IN, argv, maxprocs, info, root, comm, intercomm_CLASS_SINGLE_OUT, array_of_errcodes) MPI_SOURCEINFO_Comm_spawn( command_CLASS_SINGLE_IN, argv, maxprocs, info, root, comm, intercomm_CLASS_SINGLE_OUT, array_of_errcodes,__LINE__,__FILE__)
#define MPI_Comm_spawn_multiple( count, array_of_commands, array_of_argv, array_of_maxprocs, array_of_info_CLASS_ARRAY_IN_count, root, comm, intercomm_CLASS_SINGLE_OUT, array_of_errcodes) MPI_SOURCEINFO_Comm_spawn_multiple( count, array_of_commands, array_of_argv, array_of_maxprocs, array_of_info_CLASS_ARRAY_IN_count, root, comm, intercomm_CLASS_SINGLE_OUT, array_of_errcodes,__LINE__,__FILE__)
#define MPI_Lookup_name( service_name_CLASS_SINGLE_IN, info, port_name_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Lookup_name( service_name_CLASS_SINGLE_IN, info, port_name_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Open_port( info, port_name_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Open_port( info, port_name_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Publish_name( service_name_CLASS_SINGLE_IN, info, port_name_CLASS_SINGLE_IN) MPI_SOURCEINFO_Publish_name( service_name_CLASS_SINGLE_IN, info, port_name_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Unpublish_name( service_name_CLASS_SINGLE_IN, info, port_name_CLASS_SINGLE_IN) MPI_SOURCEINFO_Unpublish_name( service_name_CLASS_SINGLE_IN, info, port_name_CLASS_SINGLE_IN,__LINE__,__FILE__)

#define MPI_Alloc_mem( size, info, baseptr) MPI_SOURCEINFO_Alloc_mem( size, info, baseptr,__LINE__,__FILE__)
#define MPI_Comm_create_errhandler( function, errhandler_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Comm_create_errhandler( function, errhandler_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Comm_get_errhandler( comm, errhandler_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Comm_get_errhandler( comm, errhandler_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Comm_set_errhandler( comm, errhandler) MPI_SOURCEINFO_Comm_set_errhandler( comm, errhandler,__LINE__,__FILE__)
#define MPI_Finalized( flag) MPI_SOURCEINFO_Finalized( flag,__LINE__,__FILE__)
#define MPI_Free_mem( base) MPI_SOURCEINFO_Free_mem( base,__LINE__,__FILE__)
#define MPI_Get_address( location, address_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Get_address( location, address_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Info_create( info_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Info_create( info_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Info_delete( info, key_CLASS_SINGLE_IN) MPI_SOURCEINFO_Info_delete( info, key_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Info_dup( info, newinfo_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Info_dup( info, newinfo_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Info_free( info_CLASS_SINGLE_IO) MPI_SOURCEINFO_Info_free( info_CLASS_SINGLE_IO,__LINE__,__FILE__)
#define MPI_Info_get( info, key_CLASS_SINGLE_IN, valuelen, value_CLASS_SINGLE_OUT, flag) MPI_SOURCEINFO_Info_get( info, key_CLASS_SINGLE_IN, valuelen, value_CLASS_SINGLE_OUT, flag,__LINE__,__FILE__)
#define MPI_Info_get_nkeys( info, nkeys) MPI_SOURCEINFO_Info_get_nkeys( info, nkeys,__LINE__,__FILE__)
#define MPI_Info_get_nthkey( info, n, key_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Info_get_nthkey( info, n, key_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Info_get_valuelen( info, key_CLASS_SINGLE_IN, valuelen, flag) MPI_SOURCEINFO_Info_get_valuelen( info, key_CLASS_SINGLE_IN, valuelen, flag,__LINE__,__FILE__)
#define MPI_Info_set( info, key_CLASS_SINGLE_IN, value_CLASS_SINGLE_IN) MPI_SOURCEINFO_Info_set( info, key_CLASS_SINGLE_IN, value_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Pack_external( datarep_CLASS_SINGLE_IN, inbuf, incount, datatype, outbuf, outsize, position_CLASS_SINGLE_IO) MPI_SOURCEINFO_Pack_external( datarep_CLASS_SINGLE_IN, inbuf, incount, datatype, outbuf, outsize, position_CLASS_SINGLE_IO,__LINE__,__FILE__)
#define MPI_Pack_external_size( datarep_CLASS_SINGLE_IN, incount, datatype, size_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Pack_external_size( datarep_CLASS_SINGLE_IN, incount, datatype, size_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Request_get_status( request, flag, status_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Request_get_status( request, flag, status_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_darray( size, rank, ndims, array_of_gsizes, array_of_distribs, array_of_dargs, array_of_psizes, order, oldtype, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_darray( size, rank, ndims, array_of_gsizes, array_of_distribs, array_of_dargs, array_of_psizes, order, oldtype, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_hindexed( count, array_of_blocklengths, array_of_displacements_CLASS_ARRAY_IN_count, oldtype, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_hindexed( count, array_of_blocklengths, array_of_displacements_CLASS_ARRAY_IN_count, oldtype, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_hvector( count, blocklength, stride, oldtype, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_hvector( count, blocklength, stride, oldtype, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_indexed_block( count, blocklength, array_of_displacements, oldtype, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_indexed_block( count, blocklength, array_of_displacements, oldtype, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_resized( oldtype, lb, extent, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_resized( oldtype, lb, extent, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_struct( count, array_of_blocklengths, array_of_displacements_CLASS_ARRAY_IN_count, array_of_types_CLASS_ARRAY_IN_count, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_struct( count, array_of_blocklengths, array_of_displacements_CLASS_ARRAY_IN_count, array_of_types_CLASS_ARRAY_IN_count, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_subarray( ndims, array_of_sizes, array_of_subsizes, array_of_starts, order, oldtype, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_subarray( ndims, array_of_sizes, array_of_subsizes, array_of_starts, order, oldtype, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_get_extent( datatype, lb_CLASS_SINGLE_OUT, extent_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_get_extent( datatype, lb_CLASS_SINGLE_OUT, extent_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_get_true_extent( datatype, true_lb_CLASS_SINGLE_OUT, true_extent_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_get_true_extent( datatype, true_lb_CLASS_SINGLE_OUT, true_extent_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Unpack_external( datarep_CLASS_SINGLE_IN, inbuf, insize, position_CLASS_SINGLE_IO, outbuf, outcount, datatype) MPI_SOURCEINFO_Unpack_external( datarep_CLASS_SINGLE_IN, inbuf, insize, position_CLASS_SINGLE_IO, outbuf, outcount, datatype,__LINE__,__FILE__)
#define MPI_Win_create_errhandler( function, errhandler_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Win_create_errhandler( function, errhandler_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Win_get_errhandler( win, errhandler_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Win_get_errhandler( win, errhandler_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Win_set_errhandler( win, errhandler) MPI_SOURCEINFO_Win_set_errhandler( win, errhandler,__LINE__,__FILE__)
#define MPI_File_call_errhandler( fh, errorcode) MPI_SOURCEINFO_File_call_errhandler( fh, errorcode,__LINE__,__FILE__)
#define MPI_File_create_errhandler( function_CLASS_SINGLE_IN, errhandler_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_create_errhandler( function_CLASS_SINGLE_IN, errhandler_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_get_errhandler( file, errhandler_CLASS_SINGLE_OUT) MPI_SOURCEINFO_File_get_errhandler( file, errhandler_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_File_set_errhandler( file, errhandler) MPI_SOURCEINFO_File_set_errhandler( file, errhandler,__LINE__,__FILE__)

/* External Interfaces */
#define MPI_Add_error_class( errorclass) MPI_SOURCEINFO_Add_error_class( errorclass,__LINE__,__FILE__)
#define MPI_Add_error_code( errorclass, errorcode) MPI_SOURCEINFO_Add_error_code( errorclass, errorcode,__LINE__,__FILE__)
#define MPI_Add_error_string( errorcode, string_CLASS_SINGLE_IN) MPI_SOURCEINFO_Add_error_string( errorcode, string_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Comm_call_errhandler( comm, errorcode) MPI_SOURCEINFO_Comm_call_errhandler( comm, errorcode,__LINE__,__FILE__)
#define MPI_Comm_create_keyval( comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state) MPI_SOURCEINFO_Comm_create_keyval( comm_copy_attr_fn, comm_delete_attr_fn, comm_keyval, extra_state,__LINE__,__FILE__)
#define MPI_Comm_delete_attr( comm, comm_keyval) MPI_SOURCEINFO_Comm_delete_attr( comm, comm_keyval,__LINE__,__FILE__)
#define MPI_Comm_free_keyval( comm_keyval) MPI_SOURCEINFO_Comm_free_keyval( comm_keyval,__LINE__,__FILE__)
#define MPI_Comm_get_attr( comm, comm_keyval, attribute_val, flag) MPI_SOURCEINFO_Comm_get_attr( comm, comm_keyval, attribute_val, flag,__LINE__,__FILE__)
#define MPI_Comm_get_name( comm, comm_name_CLASS_SINGLE_OUT, resultlen) MPI_SOURCEINFO_Comm_get_name( comm, comm_name_CLASS_SINGLE_OUT, resultlen,__LINE__,__FILE__)
#define MPI_Comm_set_attr( comm, comm_keyval, attribute_val) MPI_SOURCEINFO_Comm_set_attr( comm, comm_keyval, attribute_val,__LINE__,__FILE__)
#define MPI_Comm_set_name( comm, comm_name_CLASS_SINGLE_IN) MPI_SOURCEINFO_Comm_set_name( comm, comm_name_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Grequest_complete( request) MPI_SOURCEINFO_Grequest_complete( request,__LINE__,__FILE__)
#define MPI_Grequest_start( query_fn, free_fn, cancel_fn, extra_state, request_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Grequest_start( query_fn, free_fn, cancel_fn, extra_state, request_CLASS_SINGLE_OUT,__LINE__,__FILE__)
/*#define MPI_Init_thread( argc, argv, required, provided) MPI_SOURCEINFO_Init_thread( argc, argv, required, provided)*/ /*Stays in mpi_standard.h for historic reasons!*/
#define MPI_Is_thread_main( flag) MPI_SOURCEINFO_Is_thread_main( flag,__LINE__,__FILE__)
#define MPI_Query_thread( provided) MPI_SOURCEINFO_Query_thread( provided,__LINE__,__FILE__)
#define MPI_Status_set_cancelled( status_CLASS_SINGLE_IO, flag) MPI_SOURCEINFO_Status_set_cancelled( status_CLASS_SINGLE_IO, flag,__LINE__,__FILE__)
#define MPI_Status_set_elements( status_CLASS_SINGLE_IO, datatype, count) MPI_SOURCEINFO_Status_set_elements( status_CLASS_SINGLE_IO, datatype, count,__LINE__,__FILE__)
#define MPI_Type_create_keyval( type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state) MPI_SOURCEINFO_Type_create_keyval( type_copy_attr_fn, type_delete_attr_fn, type_keyval, extra_state,__LINE__,__FILE__)
#define MPI_Type_delete_attr( type, type_keyval) MPI_SOURCEINFO_Type_delete_attr( type, type_keyval,__LINE__,__FILE__)
#define MPI_Type_dup( type, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_dup( type, newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_free_keyval( type_keyval) MPI_SOURCEINFO_Type_free_keyval( type_keyval,__LINE__,__FILE__)
#define MPI_Type_get_attr( type, type_keyval, attribute_val, flag) MPI_SOURCEINFO_Type_get_attr( type, type_keyval, attribute_val, flag,__LINE__,__FILE__)
#define MPI_Type_get_contents( datatype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses_CLASS_ARRAY_OUT_max_addresses, array_of_datatypes_CLASS_ARRAY_OUT_max_datatypes) MPI_SOURCEINFO_Type_get_contents( datatype, max_integers, max_addresses, max_datatypes, array_of_integers, array_of_addresses_CLASS_ARRAY_OUT_max_addresses, array_of_datatypes_CLASS_ARRAY_OUT_max_datatypes,__LINE__,__FILE__)
#define MPI_Type_get_envelope( datatype, num_integers, num_addresses, num_datatypes, combiner) MPI_SOURCEINFO_Type_get_envelope( datatype, num_integers, num_addresses, num_datatypes, combiner,__LINE__,__FILE__)
#define MPI_Type_get_name( type, type_name_CLASS_SINGLE_OUT, resultlen) MPI_SOURCEINFO_Type_get_name( type, type_name_CLASS_SINGLE_OUT, resultlen,__LINE__,__FILE__)
#define MPI_Type_set_attr( type, type_keyval, attribute_val) MPI_SOURCEINFO_Type_set_attr( type, type_keyval, attribute_val,__LINE__,__FILE__)
#define MPI_Type_set_name( type, type_name_CLASS_SINGLE_IN) MPI_SOURCEINFO_Type_set_name( type, type_name_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Win_call_errhandler( win, errorcode) MPI_SOURCEINFO_Win_call_errhandler( win, errorcode,__LINE__,__FILE__)
#define MPI_Win_create_keyval( win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state) MPI_SOURCEINFO_Win_create_keyval( win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state,__LINE__,__FILE__)
#define MPI_Win_delete_attr( win, win_keyval) MPI_SOURCEINFO_Win_delete_attr( win, win_keyval,__LINE__,__FILE__)
#define MPI_Win_free_keyval( win_keyval) MPI_SOURCEINFO_Win_free_keyval( win_keyval,__LINE__,__FILE__)
#define MPI_Win_get_attr( win, win_keyval, attribute_val, flag) MPI_SOURCEINFO_Win_get_attr( win, win_keyval, attribute_val, flag,__LINE__,__FILE__)
#define MPI_Win_get_name( win, win_name_CLASS_SINGLE_OUT, resultlen) MPI_SOURCEINFO_Win_get_name( win, win_name_CLASS_SINGLE_OUT, resultlen,__LINE__,__FILE__)
#define MPI_Win_set_attr( win, win_keyval, attribute_val) MPI_SOURCEINFO_Win_set_attr( win, win_keyval, attribute_val,__LINE__,__FILE__)
#define MPI_Win_set_name( win, win_name_CLASS_SINGLE_IN) MPI_SOURCEINFO_Win_set_name( win, win_name_CLASS_SINGLE_IN,__LINE__,__FILE__)
#define MPI_Type_create_f90_complex( p,  r, newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_f90_complex(p, r,  newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_f90_integer( r,  newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_f90_integer( r,  newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_create_f90_real(p, r,  newtype_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_create_f90_real( p,  r,  newtype_CLASS_SINGLE_OUT,__LINE__,__FILE__)
#define MPI_Type_match_size( typeclass,  size,  type_CLASS_SINGLE_OUT) MPI_SOURCEINFO_Type_match_size( typeclass,  size,  type_CLASS_SINGLE_OUT,__LINE__,__FILE__)
