/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  MARMOT public interface
 *
 *  @author Julian Stecklina, Tobias Hilbrich
 *
 *  $Id: marmot-interface.h 903 2008-12-12 12:19:28Z tobias $
 */
#ifndef __MARMOT_INTERFACE_H_
#define __MARMOT_INTERFACE_H_

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Describes all types of messages.
 */
typedef enum
{
    MARMOT_MSG_TYPE_INFO = 0,
    MARMOT_MSG_TYPE_WARNING,
    MARMOT_MSG_TYPE_ERROR
} MARMOT_MSG_TYPE;

/**
 * Describes a message returned by Marmots public interface.
 * Free it with marmot_free_msg.
 * @see marmot_free_msg
 */
typedef struct
{
    /** The Type of the message, @see UNIMCI_MSG_TYPE. */
    MARMOT_MSG_TYPE       msgType;

    char                  *strCallInfo, /**< String describing the MPI call. */
                          *strText, /**< String that contains the message text. */
                          *strHTMLReference; /**< String that contains the address of a HTML site with MPI standard information. */

    int                   isGlobal /**< True if this is a global message (involves more than just one rank). */,
                          numParticipatingRanks, /**< If 'isGlobal', number of ranks that are involved in the global message. */
                          *participatingRanks; /**< If 'isGlobal', ranks of the involved ranks. */

    unsigned long long    eventDescriptor; /**< The id that was given to the UniMCI check call that triggered this message. */
} MARMOT_MSG;

/* An enum that describes MARMOT's current mode of operation.
 */
typedef enum {
	/* Normal operation. Intercept MPI calls and act upon them. */
	MARMOT_STATE_NORMAL,
	/* Just pass MPI calls through. Useful for implementing timing
	 * sensitive parts in other wrappers. */
	MARMOT_STATE_IGNORE
} marmot_state;

void marmot_set_state(marmot_state);

/**
 * Returns whether Marmot has a not yet queried
 * message.
 * @return 1 if a outstanding message exists,
 *         0 otherwise.
 */
int marmot_has_msg (void);

/**
 * Returns number of outstanding Marmot messages.
 * @return number of outstanding messages.
 */
int marmot_get_num_msgs (void);

/**
 * Returns the first not yet queried Marmot msg.
 * @param marmot_msg address that is set to a pointer
 *        containing the retrieved Marmot msg.
 * @return 0 if successful, 1 otherwise.
 */
int marmot_pop_msg (MARMOT_MSG **marmot_msg);

/**
 * @brief Sets whether the Fortran MPI binding is
 *        currently used or not.
 * @param yes 1 if Fortran binding is used, 0 otherwise (for C).
 * @return 0 if successful, 1 otherwise.
 */
int marmot_set_fortran (int yes);

/**
 * Frees a Marmot msg.
 * @param marmot_msg the message to be freed.
 * @return 0 if successful, 1 otherwise.
 */
int marmot_free_msg (MARMOT_MSG **marmot_msg);

#ifdef __cplusplus
}
#endif

#endif	/* __MARMOT_INTERFACE_H_ */
