/**
 * MPI-calls of MPI-2-standard (One-Sided Communications)
 *
 * @see MPI-2 standard
 *
 * @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *        Matthias Jurenz
 *
  * \note For Fortran we need additional information for the arguments.
 * 		 That is necessary for the handle conversion for MPI-Implementations like
 * 		 OpenMPI.
 * 		 We need for all MPI-Handles of pointer type information how the argument
 * 		 is used. That means we have to know whether it is:
 * 			* an out single value
 * 			* an in-out single value
 * 			* an array for input purpoeses
 * 			* an out array for output purposses
 * 			* an in-out array
 * 		For the arrays we additionally need to know their size !
 * 		We have to denote all this in the argument name, so we will append at the
 * 		end of these argument names:
 * 			* _CLASS_SINGLE_IN for an in single value (this is often used for requests)
 * 			* _CLASS_SINGLE_OUT for an out single value
 * 			* _CLASS_SINGLE_IO for an in-out single value
 * 			* _CLASS_ARRAY_IN_sizeargument an array for input purpoeses
 * 			* _CLASS_ARRAY_OUT_sizeargument an out array for output purposses
 * 			* _CLASS_ARRAY_IO_sizeargument an in-out array
 * 		Where "sizeargument" is the fixed numeric size or the argument speci-
 * 		fing the array size.
 * 		Otherwise we would have to create the wrapper manually.
 *
 * \note For Fortran we have to modify indizes into arrays.
 * 	     As Fortran arrays start with 1 and C arrays with 0.
 *       This affects the MPI-1.2 functions MPI_Wait[some,any] and MPI_Test[some,any].
 *       We define the following definitions that can be appended to an argument name:
 *          * _CLASS_ARRAYINDEX_OUT_SINGLE_NOCONDITION
 *             -> for a single index into an array which is used as a output argument
 *          * _CLASS_ARRAYINDEX_OUT_SINGLE_CONDITION_conditionargument
 *             -> for a single index into an array which is used as a output argument
 *                but only if the argument "*conditionargument" is true
 *                As this is a output value we assume conditionargument also to be a
 *                output argument !
 *          * _CLASS_ARRAYINDEX_OUT_ARRAY_sizeargument
 *             -> for an array of indices into a array which is used as a output argument
 *                the size of the index array is set by "*sizeargument"
 *                As this is a output value we assume sizeargument also to be a output
 *                argument !
 *       These definitions are only a subset of the imaginable cases and the ones currently
 *       needed for the MPI-1.2 functions.
 *
 **/

int MPI_Accumulate(void* origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
int MPI_Get(void* origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
int MPI_Put(void* origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
int MPI_Win_complete(MPI_Win win);
int MPI_Win_create(void* base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win* win_CLASS_SINGLE_OUT);
int MPI_Win_fence(int assert, MPI_Win win);
int MPI_Win_free(MPI_Win* win_CLASS_SINGLE_IO);
int MPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win);
int MPI_Win_post(MPI_Group group, int assert, MPI_Win win);
int MPI_Win_start(MPI_Group group, int assert, MPI_Win win);
int MPI_Win_test(MPI_Win win, int* flag);
int MPI_Win_unlock(int rank, MPI_Win win);
int MPI_Win_wait(MPI_Win win);
