/**
 * MPI-calls of MPI-2-standard (Extended Collective Operations)
 *
 * @see MPI-2 standard
 *
 * @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *        Matthias Jurenz
 *
  * \note For Fortran we need additional information for the arguments.
 * 		 That is necessary for the handle conversion for MPI-Implementations like
 * 		 OpenMPI.
 * 		 We need for all MPI-Handles of pointer type information how the argument
 * 		 is used. That means we have to know whether it is:
 * 			* an out single value
 * 			* an in-out single value
 * 			* an array for input purpoeses
 * 			* an out array for output purposses
 * 			* an in-out array
 * 		For the arrays we additionally need to know their size !
 * 		We have to denote all this in the argument name, so we will append at the
 * 		end of these argument names:
 * 			* _CLASS_SINGLE_IN for an in single value (this is often used for requests)
 * 			* _CLASS_SINGLE_OUT for an out single value
 * 			* _CLASS_SINGLE_IO for an in-out single value
 * 			* _CLASS_ARRAY_IN_sizeargument an array for input purpoeses
 * 			* _CLASS_ARRAY_OUT_sizeargument an out array for output purposses
 * 			* _CLASS_ARRAY_IO_sizeargument an in-out array
 * 		Where "sizeargument" is the fixed numeric size or the argument speci-
 * 		fing the array size.
 * 		Otherwise we would have to create the wrapper manually.
 *
 * \note For Fortran we have to modify indizes into arrays.
 * 	     As Fortran arrays start with 1 and C arrays with 0.
 *       This affects the MPI-1.2 functions MPI_Wait[some,any] and MPI_Test[some,any].
 *       We define the following definitions that can be appended to an argument name:
 *          * _CLASS_ARRAYINDEX_OUT_SINGLE_NOCONDITION
 *             -> for a single index into an array which is used as a output argument
 *          * _CLASS_ARRAYINDEX_OUT_SINGLE_CONDITION_conditionargument
 *             -> for a single index into an array which is used as a output argument
 *                but only if the argument "*conditionargument" is true
 *                As this is a output value we assume conditionargument also to be a
 *                output argument !
 *          * _CLASS_ARRAYINDEX_OUT_ARRAY_sizeargument
 *             -> for an array of indices into a array which is used as a output argument
 *                the size of the index array is set by "*sizeargument"
 *                As this is a output value we assume sizeargument also to be a output
 *                argument !
 *       These definitions are only a subset of the imaginable cases and the ones currently
 *       needed for the MPI-1.2 functions.
 *
 **/

int MPI_Alltoallw(void* sendbuf, int* sendcounts, int* sdispls, MPI_Datatype* sendtypes_CLASS_ARRAY_IN_sendcounts, void* recvbuf, int* recvcounts, int* rdispls, MPI_Datatype* recvtypes_CLASS_ARRAY_IN_recvcounts, MPI_Comm comm);
int MPI_Exscan(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
