/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  MPI-calls of MPI-1.2-standard (complete C language binding.)
 *
 *
 *  @see MPI-1.2 standard
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 * \note For Fortran we need additional information for the arguments.
 * 		 That is necessary for the handle conversion for MPI-Implementations like
 * 		 OpenMPI.
 * 		 We need for all MPI-Handles of pointer type information how the argument
 * 		 is used. That means we have to know whether it is:
 * 			* an out single value
 * 			* an in-out single value
 * 			* an array for input purpoeses
 * 			* an out array for output purposses
 * 			* an in-out array
 * 		For the arrays we additionally need to know their size !
 * 		We have to denote all this in the argument name, so we will append at the
 * 		end of these argument names:
 * 			* _CLASS_SINGLE_IN for an in single value (this is often used for requests)
 * 			* _CLASS_SINGLE_OUT for an out single value
 * 			* _CLASS_SINGLE_IO for an in-out single value
 * 			* _CLASS_ARRAY_IN_sizeargument an array for input purpoeses
 * 			* _CLASS_ARRAY_OUT_sizeargument an out array for output purposses
 * 			* _CLASS_ARRAY_IO_sizeargument an in-out array
 * 		Where "sizeargument" is the fixed numeric size or the argument speci-
 * 		fing the array size.
 * 		Otherwise we would have to create the wrapper manually.
 *
 * \note For Fortran we have to modify indizes into arrays.
 * 	     As Fortran arrays start with 1 and C arrays with 0.
 *       This affects the MPI-1.2 functions MPI_Wait[some,any] and MPI_Test[some,any].
 *       We define the following definitions that can be appended to an argument name:
 *          * _CLASS_ARRAYINDEX_OUT_SINGLE_NOCONDITION
 *             -> for a single index into an array which is used as a output argument
 *          * _CLASS_ARRAYINDEX_OUT_SINGLE_CONDITION_conditionargument
 *             -> for a single index into an array which is used as a output argument
 *                but only if the argument "*conditionargument" is true
 *                As this is a output value we assume conditionargument also to be a
 *                output argument !
 *          * _CLASS_ARRAYINDEX_OUT_ARRAY_sizeargument
 *             -> for an array of indices into a array which is used as a output argument
 *                the size of the index array is set by "*sizeargument"
 *                As this is a output value we assume sizeargument also to be a output
 *                argument !
 *       These definitions are only a subset of the imaginable cases and the ones currently
 *       needed for the MPI-1.2 functions.
 *
 *  $Id: mpi_standard.h 1119 2010-05-10 08:52:10Z tobias $
 */

//=========================================================================
//                          MPI-1.2 standard
//=========================================================================

int MPI_Abort(MPI_Comm comm, int errorcode);
int MPI_Address(void* location, MPI_Aint* address_CLASS_SINGLE_OUT);
int MPI_Allgather(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
int MPI_Allgatherv(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype, MPI_Comm comm);
int MPI_Allreduce(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
int MPI_Alltoall(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
int MPI_Alltoallv(void* sendbuf, int* sendcounts, int* sdispls, MPI_Datatype sendtype, void* recvbuf, int* recvcounts, int* rdispls, MPI_Datatype recvtype, MPI_Comm comm);
int MPI_Attr_delete(MPI_Comm comm, int keyval);
int MPI_Attr_get(MPI_Comm comm, int keyval, void* attribute_val, int* flag);
int MPI_Attr_put(MPI_Comm comm, int keyval, void* attribute_val);
int MPI_Barrier(MPI_Comm comm );
int MPI_Bcast(void* buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm );
int MPI_Bsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
int MPI_Bsend_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Buffer_attach( void* buffer, int size);
int MPI_Buffer_detach( void* buffer, int* size);
int MPI_Cancel(MPI_Request* request_CLASS_SINGLE_IN);
int MPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int* coords);
int MPI_Cart_create(MPI_Comm comm_old, int ndims, int* dims, int* periods, int reorder, MPI_Comm* comm_cart_CLASS_SINGLE_OUT);
int MPI_Cart_get(MPI_Comm comm, int maxdims, int* dims, int* periods, int* coords);
int MPI_Cart_map(MPI_Comm comm, int ndims, int* dims, int* periods, int* newrank);
int MPI_Cart_rank(MPI_Comm comm, int* coords, int* rank);
int MPI_Cart_shift(MPI_Comm comm, int direction, int disp, int* rank_source, int* rank_dest);
int MPI_Cart_sub(MPI_Comm comm, int* remain_dims, MPI_Comm* newcomm_CLASS_SINGLE_OUT);
int MPI_Cartdim_get(MPI_Comm comm, int* ndims);
int MPI_Comm_compare(MPI_Comm comm1,MPI_Comm comm2, int* result);
int MPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm* newcomm_CLASS_SINGLE_OUT);
int MPI_Comm_dup(MPI_Comm comm, MPI_Comm* newcomm_CLASS_SINGLE_OUT);
int MPI_Comm_free(MPI_Comm* comm_CLASS_SINGLE_IO);
int MPI_Comm_group(MPI_Comm comm, MPI_Group* group_CLASS_SINGLE_OUT);
int MPI_Comm_rank(MPI_Comm comm, int* rank);
int MPI_Comm_remote_group(MPI_Comm comm, MPI_Group* group_CLASS_SINGLE_OUT);
int MPI_Comm_remote_size(MPI_Comm comm, int* size);
int MPI_Comm_size(MPI_Comm comm, int* size);
int MPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm* newcomm_CLASS_SINGLE_OUT);
int MPI_Comm_test_inter(MPI_Comm comm, int* flag);
int MPI_Dims_create(int nnodes, int ndims, int* dims);
int MPI_Errhandler_create(MPI_Handler_function* function, MPI_Errhandler* errhandler);
int MPI_Errhandler_free(MPI_Errhandler* errhandler);
int MPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler* errhandler);
int MPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler);
int MPI_Error_class(int errorcode, int* errorclass);
int MPI_Error_string(int errorcode, char* string_CLASS_SINGLE_OUT, int* resultlen);
int MPI_Finalize(void);
int MPI_Gather(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
int MPI_Gatherv(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int* recvcounts, int* displs, MPI_Datatype recvtype, int root, MPI_Comm comm);
int MPI_Get_count(MPI_Status* status_CLASS_SINGLE_IN, MPI_Datatype datatype, int* count);
int MPI_Get_elements(MPI_Status* status_CLASS_SINGLE_IN, MPI_Datatype datatype, int* count);
int MPI_Get_processor_name(char* name_CLASS_SINGLE_OUT, int* resultlen);
int MPI_Get_version(int* version, int* subversion);
int MPI_Graph_create(MPI_Comm comm_old, int nnodes, int* index, int* edges, int reorder, MPI_Comm* comm_graph_CLASS_SINGLE_OUT);
int MPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int* index, int* edges);
int MPI_Graph_map(MPI_Comm comm, int nnodes, int* index, int* edges, int* newrank);
int MPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int* neighbors);
int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int* nneighbors);
int MPI_Graphdims_get(MPI_Comm comm, int* nnodes, int* nedges);
int MPI_Group_compare(MPI_Group group1,MPI_Group group2, int* result);
int MPI_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group* newgroup_CLASS_SINGLE_OUT);
int MPI_Group_excl(MPI_Group group, int n, int* ranks, MPI_Group* newgroup_CLASS_SINGLE_OUT);
int MPI_Group_free(MPI_Group* group_CLASS_SINGLE_IO);
int MPI_Group_incl(MPI_Group group, int n, int* ranks, MPI_Group* newgroup_CLASS_SINGLE_OUT);
int MPI_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group* newgroup_CLASS_SINGLE_OUT);
int MPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group* newgroup_CLASS_SINGLE_OUT);
int MPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group* newgroup_CLASS_SINGLE_OUT);
int MPI_Group_rank(MPI_Group group, int* rank);
int MPI_Group_size(MPI_Group group, int* size);
int MPI_Group_translate_ranks(MPI_Group group1, int n, int* ranks1, MPI_Group group2, int* ranks2);
int MPI_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group* newgroup_CLASS_SINGLE_OUT);
int MPI_Ibsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Init(int* argc, char*** argv);
int MPI_Init_thread(int* argc, char*** argv,int required,int* provided);
int MPI_Initialized(int* flag);
int MPI_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm peer_comm, int remote_leader, int tag, MPI_Comm* newintercomm_CLASS_SINGLE_OUT);
int MPI_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm* newintracomm_CLASS_SINGLE_OUT);
int MPI_Iprobe(int source, int tag, MPI_Comm comm, int* flag, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Irecv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Irsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Isend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Issend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Keyval_create(MPI_Copy_function* copy_fn, MPI_Delete_function* delete_fn, int* keyval, void* extra_state);
int MPI_Keyval_free(int* keyval);
int MPI_Op_create(MPI_User_function* function, int commute, MPI_Op* op_CLASS_SINGLE_OUT);
int MPI_Op_free( MPI_Op* op_CLASS_SINGLE_IO);
int MPI_Pack(void* inbuf, int incount, MPI_Datatype datatype, void* outbuf, int outsize, int* position,  MPI_Comm comm);
int MPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int* size);
int MPI_Pcontrol(int level, ...);
int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Recv(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Recv_init(void* buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Reduce(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);
int MPI_Reduce_scatter(void* sendbuf, void* recvbuf, int* recvcounts, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
int MPI_Request_free(MPI_Request* request_CLASS_SINGLE_IO);
int MPI_Rsend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
int MPI_Rsend_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Scan(void* sendbuf, void* recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm );
int MPI_Scatter(void* sendbuf, int sendcount, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
int MPI_Scatterv(void* sendbuf, int* sendcounts, int* displs, MPI_Datatype sendtype, void* recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
int MPI_Send(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
int MPI_Send_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Sendrecv(void* sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void* recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Sendrecv_replace(void* buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Ssend(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
int MPI_Ssend_init(void* buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request* request_CLASS_SINGLE_OUT);
int MPI_Start(MPI_Request* request_CLASS_SINGLE_IO);
int MPI_Startall(int count, MPI_Request* array_of_requests_CLASS_ARRAY_IO_count);
int MPI_Test(MPI_Request* request_CLASS_SINGLE_IO, int* flag, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Test_cancelled(MPI_Status* status_CLASS_SINGLE_IN, int* flag);
int MPI_Testall(int count, MPI_Request* array_of_requests_CLASS_ARRAY_IO_count, int* flag, MPI_Status* array_of_statuses_CLASS_ARRAY_OUT_count);
int MPI_Testany(int count, MPI_Request* array_of_requests_CLASS_ARRAY_IO_count, int* index_CLASS_ARRAYINDEX_OUT_SINGLE_CONDITION_flag, int* flag, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Testsome(int incount, MPI_Request* array_of_requests_CLASS_ARRAY_IO_incount, int* outcount, int* array_of_indices_CLASS_ARRAYINDEX_OUT_ARRAY_outcount, MPI_Status* array_of_statuses_CLASS_ARRAY_OUT_outcount);
int MPI_Topo_test(MPI_Comm comm, int* status);
int MPI_Type_commit(MPI_Datatype* datatype_CLASS_SINGLE_IO);
int MPI_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype* newtype_CLASS_SINGLE_OUT);
int MPI_Type_extent(MPI_Datatype datatype, MPI_Aint* extent_CLASS_SINGLE_OUT);
int MPI_Type_free(MPI_Datatype* datatype_CLASS_SINGLE_IO);
int MPI_Type_hindexed(int count, int* array_of_blocklengths, MPI_Aint* array_of_displacements_CLASS_ARRAY_IN_count, MPI_Datatype oldtype, MPI_Datatype* newtype_CLASS_SINGLE_OUT);
int MPI_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype* newtype_CLASS_SINGLE_OUT);
int MPI_Type_indexed(int count, int* array_of_blocklengths, int* array_of_displacements, MPI_Datatype oldtype, MPI_Datatype* newtype_CLASS_SINGLE_OUT);
int MPI_Type_lb(MPI_Datatype datatype, MPI_Aint* displacement_CLASS_SINGLE_OUT);
int MPI_Type_size(MPI_Datatype datatype, int* size);
int MPI_Type_struct(int count, int* array_of_blocklengths, MPI_Aint* array_of_displacements_CLASS_ARRAY_IN_count, MPI_Datatype* array_of_types_CLASS_ARRAY_IN_count, MPI_Datatype* newtype_CLASS_SINGLE_OUT);
int MPI_Type_ub(MPI_Datatype datatype, MPI_Aint* displacement_CLASS_SINGLE_OUT);
int MPI_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype* newtype_CLASS_SINGLE_OUT);
int MPI_Unpack(void* inbuf, int insize, int* position, void* outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);
int MPI_Wait(MPI_Request* request_CLASS_SINGLE_IO, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Waitall(int count, MPI_Request* array_of_requests_CLASS_ARRAY_IO_count, MPI_Status* array_of_statuses_CLASS_ARRAY_OUT_count);
int MPI_Waitany(int count, MPI_Request* array_of_requests_CLASS_ARRAY_IO_count, int* index_CLASS_ARRAYINDEX_OUT_SINGLE_NOCONDITION, MPI_Status* status_CLASS_SINGLE_OUT);
int MPI_Waitsome(int incount, MPI_Request* array_of_requests_CLASS_ARRAY_IO_incount, int* outcount, int* array_of_indices_CLASS_ARRAYINDEX_OUT_ARRAY_outcount, MPI_Status* array_of_statuses_CLASS_ARRAY_OUT_outcount);
double MPI_Wtick(void);
double MPI_Wtime(void);

/*
 * prototypes for user-defined functions
 */

typedef int MPI_Copy_function(MPI_Comm oldcomm, int keyval, void* extra_state, void* attribute_val_in, void* attribute_val_out, int* flag);
typedef int MPI_Delete_function(MPI_Comm comm, int keyval, void* attribute_val, void* extra_state);
typedef void MPI_Handler_function(MPI_Comm* comm, int* flag, ...);
typedef void MPI_User_function(void* invec, void* inoutvec, int* len, MPI_Datatype* datatype);




