//#include "mpo.h"
#ifndef MPO_2_MISC_H
#define MPO_2_MISC_H

#include <string>
using namespace std;

namespace MPO
{

#ifdef IS_MPI_GET_ADDRESS_PRESENT
	//************************************************************************
    //
    //                          MPO_Get_address
    //
    class MPO_Get_address : public MPO_Address
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Get_address();

    	/** Constructor taking proper arguments. */
    	MPO_Get_address(void *location, MPI_Aint * address);

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;
    };
#endif /*IS_MPI_GET_ADDRESS_PRESENT*/

#ifdef IS_MPI_TYPE_CREATE_STRUCT_PRESENT
	//************************************************************************
    //
    //                          MPO_Type_create_struct
    //
    class MPO_Type_create_struct : public MPO_Type_struct
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_create_struct();

    	/** Constructor taking proper arguments. */
    	MPO_Type_create_struct(int count, int *blocklength, MPI_Aint * displ_array,
    			MPI_Datatype * oldtype, MPI_Datatype * newtype);

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;
    };
#endif /*IS_MPI_TYPE_CREATE_STRUCT_PRESENT*/

#ifdef IS_MPI_TYPE_CREATE_HINDEXED_PRESENT
    //************************************************************************
    //
    //                          MPO_Type_create_hindexed
    //
    class MPO_Type_create_hindexed : public MPO_Type_hindexed
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_create_hindexed();

    	/** Constructor taking proper arguments. */
    	MPO_Type_create_hindexed(int count, int *blocklength, MPI_Aint * displ_array,
    			MPO_Datatype oldtype, MPI_Datatype * newtype);

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;
    };
#endif /*IS_MPI_TYPE_CREATE_HINDEXED_PRESENT*/

#ifdef IS_MPI_TYPE_CREATE_HVECTOR_PRESENT
    //************************************************************************
    //
    //                          MPO_Type_create_hvector
    //
    class MPO_Type_create_hvector : public MPO_Type_hvector
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_create_hvector();

    	/** Constructor taking proper arguments. */
    	MPO_Type_create_hvector(int count, int blocklength, MPI_Aint stride,
    			MPO_Datatype oldtype, MPI_Datatype * newtype);

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;
    };
#endif /*IS_MPI_TYPE_CREATE_HVECTOR_PRESENT*/


#ifdef IS_MPI_TYPE_CREATE_DARRAY_PRESENT
    //************************************************************************
    //
    //                          MPO_Type_create_darray
    //
    class MPO_Type_create_darray : public MPO_Typeconstructors
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_create_darray();

    	/** Constructor taking proper arguments. */
    	MPO_Type_create_darray(
    			int size,
    			int rank,
    			int ndims,
    			int* array_of_gsizes,
    			int* array_of_distribs,
    			int* array_of_dargs,
    			int* array_of_psizes,
    			int order,
    			MPI_Datatype oldtype,
    			MPI_Datatype* newtype
    			);

    	/** The destructor is virtual because there are virtual members here.*/
    	virtual ~MPO_Type_create_darray();

    	/** Checks correctness of parameters etc.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPrerequisites();

    	/** Does the MPI call after checking it.
    	 *
    	 *  @return  PMPI_Type_struct
    	 */
    	virtual int callNativeMPI();

    	/** Checks correctness of parameters etc.
    	 *  Executed after the actual MPI call.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPost();

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;

    	/** Overloads << to print call-description. */
    	virtual std::ostream & debugout(std::ostream & output) const;

    	/** Serializes an object to ostream.*/
    	virtual std::ostream & save(std::ostream & out) const;

    	/** Deserializes an object from istream.*/
    	virtual std::istream & restore(std::istream & in);

    	/** A check. */
    	int check_size_and_rank (void);

    	/** A check. */
    	int check_gsizes (void);

    	/** A check. */
    	int check_distribs (void);

    	/** A check. */
    	int check_dargs (void);

    	/** A check. */
    	int check_psizes (void);

    	/** A check. */
    	int check_order (void);

    protected:
    	int my_size;
    	int my_rank;
    	int* my_array_of_gsizes;
    	int* my_array_of_distribs;
    	int* my_array_of_dargs;
    	int* my_array_of_psizes;
    	int my_order;
    	MPI_Datatype* my_newtype;
    };
#endif /*IS_MPI_TYPE_CREATE_DARRAY_PRESENT*/

#ifdef IS_MPI_TYPE_CREATE_SUBARRAY_PRESENT
    //************************************************************************
    //
    //                          MPO_Type_create_subarray
    //
    class MPO_Type_create_subarray : public MPO_Typeconstructors
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_create_subarray();

    	/** Constructor taking proper arguments. */
    	MPO_Type_create_subarray(
    			int ndims,
    			int* array_of_sizes,
    			int* array_of_subsizes,
    			int* array_of_starts,
    			int order,
    			MPI_Datatype oldtype,
    			MPI_Datatype* newtype
    	);

    	/** The destructor is virtual because there are virtual members here.*/
    	virtual ~MPO_Type_create_subarray();

    	/** Checks correctness of parameters etc.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPrerequisites();

    	/** Does the MPI call after checking it.
    	 *
    	 *  @return  PMPI_Type_struct
    	 */
    	virtual int callNativeMPI();

    	/** Checks correctness of parameters etc.
    	 *  Executed after the actual MPI call.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPost();

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;

    	/** Overloads << to print call-description. */
    	virtual std::ostream & debugout(std::ostream & output) const;

    	/** Serializes an object to ostream.*/
    	virtual std::ostream & save(std::ostream & out) const;

    	/** Deserializes an object from istream.*/
    	virtual std::istream & restore(std::istream & in);

    	/** A check. */
    	int check_sizes (void);

    	/** A check. */
    	int check_subsizes (void);

    	/** A check. */
    	int check_starts (void);

    	/** A check. */
    	int check_order (void);

    protected:
    	int* my_array_of_sizes;
    	int* my_array_of_subsizes;
    	int* my_array_of_starts;
    	int my_order;
    	MPI_Datatype* my_newtype;
    };
#endif /*IS_MPI_TYPE_CREATE_SUBARRAY_PRESENT*/

#ifdef IS_MPI_TYPE_CREATE_INDEXED_BLOCK_PRESENT
    //************************************************************************
    //
    //                          MPO_Type_create_indexed_block
    //
    class MPO_Type_create_indexed_block : public MPO_Typeconstructors
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_create_indexed_block();

    	/** Constructor taking proper arguments. */
    	MPO_Type_create_indexed_block(
    			int count,
    			int blocklength,
    			int* array_of_displacements,
    			MPI_Datatype oldtype,
    			MPI_Datatype* newtype
    	);

    	/** The destructor is virtual because there are virtual members here.*/
    	virtual ~MPO_Type_create_indexed_block();

    	/** Checks correctness of parameters etc.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPrerequisites();

    	/** Does the MPI call after checking it.
    	 *
    	 *  @return  PMPI_Type_struct
    	 */
    	virtual int callNativeMPI();

    	/** Checks correctness of parameters etc.
    	 *  Executed after the actual MPI call.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPost();

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;

    	/** Overloads << to print call-description. */
    	virtual std::ostream & debugout(std::ostream & output) const;

    	/** Serializes an object to ostream.*/
    	virtual std::ostream & save(std::ostream & out) const;

    	/** Deserializes an object from istream.*/
    	virtual std::istream & restore(std::istream & in);

    	/** A check. */
    	int check_blocklength (void);

    	/** A check. */
    	int check_displacements (void);

    protected:
    	int my_blocklength;
    	int* my_array_of_displacements;
    	MPI_Datatype* my_newtype;
    };
#endif /*IS_MPI_TYPE_CREATE_INDEXED_BLOCK_PRESENT*/

#ifdef IS_MPI_TYPE_CREATE_RESIZED_PRESENT
    //************************************************************************
    //
    //                          MPO_Type_create_resized
    //
    class MPO_Type_create_resized : public MPO_Typeconstructors
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_create_resized();

    	/** Constructor taking proper arguments. */
    	MPO_Type_create_resized(
    			MPI_Datatype oldtype,
    			MPI_Aint lb,
    			MPI_Aint extent,
    			MPI_Datatype* newtype
    	);

    	/** The destructor is virtual because there are virtual members here.*/
    	virtual ~MPO_Type_create_resized();

    	/** Checks correctness of parameters etc.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPrerequisites();

    	/** Does the MPI call after checking it.
    	 *
    	 *  @return  PMPI_Type_struct
    	 */
    	virtual int callNativeMPI();

    	/** Checks correctness of parameters etc.
    	 *  Executed after the actual MPI call.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPost();

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;

    	/** Overloads << to print call-description. */
    	virtual std::ostream & debugout(std::ostream & output) const;

    	/** Serializes an object to ostream.*/
    	virtual std::ostream & save(std::ostream & out) const;

    	/** Deserializes an object from istream.*/
    	virtual std::istream & restore(std::istream & in);

    protected:
    	MPI_Aint my_lb;
    	MPI_Aint my_extent;
    	MPI_Datatype* my_newtype;
    };
#endif /*IS_MPI_TYPE_CREATE_RESIZED_PRESENT*/

#ifdef IS_MPI_TYPE_GET_EXTENT_PRESENT
    //************************************************************************
    //
    //                          MPO_Type_get_extent
    //
    class MPO_Type_get_extent : public MPO_Typehandlers
    {
    public:
    	/** The default constructor initializes an invalid object. */
    	MPO_Type_get_extent();

    	/** Constructor taking proper arguments. */
    	MPO_Type_get_extent(
    			MPI_Datatype datatype,
    			MPI_Aint* lb,
    			MPI_Aint* extent
    	);

    	/** The destructor is virtual because there are virtual members here.*/
    	virtual ~MPO_Type_get_extent();

    	/** Checks correctness of parameters etc.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPrerequisites();

    	/** Does the MPI call after checking it.
    	 *
    	 *  @return  PMPI_Type_struct
    	 */
    	virtual int callNativeMPI();

    	/** Checks correctness of parameters etc.
    	 *  Executed after the actual MPI call.
    	 *
    	 *  @return  0
    	 */
    	virtual int checkPost();

    	/** @return  the unique call identity set by enum MPO_CALL_ID */
    	virtual int getId() const;

    	/** @return  the name of the MPI-call (C or Fortran spelling). */
    	virtual string getCallname() const;

    	/** Overloads << to print call-description. */
    	virtual std::ostream & debugout(std::ostream & output) const;

    	/** Serializes an object to ostream.*/
    	virtual std::ostream & save(std::ostream & out) const;

    	/** Deserializes an object from istream.*/
    	virtual std::istream & restore(std::istream & in);

    protected:
    	MPI_Aint* my_lb;
    	MPI_Aint* my_extent;
    };
#endif /*IS_MPI_TYPE_GET_EXTENT_PRESENT*/

} /*namespace MPO*/

#endif /*MPO_2_MISC_H*/
