   // parallel file I/O
    //=======================================================================




namespace MPO
{

    //************************************************************************
    //
    //                          MPO_File_open
    //

    /** int MPI_File_open(MPI_Comm comm, char *filename, int amode,
     *                    MPI_Info info, MPI_File *fh)
     *
     *  collective call
     *
     *  @param  comm      communicator (handle)
     *  @param  filename  name of file to open (string)
     *  @param  amode     file access mode (integer)
     *  @param  info      info object (handle)
     *  @param  fh        file handle (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COMM_NULL
     *           - MARMOT_ERR_COMM_NOT_VALID
     *           - MARMOT_ERR_COMM_INTERCOMM
     *           - MARMOT_ERR_FILE_DIFF
     *           - MARMOT_ERR_AMODE_DIFF
     *           - MARMOT_ERR_AMODE_RDONLY_RDWR_WRONLY_NOT_SET_EXACTLY_ONE
     *           - MARMOT_ERR_AMODE_RDONLY_AND_CREATE
     *           - MARMOT_ERR_AMODE_RDONLY_AND_EXCL
     *           - MARMOT_ERR_AMODE_RDWR_AND_SEQUENTIAL
     *           - MARMOT_WARN_FILE_OPEN_LEFT (MPI_Finalize)
     */



    //************************************************************************
    //
    //                          MPO_File_close
    //

    /** int MPI_File_close(MPI_File *fh)
     *
     *  collective call
     *
     *  @param  fh        file handle (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_WARN_REQUESTS_NONBLOCKING_LEFT
     *           - MARMOT_WARN_OPERATION_SPLIT_COLLECTIVE_LEFT
     *           - MARMOT_NOTE_AMODE_DELETE_ON_CLOSE
     */



    //************************************************************************
    //
    //                          MPO_File_delete
    //

    /** int MPI_File_delete(char *filename, MPI_Info info)
     *
     *  @param  filename  name of file to delete (string)
     *  @param  info      info object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_FILE_DOES_NOT_EXIST
     *           - MARMOT_WARN_FILE_IN_USE
     */



    //************************************************************************
    //
    //                          MPO_File_set_size
    //

    /** int MPI_File_set_size(MPI_File fh, MPI_Offset size)
     *
     *  collective call
     *
     *  @param  filename  name of file to delete (string)
     *  @param  info      info object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_FILE_SIZE_NEG
     *           - MARMOT_ERR_FILE_SIZE_DIFF
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_REQUESTS_NONBLOCKING_LEFT
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_LEFT
     *           - MARMOT_WARN_DATA_IN_NEW_REGION_UNDEFINED
     *           - MARMOT_NOTE_FILE_SIZE_LT_OR_EQ_CURRENT_SIZE
     */



    //************************************************************************
    //
    //                          MPO_File_preallocate
    //

    /** int MPI_File_preallocate(MPI_File fh, MPI_Offset size)
     *
     *  collective call
     *
     *  @param  fh        file handle (handle)
     *  @param  size      size to preallocate (nonnegative integer)
     *  @see  MPO_File_set_size
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_FILE_SIZE_NEG
     *           - MARMOT_ERR_FILE_SIZE_DIFF
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_REQUESTS_NONBLOCKING_LEFT
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_LEFT
     *           - MARMOT_WARN_DATA_IN_NEW_REGION_UNDEFINED
     *           - MARMOT_NOTE_FILE_SIZE_LT_OR_EQ_CURRENT_SIZE
     */



    //************************************************************************
    //
    //                          MPO_File_get_size
    //

    /** int MPI_File_get_size(MPI_File fh, MPI_Offset *size)
     *
     *  @param  fh        file handle (handle)
     *  @param  size      size of the file in bytes (nonnegative integer)
     *  @see  MPO_File_set_size
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */


    //************************************************************************
    //
    //                          MPO_File_get_group
    //

    /** int MPI_File_get_group(MPI_File fh, MPI_Group *group)
     *
     *  @param  fh        file handle (handle)
     *  @param  group     group that opened the file (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */



    //************************************************************************
    //
    //                          MPO_File_get_amode
    //

    /**  int MPI_File_get_amode(MPI_File fh, int *amode)
     *
     *  @param  fh        file handle (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */



    //************************************************************************
    //
    //                          MPO_File_set_info
    //

    /**  int MPI_File_set_info(MPI_File fh, MPI_Info info)
     *
     *  collective call
     *
     *  @param  fh        file handle (handle)
     *  @param  info      info object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_FILEHANDLE_DIFF
     *           - MARMOT_ERR_INFO_DIFF
     *           - MARMOT_NOTE_INFO_IGNORED
     */



    //************************************************************************
    //
    //                          MPO_File_get_info
    //

    /**  int MPI_File_get_info(MPI_File fh, MPI_Info *info_used)
     *
     *  @param  fh         file handle (handle)
     *  @param  info_used  info object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */



    //************************************************************************
    //
    //                          MPO_File_set_view
    //

    /**  int MPI_File_set_view(MPI_File fh, MPI_Offset disp,
     *                         MPI_Datatype etype,
     *                         MPI_Datatype filetype, char *datarep,
     *                         MPI_Info info)
     *
     *  collective call
     *
     *  @param  fh        file handle (handle)
     *  @param  disp      displacement (nonnegative integer)
     *  @param  etype     elementary datatype (handle)
     *  @param  filetype  filetype (handle)
     *  @param  datarep   data representation (string)
     *  @param  info      info object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_FILETYPE_DISPLACEMENTS_NEG
     *           - MARMOT_ERR_DATAREP_DIFF
     *           - MARMOT_ERR_AMODE_SEQUENTIAL_AND_NOT_DISPLACEMENT_CURRENT
     *           - MARMOT_ERR_AMODE_SEQUENTIAL_NOT_SET_BUT_DISPLACEMENT_CURRENT
     *           - MARMOT_ERR_ETYPE_NOT_VALID
     *           - MARMOT_ERR_ETYPE_OVERLAPPING
     *           - MARMOT_ERR_ETYPE_EXTENT_DIFF
     *           - MARMOT_ERR_FILETYPE_EXTENT_HOLE_NO_MULTIPLE_EXTENT_ETYPE
     *           - MARMOT_ERR_FILETYPE_NOT_VALID
     *           - MARMOT_ERR_FILETYPE_OVERLAPPING
     *           - MARMOT_ERR_REQUESTS_NONBLOCKING_LEFT
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_LEFT
     *           - MARMOT_NOTE_FILETYPE_HAS_HOLES
     */


    //************************************************************************
    //
    //                          MPO_File_get_view
    //

    /**  int MPI_File_get_view(MPI_File fh, MPI_Offset *disp,
     *                         MPI_Datatype *etype,
     *                         MPI_Datatype *filetype, char *datarep)
     *
     *  @param  fh        file handle (handle)
     *  @param  disp      displacement (nonnegative integer)
     *  @param  etype     elementary datatype (handle)
     *  @param  filetype  filetype (handle)
     *  @param  datarep   data representation (string)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_DATAREP_GT_MAX_DATAREP_STRING
     *           - MARMOT_WARN_DATAREP_TOO_SMALL
     */


    //************************************************************************
    //
    //                          MPO_File_read_at
    //

    /**  int MPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf,
     *                        int count, MPI_Datatype datatype,
     *                        MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  offset    file offset (nonnegative integer)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_read_at_all
    //

    /**  int MPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf,
     *                            int count, MPI_Datatype datatype,
     *                            MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  offset    file offset (nonnegative integer)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_write_at
    //

    /**  int MPI_File_write_at(MPI_File fh, MPI_Offset offset, void *buf,
     *                         int count, MPI_Datatype datatype,
     *                         MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  offset    file offset (nonnegative integer)
     *  @param  buf       initial address of buffer (choice)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */


    //************************************************************************
    //
    //                          MPO_File_write_at_all
    //

    /**  int MPI_File_write_at_all(MPI_File fh, MPI_Offset offset, void *buf,
     *                             int count, MPI_Datatype datatype,
     *                             MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  offset    file offset (nonnegative integer)
     *  @param  buf       initial address of buffer (choice)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_iread_at
    //

    /**  int MPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf,
     *                         int count, MPI_Datatype datatype,
     *                         MPIO_Request *request)
     *
     *  @param  fh        file handle (handle)
     *  @param  offset    file offset (nonnegative integer)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  request   request object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_iwrite_at
    //

    /**  int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, void *buf,
     *                          int count, MPI_Datatype datatype,
     *                          MPIO_Request *request)
     *
     *  @param  fh        file handle (handle)
     *  @param  offset    file offset (nonnegative integer)
     *  @param  buf       initial address of buffer (choice)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  request   request object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_read
    //

    /**  int MPI_File_read(MPI_File fh, void *buf, int count,
     *                     MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_read_all
    //

    /**  int MPI_File_read_all(MPI_File fh, void *buf, int count,
     *                         MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */


    //************************************************************************
    //
    //                          MPO_File_write
    //

    /**  int MPI_File_write(MPI_File fh, void *buf, int count,
     *                      MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_write_all
    //

    /**  int MPI_File_write_all(MPI_File fh, void *buf, int count,
     *                          MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh        file handle (handle)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_iread
    //

    /**  int MPI_File_iread(MPI_File fh, void *buf, int count,
     *                      MPI_Datatype datatype, MPIO_Request *request)
     *
     *  @param  fh        file handle (handle)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  request   request object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_iwrite
    //

    /**  int MPI_File_iwrite(MPI_File fh, void *buf, int count,
     *                       MPI_Datatype datatype, MPIO_Request *request)
     *
     *  @param  fh        file handle (handle)
     *  @param  count     number of elements in buffer (nonnegative integer)
     *  @param  datatype  datatype of each buffer element (handle)
     *  @param  buf       initial address of buffer (choice)
     *  @param  request   request object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_seek
    //

    /** int MPI_File_seek(MPI_File fh, MPI_Offset offset, int whence)
     *
     *  @param  fh         file handle (handle)
     *  @param  count      number of elements in buffer (nonnegative integer)
     *  @param  datatype   datatype of each buffer element (handle)
     *  @param  buf        initial address of buffer (choice)
     *  @param  request    request object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_OFFSET_TO_NEG_POS_IN_VIEW
     */



    //************************************************************************
    //
    //                          MPO_File_get_position
    //

    /** int MPI_File_get_position(MPI_File fh, MPI_Offset *offset)
     *
     *  @param  fh      file handle (handle)
     *  @param  offset  offset of individual file pointer (nonnegative integer)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     */


    //************************************************************************
    //
    //                          MPO_File_get_byte_offset
    //

    /** int MPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset,
     *                               MPI_Offset *disp)
     *
     *  @param  fh      file handle (handle)
     *  @param  offset  offset (nonnegative integer)
     *  @param  disp    absolute byte position of offset (nonnegative integer)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG ???
     */


    //************************************************************************
    //
    //                          MPO_File_read_shared
    //

    /** int MPI_File_read_shared(MPI_File fh, void *buf, int count,
     *                           MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     *  @param status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL ???
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_write_shared
    //

    /** int MPI_File_write_shared(MPI_File fh, void *buf, int count,
     *                           MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     *  @param status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_iread_shared
    //

    /** int MPI_File_iread_shared(MPI_File fh, void *buf, int count,
     *                            MPI_Datatype datatype, MPIO_Request *request)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     *  @param request   request object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_iwrite_shared
    //

    /** int MPI_File_iwrite_shared(MPI_File fh, void *buf, int count,
     *                             MPI_Datatype datatype,
     *                             MPIO_Request *request)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     *  @param request   request object (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */



    //************************************************************************
    //
    //                          MPO_File_read_ordered
    //

    /** int MPI_File_read_ordered(MPI_File fh, void *buf, int count,
     *                            MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     *  @param status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL ???
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_write_ordered
    //

    /** int MPI_File_write_ordered(MPI_File fh, void *buf, int count,
     *                            MPI_Datatype datatype, MPI_Status *status)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     *  @param status    status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL ???
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_COUNT_GT_ONE
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_seek_shared
    //

    /** int MPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence)
     *
     *  collective call
     *
     *  @param  fh      file handle (handle)
     *  @param offset   file offset (integer)
     *  @param whence   update mode (state)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_OFFSET_DIFF
     *           - MARMOT_ERR_OFFSET_TO_NEG_POS_IN_VIEW
     *           - MARMOT_ERR_WHENCE_DIFF
     */



    //************************************************************************
    //
    //                          MPO_File_get_position_shared
    //

    /** int MPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset)
     *
     *  @param  fh      file handle (handle)
     *  @param offset   offset of shared file pointer (nonnegative integer)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */



    //************************************************************************
    //
    //                          MPO_File_read_at_all_begin
    //

    /** int MPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset,
     *                                 void *buf,
     *                                 int count, MPI_Datatype datatype)
     *
     *  collective call
     *
     *  @param  fh       file handle (handle)
     *  @param offset    file offset (nonnegative integer)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_COUNT_GT_ONE
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_read_at_all_end
    //

    /** int MPI_File_read_at_all_end(MPI_File fh, void *buf,
     *                               MPI_Status *status)
     *
     *  @param  fh      file handle (handle)
     *  @param buf      initial address of buffer (choice)
     *  @param status   status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_NO_MATCHING_BEGIN
     */



    //************************************************************************
    //
    //                          MPO_File_write_at_all_begin
    //

    /** int MPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset,
     *                                  void *buf,
     *                                  int count, MPI_Datatype datatype)
     *
     *  @param  fh       file handle (handle)
     *  @param offset    file offset (nonnegative integer)
     *  @param buf       initial address of buffer (choice)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_OFFSET_NEG
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_COUNT_GT_ONE
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_write_at_all_end
    //

    /** int MPI_File_write_at_all_end(MPI_File fh, void *buf,
     *                                MPI_Status *status)
     *
     *  @param  fh      file handle (handle)
     *  @param buf      initial address of buffer (choice)
     *  @param status   status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_NO_MATCHING_BEGIN
     */



    //************************************************************************
    //
    //                          MPO_File_read_all_begin
    //

    /** int MPI_File_read_all_begin(MPI_File fh, void *buf, int count,
     *                              MPI_Datatype datatype)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_COUNT_GT_ONE
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_read_all_end
    //

    /** int MPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status)
     *
     *  @param  fh      file handle (handle)
     *  @param buf      initial address of buffer (choice)
     *  @param status   status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_NO_MATCHING_BEGIN
     */



    //************************************************************************
    //
    //                          MPO_File_write_all_begin
    //

    /** int MPI_File_write_all_begin(MPI_File fh, void *buf, int count,
     *                               MPI_Datatype datatype)
     *
     *  @param  fh       file handle (handle)
     *  @param buf       initial address of buffer (choice)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_COUNT_GT_ONE
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_write_all_end
    //

    /** int MPI_File_write_all_end(MPI_File fh, void *buf, MPI_Status *status)
     *
     *  @param  fh      file handle (handle)
     *  @param buf      initial address of buffer (choice)
     *  @param status   status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_NO_MATCHING_BEGIN
     */



    //************************************************************************
    //
    //                          MPO_File_read_ordered_begin
    //

    /** int MPI_File_read_ordered_begin(MPI_File fh, void *buf, int count,
     *          			MPI_Datatype datatype)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_WRONLY
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_COUNT_GT_ONE
     *           - MARMOT_WARN_COUNT_ZERO
     */



    //************************************************************************
    //
    //                          MPO_File_read_ordered_end
    //

    /** int MPI_File_read_ordered_end(MPI_File fh, void *buf,
     *                                MPI_Status *status)
     *
     *  @param  fh      file handle (handle)
     *  @param buf      initial address of buffer (choice)
     *  @param status   status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_NO_MATCHING_BEGIN
     */




    //************************************************************************
    //
    //                          MPO_File_write_ordered_begin
    //

    /** int MPI_File_write_ordered_begin(MPI_File fh, void *buf, int count,
     *   				 MPI_Datatype datatype)
     *
     *  @param  fh       file handle (handle)
     *  @param count     number of elements in buffer (nonnegative integer)
     *  @param datatype  datatype of each buffer element (handle)
     *  @param buf       initial address of buffer (choice)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_COUNT_NEG
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_NOT_VALID
     *           - MARMOT_ERR_TYPE_OVERLAPPING
     *           - MARMOT_ERR_AMODE_SEQUENTIAL
     *           - MARMOT_ERR_AMODE_RDONLY
     *           - MARMOT_ERR_OPERATION_SPLIT_COLLECTIVE_COUNT_GT_ONE
     *           - MARMOT_WARN_COUNT_ZERO
     */




    //************************************************************************
    //
    //                          MPO_File_write_ordered_end
    //

    /** int MPI_File_write_ordered_end(MPI_File fh, void *buf,
     *                                 MPI_Status *status)
     *
     *  @param  fh      file handle (handle)
     *  @param buf      initial address of buffer (choice)
     *  @param status   status object (Status)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_NO_MATCHING_BEGIN
     */




    //************************************************************************
    //
    //                          MPO_File_get_type_extent
    //

    /** int MPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype,
     *                               MPI_Aint *extent)
     *
     *  @param  fh       file handle (handle)
     *  @param datatype  datatype (handle)
     *  @param extent    extent of the datatype (nonnegative integer)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_TYPE_NULL
     *           - MARMOT_ERR_TYPE_INVALID
     */




    //************************************************************************
    //
    //                          MPO_Register_datarep
    //

    /**
     *
     *  @param
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */




    //************************************************************************
    //
    //                          MPO_File_set_atomicity
    //

    /** int MPI_File_set_atomicity(MPI_File fh, int flag)
     *
     *  @param fh       file handle (handle)
     *  @param flag     true to set atomic mode, false to set nonatomic mode
     *                  (logical)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     *           - MARMOT_ERR_FLAG_DIFF
     */




    //************************************************************************
    //
    //                          MPO_File_get_atomicity
    //

    /** int MPI_File_get_atomicity(MPI_File fh, int *flag)
     *
     *  @param  fh      file handle (handle)
     *  @param flag     true if atomic mode, false if nonatomic mode (logical)
     */

    /** Checks prerequisites such as correctness of parameters.
     *
     *  @return  unique error code set by enum MARMOT_ERR_CODES:
     *           - MARMOT_SUCCESS
     */

}
// end of MARMOT_IO

