/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Header file describes the MPO_LogQueue log file.
 * Rather than actually logging messages, it stores them in a
 * queue. The marmot external interface (@see marmot-interface.h)
 * is used to retrieve them.
 *
 * @author Tobias Hilbrich
 * @date 04.12.2008
 *
 * $ID$
 */

#ifndef MPO_LOGQUEUE_H
#define MPO_LOGQUEUE_H

#include "MPO_LogFile.h"
#include <queue>

namespace MPO
{
    /**
     * Log file implementation that stores messages in a queue.
     * One Queue is used per process, and messages may be
     * retrieved by using marmots public interface.
     * @see marmot-interface.h
     */
    class MPO_LogQueue : public MPO_LogFile
    {
    private:

        //==========================
        // Data
        //==========================
        std::queue<std::pair<MPO_MsgObject,std::string> > m_queue;

    public:
        //==========================
        // Constructors/Destructors
        //==========================
        MPO_LogQueue();
        ~MPO_LogQueue ();

        //==========================
        // Methods own
        //==========================
    public:
        //! Logs a message.
        bool getMsg (MPO_MsgObject &pMsg);
        //! Closes the log.
        bool closeLog (void);
        //! Queries whether a message is available.
        bool hasMsg (void);
        //! Returns number of messages in queue.
        int numMsgs (void);
        //! Returns the first message in the queue together with a link to its HTML standard link.
        std::pair<MPO_MsgObject,std::string> popMsg (void);
    };//end of class MPO_LogFile

}//end of namespace MPO

#endif /*MPO_LOGQUEUE_H*/
