/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of the base class MPO_WinCommunicator.
 *
 *  @author Bettina Krammer
 *
 *  $Id: MPO_WinCommunicator.cc 1112 2010-04-19 11:09:52Z tobias $
 */

#include "config.h"
#include "mpo-objects.h"
#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"

#ifdef MARMOT_1SIDED

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_WinCommunicator::MPO_WinCommunicator()
        : MPO_Winhandler(MPI_WIN_NULL),
          my_origin_addr(0),
          my_origin_count(-1),
          my_origin_datatype(MPI_DATATYPE_NULL),
          my_target_rank(MPI_PROC_NULL),
          my_target_disp(-1),
          my_target_count(-1),
          my_target_datatype(MPI_DATATYPE_NULL)
    {
    }

    // proper constructor
    //========================================================================
    MPO_WinCommunicator::MPO_WinCommunicator(void *origin_addr,
                                             int origin_count,
                                             MPO_Datatype origin_datatype,
                                             int target_rank,
                                             MPI_Aint target_disp,
                                             int target_count,
                                             MPO_Datatype target_datatype,
                                             MPO_Win win)
        : MPO_Winhandler(win),
          my_origin_addr(origin_addr),
          my_origin_count(origin_count),
          my_origin_datatype(origin_datatype),
          my_target_rank(target_rank),
          my_target_disp(target_disp),
          my_target_count(target_count),
          my_target_datatype(target_datatype)
    {
    }


    // destructor
    //========================================================================
    MPO_WinCommunicator::~MPO_WinCommunicator()
    {
    }


    /** There are some pure virtual methods, see mpo-2.h.
     */


    // getFamilyId
    //========================================================================
    int MPO_WinCommunicator::getFamilyId() const
    {
        // returns the unique family identity set by enum MPO_FAMILY_ID
        return MPO_CALLOBJECT_ID;
    }


    // checkPrerequisites2
    //========================================================================
    int MPO_WinCommunicator::checkPrerequisites2()
    {
        // check correctness of parameters
        checkCountOrigin();
        checkDatatypeOrigin();
        checkRankTarget();
        checkDispTarget();
        checkCountTarget();
        checkDatatypeTarget();

        return 0;
    }


    // checkCountOrigin
    //========================================================================
    int MPO_WinCommunicator::checkCountOrigin()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;


        return rvalue;
    }


    // checkDatatypeOrigin
    //========================================================================
    int MPO_WinCommunicator::checkDatatypeOrigin()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;


        return rvalue;
    }


    // checkRankTarget
    //========================================================================
    int MPO_WinCommunicator::checkRankTarget()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;


        return rvalue;
    }


    // checkDispTarget
    //========================================================================
    int MPO_WinCommunicator::checkDispTarget()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;


        return rvalue;
    }


    // checkCountTarget
    //========================================================================
    int MPO_WinCommunicator::checkCountTarget()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;


        return rvalue;
    }


    // checkDatatypeTarget
    //========================================================================
    int MPO_WinCommunicator::checkDatatypeTarget()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;


        return rvalue;
    }

}                               // end of namespace MPO

#endif /*MARMOT_1SIDED*/
