/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of the base class MPO_Winhandler.
 *
 *  @author Bettina Krammer
 *
 *  $Id: MPO_Winhandler.cc 1112 2010-04-19 11:09:52Z tobias $
 */

#include "config.h"
#include "mpo-objects.h"
#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"

#ifdef MARMOT_1SIDED

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Winhandler::MPO_Winhandler()
        : my_win(MPI_WIN_NULL)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Winhandler::MPO_Winhandler(MPO_Win win)
        : my_win(win)
    {
    }


    // destructor
    //========================================================================
    MPO_Winhandler::~MPO_Winhandler()
    {
    }


    /** There are some pure virtual methods, see mpo-2.h.
     */


    // getFamilyId
    //========================================================================
    int MPO_Winhandler::getFamilyId() const
    {
        // returns the unique family identity set by enum MPO_FAMILY_ID
        return MPO_CALLOBJECT_ID;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Winhandler::checkPrerequisites()
    {
        // check correctness of parameters
        checkWin();

        return 0;
    }


    // checkWin
    //========================================================================
    int MPO_Winhandler::checkWin()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if window is null handle.
        if ((MPI_WIN_NULL == my_win) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": window is MPI_WIN_NULL!"
                << std::endl;
            rvalue = MARMOT_ERR_WIN_NULL;

            MPO_String text;
            text.assign(&stream);
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation());
            our_log->getMsg(msg);
        }

        // (2) Check if window is valid, i.e. registered in our_wins.
        else if ((MPO_Win::our_wins.end() == MPO_Win::our_wins.find(my_win)) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << " : Window is not valid!"
                << std::endl;
            rvalue = MARMOT_ERR_WIN_NOT_VALID;

            MPO_String text;
            text.assign(&stream);
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation());
            our_log->getMsg(msg);
        }

        return rvalue;
    }                           // end checkWin


}                               // end of namespace MPO

#endif /*MARMOT_1SIDED*/
