/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of MPO_Alloc_mem class.
 *
 *  @author Blasius Czink
 *
 *  $Id: mpo_Alloc_mem.cc 1020 2009-10-22 14:03:02Z hpczink $
 */

#include "config.h"
#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"

using namespace MPO;

#ifdef IS_MPI_ALLOC_MEM_PRESENT

// default constructor
//========================================================================
MPO_Alloc_mem::MPO_Alloc_mem()
{
}


// proper constructor
//========================================================================
MPO_Alloc_mem::MPO_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
  : m_size(size), m_info(info), m_baseptr(baseptr)
{
}


// destructor
//========================================================================
MPO_Alloc_mem::~MPO_Alloc_mem()
{
}

int MPO_Alloc_mem::checkPrerequisites()
{
  return MARMOT_SUCCESS;
}

int MPO_Alloc_mem::checkPost()
{
  return MARMOT_SUCCESS;
}


// getId
//========================================================================
int MPO_Alloc_mem::getId() const
{
  // returns the unique call identity set by enum MPO_CALL_ID
  return MPO_ALLOC_MEM_ID;
}


// callNativeMPI
//========================================================================
int MPO_Alloc_mem::callNativeMPI()
{
  int rvalue = 1;

  #ifdef MARMOT_USE_THREADSYNC
    MPO_Space::getSyncObject()->enterPMPI();
  #endif
  rvalue = PMPI_Alloc_mem(m_size, m_info, m_baseptr);
  #ifdef MARMOT_USE_THREADSYNC
    MPO_Space::getSyncObject()->leavePMPI();
  #endif

  return rvalue;
}


// getCallname
//========================================================================
string MPO_Alloc_mem::getCallname() const
{
  // Distinguish between C and Fortran interface, default: C interface.

  string rvalue = "MPI_Alloc_mem";

  if (F_INTERFACE == INTERFACE_MODE)
  {
      rvalue = "MPI_ALLOC_MEM";
  }

  return rvalue;
}


// debugout
//========================================================================
std::ostream & MPO_Alloc_mem::debugout(std::ostream & output) const
{
  // Distinguish between C and Fortran interface, default: C interface.

  output << getCallname()
      << "(size = "  << m_size
      << ", info = " << m_info
      << ", baseptr = " << m_baseptr;

  if (F_INTERFACE == INTERFACE_MODE)
  {
    output << ", ierror";
  }

  output << ")" << std::endl;

  return output;
}


// save
//========================================================================
std::ostream & MPO_Alloc_mem::save(std::ostream & out) const
{
  // we don't save and restore pointers
  MPO_Callobject::save(out);
  //out < my_win;

  return out;
}


// restore
//========================================================================
std::istream & MPO_Alloc_mem::restore(std::istream & in)
{
  // we don't save and restore pointers
  MPO_Callobject::restore(in);
  //in > my_win;

  return in;
}

#endif /*IS_MPI_ALLOC_MEM_PRESENT*/

/*EOF*/
