/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Win representing MPI-windows.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Win.cc 1112 2010-04-19 11:09:52Z tobias $
 */

#include "config.h"
#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-2-objects.h"

#ifdef MARMOT_1SIDED

namespace MPO
{
    std::map<MPI_Win, MPO::mpi_win_marmot_states_t> MPO_Win::our_wins;

    // default constructor
    //========================================================================
    MPO_Win::MPO_Win()
        : my_win(MPI_WIN_NULL), my_name ("")
    {
    }


    // proper constructor
    //========================================================================
    MPO_Win::MPO_Win(MPI_Win win)
		: my_name ("")
    {
        my_win = win;
    }


    // destructor
    //========================================================================
    MPO_Win::~MPO_Win()
    {
    }


    // MPI_Win
    //========================================================================
    MPO_Win::operator  MPI_Win() const
    {
        return my_win;
    }


    // getId
    //========================================================================
    int MPO_Win::getId() const
    {
        // returns the unique type identity set by enum MPO_TYPE_ID
        return MPO_WIN_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Win::debugout(std::ostream & output) const
    {
    	if (my_name == "")
    	{
			if (my_win == MPI_WIN_NULL)
			{
				output << "MPI_WIN_NULL";
			}
			else
			{
				output << "self-defined win";
			}
    	}
    	else
    	{
    		output << my_name;
    	}

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Win::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifndef MPI_WIN_NON_BASIC
        out < my_win;
#else
        if (my_win == MPI_WIN_NULL)
        	out < (int) MPO_WIN_NULL;
        else
        	out < (int) MPO_LAST_WIN_TYPE;
#endif

      return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Win::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifndef MPI_WIN_NON_BASIC
    	in > my_win;
#else
    	int x;
    	in > x;
    	MPO_WIN_TYPE t = (MPO_WIN_TYPE) x;

    	if (t == MPO_WIN_NULL)
    		my_name = "MPI_WIN_NULL";
    	else
    		my_name = "self-defined win";

    	my_win = MPI_WIN_NULL;
#endif

        return in;
    }

}                               // end of namespace MPO

#endif
// end of MARMOT_1SIDED
