/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of MPO_Win_get_group class.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Win_get_group.cc 1112 2010-04-19 11:09:52Z tobias $
 */

#include "config.h"
#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"

#ifdef IS_MPI_WIN_GET_GROUP_PRESENT

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Win_get_group::MPO_Win_get_group()
        : MPO_Winhandler(MPI_WIN_NULL),
          my_group(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Win_get_group::MPO_Win_get_group(MPO_Win win, MPI_Group *group)
        : MPO_Winhandler(win),
          my_group(group)
    {
    }


    // destructor
    //========================================================================
    MPO_Win_get_group::~MPO_Win_get_group()
    {
    }


    // getId
    //========================================================================
    int MPO_Win_get_group::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_WIN_GET_GROUP_ID;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Win_get_group::callNativeMPI()
    {
        int rvalue = 1;

        checkPrerequisites();
        rvalue = PMPI_Win_get_group(my_win, my_group);

        return rvalue;
    }

    // checkPost
    //========================================================================
    int MPO_Win_get_group::checkPost()
    {
    	//TODO
      return MARMOT_SUCCESS;
    }

    // getCallname
    //========================================================================
    string MPO_Win_get_group::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Win_get_group";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_WIN_GET_GROUP";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Win_get_group::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(win = "
            << my_win
            << ", *group";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Win_get_group::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_win;
        // out < my_group;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Win_get_group::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_win;
        // in > my_group;

        return in;
    }

}                               // end of namespace MPO


// mpich implementation
//========================================================================
// The following comment is copied from mpich2-1.0/src/mpi/rma/win_get_group.c

/*@
   MPI_Win_get_group - Get the MPI Group of the window object

   Input Parameter:
. win - window object (handle)

   Output Parameter:
. group - group of processes which share access to the window (handle)

   Notes:
   The group is a duplicate of the group from the communicator used to
   create the MPI window, and should be freed with 'MPI_Group_free' when
   it is no longer needed.  This group can be used to form the group of
   neighbors for the routines 'MPI_Win_post' and 'MPI_Win_start'.

.N ThreadSafe

.N Fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_WIN
.N MPI_ERR_ARG
.N MPI_ERR_OTHER

int MPI_Win_get_group(MPI_Win win, MPI_Group *group)
@*/

#endif /*IS_MPI_WIN_GET_GROUP_PRESENT*/

/*EOF*/
