/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Type_create_resized.
 *
 *  @author Tobias Hilbrich
 *
 *  $Id$
 */


#include "mpo-objects.h"
#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"
#include <stdio.h>

#ifdef IS_MPI_TYPE_CREATE_RESIZED_PRESENT

namespace MPO
{
    // default constructor
    //========================================================================
	MPO_Type_create_resized::MPO_Type_create_resized()
		: MPO_Typeconstructors (-1, MPI_DATATYPE_NULL, -1)
	{
		//nothing to do
	}

	// proper constructor
	//========================================================================
	MPO_Type_create_resized::MPO_Type_create_resized(
			MPI_Datatype oldtype,
    		MPI_Aint lb,
    		MPI_Aint extent,
    		MPI_Datatype* newtype
    		)
		: MPO_Typeconstructors (1, oldtype, MPO_TYPE_CREATE_RESIZED_ID),
		  my_lb (lb),
		  my_extent (extent),
		  my_newtype (newtype)
	{
		//nothing to do
	}

    // default destructor
    //========================================================================
	MPO_Type_create_resized::~MPO_Type_create_resized()
    {
    	//Nothing to do
    }

    // checkPrerequisites
    //========================================================================
    int MPO_Type_create_resized::checkPrerequisites()
    {
    	int ret;

    	//Todo think about checks here ...

    	return ret;
    }

    // callNativeMPI
    //========================================================================
    int MPO_Type_create_resized::callNativeMPI()
    {
    	int rvalue = 1;

#ifdef MARMOT_USE_THREADSYNC
    	MPO_Space::getSyncObject()->enterPMPI();
#endif
    	rvalue = PMPI_Type_create_resized (
    			my_oldtype,
    			my_lb,
    			my_extent,
    			my_newtype
    			);
#ifdef MARMOT_USE_THREADSYNC
    	MPO_Space::getSyncObject()->leavePMPI();
#endif

    	return rvalue;
    }

    // checkPost
    //========================================================================
   	int MPO_Type_create_resized::checkPost()
   	{
   		our_datatypes.insert(std::make_pair(MPO_Datatype(*my_newtype),
   				DATATYPE_CREATED_NOT_COMMITTED));

   		our_datatype_traces->addAndSetCreation(*my_newtype,getSourceLocation());
   		our_datatype_traces->findAndTryAddDependendency(*my_newtype,my_oldtype);

   		return 0;
   	}

    // debugout
    //========================================================================
  	std::ostream & MPO_Type_create_resized::debugout(std::ostream & output) const
  	{
  		output
  		<< getCallname()
  		<< "(oldtype = "
  		<< my_oldtype
  		<< ", lb = "
  		<< my_lb << "(Address!)"
  		<< ", extent = "
  		<< my_extent << "(Address!)"
  		<< ", *newtype";

  		if (F_INTERFACE == INTERFACE_MODE)
  		{
  			output << ", ierror";
  		}

  		output
  		<< ")"
  		<< std::endl;

  		return output;
  	}

  	// save
  	//========================================================================
    std::ostream & MPO_Type_create_resized::save(std::ostream & out) const
    {
    	// we don't save and restore pointers
    	MPO_Callobject::save(out);
    	out < my_oldtype;
    	out < my_lb;
    	out < my_extent;
    	//out < my_newtype;

    	return out;
    }

    // restore
    //========================================================================
    std::istream & MPO_Type_create_resized::restore(std::istream & in)
    {
    	// we don't save and restore pointers
    	MPO_Callobject::restore(in);
    	in > my_oldtype;
    	in > my_lb;
    	in > my_extent;
    	//in < my_newtype;

    	return in;
    }

	// getCallname
	//========================================================================
	string MPO_Type_create_resized::getCallname() const
	{
		/*  Distinguish between C and Fortran interface, default: C interface.
		 */
		string rvalue = "MPI_Type_create_resized";

		if (F_INTERFACE == INTERFACE_MODE)
		{
			rvalue = "MPI_TYPE_CREATE_RESIZED";
		}

		return rvalue;
	}

	// getId
	//========================================================================
	int MPO_Type_create_resized::getId() const
	{
		// returns the unique call identity set by enum MPO_CALL_ID
		return MPO_TYPE_CREATE_RESIZED_ID;
	}

}

#endif /*IS_MPI_TYPE_CREATE_HVECTOR_PRESENT*/

/*EOF*/
