/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Type_get_extent.
 *
 *  @author Tobias Hilbrich
 *
 *  $Id$
 */


#include "mpo-objects.h"
#include "mpo-2-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"
#include <stdio.h>

#ifdef IS_MPI_TYPE_GET_EXTENT_PRESENT

namespace MPO
{
    // default constructor
    //========================================================================
	MPO_Type_get_extent::MPO_Type_get_extent()
		: MPO_Typehandlers (MPI_DATATYPE_NULL)
	{
		//nothing to do
	}

	// proper constructor
	//========================================================================
	MPO_Type_get_extent::MPO_Type_get_extent(
			MPI_Datatype datatype,
    		MPI_Aint* lb,
    		MPI_Aint* extent
    		)
		: MPO_Typehandlers (datatype),
		  my_lb (lb),
		  my_extent (extent)
	{
		//nothing to do
	}

    // default destructor
    //========================================================================
	MPO_Type_get_extent::~MPO_Type_get_extent()
    {
    	//Nothing to do
    }

    // checkPrerequisites
    //========================================================================
    int MPO_Type_get_extent::checkPrerequisites()
    {
    	int ret;

    	our_datatype_traces->findAndAddUsage(my_datatype,getSourceLocation());

    	//Todo think about checks here ...

    	return ret;
    }

    // callNativeMPI
    //========================================================================
    int MPO_Type_get_extent::callNativeMPI()
    {
    	int rvalue = 1;

#ifdef MARMOT_USE_THREADSYNC
    	MPO_Space::getSyncObject()->enterPMPI();
#endif
    	rvalue = PMPI_Type_get_extent (
    			my_datatype,
    			my_lb,
    			my_extent
    			);
#ifdef MARMOT_USE_THREADSYNC
    	MPO_Space::getSyncObject()->leavePMPI();
#endif

    	return rvalue;
    }

    // checkPost
    //========================================================================
   	int MPO_Type_get_extent::checkPost()
   	{
   		//Nothing to do

   		return 0;
   	}

    // debugout
    //========================================================================
  	std::ostream & MPO_Type_get_extent::debugout(std::ostream & output) const
  	{
  		output
  		<< getCallname()
  		<< "(datatype = "
  		<< my_datatype
  		<< ", *lb "
  		<< ", *extent ";

  		if (F_INTERFACE == INTERFACE_MODE)
  		{
  			output << ", ierror";
  		}

  		output
  		<< ")"
  		<< std::endl;

  		return output;
  	}

  	// save
  	//========================================================================
    std::ostream & MPO_Type_get_extent::save(std::ostream & out) const
    {
    	// we don't save and restore pointers
    	MPO_Callobject::save(out);
    	out < my_datatype;
    	//out < my_lb;
    	//out < my_extent;

    	return out;
    }

    // restore
    //========================================================================
    std::istream & MPO_Type_get_extent::restore(std::istream & in)
    {
    	// we don't save and restore pointers
    	MPO_Callobject::restore(in);
    	in > my_datatype;
    	//in > my_lb;
    	//in > my_extent;

    	return in;
    }

	// getCallname
	//========================================================================
	string MPO_Type_get_extent::getCallname() const
	{
		/*  Distinguish between C and Fortran interface, default: C interface.
		 */
		string rvalue = "MPI_Type_get_extent";

		if (F_INTERFACE == INTERFACE_MODE)
		{
			rvalue = "MPI_TYPE_GET_EXTENT";
		}

		return rvalue;
	}

	// getId
	//========================================================================
	int MPO_Type_get_extent::getId() const
	{
		// returns the unique call identity set by enum MPO_CALL_ID
		return MPO_TYPE_GET_EXTENT_ID;
	}

}

#endif /*IS_MPI_TYPE_GET_EXTENT_PRESENT*/

/*EOF*/
