/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of base class MPO_Space.
 *
 *  MPO_Space represents the MPI library with all resources.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: MPO_Space.cc 1112 2010-04-19 11:09:52Z tobias $
 */

#include "config.h"
#include "mpo-objects.h"
#include "mpo-2-objects.h"
#include "mpo-messenger.h"
#include "mpo-autogenerated.h"
#include "mpo-debugserver.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"
#include "mpo_SourceLocation.h"
#include "MPO_ThreadSynchronisationSyncBase.h"
#include "mpo_ThreadSynchronisationDummy.h"
#include "mpo_ThreadSynchronisationOpenMP.h"
#include "mpo-fortran-datatypehelpers.h"



//***************************************************************************
//                              our objects
//***************************************************************************

//========================
// Requests
//========================

std::map < MPO::MPO_Request, enum MPO::KIND_OF_REQUEST >
MPO::MPO_Space::our_requests;

//========================
// Datatypes
//========================

std::map < MPO::MPO_Datatype, enum MPO::KIND_OF_DATATYPE >
MPO::MPO_Space::our_datatypes;

std::map < MPO::MPO_Datatype, std::string >
MPO::MPO_Space::our_datatype_names;

//========================
// Errhandlers
//========================

std::map < MPI_Errhandler, int * >
MPO::MPO_Space::our_errhandlers;

std::vector<int>
MPO::MPO_Space::our_errcodes;

//========================
// Communicators
//========================

MPI_Comm MPO::MPO_Space::our_mpi_comm_world = MPI_COMM_NULL;

std::set < MPO::MPO_Comm >
MPO::MPO_Space::our_comms;

//========================
// Groups
//========================

std::set < MPO::MPO_Group >
MPO::MPO_Space::our_groups;

//========================
// Operators
//========================

std::set < MPO::MPO_Op >
MPO::MPO_Space::our_ops;

std::map < MPO::MPO_Op, std::string >
MPO::MPO_Space::our_op_names;

//========================
// Others
//========================

MPO::MessengerUsingMPI MPO::MPO_Space::our_channel;

MPO::Factory < int, MPO::MPO_Callobject > MPO::MPO_Space::our_call_factory;

int MPO::MPO_Space::buffer_attached = 0;

MPO::MPO_LogFile *MPO::MPO_Space::our_log = NULL;

MPO::MPO_ResourceGatherer<MPO::MPO_Datatype> *MPO::MPO_Space::our_datatype_traces = NULL;
MPO::MPO_ResourceGatherer<MPO::MPO_Op> *MPO::MPO_Space::our_op_traces = NULL;
MPO::MPO_ResourceGatherer<MPO::MPO_Group> *MPO::MPO_Space::our_group_traces = NULL;
MPO::MPO_ResourceGatherer<MPO::MPO_Comm> *MPO::MPO_Space::our_comm_traces = NULL;
MPO::MPO_ResourceGatherer<MPI_Errhandler> *MPO::MPO_Space::our_errhandler_traces = NULL;
MPO::MPO_ResourceGatherer<MPO::MPO_Request> *MPO::MPO_Space::our_request_traces = NULL;

#ifdef MARMOT_THREADED
int MPO::MPO_Space::our_threadlevel_required = MPI_THREAD_SINGLE;
int MPO::MPO_Space::our_threadlevel_provided = MPI_THREAD_SINGLE;
#endif

MPO::MPO_IThreadSynchronisation* MPO::MPO_Space::our_thread_sync = NULL;

bool MPO::MPO_Space::our_finalize_was_called = false;
bool MPO::MPO_Space::our_init_was_called = false;

int MPO::MPO_Space::our_mpi_status_size = 0;

bool MPO::MPO_Space::our_no_debugserver = false;
MPO::MPO_LOGFILE_TYPE MPO::MPO_Space::our_forced_logtype = LAST_LOGFILE_TYPE;

namespace MPO
{

    //************************************************************************
    //                          MPO_Space
    //************************************************************************


    // default constructor
    //========================================================================
    MPO_Space::MPO_Space()
    {
    }


    // destructor
    //========================================================================
    MPO_Space::~MPO_Space()
    {
    }


    // initFactories
    //========================================================================
    // compilation error:
    // Some compilers (e.g. xlC on ibm) require this
    // explicit instantiation here!
    template class Factory < int, MPO_Callobject >;

    void MPO_Space::initFactories()
    {

        /*  We initialize the factories by registrating EVERY object
         *  in the MPO_Space universe. This is a mess, but there is not much
         *  choice, because C++ does not offer persistency as part of the
         *  language.
         */

        // initialize factory for Callobjects:

        // MPI-1.2 standard
        our_call_factory.add < MPO_Abort > ();
        our_call_factory.add < MPO_Address > ();
        our_call_factory.add < MPO_Allgather > ();
        our_call_factory.add < MPO_Allgatherv > ();
        our_call_factory.add < MPO_Allreduce > ();
        our_call_factory.add < MPO_Alltoall > ();
        our_call_factory.add < MPO_Alltoallv > ();
        our_call_factory.add < MPO_Attr_delete > ();
        our_call_factory.add < MPO_Attr_get > ();
        our_call_factory.add < MPO_Attr_put > ();
        our_call_factory.add < MPO_Barrier > ();
        our_call_factory.add < MPO_Bcast > ();
        our_call_factory.add < MPO_Bsend > ();
        our_call_factory.add < MPO_Bsend_init > ();
        our_call_factory.add < MPO_Buffer_attach > ();
        our_call_factory.add < MPO_Buffer_detach > ();
        our_call_factory.add < MPO_Cancel > ();
        our_call_factory.add < MPO_Cart_coords > ();
        our_call_factory.add < MPO_Cart_create > ();
        our_call_factory.add < MPO_Cartdim_get > ();
        our_call_factory.add < MPO_Cart_get > ();
        our_call_factory.add < MPO_Cart_map > ();
        our_call_factory.add < MPO_Cart_rank > ();
        our_call_factory.add < MPO_Cart_shift > ();
        our_call_factory.add < MPO_Cart_sub > ();
        our_call_factory.add < MPO_Comm_compare > ();
        our_call_factory.add < MPO_Comm_create > ();
        our_call_factory.add < MPO_Comm_dup > ();
        our_call_factory.add < MPO_Comm_free > ();
        our_call_factory.add < MPO_Comm_group > ();
        our_call_factory.add < MPO_Comm_rank > ();
        our_call_factory.add < MPO_Comm_remote_group > ();
        our_call_factory.add < MPO_Comm_remote_size > ();
        our_call_factory.add < MPO_Comm_size > ();
        our_call_factory.add < MPO_Comm_split > ();
        our_call_factory.add < MPO_Comm_test_inter > ();
        our_call_factory.add < MPO_Dims_create > ();
        our_call_factory.add < MPO_Errhandler_create > ();
        our_call_factory.add < MPO_Errhandler_free > ();
        our_call_factory.add < MPO_Errhandler_get > ();
        our_call_factory.add < MPO_Errhandler_set > ();
        our_call_factory.add < MPO_Error_class > ();
        our_call_factory.add < MPO_Error_string > ();
        our_call_factory.add < MPO_Finalize > ();
        our_call_factory.add < MPO_Gather > ();
        our_call_factory.add < MPO_Gatherv > ();
        our_call_factory.add < MPO_Get_count > ();
        our_call_factory.add < MPO_Get_elements > ();
        our_call_factory.add < MPO_Get_processor_name > ();
        our_call_factory.add < MPO_Get_version > ();
        our_call_factory.add < MPO_Graph_create > ();
        our_call_factory.add < MPO_Graphdims_get > ();
        our_call_factory.add < MPO_Graph_get > ();
        our_call_factory.add < MPO_Graph_map > ();
        our_call_factory.add < MPO_Graph_neighbors > ();
        our_call_factory.add < MPO_Graph_neighbors_count > ();
        our_call_factory.add < MPO_Group_compare > ();
        our_call_factory.add < MPO_Group_difference > ();
        our_call_factory.add < MPO_Group_excl > ();
        our_call_factory.add < MPO_Group_free > ();
        our_call_factory.add < MPO_Group_incl > ();
        our_call_factory.add < MPO_Group_intersection > ();
        our_call_factory.add < MPO_Group_range_excl > ();
        our_call_factory.add < MPO_Group_range_incl > ();
        our_call_factory.add < MPO_Group_rank > ();
        our_call_factory.add < MPO_Group_size > ();
        our_call_factory.add < MPO_Group_translate_ranks > ();
        our_call_factory.add < MPO_Group_union > ();
        our_call_factory.add < MPO_Ibsend > ();
        our_call_factory.add < MPO_Init > ();
        our_call_factory.add < MPO_Initialized > ();
        our_call_factory.add < MPO_Intercomm_create > ();
        our_call_factory.add < MPO_Intercomm_merge > ();
        our_call_factory.add < MPO_Iprobe > ();
        our_call_factory.add < MPO_Irecv > ();
        our_call_factory.add < MPO_Irsend > ();
        our_call_factory.add < MPO_Isend > ();
        our_call_factory.add < MPO_Issend > ();
        our_call_factory.add < MPO_Keyval_create > ();
        our_call_factory.add < MPO_Keyval_free > ();
        our_call_factory.add < MPO_Op_create > ();
        our_call_factory.add < MPO_Op_free > ();
        our_call_factory.add < MPO_Pack > ();
        our_call_factory.add < MPO_Pack_size > ();
        // our_call_factory.add < MPO_Pcontrol > ();
        our_call_factory.add < MPO_Probe > ();
        our_call_factory.add < MPO_Recv > ();
        our_call_factory.add < MPO_Recv_init > ();
        our_call_factory.add < MPO_Reduce > ();
        our_call_factory.add < MPO_Reduce_scatter > ();
        our_call_factory.add < MPO_Request_free > ();
        our_call_factory.add < MPO_Rsend > ();
        our_call_factory.add < MPO_Rsend_init > ();
        our_call_factory.add < MPO_Scan > ();
        our_call_factory.add < MPO_Scatter > ();
        our_call_factory.add < MPO_Scatterv > ();
        our_call_factory.add < MPO_Send > ();
        our_call_factory.add < MPO_Send_init > ();
        our_call_factory.add < MPO_Sendrecv > ();
        our_call_factory.add < MPO_Sendrecv_replace > ();
        our_call_factory.add < MPO_Ssend > ();
        our_call_factory.add < MPO_Ssend_init > ();
        our_call_factory.add < MPO_Start > ();
        our_call_factory.add < MPO_Startall > ();
        our_call_factory.add < MPO_Test > ();
        our_call_factory.add < MPO_Testall > ();
        our_call_factory.add < MPO_Testany > ();
        our_call_factory.add < MPO_Test_cancelled > ();
        our_call_factory.add < MPO_Testsome > ();
        our_call_factory.add < MPO_Topo_test > ();
        our_call_factory.add < MPO_Type_commit > ();
        our_call_factory.add < MPO_Type_contiguous > ();
        our_call_factory.add < MPO_Type_extent > ();
        our_call_factory.add < MPO_Type_free > ();
        our_call_factory.add < MPO_Type_hindexed > ();
        our_call_factory.add < MPO_Type_hvector > ();
        our_call_factory.add < MPO_Type_indexed > ();
        our_call_factory.add < MPO_Type_lb > ();
        our_call_factory.add < MPO_Type_size > ();
        our_call_factory.add < MPO_Type_struct > ();
        our_call_factory.add < MPO_Type_ub > ();
        our_call_factory.add < MPO_Type_vector > ();
        our_call_factory.add < MPO_Unpack > ();
        our_call_factory.add < MPO_Wait > ();
        our_call_factory.add < MPO_Waitall > ();
        our_call_factory.add < MPO_Waitany > ();
        our_call_factory.add < MPO_Waitsome > ();
        // our_call_factory.add < MPO_Wtick >();
        // our_call_factory.add < MPO_Wtime >();

        // MPI-2 standard - file I/O
        // not implemented yet
#ifdef MARMOT_IO
//         our_call_factory.add < MPO_File_close > ();
//         our_call_factory.add < MPO_File_delete > ();
//         our_call_factory.add < MPO_File_get_amode > ();
//         our_call_factory.add < MPO_File_get_atomicity > ();
//         our_call_factory.add < MPO_File_get_byte_offset > ();
//         our_call_factory.add < MPO_File_get_group > ();
//         our_call_factory.add < MPO_File_get_info > ();
//         our_call_factory.add < MPO_File_get_position > ();
//         our_call_factory.add < MPO_File_get_position_shared > ();
//         our_call_factory.add < MPO_File_get_size > ();
//         our_call_factory.add < MPO_File_get_type_extent > ();
//         our_call_factory.add < MPO_File_get_view > ();
//         our_call_factory.add < MPO_File_iread_at > ();
//         our_call_factory.add < MPO_File_iread > ();
//         our_call_factory.add < MPO_File_iread_shared > ();
//         our_call_factory.add < MPO_File_iwrite_at > ();
//         our_call_factory.add < MPO_File_iwrite > ();
//         our_call_factory.add < MPO_File_iwrite_shared > ();
//         our_call_factory.add < MPO_File_open > ();
//         our_call_factory.add < MPO_File_preallocate > ();
//         our_call_factory.add < MPO_File_read_all_begin > ();
//         our_call_factory.add < MPO_File_read_all > ();
//         our_call_factory.add < MPO_File_read_all_end > ();
//         our_call_factory.add < MPO_File_read_at_all_begin > ();
//         our_call_factory.add < MPO_File_read_at_all > ();
//         our_call_factory.add < MPO_File_read_at_all_end > ();
//         our_call_factory.add < MPO_File_read_at > ();
//         our_call_factory.add < MPO_File_read > ();
//         our_call_factory.add < MPO_File_read_ordered_begin > ();
//         our_call_factory.add < MPO_File_read_ordered > ();
//         our_call_factory.add < MPO_File_read_ordered_end > ();
//         our_call_factory.add < MPO_File_read_shared > ();
//         our_call_factory.add < MPO_File_seek > ();
//         our_call_factory.add < MPO_File_seek_shared > ();
//         our_call_factory.add < MPO_File_set_atomicity > ();
//         our_call_factory.add < MPO_File_set_info > ();
//         our_call_factory.add < MPO_File_set_size > ();
//         our_call_factory.add < MPO_File_set_view > ();
//         our_call_factory.add < MPO_File_write_all_begin > ();
//         our_call_factory.add < MPO_File_write_all > ();
//         our_call_factory.add < MPO_File_write_all_end > ();
//         our_call_factory.add < MPO_File_write_at_all_begin > ();
//         our_call_factory.add < MPO_File_write_at_all > ();
//         our_call_factory.add < MPO_File_write_at_all_end > ();
//         our_call_factory.add < MPO_File_write_at > ();
//         our_call_factory.add < MPO_File_write > ();
//         our_call_factory.add < MPO_File_write_ordered_begin > ();
//         our_call_factory.add < MPO_File_write_ordered > ();
//         our_call_factory.add < MPO_File_write_ordered_end > ();
//         our_call_factory.add < MPO_File_write_shared > ();
//         our_call_factory.add < MPO_Register_datarep > ();
//         our_call_factory.add < MPO_File_create_errhandler > ();
//         our_call_factory.add < MPO_File_get_errhandler > ();
//         our_call_factory.add < MPO_File_set_errhandler > ();
//         our_call_factory.add < MPO_File_c2f > ();
//         our_call_factory.add < MPO_File_f2c > ();
#endif

        // MPI-2 standard - one-sided communication
#ifdef IS_MPI_WIN_CREATE_PRESENT
         our_call_factory.add < MPO_Win_create > ();
#endif /*IS_MPI_WIN_CREATE_PRESENT*/
#ifdef IS_MPI_WIN_FREE_PRESENT
         our_call_factory.add < MPO_Win_free > ();
#endif /*IS_MPI_WIN_FREE_PRESENT*/
#ifdef IS_MPI_WIN_GET_GROUP_PRESENT
         our_call_factory.add < MPO_Win_get_group > ();
#endif /*IS_MPI_WIN_GET_GROUP_PRESENT*/
#ifdef IS_MPI_GET_PRESENT
         our_call_factory.add < MPO_Get > ();
#endif /*IS_MPI_GET_PRESENT*/
#ifdef IS_MPI_PUT_PRESENT
         our_call_factory.add < MPO_Put > ();
#endif /*IS_MPI_PUT_PRESENT*/
#ifdef IS_MPI_ACCUMULATE_PRESENT
         our_call_factory.add < MPO_Accumulate > ();
#endif /*IS_MPI_ACCUMULATE_PRESENT*/
#ifdef IS_MPI_ALLOC_MEM_PRESENT
         our_call_factory.add < MPO_Alloc_mem > ();
#endif /*IS_MPI_ALLOC_MEM_PRESENT*/
#ifdef IS_MPI_FREE_MEM_PRESENT
         our_call_factory.add < MPO_Free_mem > ();
#endif /*IS_MPI_FREE_MEM_PRESENT*/
#ifdef IS_MPI_WIN_FENCE_PRESENT
         our_call_factory.add < MPO_Win_fence > ();
#endif /*IS_MPI_WIN_FENCE_PRESENT*/
#ifdef IS_MPI_WIN_START_PRESENT
         our_call_factory.add < MPO_Win_start > ();
#endif /*IS_MPI_WIN_START_PRESENT*/
#ifdef IS_MPI_WIN_COMPLETE_PRESENT
         our_call_factory.add < MPO_Win_complete > ();
#endif /*IS_MPI_WIN_COMPLETE_PRESENT*/
#ifdef IS_MPI_WIN_POST_PRESENT
         our_call_factory.add < MPO_Win_post > ();
#endif /*IS_MPI_WIN_POST_PRESENT*/
#ifdef IS_MPI_WIN_WAIT_PRESENT
         our_call_factory.add < MPO_Win_wait > ();
#endif /*IS_MPI_WIN_WAIT_PRESENT*/
#ifdef IS_MPI_WIN_TEST_PRESENT
         our_call_factory.add < MPO_Win_test > ();
#endif /*IS_MPI_WIN_TEST_PRESENT*/
#ifdef IS_MPI_WIN_LOCK_PRESENT
         our_call_factory.add < MPO_Win_lock > ();
#endif /*IS_MPI_WIN_LOCK_PRESENT*/
#ifdef IS_MPI_WIN_UNLOCK_PRESENT
         our_call_factory.add < MPO_Win_unlock > ();
#endif /*IS_MPI_WIN_UNLOCK_PRESENT*/
//         our_call_factory.add < MPO_Win_create_errhandler > ();
//         our_call_factory.add < MPO_Win_get_errhandler > ();
//         our_call_factory.add < MPO_Win_set_errhandler > ();
//         our_call_factory.add < MPO_Win_call_errhandler > ();
//         our_call_factory.add < MPO_Win_create_keyval > ();
//         our_call_factory.add < MPO_Win_free_keyval > ();
//         our_call_factory.add < MPO_Win_set_attr > ();
//         our_call_factory.add < MPO_Win_get_attr > ();
//         our_call_factory.add < MPO_Win_delete_attr > ();
//         our_call_factory.add < MPO_Win_set_name > ();
//         our_call_factory.add < MPO_Win_get_name > ();
//         our_call_factory.add < MPO_Win_f2c > ();
//         our_call_factory.add < MPO_Win_c2f > ();

         // MPI-2 standard - Threading support
#ifdef MARMOT_THREADED
		   our_call_factory.add < MPO_Init_thread > ();
#endif

		   //Type creation calls of MPI2d
#ifdef IS_MPI_GET_ADDRESS_PRESENT
		   our_call_factory.add < MPO_Get_address > ();
#endif /*IS_MPI_GET_ADDRESS_PRESENT*/
#ifdef IS_MPI_TYPE_CREATE_STRUCT_PRESENT
		   our_call_factory.add < MPO_Type_create_struct > ();
#endif /*IS_MPI_TYPE_CREATE_STRUCT_PRESENT*/
#ifdef IS_MPI_TYPE_CREATE_HINDEXED_PRESENT
		   our_call_factory.add < MPO_Type_create_hindexed > ();
#endif /*IS_MPI_TYPE_CREATE_HINDEXED_PRESENT*/
#ifdef IS_MPI_TYPE_CREATE_HVECTOR_PRESENT
		   our_call_factory.add < MPO_Type_create_hvector > ();
#endif /*IS_MPI_TYPE_CREATE_HVECTOR_PRESENT*/
#ifdef IS_MPI_TYPE_CREATE_DARRAY_PRESENT
		   our_call_factory.add < MPO_Type_create_darray > ();
#endif /*IS_MPI_TYPE_CREATE_DARRAY_PRESENT*/
#ifdef IS_MPI_TYPE_CREATE_SUBARRAY_PRESENT
		   our_call_factory.add < MPO_Type_create_subarray > ();
#endif /*IS_MPI_TYPE_CREATE_SUBARRAY_PRESENT*/
#ifdef IS_MPI_TYPE_CREATE_INDEXED_BLOCK_PRESENT
		   our_call_factory.add < MPO_Type_create_indexed_block > ();
#endif /*IS_MPI_TYPE_CREATE_INDEXED_BLOCK_PRESENT*/
#ifdef IS_MPI_TYPE_CREATE_RESIZED_PRESENT
		   our_call_factory.add < MPO_Type_create_resized > ();
#endif /*IS_MPI_TYPE_CREATE_RESIZED_PRESENT*/
#ifdef IS_MPI_TYPE_GET_EXTENT_PRESENT
		   our_call_factory.add < MPO_Type_get_extent > ();
#endif /*IS_MPI_TYPE_GET_EXTENT_PRESENT*/

		   //Extended Collectives
#ifdef IS_MPI_ALLTOALLW_PRESENT
        our_call_factory.add < MPO_Alltoallw > ();
#endif /*IS_MPI_ALLTOALLW_PRESENT*/
#ifdef IS_MPI_EXSCAN_PRESENT
        our_call_factory.add < MPO_Exscan > ();
#endif /*IS_MPI_EXSCAN_PRESENT*/
    }                           // end initFactories


	// getSyncObject
    //========================================================================
	MPO_IThreadSynchronisation* MPO_Space::getSyncObject(void)
	{
		if (our_thread_sync == NULL)
		{
			// Initialize the thread synchronisation object using the global
			// defines.
	       	#ifdef MARMOT_OPENMP
	       		our_thread_sync = new MPO_ThreadSynchronisationOpenMP ();
	       	#else
	       		our_thread_sync = new MPO_ThreadSynchronisationDummy ();
	       	#endif
		}

		return our_thread_sync;
	}

	// getLogFile
    //========================================================================
	MPO_LogFile* MPO_Space::getLogFile(void)
	{
		return our_log;
	}

#ifdef MARMOT_THREADED
   	int MPO_Space::getProvidedThreadLevel (void)
   	{
   		return our_threadlevel_provided;
   	}
#endif

    // getLogFile
    //========================================================================
    const char* MPO_Space::getThreadLevelName(int level)
    {
    	switch (level)
	    {
	        case MPI_THREAD_SINGLE:
	            return "MPI_THREAD_SINGLE";
	        case MPI_THREAD_FUNNELED:
	            return "MPI_THREAD_FUNNELED";
	        case MPI_THREAD_SERIALIZED:
	            return "MPI_THREAD_SERIALIZED";
	        case MPI_THREAD_MULTIPLE:
    		    return "MPI_THREAD_MULTIPLE";
	    }

    	return "UNKNOWN Thread Level";
    }

    // wasFinalizeCalled
    //========================================================================
    bool MPO_Space::wasFinalizeCalled (void)
    {
        return our_finalize_was_called;
    }

    // wasInitCalled
    //========================================================================
    bool MPO_Space::wasInitCalled (void)
    {
        return our_init_was_called;
    }

    // wasInitCalled
    //========================================================================
    int MPO_Space::getStatusSizeFortran (void)
    {
        if (our_mpi_status_size == 0)
        {
#ifndef MARMOT_NO_FORTRAN
            GET_MPI_STATUS_SIZE(&our_mpi_status_size);
#else
            our_mpi_status_size = sizeof(MPI_Status)/sizeof(int);
#endif
        }

        return our_mpi_status_size;
    }

    // disableDebugServer
    //========================================================================
    void MPO_Space::disableDebugServer (void)
    {
        our_no_debugserver = true;
    }

    // isDebugServerEnabled
    //========================================================================
    bool MPO_Space::isDebugServerEnabled (void)
    {
        return (!our_no_debugserver);
    }

    // forceLogType
    //========================================================================
    void MPO_Space::forceLogType (MPO_LOGFILE_TYPE type)
    {
        our_forced_logtype = type;
    }

    // isLogTypeForced
    //========================================================================
    bool MPO_Space::isLogTypeForced (void)
    {
        if (our_forced_logtype != LAST_LOGFILE_TYPE)
            return true;
        return false;
    }

    // getForcedLogType
    //========================================================================
    MPO_LOGFILE_TYPE MPO_Space::getForcedLogType (void)
    {
        return our_forced_logtype;
    }
}                               // end of namespace MPO
