#include "ParseErrCodesEnv.h"


ParseErrCodesEnv::ParseErrCodesEnv(std::string str, int numcodes, int defaultval)
{
	ClassName.assign("ParseErrCodesEnv");			//name of the class
	StdOutPrefix.assign("[");				//prefix for output to stdout
	StdErrPrefix.assign("[");				//prefix for output to stderr
	StdOutPrefix.append(ClassName);
	StdErrPrefix.append(ClassName);
	StdOutPrefix.append("]:");
	StdErrPrefix.append(" - ERROR]:");
	expr="";
	expr+=str;						//store expression in expr
	code_vector=new std::vector <int> (numcodes,-1);	//initialize code_vector
	this->numcodes=numcodes;				//and code_count
	if((defaultval==1)||(defaultval==0))			//valid defaultval provided?
	{
		this->defaultval=defaultval;			//then use it
	}
	else							//if not...
	{
		std::cerr << StdErrPrefix << "Default value must be 0 or 1 (now set to 1)" << std::endl;
		this->defaultval=CODES_DEFAULTVALUE;		//...set it to CODES_DEFAULTVALUE;
	}
#ifdef DEBUG
	std::cout << StdOutPrefix << "Constructor called, code_vector initialized for "<<this->numcodes<< " codes, default value =  "<< this->defaultval<<std::endl;
#endif
}
void ParseErrCodesEnv::printExpr()
{
	std::cout << StdOutPrefix << "expr: " <<expr<<std::endl;//just print the expression
}
void ParseErrCodesEnv::printCodeVector()
{
	std::cout << StdOutPrefix << "CodeVector entries: "<<std::endl<<StdOutPrefix;
	for (unsigned int i=0;i<(*code_vector).size();i++)		//print all entries of the code vector
	{
		if((*code_vector)[i]!=-1)
			std::cout << "cv["<<i<<"]="<<(*code_vector)[i]<<", ";
	}
	std::cout<<std::endl;

}
std::vector <int> *ParseErrCodesEnv::getCodeVector()
{
	return code_vector;
}
void ParseErrCodesEnv::evaluate()
{
#ifdef DEBUG
	std::cout << StdOutPrefix << "Starting evaluation"<<std::endl;
#endif
	std::vector <int> *defvec=new std::vector <int> (numcodes,defaultval); //vector with the default values set
	std::string exprwow = "";					//expression without whitespace
	if(expr.size()==0)				//expression empty?
	{
		record(defvec);				//then use default values
		return;
	}
	for(unsigned int i=0;i<expr.size();i++)			//look at expression, token by token
	{
		if(!isspace(expr[i]))			//if it's not whitespace, append it to exprwow
			exprwow.append(1,expr[i]);
	}
	std::vector <int> *tmp=evaluate(exprwow);		//the vector (w/o whitespaces) w ith the entries defined by the expression
	if(tmp)							//no parse error
	{
		record(defvec);					//first apply the default values
		record(tmp);					//now overwrite where necessary
		delete tmp;					//and delete the vector
	}
	else							//parse error
	{
		std::cerr << StdErrPrefix << "Error while parsing. Will use default values" << std::endl;
		record(defvec);					//apply the default values and do nothing else
	}
	delete defvec;						//we don't need this vector anymore
	return;
}
std::vector <int> *ParseErrCodesEnv::evaluate(std::string str)
{
	std::vector <int> seppositions;	//vector with the positions of the separators (",")
	std::vector <int> *tmp,*ret;	//pointers we need later
#ifdef DEBUG
	std::cout<< StdOutPrefix <<"evaluate called with: "<<str<<std::endl;
#endif
	int errorcount;			//if any of the comma separated expressions fails to parse this value is incremented
	int inside_bracket=0;		//to see whether we are inside some brackets
	seppositions.push_back(-1);	//we set the first separator so the for loop gets easier
	for (unsigned int i=0;i<str.size();i++)	//have a look at the expression, token by token
	{

		if(str[i]=='(')inside_bracket++;	//now we are inside some brackets
		if(str[i]==')')inside_bracket--;	//one bracket closed (we can still be inside some brackets)
		if((str[i]==',')&&(!inside_bracket))	//we are not inside brackets and have found a seperator
		{
#ifdef DEBUG
			std::cout<< StdOutPrefix <<"Found separator at position "<<i<<std::endl;
#endif
			seppositions.push_back(i);	//add the separator's position to the position's vector
		}
	}
	seppositions.push_back(str.size());		//we set the last separator to be immediatley after the string
	if(seppositions.size()>2)			//do we have at least one separator?
	{
		ret=new std::vector <int> (numcodes,-1); //initialize the return vector
		errorcount=0;					//no errors yet
		for (unsigned int i=0;i<seppositions.size()-1;i++)	//check all the sections
		{
#ifdef DEBUG
			std::cout << StdOutPrefix<< "Now evaluating from position "<<seppositions[i]+1<< " to position "<< seppositions[i]+seppositions[i+1]-seppositions[i]-1<<std::endl;
#endif
			tmp=evaluate(str.substr(seppositions[i]+1,seppositions[i+1]-seppositions[i]-1)); //evaluate the section
			if(tmp)				//no parse error
			{
				for (unsigned int j=0;j<(*tmp).size();j++)	//update the return vector
				{
					if((*tmp)[j]==1)(*ret)[j]=1;	//only update if the entry is 1...
				       	if((*tmp)[j]==0)(*ret)[j]=0;	//...or 0
				}
				delete tmp;				//delete this vector, we don't need it anymore
			}
			else				//parse error
			{
				errorcount++;		//increment the error count
			}

		}
		if(errorcount)				//at least one section had a parse error
		{
			delete ret;			//we don't need this vector anymore, now that we have an error
			return NULL;			//return NULL instead
		}
		return ret;				//otherwise (no errors) return the "merged" vector
	}
	else						//we didn't find a separator at the top level
	{
		if(str[0]=='^')			//is the next token the negation symbol?
		{
			tmp=evaluate(str.substr(1));	//then evaluate the substring after this token
			if(tmp)			//np parse error
			{
				for (unsigned int i=0;i<(*tmp).size();i++)	//have a look at the returned vector
				{
					if((*tmp)[i]==0)		//negate the 0s...
						(*tmp)[i]=1;
					else if((*tmp)[i]==1)		//and the 1s
						(*tmp)[i]=0;
				}
				return tmp;				//and return this vector
			}
			return NULL;			//parse error, return NULL
		}
		if(str[0]=='(')			//next token opening bracket?
		{
			if(str[str.size()-1]==')')	//then the last token of the string must be a closing bracket
				return evaluate(str.substr(1,str.size()-2)); //evaluate the inside
			else				//the last token of the string is not a closing bracket...
			{
				std::cerr << StdErrPrefix << "Syntax error: ) expected"<<std::endl;
				return NULL;		//then we have a syntax error, return NULL
			}
		}
		//now only numbers and ranges are allowed
		if(str.find_first_not_of("0123456789-")!=std::string::npos)	//do we have other tokens?
		{
			std::cerr << StdErrPrefix << "Syntax error: Invalid token"<<std::endl;
			return NULL;	//if yes, we have a syntax error, return NULL
		}
		//the string must now start with a digit
		if(!isdigit(str[0]))			//does the string start with a digit?
		{
			std::cerr << StdErrPrefix <<"Syntax error: Digit expected"<<std::endl;
			return NULL;	//if not, we have a syntax error, return NULL
		}
		//do we have a range?
		std::string::size_type dashpos;			//for the position of the dash
		if(str.find_first_of("-")!=str.find_last_of("-"))	//more then one dash? -> not allowed
		{
			std::cerr << StdErrPrefix << "Syntax error: Too many dashes"<<std::endl;
			return NULL;	//syntax error, return NULL
		}
		dashpos=str.find_first_of("-");			//get the position of the dash
		if(dashpos!=std::string::npos)	//there is a "-"
		{
			int lvalue;			//left value
			int rvalue;			//right value
			std::istringstream lbuffer(str.substr(0,dashpos));//the string left to the dash
			std::istringstream rbuffer(str.substr(dashpos+1));//the string right to the dash
			lbuffer >> lvalue;	//get left value
			rbuffer >> rvalue;	//get right value
			if((!validCodeNumber(lvalue))||(!validCodeNumber(rvalue)))	//are the codes within the allowed range?
			{
				std::cerr << StdErrPrefix <<"Code number out of range"<<std::endl;
				return NULL;	//at least one value out of range, return NULL
			}
			if(lvalue>rvalue)	//if lvalue is greater then rvalue, we switch the values
			{
				int tvalue;
				tvalue=lvalue;
				lvalue=rvalue;
				rvalue=tvalue;
			}
#ifdef DEBUG
			std::cout<< StdOutPrefix << "I have a range, from " <<lvalue<< " to "<<rvalue<<std::endl;
#endif
			tmp=new std::vector <int> (numcodes,-1);	//create a new vector
			for(int i=lvalue;i<=rvalue;i++)
				(*tmp)[i]=1;				//set the defined range to 1
			return tmp;					//return this vector

		}
		else				//a simple number
		{
			std::istringstream buffer(str);		//the string
			int value;				//the value
			buffer >> value;			//get the value
			if(!validCodeNumber(value))		//code number valid?
			{
				std::cerr<< StdErrPrefix << "Error code out of range"<<std::endl;
				return NULL;	//if not, return NULL
			}
#ifdef DEBUG
			std::cout<< StdOutPrefix << "I have a number: "<<value<<std::endl;
#endif
			tmp=new std::vector <int> (numcodes,-1);	//create a new vector
			(*tmp)[value]=1;				//set the specified code to 1
			return tmp;					//return the vector
		}
	}

	return NULL;			//an error occured, return NULL
}

ParseErrCodesEnv::~ParseErrCodesEnv(void)
{
    if (code_vector)
        delete (code_vector);
    code_vector = NULL;
}

void ParseErrCodesEnv::record(std::vector <int> *vec)
{
	for (unsigned int i=0;i<vec->size();i++)		//have a look at the vector vec
	{
		if((*vec)[i]==1)		//copy the 1s from vec to the code_vector
			(*code_vector)[i]=1;
		if((*vec)[i]==0)		//copy the 0s from vec to the code_vector
			(*code_vector)[i]=0;
	}
}

int ParseErrCodesEnv::validCodeNumber(int number)
{
	if((number<0)||(number>=numcodes))	//is the number <0 or >=numcode
		return 0;		//then it is out of range
	return 1;				//otherwise, it's fine
}
int ParseErrCodesEnv::getValue(int code)
{
	if(validCodeNumber(code))
		return (*code_vector)[code];
	return -1;
}

void ParseErrCodesEnv::getErrCodes(bool *errcodes)
{
	for(int i=0;i<numcodes;i++)
	{
		errcodes[i]=(*code_vector)[i]==0?false:true;
	}
	return;
}

std::string *ParseErrCodesEnv::getString()
{
	//ret_str=new std::string;
	std::stringstream tmp_ss;
	int i;
	for(i=0;i<numcodes;i++)
	{
		if(!(*code_vector)[i])
		{
			tmp_ss<<"^";
		}
		tmp_ss<<i;
		if(i<numcodes-1)
		{
			tmp_ss<<",";
		}
	}
	return new std::string(tmp_ss.str());
}
