/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
#include <cstring>
#include <cstdlib>
#include <iostream>
#include <assert.h>
#include "NetUtil.h"
#include "SocketAddr.h"

using namespace std;

// disable depracation warning caused by sprintf (we know it's safe because no user defined strings are used)
#ifdef _MSC_VER
#pragma warning(disable:4996)
#endif

CSocketAddr::CSocketAddr() : m_valid(false)
{
  memset(&m_addr, 0, sizeof(m_addr));
  m_addr.sin_family = AF_INET;
#ifdef NET_HAVE_SALEN
  m_addr.sin_len = sizeof(struct sockaddr_in);
#endif
}

CSocketAddr::CSocketAddr(const std::string &host, unsigned short port) : m_valid(false)
{
  struct in_addr addr;

  memset(&m_addr, 0, sizeof(m_addr));
  m_addr.sin_family = AF_INET;
  net_hostent_ptr hostent = NetUtil::getHostEnt(host);
  if (hostent)
  {
    addr = *(struct in_addr *)hostent->h_addr;
    memcpy(&m_addr.sin_addr, &addr, sizeof(addr));
    m_addr.sin_port = htons(port);
    m_valid = true;
  } else {
    m_addr.sin_port = htons(port);
  }
}

CSocketAddr::CSocketAddr(const std::string &hostAndPort) : m_valid(false)
{
  struct in_addr addr;
  string hoststr, portstr;
  int portno=0;
  string::size_type pos = hostAndPort.find(":");
  if (pos != string::npos)
  {
    hoststr = hostAndPort.substr(0, pos);
    if ((pos+1) >= hostAndPort.length())
      return;
    portstr = hostAndPort.substr(pos+1, hostAndPort.length()-pos-1);
    if (portstr.length() > 5)  // more than 5 digits must be wrong
      return;
    portno = atoi(portstr.c_str());
    if ((portno <= 0) || (portno > 65535))
      return;
  }
  memset(&m_addr, 0, sizeof(m_addr));
  m_addr.sin_family = AF_INET;
  net_hostent_ptr hostent = NetUtil::getHostEnt(hoststr);
  if (hostent)
  {
    addr = *(struct in_addr *)hostent->h_addr;
    memcpy(&m_addr.sin_addr, &addr, sizeof(addr));
    m_addr.sin_port = htons(portno);
    m_valid = true;
  } else {
    m_addr.sin_port = htons(portno);
  }
}

CSocketAddr::CSocketAddr(const CSocketAddr &addr)
{
  memcpy(&m_addr, &addr.m_addr, sizeof(m_addr));
  m_valid = addr.m_valid;
}

CSocketAddr::CSocketAddr(const struct sockaddr_in* addr)
{
  memcpy(&m_addr, addr, sizeof(m_addr));
  m_valid = true;
}

CSocketAddr::~CSocketAddr()
{
}

CSocketAddr& CSocketAddr::operator=(const CSocketAddr &addr)
{
  if (this == &addr)
    return *this;
  memcpy(&m_addr, &addr.m_addr, sizeof(m_addr));
  m_valid = addr.m_valid;
  return *this;
}

void CSocketAddr::swap(CSocketAddr &addr)
{
  struct sockaddr_in tmp_addr;
  bool   tmp_valid;
  memcpy(&tmp_addr, &m_addr, sizeof(tmp_addr));
  tmp_valid = m_valid;
  memcpy(&m_addr, &addr.m_addr, sizeof(m_addr));
  m_valid = addr.m_valid;
  memcpy(&addr.m_addr, &tmp_addr, sizeof(addr.m_addr));
  addr.m_valid = tmp_valid;
}

std::string CSocketAddr::getHost() const
{
  string host("");
  char *hostname = inet_ntoa(m_addr.sin_addr);
  if (hostname)
    host = hostname;
  return host;
}

unsigned short CSocketAddr::getPort() const
{
  return ntohs(m_addr.sin_port);
}

net_socklen_t CSocketAddr::getLength() const
{
  return sizeof(m_addr);
}

const struct sockaddr* CSocketAddr::getAddr() const
{
  return reinterpret_cast<const struct sockaddr*>(&m_addr);
}

int CSocketAddr::getAF() const
{
  return m_addr.sin_family;
}

std::string CSocketAddr::toString() const
{
  char port[12];
  sprintf(port, "%d", getPort());
  string host_and_port = getHost();
  host_and_port.append(":");
  host_and_port.append(port);
  return host_and_port;
}

bool CSocketAddr::isValid() const
{
  return m_valid;
}
