/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definitions of environment and other global variables.
 *
 *  @author Bettina Krammer, Tobias Hilbrich
 *
 *  $Id: mpo-envvar.cc 1062 2010-02-05 13:45:16Z hpczink $
 */


#include "mpo-envvar.h"
#include "mpo-enums.h"


namespace MPO
{

    //==========================
    // Environment variables
    //==========================

    // see mpo_Init.cc and mpo-debugserver.cc

    int DEBUG_MODE = 0;
    const int DEBUG_MODE_DEFAULT = 2;

    int INTERFACE_MODE = LAST_INTERFACE_MODE;
    const int INTERFACE_MODE_DEFAULT = 0;

    int SERIALIZE = 0;
    const int SERIALIZE_DEFAULT = 0; //default 0

    long TRACE_CALLS = 0;
    const long TRACE_CALLS_DEFAULT = 0;      // default 0

    int LOGFILE_TYPE = 0;
    const int LOGFILE_TYPE_DEFAULT = 0; //default 0

    int LOGFLUSH_TYPE = 0;
    const int LOGFLUSH_TYPE_DEFAULT = 0;

    int LOGFILTER_COUNT = 0;
    const int LOGFILTER_COUNT_DEFAULT = 2;

    int RESOURCE_TRACE_TYPE = 0;
    const int RESOURCE_TRACE_TYPE_DEFAULT = 0; //default 0

    long MAX_PEND_COUNT = 0;
    const long MAX_PEND_COUNT_DEFAULT = 10;  // default 10

//clean// use MAX_TIMEOUT_DEADLOCK and MAX_TIMEOUT_SERIALIZE instead of
//clean// long MAX_TIMEOUT           = 0;
    long MAX_TIMEOUT_DEADLOCK = 0;
    const long MAX_TIMEOUT_DEADLOCK_DEFAULT = 1000000;  // default 1 sec

    long MAX_TIMEOUT_SERIALIZE = 0;
    const long MAX_TIMEOUT_SERIALIZE_DEFAULT = 1000;
    // default 1000 microsec

    const string MARMOT_LOGFILE_PATH_DEFAULT = "";

    //===========================================
    // Additional variables for MAX_TIMEOUT
    // to test performance and deadlock detection
    //===========================================

// everything related to these test variables is commented out
// with "//test p&d//" to switch them off

    long MAX_TIMEOUT_ONE = 0;
    const long MAX_TIMEOUT_ONE_DEFAULT = 0;  // default 0 microsec

    long MAX_TIMEOUT_TWO = 0;
    const long MAX_TIMEOUT_TWO_DEFAULT = 0;  // default 0 microsec

//test p&d// long MAX_TIMEOUT_THREE = 0;
//test p&d// long MAX_TIMEOUT_FOUR  = 0;
//test p&d// long MAX_TIMEOUT_FIVE  = 0;
//test p&d// long MAX_TIMEOUT_SIX   = 0;
//test p&d// long MAX_TIMEOUT_SEVEN = 0;

//clean// for the cleaned up version we use
//clean// MAX_TIMEOUT_DEADLOCK  for MAX_TIMEOUT_SEVEN
//clean// MAX_TIMEOUT_SERIALIZE for MAX_TIMEOUT_SIX


    //==========================
    // Strings for output
    //==========================

    const string MSG_ERROR = "ERROR: ";
    const string MSG_WARNING = "WARNING: ";
    const string MSG_NOTE = "NOTE: ";

    const string MSG_COUNT = "Count";
    const string MSG_SENDCOUNT = "Send count";
    const string MSG_RECVCOUNT = "Receive count";
    const string MSG_INCOUNT = "Incount";
    const string MSG_OUTCOUNT = "Outcount";

    const string MSG_INSIZE = "Insize";
    const string MSG_OUTSIZE = "Outsize";

    const string MSG_TAG = "Tag";
    const string MSG_SENDTAG = "Send tag";
    const string MSG_RECVTAG = "Receive tag";

    const string MSG_NEW_GROUP_EQ_OLD_GROUP =
    "New group will be identical to old group!";
    const string MSG_NEW_GROUP_EQ_EMPTY =
    "New group will be MPI_GROUP_EMPTY !";
    string MARMOT_LOGFILE_PATH =
    "";

}                               // end of namespace MPO


