/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Templates for our check functions. 
 *  Don't spread check functions all over the code if they do exactly 
 *  the same. Define them once here and document properly where they are used.
 *  If you declare a new check function, put it into THIS file and document
 *  exactly in the @see comment wherever it is used.
 * 
 *  HINT FOR DEVELOPERS:
 *  ====================
 *  WARNING: These functions might be out of date ! (Tobias Hilbrich) 
 *  
 *  How to use these template functions: 
 *
 *
 *  How to add new template functions (keep functions in alphabetical order):
 * 
 * (1) Define the template function. Its name should reflect what this method 
 *     does. 
 * (2) If the template function is similar but not exactly the same as 
 *     another, choose a different name for it. Use descriptive names.
 *     Describe exactly what's been changed if you create a new method by 
 *     copy and paste.   
 * (3) Add comments.
 * (3.1) The @param comment describes parameters, if any.
 * (3.2) The @return comment must contain all the error codes that are 
 *       returned.
 *       These error codes are in enum MARMOT_ERR_CODES. Add a new error code
 *       if necessary. The error codes distinguish between 
 *       MARMOT_ERR_FOO, MARMOT_WARN_FOO and MARMOT_NOTE_FOO.
 *       Choose the name of the error codes in a way that it is clear what the
 *       error is.
 * (3.3) The @see comment must contain all classes that use this function.
 *  
 *  @see enum MARMOT_ERR_CODES
 *
 *
 *  TODO: ADD SOME MORE DOCUMENTATION AND ALL THE METHODS
 *  
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo-templates.cc 838 2008-06-13 10:44:34Z hpchimml $
 */

// TODO
/* Developer's guide

derive class or define new class

default constructor:
initialize to invalid values:

proper constructor:
initialize

destructor

methods (checks)

assignment operators

data

public/private

enums (id), save, restore, getCallname, callNativeMPI
*/

#include <iostream>
#include <map>
#include <set>
#include <string>

#include <mpi.h>
#include "mpi_undef_makros.h"

#include "mpo-enums.h"
#include "mpo-envvar.h"
#include "mpo.h"



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"

#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{


/** Initialize default destructors to invalid values, such as:

: my_count(-1),
my_datatype(MPI_DATATYPE_NULL),
my_dest(MPI_PROC_NULL),
my_sendtag(MPI_ANY_TAG),
my_source(MPI_PROC_NULL), 
my_recvtag(MPI_ANY_TAG),
my_comm(MPI_COMM_NULL)





*/


/** Initialize proper destructors to our arguments:

: my_count(count),
my_datatype(datatype),
my_dest(dest),
my_sendtag(sendtag),
my_source(source),
my_recvtag(recvtag),
my_comm(comm)

*/


    // default constructor
    //========================================================================


    // proper constructor
    //========================================================================


    // destructor
    //========================================================================


    /* There are some pure virtual methods, see mpo.h.
     */


    // getFamilyId
    //========================================================================
    int MPO_::getFamilyId() const
    {
        // returns the unique family identity set by enum MPO_FAMILY_ID
        return MPO_CALLOBJECT_ID;
    }

    // eraseErrhandler()
    //========================================================================
    /** @see  MPO_Comm_free
     *        exactly like this template 
     */

    // eraseErrhandler
    //========================================================================
    int MPO_::eraseErrhandler()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        MPI_Errhandler err;
        PMPI_Errhandler_get(*my_comm, &err);

        if ((err != MPI_ERRORS_ARE_FATAL) && 
            (err != MPI_ERRORS_RETURN))
        {
            (our_errhandlers.find(err)->second[0])--;
        }

        if ((our_errhandlers.find(err)->second[0] == 0) &&
            (our_errhandlers.find(err)->second[1] == 1))
        {
            our_errhandlers.erase(err);
        }

        return rvalue;
    }                           // end eraseErrhandler


    // eraseRequest
    //========================================================================
    // exactly like in MPO_Tests, except for my_flags and warning
    /** @see  MPO_Completion
     */
    int MPO_::eraseRequest(int *my_outcount, int *my_index_array)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        std::map < MPO_Request, enum KIND_OF_REQUEST >::iterator pos;

        // (1) out_count is not MPI_UNDEFINED.
        if (*my_outcount != MPI_UNDEFINED)
        {
            // erase or change request
            for (int i = 0; i <= *my_outcount - 1; i++)
            {
                pos = our_requests.find(
                    &my_request_array[my_index_array[i]]);

                if (pos != our_requests.end())
                {
                    // all the cases from enum KIND_OF_REQUEST
                    switch (pos->second)
                    {
                        case (REQUEST_NULL):
                            // do nothing;
                            break;
                        case (REQUEST_NOT_PERSISTENT):
                            our_requests.erase(pos);
                            break;
                        case (REQUEST_ACTIVE_PERSISTENT):
                            pos->second = REQUEST_INACTIVE_PERSISTENT;
                            break;
                        case (REQUEST_INACTIVE_PERSISTENT):
                            // do nothing;                           
                            break;
                        default:
                            std::stringstream stream;
                            stream
                                << "Check mapping of requests."
                                << std::endl
                                << "Check implementation of switch statement."
                                << std::endl;
                            rvalue = MARMOT_ERR_MARMOT;              
                                        
                            MPO_String text;
                            text.assign(&stream);      
                            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                            our_log->getMsg(msg);   
                            break;
                    }
                }
            }                   // end for
        }                       // end if (1)

        // (2) out_count is MPI_UNDEFINED.
        else if ((*my_outcount == MPI_UNDEFINED) &&
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": No valid handles in requests! "
                << "thus outcount/index = MPI_UNDEFINED. "
                << std::endl;
            rvalue = MARMOT_WARN_REQUEST_OUTCOUNT_UNDEFINED;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }


    // insertGroup()
    //========================================================================
    /** @see  
     */

// TODO //

    // insertOp()
    //========================================================================
    /** @see  
     */

// TODO //






    // checkPrerequisites
    //========================================================================
    /** @see  MPO_Blocking
     *  @see  MPO_Sendrecv
     *  @see  MPO_Initiator
     */
    int MPO_::checkPrerequisites()
    {
        // check correctness of parameters
        checkCount();
        checkDatatype();
        checkRank();
        checkTag();
        checkComm();

        return 0;
    }

// TODO //

    // checkAssert()
    //========================================================================
    /** They are all alike. 
     *
     *  @see  MPO_Win_lock::checkAssert()
     *        exactly like this template, except for 
     *        the legal values for assert
     *
     *  @see  MPO_Win_fence::checkAssert()
     *        exactly like this template, except for 
     *        the legal values for assert
     *
     *  @see  MPO_Win_post::checkAssert()
     *        exactly like this template, except for 
     *        the legal values for assert
     *
     *  @see  MPO_Win_start::checkAssert()
     *        exactly like this template, except for 
     *        the legal values for assert
     */

    // checkAssert
    //========================================================================
    int MPO_::checkAssert()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // Check if assert is valid.
        if ((0 != my_assert) && 
            (MPI_MODE_NOCHECK != my_assert) &&  
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: " 
                << getCallname() 
                << ": assert is not valid!" 
                << " Legal values are only 0 or MPI_MODE_NOCHECK."
                << std::endl;
            rvalue = MARMOT_ERR_ASSERT_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkAssert



    // checkBlocklength() / checkBlocklengthArray()
    //========================================================================
    /** They are all alike. 
     *  The only difference between them is having a single blocklength or
     *  having an array of blocklengths, i.e. in the letter case we use a
     *  for loop to check the i-th blocklength.
     *
     *  @see  MPO_Type_hvector::checkBlocklength()
     *        exactly like this template
     *
     *  @see  MPO_Type_vector::checkBlocklength() 
     *        exactly like this template
     *
     *  @see  MPO_Type_struct::checkBlocklengthArray()
     *        exactly like this template
     *
     *  @see  MPO_Type_hindexed::checkBlocklengthArray()
     *        exactly like this template
     *
     *  @see  MPO_Type_indexed::checkBlocklengthArray()
     *        exactly like this template
     */

    // checkBlocklength
    //========================================================================
    int MPO_::checkBlocklength()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if blocklength is negative.
        if ((my_blocklength < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": blocklength = "
                << my_blocklength
                << " less than 0!" 
                << std::endl;
            rvalue = MARMOT_ERR_BLOCKLENGTH_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if blocklength is 0.
        else if ((my_blocklength == 0) && 
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname() 
                << ": blocklength = 0!" 
                << std::endl;
            rvalue = MARMOT_WARN_BLOCKLENGTH_ZERO;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkBlocklength


    // checkBlocklengthArray
    //========================================================================
    int MPO_::checkBlocklengthArray()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        for (int i = 0; i < my_count; i++)
        {
            // (1) Check if blocklength is negative.
            if ((my_blocklength[i] < 0) && 
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname() 
                    << ": blocklength[" 
                    << i 
                    << "] less than 0!" 
                    << std::endl;
                rvalue = MARMOT_ERR_BLOCKLENGTH_NEG;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }

            // (2) Check if blocklength is 0.
            else if ((my_blocklength[i] == 0) && 
                     (MSG_ERR_WARN <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "WARNING: "
                    << getCallname() 
                    << ": blocklength[" 
                    << i 
                    << "] = 0!" 
                    << std::endl;
                rvalue = MARMOT_WARN_BLOCKLENGTH_ZERO;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }                       // end for

        return rvalue;
    }                           // end checkBlocklengthArray


    // checkBufferAttached()
    //========================================================================
    /** @see  MPO_Buffer_detach::checkBufferAttached()
     *        exactly like this template
     *
     *  @see  MPO_Buffer_attach::checkBufferAttached()
     *        exactly like this template,
     *        exactly like MPO_Buffer_detach::checkBufferAttached(), except for
     *        - error code
     *        - replace (!buffer_attached) by (buffer_attached)
     *        - replace ": There's no buffer attached!" by
     *                  ": There's already a buffer attached!" 
     */

    // checkBufferAttached
    //========================================================================
    int MPO_:checkBufferAttached()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // Check if buffer is attached.
        if ((!buffer_attached) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": There's no buffer attached!" 
                << std::endl;
            rvalue = MARMOT_ERR_BUFFER_NOT_ATTACHED;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return 1;
    }                           // end checkBufferAttached



    // checkColor()
    //========================================================================
    /** @see  MPO_Comm_split
     *        exactly like this template
     */

    // checkColor
    //========================================================================
    int MPO_::checkColor()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if color is < 0.
        if ((my_color != MPI_UNDEFINED) &&
            (my_color < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Color( != MPI_UNDEFINED) less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_COLOR_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if color is MPI_UNDEFINED.
        else if ((my_color == MPI_UNDEFINED) &&
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": Color = MPI_UNDEFINED ! "
                << "New communicator will be MPI_COMM_NULL."
                << std::endl;
            rvalue = MARMOT_WARN_COLOR_UNDEFINED;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkColor


    // checkComm() / checkComm(MPO_Comm my_comm) / checkCommSecond()
    // checkCommFreed()
    //========================================================================
    /** They are all alike. 
     *
     *  @see  MPO_Blocking::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Cartesian::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Collective::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Communicatorhandler::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Graph::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Initiator::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Intercommunicators::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Probes::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Sendrecvs::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Pack_size::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Pack::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Unpack::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Attr_delete::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Attr_get::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Attr_put::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Abort::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Errhandler_get::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Errhandler_set::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Comm_group::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Comm_test_inter::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Graph_create::checkComm()
     *        exactly like this template
     *
     *  @see  MPO_Cart_create::checkComm()
     *        exactly like this template, except for:
     *        - replace my-comm by my_comm_old
     *
     *  @see  MPO_Cart_map::checkComm()
     *        exactly like this template, except for:
     *        - replace my_comm by my_comm_old
     * 
     *  @see  MPO_Comm_test_inter::checkCommFreed()
     *        exactly like checkComm(), except for:
     *        - the name
     *        - replace my_comm by *my_comm
     *        - replace ": Communicator is MPI_COMM_NULL!" by
     *                  ": Communicator to be freed is already MPI_COMM_NULL!" 
     *        - replace rvalue = MARMOT_ERR_COMM_NULL by
     *                  rvalue = MARMOT_WARN_COMM_NULL
     *
     *  @see  MPO_Intercomm_create::checkComm(MPO_Comm my_comm)
     *        exactly like this template, except for:
     *        -  checkComm(MPO_Comm my_comm);
     *           with parameter my_comm here
     *
     *  @see  MPO_Comm_compare::checkCommSecond()
     *        exactly like checkComm, except for:
     *        - replace my_comm by my_comm2
     *        - adjust error message 
     *          ("Second communicator" instead of "Communicator")
     */

    // checkComm
    //========================================================================
    int MPO_::checkComm()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if communicator is MPI_COMM_NULL.
        if ((MPI_COMM_NULL == my_comm) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Communicator is MPI_COMM_NULL!" 
                << std::endl;
            rvalue = MARMOT_ERR_COMM_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if communicator is valid.
        else if ((our_comms.end() == our_comms.find(my_comm)) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << " : Communicator is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_COMM_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkComm


    // checkCommCartesian() / checkCommGraph()
    // checkCommIntercomm() / checkCommIntracomm(MPO_Comm my_comm)
    //========================================================================
    /** They are all alike.
     *
     *  @see  MPO_Cartesian::checkCommCartesian()
     *        exactly like this template
     *
     *  @see  MPO_Graph::checkCommGraph()
     *        exactly like this template
     *        exactly like checkCommCartesian, except for:
     *        - replace cartesian by graph everywhere
     *        - replace MPI_CART by MPI_GRAPH
     *
     *  @see  MPO_Intercommunicators::checkCommIntercomm()
     *        exactly like this template
     *
     *  @see  MPO_Intercomm_create::checkCommIntracomm(MPO_Comm my_comm)
     *        exactly like this template
     *        exactly like checkCommIntracomm, except for:
     *        - parameter my_comm
     *        - in if statement: replace !inter by inter
     *        - in if statement: modify output
     *        - in if statement: modify rvalue
     */

    // checkCommCartesian
    //========================================================================
    int MPO_::checkCommCartesian()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        int status = MPI_UNDEFINED;

        PMPI_Topo_test(my_comm, &status);

        // Check if communicator has cartesian structure.
        if ((status != MPI_CART) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Communicator has no cartesian structure ! " 
                << std::endl;
            rvalue = MARMOT_ERR_COMM_NOT_CARTESIAN;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCommCartesian


    // checkCommGraph
    //========================================================================
    int MPO_::checkCommGraph()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        int status = MPI_UNDEFINED;

        PMPI_Topo_test(my_comm, &status);

        // Check if communicator has graph structure.
        if ((status != MPI_GRAPH) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Communicator has no graph structure ! " 
                << std::endl;
            rvalue = MARMOT_ERR_COMM_NOT_GRAPH;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCommGraph


    // checkCommIntercomm
    //========================================================================
    int MPO_::checkCommIntercomm()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;

        int inter = 0;

        PMPI_Comm_test_inter(my_comm, &inter);

        // Check if communicator is intercommunicator.
        if ((!inter) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Communicator is no inter-communicator !"
                << std::endl;
            rvalue = MARMOT_ERR_COMM_NOT_INTERCOMM;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCommIntercomm


    // checkCommIntracomm
    //========================================================================
    int MPO_::checkCommIntracomm(MPO_Comm my_comm)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;

        int inter = 0;

        PMPI_Comm_test_inter(my_comm, &inter);

        // Check if communicator is intercommunicator.
        if ((inter) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Communicator is inter-communicator,"
                << " but should be intra-communicator!"
                << std::endl;
            rvalue = MARMOT_ERR_COMM_INTERCOMM;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCommIntracomm


    // checkCommWorld()
    //========================================================================
    /** @see  MPO_Abort::checkCommWorld()
     *        exactly like this template
     */

    // checkCommWorld
    //========================================================================
    int MPO_::checkCommWorld()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // Check if communicator is MPI_COMM_WORLD.
        if ((my_comm != MPI_COMM_WORLD) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": Communicator is not MPI_COMM_WORLD,"
                << " but MPI_Implementation may use MCW here!" 
                << std::endl;
            rvalue = MARMOT_WARN_COMM_NOT_WORLD;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCommWorld


    // checkCount(const string KIND_OF_COUNT) 
    // checkCountIn(const string KIND_OF_COUNT) 
    // checkCount(int my_count, const string KIND_OF_COUNT) 
    // checkCountOfReceiver(const string KIND_OF_COUNT)
    // checkCountGroup(const string KIND_OF_COUNT,
    //                 const string KIND_OF_NEW_GROUP)
    // checkCountArray(int* my_counts, const string KIND_OF_COUNT)
    //========================================================================
    /** They are all alike.
     *
     *  @see  MPO_Blocking::checkCount(const string KIND_OF_COUNT)
     *        exactly like this template
     *
     *  @see  MPO_Sendrecvs::checkCount(const string KIND_OF_COUNT)
     *        exactly like this template
     *
     *  @see  MPO_Initiator::checkCount(const string KIND_OF_COUNT)
     *        exactly like this template
     *
     *  @see  MPO_Typeconstructors::checkCount(const string KIND_OF_COUNT)
     *        exactly like this template
     *
     *  @see  MPO_Pack_size::checkCountIn(const string KIND_OF_COUNT)
     *        exactly like this template, except for:
     *        - replace my_count by my_incount
     *
     *  @see  MPO_Pack::checkCountIn(const string KIND_OF_COUNT)
     *        exactly like MPO_Pack_size::checkCountIn(const string 
     *                                                 KIND_OF_COUNT)
     *
     *  @see  MPO_Unpack::checkCountOut(const string KIND_OF_COUNT)
     *        exactly like this template, except for:
     *        - replace my_count by my_outcount
     *
     *  @see  MPO_Collective::checkCount(int my_count,
     *                                   const string KIND_OF_COUNT)
     *        exactly like checkCount(const string KIND_OF_COUNT), except for:
     *        - additional parameter my_count
     *        used by checkPrerequisites2 in 
     *         - MPO_Scatterv
     *         - MPO_Scan
     *         - MPO_Gatherv
     *         - MPO_Alltoallv
     *         - MPO_Alltoall
     *         - MPO_Allgatherv
     *         - MPO_Scatter
     *         - MPO_Allreduce
     *         - MPO_Reduce
     *         - MPO_Allgather
     *         - MPO_Gather
     *         - MPO_Bcast  
     *
     *  @see  MPO_Sendrecv::checkCountOfReceiver(const string KIND_OF_COUNT)
     *        exactly like checkCount(const string KIND_OF_COUNT), except for:
     *        - replace my_count by my_recvcount
     *
     *  @see  MPO_Group_excl::checkCountGroup(const string KIND_OF_COUNT,
     *                                        const string KIND_OF_NEW_GROUP)
     *        exactly like this template
     *        exactly like checkCount(const string KIND_OF_COUNT), except for:
     *        - additional parameter KIND_OF_NEW_GROUP
     *        - adjust output
     *
     *  @see  MPO_Group_incl::checkCountGroup(const string KIND_OF_COUNT,
     *                                        const string KIND_OF_NEW_GROUP)
     *        exactly like this template
     *        exactly like checkCount(const string KIND_OF_COUNT), except for:
     *        - additional parameter KIND_OF_NEW_GROUP
     *        - adjust output
     *
     *  @see  MPO_Collective::checkCountArray(int* my_counts,
     *                                        const string KIND_OF_COUNT)
     *        exactly like checkCount(int my_count, 
     *                                const string KIND_OF_COUNT), except for:
     *        - array of counts instead of single count
     *        - for loop with my_counts[i]
     *        - adjust output 
     *        used by checkPrerequisites2 in 
     *         - MPO_Scatterv
     *         - MPO_Gatherv
     *         - MPO_Alltoallv
     *         - MPO_Reduce_scatter
     *         - MPO_Allgatherv
     */

    // checkCount
    //========================================================================
    int MPO_::checkCount(const string KIND_OF_COUNT)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if count is negative.
        if ((my_count < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": "
                << KIND_OF_COUNT
                << " = "
                << my_count
                << " less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_COUNT_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if count is 0.
        if ((my_count == 0) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: " 
                << getCallname() 
                << ": "
                << KIND_OF_COUNT
                << " = 0!" 
                << std::endl;
            rvalue = MARMOT_WARN_COUNT_ZERO;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCount


    // checkCountGroup
    //========================================================================
    int MPO_::checkCountGroup(const string KIND_OF_COUNT,
                              const string KIND_OF_NEW_GROUP)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if count is negative.
        if ((my_count < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": "
                << KIND_OF_COUNT
                << " = "
                << my_count
                << " less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_COUNT_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if count is 0.
        if ((my_count == 0) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: " 
                << getCallname() 
                << ": "
                << KIND_OF_COUNT
                << "= 0!" 
                << KIND_OF_NEW_GROUP
                << std::endl;
            rvalue = MARMOT_WARN_COUNT_ZERO;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCountGroup


    // checkCountArray
    //========================================================================
    int MPO_::checkCountArray(int* my_counts, const string KIND_OF_COUNT)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        int size = -1;
        PMPI_Comm_size(my_comm, &size);

        int testrank = -1;
        PMPI_Comm_rank(my_comm, &testrank);

        for (int i = 0; i < size; i++)
        {
            // (1) Check if count is negative.
            if ((my_counts[i] < 0) && 
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname() 
                    << ": On rank "
                    << testrank 
                    << ": "
                    << KIND_OF_COUNT
                    << "[" << i << "]"
                    << " = "
                    << my_counts[i]
                    << " less than 0 !" 
                    << std::endl;
                rvalue = MARMOT_ERR_COUNT_NEG;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }

            // (2) Check if count is 0.
            if ((my_counts[i] == 0) && 
                (MSG_ERR_WARN <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream 
                    << "WARNING: " 
                    << getCallname() 
                    << ": On rank "
                    << testrank 
                    << ": "
                    << KIND_OF_COUNT
                    << "[" << i << "]"
                    << "= 0!" 
                    << std::endl;
                rvalue = MARMOT_WARN_COUNT_ZERO;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }

        return rvalue;
    }                           // end checkCountArray


    // checkCountsAreEqual()
    //========================================================================
    /** @see  MPO_Alltoall
     *        exactly like this template
     */

    // checkCountsAreEqual
    //========================================================================
    int MPO_::checkCountsAreEqual()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        if ((my_recvcount != my_sendcount) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname() 
                << ": Send and receive count are NOT equal!" 
                << std::endl;
            rvalue = MARMOT_WARN_COUNTS_NOT_EQUAL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }


    // checkCountUndefined()
    //========================================================================
    /** @see  MPO_Get_elements
     *        exactly like this template
     *
     *  @see  MPO_Get_count 
     *        exactly like this template
     */

    // checkCountUndefined
    //========================================================================
    int MPO_::checkCountUndefined()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // Check if count is MPI_UNDEFINED.
        if ((*my_count == MPI_UNDEFINED) &&
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname() 
                << ": Count is MPI_UNDEFINED! " 
                << std::endl;
            rvalue = MARMOT_WARN_COUNT_UNDEFINED;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkCountUndefined




    // checkDatatype() / checkDatatypeOfSender() / checkDatatypeSendRecv()
    //========================================================================
    /** @see  MPO_Blocking::checkDatatype()
     *
     *  @see  MPO_Sendrecvs::checkDatatypeOfSender()
     *        exactly like checkDatatype, except for:
     *        - replace datatype by datatype (sendtype)
     *
     *  @see  MPO_Initiator::checkDatatype()
     *        exactly like this template
     *
     *  @see  MPO_Collective::checkDatatypeSendRecv()
     *        exactly like checkDatatype, except for:
     *        - replace datatype by datatype, sendtype, receivtype 
     *          (switch kind_of_count)
     *
     *  @see  MPO_Typeconstructors::checkDatatype()
     *        exactly like checkDatatype, except for:
     *        - replace datatype by oldtype 
     *
     *  @see  MPO_Typehandlers::checkDatatype()
     *        exactly like checkDatatype
     *
     *  @see  MPO_Pack_size::checkDatatype()
     *        exactly like this template, except for:
     *        - TODO
     *
     *  @see  MPO_Pack::checkDatatype()
     *        exactly like this template, except for:
     *        - TODO
     *
     *  @see  MPO_Unpack::checkDatatype()
     *        exactly like this template, except for:
     *        - TODO
     *
     *  @see  MPO_Get_elements::checkDatatype()
     *        exactly like this template, except for:
     *        - TODO
     *
     *  @see  MPO_Get_count::checkDatatype()
     *        exactly like this template, except for:
     *        - TODO
     */

// TODO // Type/sendtype/recvtype

    // checkDatatype
    //========================================================================
    int MPO_::checkDatatype()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        std::map < MPO_Datatype, enum KIND_OF_DATATYPE >::iterator pos;
        pos = our_datatypes.find(my_datatype);

        // (1) Datatype is not valid, i.e. not registered in
        // our MPO datatypes.
        if ((pos == our_datatypes.end()) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": datatype is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_TYPE_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Datatype is registered in our MPO datatypes.
        // In this case, we have to check different violations 
        // in the switch statement.
        else if ((pos != our_datatypes.end()) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            MPO_MSG_TYPE type = MSG_TYPE_ERROR;
                        
            // all the cases from enum KIND_OF_DATATYPE
            switch (pos->second)
            {
                case (DATATYPE_NULL):
                    stream
                        << "ERROR: "
                        << getCallname() 
                        << ": datatype is MPI_DATATYPE_NULL!" 
                        << std::endl;
                    rvalue = MARMOT_ERR_TYPE_NULL;
                    break;
                case (DATATYPE_CREATED_NOT_COMMITTED):
                    stream
                        << "ERROR: "
                        << getCallname() 
                        << ": datatype is not committed!" 
                        << std::endl;
                    rvalue = MARMOT_ERR_TYPE_NOT_COMMITTED;
                    break;
                case (DATATYPE_COMMITTED_C):
                    if (F_INTERFACE == INTERFACE_MODE)
                    {
                        stream
                            << "ERROR: "
                            << getCallname() 
                            << ": datatype is C-Type!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_C;
                    }
                    break;
                case (DATATYPE_COMMITTED_C_AND_FORTRAN):
                    // Do nothing, because everything is fine.
                    break;
                case (DATATYPE_COMMITTED_FORTRAN):
                    if (C_INTERFACE == INTERFACE_MODE)
                    {
                        stream
                            << "ERROR: "
                            << getCallname() 
                            << ": datatype is Fortran-Type!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_F;
                    }
                    break;
                case (DATATYPE_REDUCTION_C):
                    if ((MSG_ERR_WARN <= DEBUG_MODE) &&
                        (C_INTERFACE == INTERFACE_MODE))
                    {
                        stream
                            << "WARNING: "
                            << getCallname()
                            << ": datatype is for reduction functions (C)!" 
                            << std::endl;
                        rvalue = MARMOT_WARN_TYPE_REDUCTION_C;
                        type = MSG_TYPE_WARNING;
                    }
                    if (F_INTERFACE == INTERFACE_MODE)
                    {
                        stream
                            << "ERROR: "
                            << getCallname()
                            << ": datatype is for reduction functions (C)!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_REDUCTION_C;
                    }
                    break;
                case (DATATYPE_REDUCTION_FORTRAN):
                    if (C_INTERFACE == INTERFACE_MODE)
                    {
                        stream
                            << "ERROR: "
                            << getCallname()
                            << ": datatype is for reduction functions "
                            << "(Fortran)!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_REDUCTION_F;
                    }
                    if ((MSG_ERR_WARN <= DEBUG_MODE) &&
                        (F_INTERFACE == INTERFACE_MODE))
                    {
                        stream
                            << "WARNING: "
                            << getCallname()
                            << ": datatype is for reduction functions "
                            << "(Fortran)!" 
                            << std::endl;
                        rvalue = MARMOT_WARN_TYPE_REDUCTION_F;
                        type = MSG_TYPE_WARNING;
                    }
                    break;
                case (DATATYPE_OPTIONAL_C):
                    if ((MSG_ERR_WARN <= DEBUG_MODE) &&
                        (C_INTERFACE == INTERFACE_MODE))
                    {
                        stream
                            << "WARNING: "
                            << getCallname() 
                            << ": datatype is only optional!" 
                            << std::endl;
                        rvalue = MARMOT_WARN_TYPE_OPTIONAL_C;
                        type = MSG_TYPE_WARNING;
                    }
                    if (F_INTERFACE == INTERFACE_MODE)
                    {
                        stream
                            << "ERROR: "
                            << getCallname() 
                            << ": datatype is optional C-Type!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_OPTIONAL_C;
                    }
                    break;
                case (DATATYPE_OPTIONAL_FORTRAN):
                    if (C_INTERFACE == INTERFACE_MODE)
                    {
                        stream
                            << "ERROR: "
                            << getCallname() 
                            << ": datatype is optional Fortran-type!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_OPTIONAL_F;
                    }
                    if ((MSG_ERR_WARN <= DEBUG_MODE) && 
                        (F_INTERFACE == INTERFACE_MODE))
                    {
                        stream
                            << "WARNING: "
                            << getCallname() 
                            << ": datatype is only optional!" 
                            << std::endl;
                        rvalue = MARMOT_WARN_TYPE_OPTIONAL_F;
                        type = MSG_TYPE_WARNING;
                    }
                    break;
                case (DATATYPE_CONSTRUCT_DERIVED_TYPE):
                    stream
                        << "ERROR: "
                        << getCallname() 
                        << ": datatype is for constructing derived datatypes!"
                        << std::endl;
                    rvalue = MARMOT_ERR_TYPE_CONSTR_DERIVED;
                    break;
                default:
                    stream
                        << "Check mapping of datatypes."
                        << std::endl
                        << "Check implementation of switch statement."
                        << std::endl;
                    rvalue = MARMOT_ERR_MARMOT;
                    break;
            }                   // end switch
            
            if (rvalue != MARMOT_SUCCESS)
            {
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (type,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }                       // end else if (2)

        return rvalue;
    }                           // end checkDatatype


    // checkDatatypeOfSender
    //========================================================================

// TODO // Type/sendtype/recvtype


    // checkDatatypeSendRecv
    //========================================================================

// TODO // Type/sendtype/recvtype


    // checkDatatypeArrayOfSender
    //========================================================================

// TODO // Type/sendtype/recvtype


    // checkDatatypeArrayTypeStruct()
    //========================================================================
    /**
     *  @see  MPO_Type_struct::checkDatatypeArrayTypeStruct()
     *        exactly like this template
     */

    int MPO_::checkDatatypeArrayTypeStruct()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        std::map < MPO_Datatype, enum KIND_OF_DATATYPE >::iterator pos;

        // Check array of datatypes
        for (int i = 0; i < my_count; i++)
        {
            pos = our_datatypes.find(my_oldtypes[i]);

            // (1) Datatype is not valid, i.e. not registered in
            // our MPO datatypes.
            if ((pos == our_datatypes.end()) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname() 
                    << ": datatype[" 
                    << i 
                    << "] is not valid!" 
                    << std::endl;
                rvalue = MARMOT_ERR_TYPE_NOT_VALID;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }

            // (2) Datatype is registered in our MPO datatypes.
            // In this case, we have to check different violations 
            // in the switch statement.
            else if ((pos != our_datatypes.end()) &&
                     (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                MPO_MSG_TYPE type = MSG_TYPE_ERROR;
                                
                // all the cases from enum KIND_OF_DATATYPE
                switch (pos->second)
                {
                    case (DATATYPE_NULL):
                        stream 
                            << "ERROR: "
                            << getCallname() 
                            << ": datatype[" 
                            << i 
                            << "] is MPI_DATATYPE_NULL!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_NULL;
                        break;
                    case (DATATYPE_CREATED_NOT_COMMITTED):
                        stream
                            << "ERROR: "
                            << getCallname() 
                            << ": datatype[" 
                            << i 
                            << "] is not committed!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_TYPE_NOT_COMMITTED;
                        break;
                    case (DATATYPE_COMMITTED_C):
                        if (F_INTERFACE == INTERFACE_MODE)
                        {
                            stream
                                << "ERROR: "
                                << getCallname() 
                                << ": datatype[" 
                                << i 
                                << "] is C-Type!" 
                                << std::endl;
                            rvalue = MARMOT_ERR_TYPE_C;
                        }
                        break;
                    case (DATATYPE_COMMITTED_C_AND_FORTRAN):
                        // Do nothing, because everything is fine.
                        break;
                    case (DATATYPE_COMMITTED_FORTRAN):
                        if (C_INTERFACE == INTERFACE_MODE)
                        {
                            stream
                                << "ERROR: "
                                << getCallname() 
                                << ": datatype [" 
                                << i 
                                << "] is Fortran-Type!" 
                                << std::endl;
                            rvalue = MARMOT_ERR_TYPE_F;
                        }
                        break;
                    case (DATATYPE_REDUCTION_C):
                        if ((MSG_ERR_WARN <= DEBUG_MODE) &&
                            (C_INTERFACE == INTERFACE_MODE))
                        {
                            stream
                                << "WARNING: "
                                << getCallname()
                                << ": datatype [" 
                                << i 
                                << "] is for reduction functions (C)!" 
                                << std::endl;
                            rvalue = MARMOT_WARN_TYPE_REDUCTION_C;
                            type = MSG_TYPE_WARNING;
                        }
                        if (F_INTERFACE == INTERFACE_MODE)
                        {
                            stream
                                << "ERROR: "
                                << getCallname()
                                << ": datatype [" 
                                << i 
                                << "] is for reduction functions (C)!" 
                                << std::endl;
                            rvalue = MARMOT_ERR_TYPE_REDUCTION_C;
                        }
                        break;
                    case (DATATYPE_REDUCTION_FORTRAN):
                        if (C_INTERFACE == INTERFACE_MODE)
                        {
                            stream
                                << "ERROR: "
                                << getCallname()
                                << ": datatype [" 
                                << i 
                                << "] is for reduction functions "
                                << "(Fortran)!" 
                                << std::endl;
                            rvalue = MARMOT_ERR_TYPE_REDUCTION_F;
                        }
                        if ((MSG_ERR_WARN <= DEBUG_MODE) &&
                            (F_INTERFACE == INTERFACE_MODE))
                        {
                            stream
                                << "WARNING: "
                                << getCallname()
                                << ": datatype [" 
                                << i 
                                << "] is for reduction functions "
                                << "(Fortran)!" 
                                << std::endl;
                            rvalue = MARMOT_WARN_TYPE_REDUCTION_F;
                            type = MSG_TYPE_WARNING;
                        }
                        break;
                    case (DATATYPE_OPTIONAL_C):
                        if ((MSG_ERR_WARN <= DEBUG_MODE) &&
                            (C_INTERFACE == INTERFACE_MODE))
                        {
                            stream
                                << "WARNING: "
                                << getCallname() 
                                << ": datatype [" 
                                << i 
                                << "] is only optional!" 
                                << std::endl;
                            rvalue = MARMOT_WARN_TYPE_OPTIONAL_C;
                            type = MSG_TYPE_WARNING;
                        }
                        if (F_INTERFACE == INTERFACE_MODE)
                        {
                            stream
                                << "ERROR: "
                                << getCallname() 
                                << ": datatype [" 
                                << i 
                                << "] is optional C-Type!" 
                                << std::endl;
                            rvalue = MARMOT_ERR_TYPE_OPTIONAL_C;
                        }
                        break;
                    case (DATATYPE_OPTIONAL_FORTRAN):
                        if (C_INTERFACE == INTERFACE_MODE)
                        {
                            stream
                                << "ERROR: "
                                << getCallname() 
                                << ": datatype [" 
                                << i 
                                << "] is optional Fortran-type!" 
                                << std::endl;
                            rvalue = MARMOT_ERR_TYPE_OPTIONAL_F;
                        }
                        if ((MSG_ERR_WARN <= DEBUG_MODE) && 
                            (F_INTERFACE == INTERFACE_MODE))
                        {
                            stream
                                << "WARNING: "
                                << getCallname() 
                                << ": datatype [" 
                                << i 
                                << "] is only optional!" 
                                << std::endl;
                            rvalue = MARMOT_WARN_TYPE_OPTIONAL_F;
                            type = MSG_TYPE_WARNING;
                        }
                        break;
                    case (DATATYPE_CONSTRUCT_DERIVED_TYPE):
                        if ((my_oldtypes[i] == MPI_LB) &&
                            (i != 0) && 
                            (MSG_ERR <= DEBUG_MODE))
                        {
                            stream
                                << "ERROR: "
                                << getCallname()
                                << ": MPI_LB only allowed on entry[0]! "
                                << "Not on ["
                                << i 
                                << "] !" 
                                << std::endl;
                            rvalue = MARMOT_ERR_TYPE_LB;
                        }
                        else if ((my_oldtypes[i] == MPI_UB) &&
                                 (i != (my_count - 1)) && 
                                 (MSG_ERR <= DEBUG_MODE))
                            {
                                stream
                                    << "ERROR: "
                                    << getCallname()
                                    << ": MPI_UB only allowed on last entry! "
                                    << "Not on ["
                                    << i 
                                    << "] !" 
                                    << std::endl;
                                rvalue = MARMOT_ERR_TYPE_UB;
                            }
                        break;
                    default:
                        stream
                            << "Check mapping of datatypes."
                            << std::endl
                            << "Check implementation of switch statement."
                            << std::endl;
                        rvalue = MARMOT_ERR_MARMOT;
                        break;
                }               // end switch
            
                if (rvalue != MARMOT_SUCCESS)
                {
                    MPO_String text;
                    text.assign(&stream);      
                    MPO_MsgObject msg (type,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                    our_log->getMsg(msg);   
                }
            }                   // end else if (2)
        }                       // end for

        return rvalue;
    }                           // end checkDatatypeArrayTypeStruct


    // checkDatatypeCommitted()
    //========================================================================
    /**
     *  @see  MPO_Type_commit::checkDatatypeCommitted()
     *        exactly like this template
     */

    // checkDatatypeCommitted
    //========================================================================
    int MPO_::checkDatatypeCommitted()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        std::map < MPO_Datatype, enum KIND_OF_DATATYPE >::iterator pos;
        pos = our_datatypes.find(*my_datatype);

        // (1) Datatype is not valid, i.e. not registered in
        // our MPO datatypes.
        if ((pos == our_datatypes.end()) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream 
                << "ERROR: "
                << getCallname() 
                << ": datatype is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_TYPE_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Datatype is registered in our MPO datatypes.
        // In this case, we have to check different violations 
        // in the switch statement.
        else if ((pos != our_datatypes.end()) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            MPO_MSG_TYPE type = MSG_TYPE_ERROR;
                        
            // all the cases from enum KIND_OF_DATATYPE
            switch (pos->second)
            {
                case (DATATYPE_NULL):
                    if (MSG_ERR_WARN_NOTE <= DEBUG_MODE)
                    {
                        stream 
                            << "NOTE: "
                            << getCallname() 
                            << ": datatype is MPI_DATATYPE_NULL!" 
                            << std::endl;
                        rvalue = MARMOT_NOTE_TYPE_NULL;
                        type = MSG_TYPE_NOTE;
                    }
                    break;
                case (DATATYPE_CREATED_NOT_COMMITTED):
                    // Do nothing, because everything is fine.
                    break;
                case (DATATYPE_COMMITTED_C):
                    if (MSG_ERR_WARN_NOTE <= DEBUG_MODE)
                    {
                        stream 
                            << "NOTE: "
                            << getCallname() 
                            << ": Datatype already committed!" 
                            << std::endl;
                        rvalue = MARMOT_NOTE_TYPE_ALREADY_COMMITTED_C;
                        type = MSG_TYPE_NOTE;
                    }
                    break;
                case (DATATYPE_COMMITTED_C_AND_FORTRAN):
                    if (MSG_ERR_WARN_NOTE <= DEBUG_MODE)
                    {
                        stream 
                            << "NOTE: "
                            << getCallname() 
                            << ": Datatype already committed!" 
                            << std::endl;
                        rvalue = MARMOT_NOTE_TYPE_ALREADY_COMMITTED_C_AND_F;
                        type = MSG_TYPE_NOTE;
                    }                    
                    break;
                case (DATATYPE_COMMITTED_FORTRAN):
                    if (MSG_ERR_WARN_NOTE <= DEBUG_MODE)
                    {
                        stream 
                            << "NOTE: "
                            << getCallname() 
                            << ": Datatype already committed!" 
                            << std::endl;
                        rvalue = MARMOT_NOTE_TYPE_ALREADY_COMMITTED_F;
                        type = MSG_TYPE_NOTE;
                    }                    
                    break;
                case (DATATYPE_REDUCTION_C):
                    // Do nothing.
                    break;
                case (DATATYPE_REDUCTION_FORTRAN):
                    // Do nothing.
                    break;
                case (DATATYPE_OPTIONAL_C):
                    // Do nothing.
                    break;
                case (DATATYPE_OPTIONAL_FORTRAN):
                    // Do nothing.
                    break;
                case (DATATYPE_CONSTRUCT_DERIVED_TYPE):
                    // Do nothing.
                    break;
                default:
                    stream 
                        << "Check mapping of datatypes."
                        << std::endl
                        << "Check implementation of switch statement."
                        << std::endl;
                    rvalue = MARMOT_ERR_MARMOT;
                    break;
            }                   // end switch
            
            if (rvalue != MARMOT_SUCCESS)
            {
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (type,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }                       // end else if (2)

        return rvalue;
    }                           // end checkDatatypeCommitted


    // checkDimsArray()
    //========================================================================
    /** @see  MPO_Dims_create
     *        exactly like this template  
     */

    // checkDimsArray
    //========================================================================
    int MPO_::checkDimsArray()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        int prod = 1;
        int sum = 0;
        int nonzero = my_ndims;

        // Check array of dims.
        for (int i = 0; i < my_ndims; i++)
        {
            if (my_dims[i] != 0)
            {
                nonzero--;

                if ((my_dims[i] < 0) &&
                    (MSG_ERR <= DEBUG_MODE))
                {
                    std::stringstream stream;
                    stream 
                        << "ERROR: "
                        << getCallname() 
                        << ": dim[" 
                        << i 
                        << "] less than 0 !" 
                        << std::endl;
                    rvalue = MARMOT_ERR_DIM_NEG;
            
                    MPO_String text;
                    text.assign(&stream);      
                    MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                    our_log->getMsg(msg);   
                }
        
                else  if ((my_dims[i] >= 0) &&
                          (MSG_ERR <= DEBUG_MODE))
                {                       
                    // so the entry is valid and will persist
                    prod *= my_dims[i];
                    sum += my_dims[i];

                    if ((sum > my_nnodes) && 
                        (MSG_ERR <= DEBUG_MODE))
                    {
                        std::stringstream stream;
                        stream 
                            << "ERROR: "
                            << getCallname()
                            << ": Number of nodes in dims greater than nnodes!" 
                            << std::endl;
                        rvalue = MARMOT_ERR_DIM_TOO_BIG;
            
                        MPO_String text;
                        text.assign(&stream);      
                        MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                        our_log->getMsg(msg);   
                    }
                }
            }
        }

        if ((my_nnodes % prod != 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream 
                << "ERROR: "
                << getCallname()
                << ": Nnodes="
                << my_nnodes
                << " is not a multiple of the product of entries in dims (="
                << prod 
                << ") !" 
                << std::endl;
            rvalue = MARMOT_ERR_NODES_NEQ_PRODUCT_DIMS;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        if ((nonzero == 0) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream 
                << "WARNING: "
                << getCallname() 
                << ": No entries in dims to be set!" 
                << std::endl;
            rvalue = MARMOT_WARN_DIM_ZERO;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkDimsArray


    // checkMaxDims()
    //========================================================================
    /** @see  MPO_Cart_get
     *        exactly like this template  
     */

    // checkMaxDims
    //========================================================================
    int MPO_::checkMaxDims()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // (1) Check if maxdims is negative.
        if ((my_maxdims < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream 
                << "ERROR: " 
                << getCallname() 
                << ": Maxdims less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_DIM_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if maxdims is 0.
        if ((my_maxdims == 0) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream 
                << "WARNING: " 
                << getCallname() 
                << "Maxdims = 0 !" 
                << std::endl;
            rvalue = MARMOT_WARN_DIM_ZERO;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }


    // checkDisplacementArrayOfReceiver()
    //========================================================================
    /** @see  MPO_Alltoallv::checkDisplacementArrayOfReceiver()
     *        exactly like this template
     *
     *  @see  MPO_Scatterv::checkDisplacementArrayOfSender()
     *        exactly like this template,
     *        exactly like checkDisplacementArrayOfReceiver(), except for:
     *        - replace my_rdispls by my_displs
     *        - replace my_recvtype by my_sendtype
     *        - replace my_recvcounts by my_sendcounts
     *        - replace most inner if-loop in case (1)
     *        
     *  @see  MPO_Allgatherv::checkDisplacementArrayOfReceiver()
     *        exactly like this template, except for:
     *        - replace my_rdispls by my_displs
     *        - replace "reads data / holes read from other nodes" 
     *          by "overrides data / holes from other nodes!"
     *        - replace MARMOT_NOTE_DISPLACEMENT_READS_DATA 
     *          by MARMOT_NOTE_DISPLACEMENT_OVERRIDES_DATA
     *
     *  @see  MPO_Gatherv::checkDisplacementArrayOfRoot()
     *        exactly like MPO_Allgatherv::checkDisplacementArrayOfReceiver(),
     *        except for:
     *        - Add if (rank == my_root) loop because we only check the root   
     */

    // checkDisplacementArrayOfReceiver
    //========================================================================
    int MPO_::checkDisplacementArrayOfReceiver()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        int size = -1;
        PMPI_Comm_size(my_comm, &size);

        int rank = -1;
        PMPI_Comm_rank(my_comm, &rank);

        // compilation error:  MPI_Aint typeextent may be int or long, so
        // use instead of 
        //     int used = 0;
        // an MPI_Aint and cast typesize to MPI_Aint in the if condition 
        // below. 
        MPI_Aint used = 0;

        MPI_Aint typeextent = 0;
        PMPI_Type_extent(my_recvtype, &typeextent);

        int typesize = -1;
        PMPI_Type_size(my_recvtype, &typesize);

        // (1) Here we have a compact datatype without holes
        // so overriding are errors.
        if (typeextent == (MPI_Aint) typesize)
        {
            std::stringstream stream;
            bool bErrorOccured = false;
                        
            // Check array of displacements.
            for (int i = 0; i < size; i++)
            {
                if ((my_rdispls[i] < used) && 
                    (MSG_ERR <= DEBUG_MODE))
                {
                    stream 
                        << "ERROR: "
                        << getCallname()
                        << ": On rank "
                        << rank
                        << ": displs["
                        << i
                        << "] = "
                        << my_rdispls[i]
                        << "  (used = " 
                        << used 
                        << ") overrides data!" 
                        << std::endl;
                    rvalue = MARMOT_ERR_DISPLACEMENT_OVERRIDES_DATA;
                    bErrorOccured = true;
                }
                used = my_rdispls[i] + (my_recvcounts[i]);
            }
            
            if (bErrorOccured)
            {
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }

        // (2) Here we have a datatype with holes
        // so we don't check whether it is correct.
        else
        {
            std::stringstream stream;
            bool bErrorOccured = false;
            
            // Check array of displacements.
            for (int i = 0; i < size; i++)
            {
                if ((my_rdispls[i] < used) && 
                    (MSG_ERR_WARN_NOTE <= DEBUG_MODE))
                {
                    stream
                        << "NOTE: "
                        << getCallname()
                        << ": Datatype has holes, "
                        << "so we don't check whether it's correct,"
                        << "but on rank "
                        << rank
                        << ": displs["
                        << i 
                        << "] reads data / holes read from other nodes!" 
                        << std::endl;
                    rvalue = MARMOT_NOTE_DISPLACEMENT_READS_DATA;
                    bErrorOccured = true;
                }
                used = my_rdispls[i] + (typeextent * my_recvcounts[i]);
            }
            
            if (bErrorOccured)
            {
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_NOTE,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }

        return rvalue;
    }                           // end checkDisplacementArrayOfReceiver


    // checkDisplacementArrayOfSender
    //========================================================================
    int MPO_::checkDisplacementArrayOfSender()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        int size = -1;
        PMPI_Comm_size(my_comm, &size);

        int rank = -1;
        PMPI_Comm_rank(my_comm, &rank);

        // compilation error:  MPI_Aint typeextent may be int or long, so
        // use instead of 
        //     int used = 0;
        // an MPI_Aint and cast typesize to MPI_Aint in the if condition 
        // below. 
        MPI_Aint used = 0;

        MPI_Aint typeextent = 0;
        PMPI_Type_extent(my_sendtype, &typeextent);

        int typesize = -1;
        PMPI_Type_size(my_sendtype, &typesize);

        // (1) Here we have a compact datatype without holes.
        if (typeextent == (MPI_Aint) typesize)
        {
            std::stringstream stream;
            bool bErrorOccured = false;
            
            // Check array of displacements.
            for (int i = 0; i < size; i++)
            {
                if ((my_displs[i] < used) && 
                    (MSG_ERR_WARN_NOTE <= DEBUG_MODE))
                {
                    stream
                        << "REMARK: "
                        << getCallname()
                        << ": displs["
                        << i
                        << "] = "
                        << my_displs[i]
                        << "  (used = "
                        << used 
                        << ") sends data already sent!" 
                        << std::endl;
                    rvalue = MARMOT_NOTE_DISPLACEMENT_DATA_SENT_TWICE;
                    bErrorOccured = true;
                }
            }
            
            if (bErrorOccured)
            {
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_NOTE,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }

        // (2) Here we have a datatype with holes.
        else
        {
            std::stringstream stream;
            bool bErrorOccured = false;
            
            // Check array of displacements.
            for (int i = 0; i < size; i++)
            {
                if ((my_displs[i] < used) && 
                    (MSG_ERR_WARN_NOTE <= DEBUG_MODE))
                {
                    stream
                        << "REMARK: "
                        << getCallname()
                        << ": Datatype has holes, "
                        << "so we don't check whether it's correct,"
                        << "but on rank "
                        << rank
                        << ": displs["
                        << i 
                        << "] reads data/ holes read from other nodes!" 
                        << std::endl;
                    rvalue = MARMOT_NOTE_DISPLACEMENT_READS_DATA;
                    bErrorOccured = true;
                }
                used = my_displs[i] + (typeextent * my_sendcounts[i]);
            }
            
            if (bErrorOccured)
            {
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_NOTE,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);   
            }
        }

        return rvalue;
    }                           // end checkDisplacementArrayOfSender


    // checkEdgeArray()
    //========================================================================
    /** @see  MPO_Graph_map::checkEdgeArray()
     *        exactly like this template
     *  
     *  @see  MPO_Graph_create::checkEdgeArray()
     *        exactly like this template
     */

    // checkEdgeArray
    //========================================================================
    int MPO_::checkEdgeArray()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        int comm_size = -1;
        PMPI_Comm_size(my_comm, &comm_size);

        std::stringstream stream;
        bool bErrorOccured = false;
        
        // Check if all entries in my_edges are valid
        for (int i = 0; i < my_index[my_nnodes - 1]; i++)
        {
            if ((my_edges[i] >= comm_size) && 
                (MSG_ERR <= DEBUG_MODE))
            {
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": Invalid rank: edges["
                    << i 
                    << "] = " 
                    << my_edges[i] 
                    << std::endl;
                rvalue = MARMOT_ERR_EDGE_INVALID;
                bErrorOccured = true;
            }
        }
        
        if (bErrorOccured)
        {
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkEdgeArray


    // checkErrhandler() / checkAndEraseErrhandler()
    //========================================================================
    /** @see  MPO_Errhandler_get::checkErrhandler()
     *        exactly like this template
     *
     *  @see  MPO_Errhandler_set::checkErrhandler()
     *        exactly like this template, except for:
     *        - replace our_errhandlers.find(*my_errhandler) by
     *                  our_errhandlers.find(my_errhandler)
     *
     *  @see  MPO_Errhandler_free::checkAndEraseErrhandler()
     *        exactly like this template, 
     *        exactly like checkErrhandler(), except for:
     *        - add erase ...
     *        
     */

    // checkErrhandler
    //========================================================================
    int MPO_::checkErrhandler()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // Check if errhandler is valid.
        if ((our_errhandlers.find(*my_errhandler) == our_errhandlers.end()) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": errhandler is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_ERRHANDLER_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkErrhandler


    // checkAndEraseErrhandler
    //========================================================================
    int MPO_::checkAndEraseErrhandler()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // Check if errhandler is valid.
        if ((our_errhandlers.find(*my_errhandler) == our_errhandlers.end()) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": errhandler is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_ERRHANDLER_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        (our_errhandlers.find(*my_errhandler)->second[1]) = 1;

        // Erase errhandler from our map.
        if (our_errhandlers.find(*my_errhandler)->second[0] == 0)
        {
            our_errhandlers.erase(*my_errhandler);
        }

        return rvalue;
    }                           // end checkAndEraseErrhandler


    // checkErrorcode()
    //========================================================================
    /** 
     *  @see  MPO_Error_class::checkErrorcode()
     *        exactly like this template
     *  
     *  @see  MPO_Error_string::checkErrorcode()
     *        exactly like this template
     *  
     */

    // checkErrorcode
    //========================================================================
    int MPO_::checkErrorcode()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // Check if errorcode is valid.
        if (((my_errorcode < 0) ||
             (my_errorcode > MPI_ERR_LASTCODE)) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": errorcode is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_ERRORCODE_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkErrorcode


    // checkGroup() / checkGroup2()
    //========================================================================
    /** 
     *  @see  MPO_Groupconstructor::checkGroup()
     *        exactly like this template
     *  
     *  @see  MPO_Grouphandler::checkGroup()
     * 
     *  @see  MPO_Comm_create::
     *
     *  @see  MPO_Win_post::checkGroup
     */

    // checkGroup
    //========================================================================
    int MPO_::checkGroup(int valid)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // (1) Check if group is null handle.
        if ((my_group == MPI_GROUP_NULL) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname() 
                << ": Group1 is MPI_GROUP_NULL!" 
                << std::endl;
            rvalue = MARMOT_WARN_GROUP_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if group is valid, i.e. registered in our_groups.
        if ((our_groups.find(my_group) == our_groups.end()) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Group1 not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_GROUP_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }
        valid--;

        return rvalue;
    }                           // end checkGroup


    // checkGroup2
    //========================================================================
    /** 
     *  @see  MPO_Groupconstructor::checkGroup2
     *        exactly like this template
     *
     *  @see  MPO_Grouphandler
     */
    int MPO_::checkGroup2(int valid)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // (1) Check if group is null handle.
        if ((my_group2 == MPI_GROUP_NULL) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname() 
                << ": Group2 is MPI_GROUP_NULL!" 
                << std::endl;
            rvalue = MARMOT_WARN_GROUP_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if group is valid, i.e. registered in our_groups.
        if ((our_groups.find(my_group2) == our_groups.end()) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Group2 not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_GROUP_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }
        valid--;

        return rvalue;
    }                           // end checkGroup2


    // checkLeaderLocal() / checkLeaderRemote()
    //========================================================================
    /** @see  MPO_Intercomm_create::checkLeaderLocal()
     *        exactly like this template
     *
     *  @see  MPO_Intercomm_create::checkLeaderRemote()
     *        exactly like this template
     */
    // checkLeaderLocal
    //========================================================================
    int MPO_::checkLeaderLocal()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // local_leader in local_comm:
        int local_size = 0;
        PMPI_Comm_size(my_local_comm, &local_size);

        int local_rank = MPI_PROC_NULL;
        PMPI_Comm_rank(my_local_comm, &local_rank);
           
        // remote_leader in peer_comm 
        // (only significant at local_leader):
        int peer_size = -1;
        PMPI_Comm_size(my_peer_comm, &peer_size);

        // (1) Check local leader
        // (1.1) Check if local leader is MPI_ANY_SOURCE
        if ((my_local_leader == MPI_ANY_SOURCE) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Local_leader is MPI_ANY_SOURCE! "
                << "Not allowed in this call."
                << std::endl;
            rvalue = MARMOT_ERR_LEADER_ANY_SOURCE;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (1.2) Check if local leader is negative
        else if ((my_local_leader != MPI_ANY_SOURCE) &&
                 (my_local_leader < 0) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Local_leader less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_LEADER_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (1.3) Check if local leader is out of local communicator
        else if ((my_local_leader != MPI_ANY_SOURCE) &&
                 (my_local_leader >= local_size) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Local_leader out of local_comm!" 
                << std::endl;
            rvalue = MARMOT_ERR_LEADER_OUT_OF_COMM;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // both leaders in peer_comm:
        // on nodes that are no member of local_comm: 
        // local_comm==MPI_COMM_NULL
        else if ((my_local_comm != MPI_COMM_NULL) &&
                 (local_rank == my_local_leader) &&
// ???
                 (my_peer_comm == MPI_COMM_NULL) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Local_leader does not belong to peer_comm!"
                << std::endl;
            rvalue = MARMOT_ERR_LEADER_OUT_OF_COMM;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }    

        return rvalue;
    }                             // end checkLeaderLocal


    // checkLeaderRemote
    //========================================================================
    int MPO_::checkLeaderRemote()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // local_leader in local_comm:
        int local_size = 0;
        PMPI_Comm_size(my_local_comm, &local_size);

        int local_rank = MPI_PROC_NULL;
        PMPI_Comm_rank(my_local_comm, &local_rank);
           
        // remote_leader in peer_comm 
        // (only significant at local_leader):
        int peer_size = -1;
        PMPI_Comm_size(my_peer_comm, &peer_size);

        // (2) Check remote leader
        // (2.1) Check if remote leader is MPI_ANY_SOURCE
        if ((my_remote_leader == MPI_ANY_SOURCE) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Remote_leader is MPI_ANY_SOURCE! "
                << "Not allowed in this call."
                << std::endl;
            rvalue = MARMOT_ERR_LEADER_ANY_SOURCE;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2.2) Check if remote leader is negative
        else if ((my_remote_leader != MPI_ANY_SOURCE) &&
                 (my_remote_leader < 0) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Remote_leader less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_LEADER_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2.3) Check if remote leader is out of peer communicator
        else if ((my_remote_leader != MPI_ANY_SOURCE) &&
                 (my_remote_leader >= peer_size) && 
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Remote_leader is out of peer_comm!" 
                << std::endl;
            rvalue = MARMOT_ERR_LEADER_OUT_OF_COMM;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                             // end checkLeaderRemote


    // checkLockType()
    //========================================================================
    /** @see  MPO_Win_lock
     *        exactly like this template
     */

    // checkLockType
    //========================================================================
    int MPO_::checkLockType()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // Check if locktype is valid.
        if ((MPI_LOCK_EXCLUSIVE != my_lock_type) && 
            (MPI_LOCK_SHARED != my_lock_type) &&  
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: " 
                << getCallname() 
                << ": lock type is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_LOCKTYPE_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkLockType



    // checkMaxEdges()
    //========================================================================
    /** @see  MPO_Graph_get
     *        exactly like this template
     */

    // checkMaxEdges
    //========================================================================
    int MPO_::checkMaxEdges()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // Check if maxedges is negative.
        if ((my_maxedges < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: " 
                << getCallname() 
                << ": maxedges less than 0!" 
                << std::endl;
            rvalue = MARMOT_ERR_EDGE_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkMaxEdges


    // checkMaxIndex()
    //========================================================================
    /** @see  MPO_Graph_get
     *        exactly like this template
     */

    // checkMaxIndex
    //========================================================================
    int MPO_::checkMaxIndex()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        int size = -1;
        PMPI_Comm_size(my_comm, &size);

        // (1) Check if maxindex is negative.
        if ((my_maxindex < 0) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: " 
                << getCallname() 
                << ": maxindex less than 0!" 
                << std::endl;
            rvalue = MARMOT_ERR_INDEX_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if maxindex is less than size of communicator.
        else if ((my_maxindex < size) && 
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": maxindex is less than size of communicator!" 
                << std::endl;
            rvalue = MARMOT_WARN_INDEX_LT_SIZE;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkMaxIndex


    // checkMaxNeighbors()
    //========================================================================
    /** @see  MPO_Graph_neighbors
     *        exactly like this template
     */

    // checkMaxNeighbors
    //========================================================================
    int MPO_::checkMaxNeighbors()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // Check if maxneighbors is negative.
        if ((my_maxneighbors < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": maxneighbors less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_NEIGHBORS_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkMaxNeighbors


    // checkOp() / checkOp(MPO_Op op)
    //========================================================================
    /** @see  MPO_Op_free::checkOp()
     *        exactly like this template
     *
     *  @see  MPO_Collective::checkOp(MPO_Op op)
     *        exactly like this template
     */

    // checkOp
    //========================================================================
    int MPO_::checkOp()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // Check if op exists.
        if ((our_ops.find(*my_op) == our_ops.end()) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Operation to free doesn't exist (or is already freed)!"
                << std::endl;
            rvalue = MARMOT_ERR_OP_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkOp


    // checkOp
    //========================================================================
    int MPO_::checkOp(MPO_Op op)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // (1) Check if op is null handle.
        if ((op == MPI_OP_NULL) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Operation is MPI_OP_NULL!" 
                << std::endl;
            rvalue = MARMOT_ERR_OP_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if op is valid, i.e. registered in our_ops.
        if ((our_ops.find(op) == our_ops.end()) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Operation not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_OP_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkOp


    // checkPendingMessagesLeft()
    //========================================================================
    /** @see  MPO_Finalize::checkPendingMessagesLeft()
     *        exactly like this template
     */

    // checkPendingMessagesLeft
    //========================================================================
    int MPO_::checkPendingMessagesLeft()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        std::set < MPO_Comm >::iterator pos;
        int flag = 0;
        MPI_Status status;

        /** at creator of this method we might create the same message again and again here - think about a loop abort after the first message ! */
        // Check whether there are pending messages. 
        for (pos = our_comms.begin(); pos != our_comms.end(); pos++)
        {
            //If node is not member of communicator: *pos == MPI_COMM_NULL
            if (*pos != MPI_COMM_NULL)
            {
                PMPI_Iprobe(MPI_ANY_SOURCE, MPI_ANY_TAG, *pos, &flag, &status);

                if ((flag) && 
                    (MSG_ERR_WARN <= DEBUG_MODE))
                {
                    std::stringstream stream;
                    stream
                        << "WARNING: "
                        << getCallname()
                        << ": There are still pending messages!" 
                        << std::endl;
                    rvalue = MARMOT_WARN_PENDING_MESSAGES_LEFT;
            
                    MPO_String text;
                    text.assign(&stream);      
                    MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                    our_log->getMsg(msg);  
                }
            }
        }

        return rvalue;
    }                           // end checkPendingMessagesLeft


    // checkCoordinates()
    //========================================================================
    /** @see  MPO_Cart_rank
     *        exactly like this template
     */

    // checkCoordinates
    //========================================================================
    int MPO_::checkCoordinates()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        int ndims = -1;
        PMPI_Cartdim_get(my_comm, &ndims);

        int *dims = 0;
        int *periods = 0;
        int *coords = 0;

        dims = new int[ndims];
        periods = new int[ndims];
        coords = new int[ndims];

        PMPI_Cart_get(my_comm, ndims, dims, periods, coords);

        // Check array of coordinates
        for (int i = 0; i < ndims; i++)
        {
            // (1) Check if coordinate is negative. 
            if ((!periods[i]) && 
                (coords[i] < 0) && 
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": coordinate("
                    << i 
                    << ") less than 0 in non-periodic dimension!" 
                    << std::endl;
                rvalue = MARMOT_ERR_COORD_NEG;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (2) Check if coordinate is out of range. 
            else if ((!periods[i]) && 
                     (coords[i] >= dims[i]) && 
                     (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": coordinate("
                    << i 
                    << ") out of range in non-periodic dimension!" 
                    << std::endl;
                rvalue = MARMOT_ERR_COORD_TOO_BIG;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }
        }                       // end for

        delete[] dims;
        delete[] periods;
        delete[] coords;

        return rvalue;
    }                           // end checkCoordinates


    // checkRangeArray()
    //========================================================================
    /** @see  MPO_Group_range_excl
     *        exactly like this template
     *
     *  @see  MPO_Group_range_incl
     *        exactly like this template
     *
     *  This is very complex, may be better to split this into 
     *  checkFirstRank, checkLastRank, checkStride, checkComputedRank or so.
     */

    // checkRangeArray()
    //========================================================================
    int MPO_::checkRangeArray()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        int size = -1;
        int i = 0;
        int j = 0;
        int comprank = -1;
        int valid = 1;
        int *testarray = 0;

        PMPI_Group_size(my_group, &size);

        // Initialize testarray to 0.
        testarray = new int[size];
        for (j = 0; j <= size - 1; j++)
        {
            testarray[j] = 0;
        }

        // Check array of ranges
        for (i = 0; i <= my_n - 1; i++)
        {
            // (1.1) Check if first rank is negative.
            if ((my_ranges[i][0] < 0) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR :"
                    << getCallname()
                    << " : First rank in triplet "
                    << i 
                    << " is less than zero!" 
                    << std::endl;
                rvalue = MARMOT_ERR_RANK_FIRST_NEG;
                valid = 0;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (1.2) Check if last rank is negative.
            if ((my_ranges[i][1] < 0) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR :"
                    << getCallname()
                    << " : Last rank in triplet "
                    << i 
                    << " is less than zero!" 
                    << std::endl;
                rvalue = MARMOT_ERR_RANK_LAST_NEG;
                valid = 0;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (2.1) Check if first rank is within group.
            if ((my_ranges[i][0] > size) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": First rank in triplet "
                    << i 
                    << " is out of Group!" 
                    << std::endl;
                rvalue = MARMOT_ERR_RANK_FIRST_TOO_BIG;
                valid = 0;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (2.2) Check if last rank is within group.
            if (((my_ranges[i][0] +
                  ((my_ranges[i][1] - my_ranges[i][0]) / my_ranges[i][2]) /
                  my_ranges[i][2]) > size) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": Last rank in triplet "
                    << i 
                    << " is out of Group!" 
                    << std::endl;
                rvalue = MARMOT_ERR_RANK_LAST_TOO_BIG;
                valid = 0;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (3.1) Check if stride is negative.
            if ((my_ranges[i][0] < my_ranges[i][1]) && 
                (my_ranges[i][2] < 0) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": In triplet "
                    << i 
                    << ": First rank less than last rank, but stride less than 0 !" 
                    << std::endl;
                rvalue = MARMOT_ERR_STRIDE_NEG;
                valid = 0;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (3.2) Check if stride is 0.
            if ((my_ranges[i][2] == 0) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": Stride of triplet " 
                    << i 
                    << " is zero!" 
                    << std::endl;
                rvalue = MARMOT_ERR_STRIDE_ZERO;               
                valid = 0;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (3.3) Check if stride is positive.
            if ((my_ranges[i][0] > my_ranges[i][1]) && 
                (my_ranges[i][2] > 0) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": In triplet "
                    << i 
                    << ": First rank greater than last rank, but stride greater than 0 !" 
                    << std::endl;
                rvalue = MARMOT_ERR_STRIDE_POS;               
                valid = 0;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (4) Check if all computed ranges are distinct and valid.
            if (valid)
            {
                comprank = my_ranges[i][0];

                while (((my_ranges[i][0] <= my_ranges[i][1]) &&
                        (comprank <= my_ranges[i][1])) ||
                       ((my_ranges[i][0] > my_ranges[i][1]) &&
                        (comprank >= my_ranges[i][1])))
                {
                    // (4.1) Check if computed rank is negative.
                    if ((comprank < 0) && 
                        (MSG_ERR <= DEBUG_MODE))
                    {
                        std::stringstream stream;
                        stream
                            << "ERROR: "
                            << getCallname() 
                            << " Computed rank less than 0" 
                            << std::endl;
                        rvalue = MARMOT_ERR_RANK_COMPUTED_NEG; 
            
                        MPO_String text;
                        text.assign(&stream);      
                        MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                        our_log->getMsg(msg);     
                    }

                    // (4.2) Check if computed rank is out of group.
                    if ((comprank > size) && 
                        (MSG_ERR <= DEBUG_MODE))
                    {
                        std::stringstream stream;
                        stream
                            << "ERROR: "
                            << getCallname()
                            << " Computed rank out of group: "
                            << std::endl
                            << "Rank = "
                            << comprank 
                            << ", size of comm = " 
                            << size 
                            << std::endl;
                        rvalue = MARMOT_ERR_RANK_COMPUTED_TOO_BIG;   
            
                        MPO_String text;
                        text.assign(&stream);      
                        MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                        our_log->getMsg(msg);   
                    }

                    if (testarray[comprank] == 0)
                    {
                        testarray[comprank] = 1;
                    }
                    // (4.3) Check if computed rank appears twice.
                    else if ((testarray[comprank] != 0) &&
                             (MSG_ERR <= DEBUG_MODE))
                    {
                        std::stringstream stream;
                        stream
                            << "ERROR: "
                            << getCallname()
                            << ": Rank " 
                            << comprank 
                            << " twice to exclude! " 
                            << std::endl;
                        rvalue = MARMOT_ERR_RANK_COMPUTED_TWICE;
            
                        MPO_String text;
                        text.assign(&stream);      
                        MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                        our_log->getMsg(msg);      
                    }

                    comprank = comprank + (my_ranges[i][2]);
                }               // end while
            }                   // end if (4)
        }                       // end for

        delete[] testarray;

        return rvalue;
    }                           // end checkRangeArray


    // checkRank() / checkRankTooBig()
    // checkRankOfDestination() / checkRankOfDestTooBig()
    // checkRankOfSource() / checkRankOfSourceTooBig() 
    //========================================================================
    /** @see  
     */

// TODO //


    // checkRank
    //========================================================================
    /** @see  MPO_Blocking
     *  @see  MPO_Sendrecv::checkRankSendRecv
     *  @see  MPO_Initiator::checkRank
     *  @see  MPO_Win_unlock::checkRank
     *  @see  MPO_Win_lock::checkRank
     */
    int MPO_::checkRank()
    {
        // We distinct between MPO_Recv and other blocking calls, 
        // because my_rank may be source or destination.

        if (getId() == MPO_RECV_ID)
        {
            checkRankOfSource();
        }
        else 
        {
            checkRankOfDestination();
        }

        checkRankTooBig();

        return 0;
    }


    // checkRankOfSource
    //========================================================================
    /** @see  MPO_Blocking
     *
     *  @see  MPO_Sendrecv
     *        exactly like checkRankOfSource, except for:
     *        - replace my_rank by my_source
     *
     *  @see  MPO_Initiator
     *        exactly like this template, except for:
     *  @see  MPO_Collective::checkRoot(int root)
     *        exactly like this template, except for:
     *        - replace my_rank by root
     *  @see  MPO_Probes::checkRankOfSource
     */
    int MPO_::checkRankOfSource()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if rank of source < 0.
        if ((my_rank != MPI_ANY_SOURCE) &&
            (my_rank != MPI_PROC_NULL) && 
            (my_rank < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Rank of source = "
                << my_rank
                << " less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_RANK_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if rank of source is MPI_PROC_NULL.
        else if ((my_rank == MPI_PROC_NULL) && 
                 (MSG_ERR_WARN_NOTE <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "NOTE: "
                << getCallname() 
                << ": Source = MPI_PROC_NULL !" 
                << std::endl;
            rvalue = MARMOT_NOTE_RANK_PROC_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_NOTE,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (3) Warn of race conditions if MPI_ANY_SOURCE is used.
        else if ((my_rank == MPI_ANY_SOURCE) && 
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": Use of MPI_ANY_SOURCE may cause race conditions!"
                << std::endl;
            rvalue = MARMOT_WARN_RANK_ANY_SOURCE;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }                      

        return rvalue;
    }                           // end checkRankOfSource


    // checkRankOfDestination
    //========================================================================
    /** @see  MPO_Blocking
     *
     *  @see  MPO_Sendrecv
     *        exactly like checkRankOfDestination, except for:
     *        - replace my_rank by my_dest
     *
     *  @see  MPO_Initiator
     *        exactly like this template, except for:
     */
    int MPO_::checkRankOfDestination()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if rank of destination < 0.
        if ((my_rank < 0) && 
            (my_rank != MPI_PROC_NULL) && 
            (my_rank != MPI_ANY_SOURCE) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": Rank of destination = "
                << my_rank
                << " less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_RANK_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if rank of destination is MPI_PROC_NULL.
        else if ((my_rank == MPI_PROC_NULL) && 
                 (MSG_ERR_WARN_NOTE <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "NOTE: "
                << getCallname()
                << ": Rank of destination = MPI_PROC_NULL !" 
                << std::endl;
            rvalue = MARMOT_NOTE_RANK_PROC_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_NOTE,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (3) Check if rank of destination is MPI_ANY_SOURCE.
        else if ((my_rank == MPI_ANY_SOURCE) && 
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR "
                << getCallname()
                << ": Rank of destination = MPI_ANY_SOURCE !" 
                << std::endl;
            rvalue = MARMOT_ERR_RANK_ANY_SOURCE;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkRankOfDestination


    // checkRankTooBig
    //========================================================================
    /** @see  MPO_Blocking
     *
     *  @see  MPO_Sendrecv::checkRankOfSourceTooBig, checkRankOfDestTooBig
     *        exactly like checkRankTooBig, except for:
     *        - replace my_rank by my_source and by my_dest, respectively.
     *
     *  @see  MPO_Initiator::checkRankOfSourceTooBig
     *        exactly like this template, except for:
     *
     *  @see  MPO_Initiator::checkRankOfDestTooBig
     *        exactly like this template, except for:
     *  @see  MPO_Probes::checkRankOfSourceTooBig
     *        exactly like this template, except for:
     *        - replace my_rank by source
     */
    int MPO_::checkRankTooBig()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;        

        // Check if rank is out of communicator.     
        int size = -1;
        PMPI_Comm_size(my_comm, &size);

        if ((my_rank >= size) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Rank is out of communicator: rank = "
                << my_rank 
                << ", size of communicator = " 
                << size 
                << std::endl;
            rvalue = MARMOT_ERR_RANK_TOO_BIG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }


    // checkRecv()
    //========================================================================
    /** @see  
     */

// TODO //


    // checkRequest() / checkRequestArray() / checkRequestCountArray()
    //========================================================================
    /** @see  
     */

// TODO //

    // checkRequest
    //========================================================================
    /** 
     *  @see  MPO_Initiator::checkRequest
     *        exactly like this template
     */
    int MPO_::checkRequest()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES.
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if request is still in use
        if (our_requests.find(my_request_p) != our_requests.end())
        {
            if (MSG_ERR <= DEBUG_MODE)
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname() 
                    << " Request is still in use !! " 
                    << std::endl;
            }
            rvalue = MARMOT_ERR_REQUEST_STILL_USED;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Insert request
        /**
         * Moved to the various child classes registration by value requires registration
         * after the MPI Call !
         */
        //else
        //{
        //    our_requests.insert(std::make_pair(my_request_p, my_persistent));
        //}

        return rvalue;
    }                           // end checkRequest


    // checkRequestCountArray
    //========================================================================
    /** @see  MPO_Completion
     *  @see  MPO_Tests
     *  @see  MPO_Starts
     */
    int MPO_::checkRequestCountArray()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if count is negative.
        if ((my_count < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": count of requests = " 
                << my_count
                << " less than 0!"
                << std::endl;
            rvalue = MARMOT_ERR_REQUEST_COUNT_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if count is 0.
        if ((my_count == 0) && 
            (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname() 
                << ": count of requests is zero!" 
                << std::endl;
            rvalue = MARMOT_WARN_REQUEST_COUNT_ZERO;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }


    // checkRequestArray
    //========================================================================
    /** @see  MPO_Starts
     *  @see  MPO_Completion
     *  @see  MPO_Tests
     */
    int MPO_::checkRequestArray()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        std::map < MPO_Request, enum KIND_OF_REQUEST >::iterator pos;

        // Check if request_array is correct.
        for (int i = 0; i <= my_count - 1; i++)
        {
            pos = our_requests.find(&my_request_array[i]);

            // (1) Request is not valid, i.e. not registered in the map
            // of our_requests.  
            if ((pos == our_requests.end()) &&
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname()
                    << ": Request " 
                    << i
                    << " does not exist, is not / no more valid! " 
                    << std::endl;
                rvalue = MARMOT_ERR_REQUEST_NOT_VALID;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            // (2) Request is registered in the map of our_requests.
            // In this case, we have to check different violations 
            // in the switch statement.
            else if ((pos != our_requests.end()) &&
                     (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                MPO_MSG_TYPE type = MSG_TYPE_ERROR;
                                
                // all the cases from enum KIND_OF_REQUEST
                switch (pos->second)
                {
                    case (REQUEST_NULL):
                        if (MSG_ERR_WARN <= DEBUG_MODE)
                        {
                            stream
                                << "WARNING: "
                                << getCallname()
                                << ": Request " 
                                << i
                                << " is MPI_REQUEST_NULL! " 
                                << std::endl;
                            rvalue = MARMOT_WARN_REQUEST_NULL;
                            type = MSG_TYPE_WARNING;
                        }
                        break;
                    case (REQUEST_NOT_PERSISTENT):
                        stream
                            << "ERROR: "
                            << getCallname()
                            << ": Request " 
                            << i
                            << " is no persistent request! " 
                            << std::endl;
                        rvalue = MARMOT_ERR_REQUEST_NOT_PERSISTENT;
                        break;
                    case (REQUEST_ACTIVE_PERSISTENT):
                        if (MSG_ERR_WARN <= DEBUG_MODE)
                        {
                            stream
                                << "WARNING: "
                                << getCallname()
                                << ": Request "
                                << i 
                                << " is persistent request, but still active! "
                                << std::endl;
                            rvalue = MARMOT_WARN_REQUEST_ACTIVE_PERSISTENT;
                            type = MSG_TYPE_WARNING;
                        }
                        break;
                    case (REQUEST_INACTIVE_PERSISTENT):
                        // Everything is fine. Set request to active.
                        pos->second = REQUEST_ACTIVE_PERSISTENT;
                        break;
                    default:
                        stream
                            << "Check mapping of requests."
                            << std::endl
                            << "Check implementation of switch statement."
                            << std::endl;
                        rvalue = MARMOT_ERR_MARMOT;
                        break;
                }               // end switch
                
                if (rvalue != MARMOT_SUCCESS)
                {            
                    MPO_String text;
                    text.assign(&stream);      
                    MPO_MsgObject msg (type,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                    our_log->getMsg(msg);  
                }
            }                   // end else if
        }                       // end for

        return rvalue;
    }                           // end checkRequestArray

 
    // checkRequestsLeft()
    //========================================================================
    /** @see  MPO_Finalize::checkRequestsLeft()
     *        exactly like this template
     */

    // checkRequestsLeft
    //========================================================================
    int MPO_::checkRequestsLeft()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        int count = 0;

        // Check whether there are active or non-freed requests left.
        if ((!our_requests.empty()) &&
            (MSG_ERR_WARN <= DEBUG_MODE))
            {
                count = our_requests.size();
                std::stringstream stream;
                stream
                    << "WARNING: "
                    << getCallname()
                    << ": There are still "
                    << count 
                    << " active/non-freed Requests! " 
                    << std::endl;
                rvalue = MARMOT_WARN_REQUESTS_LEFT;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

       return rvalue;
    }                           // end checkRequestsLeft


    // checkRoot()
    //========================================================================
    /** @see  MPO_Collective
     *        exactly like this template
     */

    // checkRoot
    //========================================================================
    int MPO_::checkRoot(int root)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // (1) Check if rank of root < 0.
        if ((root != MPI_ANY_SOURCE) &&
            (root != MPI_PROC_NULL) && 
            (root < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname()
                << ": Rank of source = "
                << root
                << " less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_RANK_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if rank of root is MPI_PROC_NULL.
        else if ((root == MPI_PROC_NULL) && 
                 (MSG_ERR_WARN_NOTE <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "NOTE: "
                << getCallname() 
                << ": Source = MPI_PROC_NULL !" 
                << std::endl;
            rvalue = MARMOT_NOTE_RANK_PROC_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_NOTE,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (3) Warn of race conditions if MPI_ANY_SOURCE is used.
        else if ((root == MPI_ANY_SOURCE) && 
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": Use of MPI_ANY_SOURCE may cause race conditions!"
                << std::endl;
            rvalue = MARMOT_WARN_RANK_ANY_SOURCE;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }                      

        return rvalue;
    }                           // end checkRoot


    // checkSizeIn(const string KIND_OF_SIZE) 
    // checkSizeOut(const string KIND_OF_SIZE)
    //========================================================================
    /** @see  MPO_Unpack::checkSizeIn(const string KIND_OF_SIZE)
     *        exactly like this template
     *
     *  @see  MPO_Pack::checkSizeOut(const string KIND_OF_SIZE)
     *        exactly like this template, except for:
     *        - replace my_insize by my_outsize 
     */

    // checkSizeIn
    //========================================================================
    int MPO_::checkSizeIn(const string KIND_OF_SIZE)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS; 

        // (1) Check if size is negative.
        if ((my_insize < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: " 
                << getCallname() 
                << ": "
                << KIND_OF_SIZE
                << " = "
                << my_insize
                << " less than 0 !" 
                << std::endl;
            rvalue = MARMOT_ERR_SIZE_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if size is 0.
        else if ((my_insize == 0) && 
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: " 
                << getCallname() 
                << ": "
                << KIND_OF_SIZE
                << " = 0 !" 
                << std::endl;
            rvalue = MARMOT_WARN_SIZE_ZERO;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkSizeIn


    // checkTag() / checkTagOfSender() / checkTagOfReceiver()
    //========================================================================
    /** @see  
     */

// TODO //


    // checkTag(const string KIND_OF_TAG)
    //========================================================================
    /** @see  MPO_Blocking::checkTag(const string KIND_OF_TAG)
     *        exactly like this template
     *
     *  @see  MPO_Probes::checkTag(const string KIND_OF_TAG)
     *        exactly like this template
     *
     *  @see MPO_Initiator::checkTag(const string KIND_OF_TAG)
     *        exactly like this template
     *
     *  @see MPO_Intercomm_create::checkTag(const string KIND_OF_TAG)
     *        exactly like this template, except for:
     *        -add 
        // (3) Check if Tag is MPI_ANY_TAG.
        else if ((my_tag == MPI_ANY_TAG) && 
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::cerr 
                << "ERROR: "
                << getCallname()
                << ": Tag is MPI_ANY_TAG which is not allowed in this call!"
                << std::endl;
            rvalue = MARMOT_ERR_TAG_ANY_TAG;
        }
     *
     *  @see  MPO_Sendrecvs::checkTagOfSender(const string KIND_OF_TAG)
     *        exactly like this template, except for:
     *        - replace my_tag by my_sendtag
     *
     *  @see  MPO_Sendrecvs::checkTagOfReceiver(const string KIND_OF_TAG)
     *        exactly like this template, except for:
     *        - replace my_tag by my_recvtag
     */

    // checkTag
    //========================================================================
    int MPO_::checkTag(const string KIND_OF_TAG)
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if tag is < 0.
        if ((my_tag != MPI_ANY_TAG) && 
            (my_tag < 0) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": "
                << KIND_OF_TAG
                << " = "
                << my_tag
                << " less than 0!" 
                << std::endl;
            rvalue = MARMOT_ERR_TAG_NEG;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if tag is within guaranteed range.
        else if ((my_tag != MPI_ANY_TAG) && 
                 (my_tag > 32767) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            int ub = 0;
            int flag = 1;
            PMPI_Attr_get(MPI_COMM_WORLD, MPI_TAG_UB, &ub, &flag);

            if ((my_tag > ub) && 
                (MSG_ERR <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "ERROR: "
                    << getCallname() 
                    << ": "
                    << KIND_OF_TAG
                    << " = "
                    << my_tag 
                    << " is larger than allowed!" 
                    << std::endl
                    << "THIS implementation allows tags up to "
                    << ub 
                    << "."
                    << std::endl;
                rvalue = MARMOT_ERR_TAG_TOO_BIG;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }

            else if ((my_tag <= ub) && 
                     (MSG_ERR_WARN <= DEBUG_MODE))
            {
                std::stringstream stream;
                stream
                    << "WARNING: "
                    << getCallname()
                    << ": "
                    << KIND_OF_TAG
                    << " = "
                    << my_tag
                    << " greater than 32767 ! MPI only guarantees tags up to this." 
                    << std::endl
                    << "THIS implementation allows tags up to "
                    << ub 
                    << "."
                    << std::endl;
                rvalue = MARMOT_WARN_TAG_TOO_BIG;
            
                MPO_String text;
                text.assign(&stream);      
                MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
                our_log->getMsg(msg);  
            }
        }                       // end else if (2)

        return rvalue;
    }                           // end checkTag


    // checkToplogyNew() / checkTopologyCartNew()
    //========================================================================
    /** @see  MPO_Graph_map
     *        exactly like this template
     *
     *  @see  MPO_Graph_map::checkTopologyNew()
     *        exactly like this template, except for:
     *        - replace my_comm by my_comm_old
     *
     *  @see  MPO_Cart_create::checkTopologyCartNew()
     *        exactly like this template, 
     *        exactly like checkTopologynew(), except for:
     *        - replace my_comm by my_comm_old
     *        - TODO
     *
     *  @see  MPO_Cart_map::checkTopologyCartNew()
     *        exactly like this template, except for:
     *        - replace my_comm by my_comm_old
     */

    // checkToplogyNew
    //========================================================================
    int MPO_::checkTopologyNew()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        int comm_size = -1;
        PMPI_Comm_size(my_comm, &comm_size);

        // (1) Check if nnodes > size of old communicator
        if ((comm_size < my_nnodes) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream    << "ERROR: "
                      << getCallname()
                      << ": New topology is larger than old communicator:"
                      << std::endl
                      << " comm_size = "
                      << comm_size 
                      << ", nnodes = " 
                      << my_nnodes 
                      << std::endl;
            rvalue = MARMOT_ERR_TOPOLOGY_NEW_GT_COMM_OLD;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if nnodes < size of old communicator
        else if ((comm_size > my_nnodes) &&
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": New topology is smaller than old communicator:"
                << " Some nodes will get MPI_COMM_NULL." 
                << std::endl
                << " comm_size = "
                << comm_size 
                << ", nnodes = " 
                << my_nnodes 
                << std::endl;
            rvalue = MARMOT_WARN_TOPOLOGY_NEW_LT_COMM_OLD;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkTopologyNew


    // checkTopologyCartNew
    //========================================================================
    int MPO_::checkTopologyCartNew()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        int comm_size = -1;
        PMPI_Comm_size(my_comm_old, &comm_size);

        // Compute total size of cartesian grid
        int i = 0;
        int total_size = 1;

        for (i = 0; i <= my_ndims - 1; i++)
        {
            total_size = total_size * my_dims[i];
        }

        // (1) Check if total size > size of old communicator
        if ((comm_size < total_size) &&
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream    << "ERROR: "
                      << getCallname()
                      << ": New topology is larger than old communicator:"
                      << std::endl
                      << " comm size = "
                      << comm_size 
                      << ", total size = " 
                      << total_size 
                      << std::endl;
            rvalue = MARMOT_ERR_TOPOLOGY_CART_NEW_GT_COMM_OLD;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        // (2) Check if total_size < size of old communicator
        else if ((comm_size > total_size) &&
                 (MSG_ERR_WARN <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "WARNING: "
                << getCallname()
                << ": New topology is smaller than old communicator:"
                << " Some nodes will get MPI_COMM_NULL." 
                << std::endl
                << " comm size = "
                << comm_size 
                << ", total size = " 
                << total_size 
                << std::endl;
            rvalue = MARMOT_WARN_TOPOLOGY_CART_NEW_LT_COMM_OLD;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_WARNING,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);  
        }

        return rvalue;
    }                           // end checkTopologyCartNew



    // checkWin()
    //========================================================================
    /** @see  MPO_Winhandler::checkWin()
     *        exactly like this template
     *
     *  @see  MPO_Win_free:checkWin()
     *        exactly like this template, except for:
     *        replace my_win by *my_win
     *        replace MARMOT_ERR_WIN_NULL by MARMOT_WARN_WIN_NULL
     */

    // checkWin
    //========================================================================
    int MPO_Winhandler::checkWin()
    {
        // Return value is the unique error code set by enum MARMOT_ERR_CODES. 
        int rvalue = MARMOT_SUCCESS;

        // (1) Check if window is null handle.
        if ((MPI_WIN_NULL == my_win) && 
            (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << ": window is MPI_WIN_NULL!" 
                << std::endl;
            rvalue = MARMOT_ERR_WIN_NULL;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        // (2) Check if window is valid, i.e. registered in our_wins.
        else if ((our_wins.end() == our_wins.find(my_win)) &&
                 (MSG_ERR <= DEBUG_MODE))
        {
            std::stringstream stream;
            stream
                << "ERROR: "
                << getCallname() 
                << " : Window is not valid!" 
                << std::endl;
            rvalue = MARMOT_ERR_WIN_NOT_VALID;
            
            MPO_String text;
            text.assign(&stream);      
            MPO_MsgObject msg (MSG_TYPE_ERROR,0,text, getCallname() ,(MPO_CALL_ID) getId(),(MARMOT_ERR_CODES)rvalue,getSourceLocation()); 
            our_log->getMsg(msg);   
        }

        return rvalue;
    }                           // end checkWin




    // default constructor
    //========================================================================


    // proper constructor
    //========================================================================


    // destructor
    //========================================================================


    /* There are some pure virtual methods, see mpo.h.
     */


    // getFamilyId
    //========================================================================


    // getId
    //========================================================================


    // callNativeMPI
    //========================================================================


    // getCallname
    //========================================================================


    // debugout
    //========================================================================


    // save
    //========================================================================


    // restore
    //========================================================================

}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/env/init.c.
