/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *  
 *  Definition of classes MPO_Timer and MPO_Timestamp.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  Version:$Id: mpo-timer.cc 955 2009-05-15 14:02:17Z hpczink $
 */

#ifdef _WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

#include <iostream>

#include "mpo-timer.h"

namespace MPO
{

    //************************************************************************
    //
    //                            MPO_Timer
    // 

    // default constructor
    //========================================================================
    MPO_Timer::MPO_Timer()
    {
    }


    // destructor
    //========================================================================
    MPO_Timer::~MPO_Timer()
    {
    }

 
    // sleep
    //========================================================================
    /**
     *  So far we use the system function usleep.
     *
     *  It is included in BSD, Unix95 and Unix98. But not in Posix.
     *  In the future we might use "select" to achieve a better portability.
     */
    void MPO_Timer::sleep(long microseconds)
    {
	//in case sleep(0) is called (i.e. don't sleep) we don't call Sleep or 
	//usleep as this suspends the process
	if(0==microseconds)
		return;
#ifdef _WIN32
        Sleep(microseconds/1000);
#else
        usleep(microseconds);
#endif
    }


    //************************************************************************
    //
    //                            MPO_Timestamp
    //


    // proper constructor
    //========================================================================
    MPO_Timestamp::MPO_Timestamp()
        : my_value(0)
    {
    }


    // destructor
    //========================================================================
    MPO_Timestamp::~MPO_Timestamp()
    {
    }


    // operator ++
    //========================================================================
    void MPO_Timestamp::operator++()
    {
        my_value++;
    }


    // operator long
    //========================================================================
    /** The conversion to long is for a simple access to comparison 
     *  functions.
     */
    MPO_Timestamp::operator  long () const
    {
        return my_value;
    }


}                               // end of namespace MPO
