/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of MPO_Address class.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Address.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Address::MPO_Address()
        : my_location(0), 
          my_address(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Address::MPO_Address(void *location,
                             MPI_Aint * address)
        : my_location(location),
          my_address(address)
    {
    }


    // destructor
    //========================================================================
    MPO_Address::~MPO_Address()
    {
    }


    // getId
    //========================================================================
    int MPO_Address::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_ADDRESS_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Address::checkPost(void)
    {
            
        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Address::checkPrerequisites(void)
    {


        
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Address::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Address(my_location, my_address);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Address::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Address";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_ADDRESS";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Address::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname() 
            << "(*location, "
            << "*address";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Address::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_location;
        // out < my_address;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Address::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_location;
        // in > my_address;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/pt2pt/address.c.
/*@
  MPI_Address - Gets the address of a location in memory  

  Input Parameters:
  . location - location in caller memory (choice) 

  Output Parameter:
  . address - address of location (integer) 

  Note:
  This routine is provided for both the Fortran and C programmers.
  On many systems, the address returned by this routine will be the same
  as produced by the C '&' operator, but this is not required in C and
  may not be true of systems with word- rather than byte-oriented 
  instructions or systems with segmented address spaces.  

  .N fortran

  int MPI_Address( 
  void *location, 
  MPI_Aint *address)
  @ */

