/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Cart_sub.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Cart_sub.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Cart_sub::MPO_Cart_sub()
        : MPO_Cartesian(MPI_COMM_NULL),
          my_remain_dims(0),
          my_new_comm(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Cart_sub::MPO_Cart_sub(MPO_Comm comm,
                               int *remain_dims,
                               MPI_Comm * new_comm)
        : MPO_Cartesian(comm),
          my_remain_dims(remain_dims),
          my_new_comm(new_comm)
    {
    }


    // destructor
    //========================================================================
    MPO_Cart_sub::~MPO_Cart_sub()
    {
    }


    // getId
    //========================================================================
    int MPO_Cart_sub::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_CART_SUB_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Cart_sub::checkPost(void)
    {

        our_comms.insert(*my_new_comm);

        our_comm_traces->findAndAddUsage(my_comm,getSourceLocation());
        our_comm_traces->addAndSetCreation(*my_new_comm,getSourceLocation());
        our_comm_traces->findAndTryAddDependendency(*my_new_comm,my_comm);

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Cart_sub::checkPrerequisites(void)
    {


        MPO_Cartesian::checkPrerequisites();


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Cart_sub::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Cart_sub(my_comm, my_remain_dims, my_new_comm);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Cart_sub::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Cart_sub";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_CART_SUB";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Cart_sub::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(comm = "
            << my_comm
            << ", *remain_dims, "
            << "*newcomm";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Cart_sub::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_comm;
        // out < my_remain_dims;
        // out < my_new_comm_cart;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Cart_sub::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_comm;
        // in > my_remain_dims;
        // in > my_new_comm;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/topol/cart_sub.c.
/*@

MPI_Cart_sub - Partitions a communicator into subgroups which
form lower-dimensional cartesian subgrids

Input Parameters:
+ comm - communicator with cartesian structure (handle)
- remain_dims - the  'i'th entry of remain_dims specifies whether the
'i'th dimension is kept in the subgrid (true) or is dropped (false)
(logical vector)

Output Parameter:
. newcomm - communicator containing the subgrid that includes the calling
process (handle)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_TOPOLOGY
.N MPI_ERR_COMM
.N MPI_ERR_ARG

int MPI_Cart_sub (
MPI_Comm comm,
int *remain_dims,
MPI_Comm *comm_new )
@ */
