/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Cartdim_get.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Cartdim_get.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Cartdim_get::MPO_Cartdim_get()
        : MPO_Cartesian(MPI_COMM_NULL),
          my_ndims(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Cartdim_get::MPO_Cartdim_get(MPO_Comm communicator,
                                     int *ndims)
        : MPO_Cartesian(communicator),
          my_ndims(ndims)
    {
    }


    // destructor
    //========================================================================
    MPO_Cartdim_get::~MPO_Cartdim_get()
    {
    }


    // getId
    //========================================================================
    int MPO_Cartdim_get::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_CARTDIM_GET_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Cartdim_get::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Cartdim_get::checkPrerequisites(void)
    {


        MPO_Cartesian::checkPrerequisites();

        our_comm_traces->findAndAddUsage(my_comm,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Cartdim_get::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Cartdim_get(my_comm, my_ndims);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Cartdim_get::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Cartdim_get";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_CARTDIM_GET";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Cartdim_get::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(comm = "
            << my_comm
            << ", *ndims";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Cartdim_get::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_comm;
        // out < my_ndims;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Cartdim_get::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_comm;
        // in > my_ndims;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied
// from mpich-1.2.4/src/topol/cartdim_get.c.
/*@

MPI_Cartdim_get - Retrieves Cartesian topology information associated with
a communicator

Input Parameter:
. comm - communicator with cartesian structure (handle)

Output Parameter:
. ndims - number of dimensions of the cartesian structure (integer)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_COMM
.N MPI_ERR_ARG

int MPI_Cartdim_get (
MPI_Comm comm,
int *ndims )
@ */

