/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Datatype representing MPI-datatypes.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Datatype.cc 1078 2010-02-15 12:32:44Z tobias $
 */



#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "config.h"

namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Datatype::MPO_Datatype()
        : my_datatype(MPI_DATATYPE_NULL), my_name ("")
    {
    }


    // proper constructor
    //========================================================================
    MPO_Datatype::MPO_Datatype(MPI_Datatype datatype)
        : my_name ("")
    {
        my_datatype = datatype;
    }


    // destructor
    //========================================================================
    MPO_Datatype::~MPO_Datatype()
    {
    }


    // MPI_Datatype
    //========================================================================
    MPO_Datatype::operator  MPI_Datatype() const
    {
        return my_datatype;
    }


    // getId
    //========================================================================
    int MPO_Datatype::getId() const
    {
        // returns the unique type identity set by enum MPO_TYPE_ID
        return MPO_DATATYPE_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Datatype::debugout(std::ostream & output) const
    {
    	if (my_name == "")
    	{
			if (our_datatype_names.find(my_datatype) != our_datatype_names.end())
			{
				output << our_datatype_names.find(my_datatype)->second;
			}
			else
			{
				output << "non-predefined datatype";
			}
    	}
    	else
    	{
    		output << my_name;
    	}

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Datatype::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifndef MPI_DATATYPE_NON_BASIC
        out < my_datatype;
#else
        std::map < MPO_Datatype, std::string >::iterator
			j,
			i = our_datatype_names.find(my_datatype);
        if (i != our_datatype_names.end())
        {
        	int index = 0;
        	for (j = our_datatype_names.begin(); j != i; j++, index++);

        	out < index;
        }
        else
        {
        	out < (int)our_datatype_names.size();
        }
#endif

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Datatype::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifndef MPI_DATATYPE_NON_BASIC
        in > my_datatype;
#else
        int x;
        in > x;

        if (x == our_datatype_names.size())
        {
        	my_name = "non-predefined datatype";
        }
        else
        {
            std::map < MPO_Datatype, std::string >::iterator i = our_datatype_names.begin();
        	while (x > 0) {x--; i++;};
        	my_name = i->second;
        }

        my_datatype = MPI_DATATYPE_NULL;
#endif

        return in;
    }


}                               // end of namespace MPO
