/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Get_processor_name.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Get_processor_name.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Get_processor_name::MPO_Get_processor_name()
        : my_name(0), 
          my_resultlen(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Get_processor_name::MPO_Get_processor_name(char *name,
                                                   int
                                                   *resultlen)
        : my_name(name),
          my_resultlen(resultlen)
    {
    }


    // destructor
    //========================================================================
    MPO_Get_processor_name::~MPO_Get_processor_name()
    {
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Get_processor_name::checkPrerequisites()
    {

        

        // currently no checks implemented    

        return 0;
    }


    // getId
    //========================================================================
    int MPO_Get_processor_name::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_GET_PROCESSOR_NAME_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Get_processor_name::checkPost(void)
    {
    
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Get_processor_name::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Get_processor_name(my_name, my_resultlen);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif 

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Get_processor_name::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Get_processor_name";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_GET_PROCESSOR_NAME";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & 
    MPO_Get_processor_name::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname() 
            << "(*name, "
            << "*resultlen";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Get_processor_name::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_name;
        // out < my_resultlen;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Get_processor_name::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_name;
        // in > my_resultlen;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/env/getpname.c.
/*@
  MPI_Get_processor_name - Gets the name of the processor

  Output Parameters:
  + name - A unique specifier for the actual (as opposed to virtual) node. 
  This must be an array of size at least 'MPI_MAX_PROCESSOR_NAME'.
  - resultlen - Length (in characters) of the name 

  Notes:
  The name returned should identify a particular piece of hardware; 
  the exact format is implementation defined.  This name may or may not
  be the same as might be returned by 'gethostname', 'uname', or 'sysinfo'.

  .N fortran

  int MPI_Get_processor_name( 
  char *name,
  int *resultlen)
  @ */

