/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Get_version.
 *
 *  @author Bettina Krammer
 *
 *  $Id: mpo_Get_version.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // MPI_Get_version can be called before MPI_Init and after MPI_Finalize.


    // default constructor
    //========================================================================
    MPO_Get_version::MPO_Get_version()
        : my_version(0), 
          my_subversion(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Get_version::MPO_Get_version(int *version,
                                     int *subversion)
        : my_version(version),
          my_subversion(subversion)
    {
    }


    // destructor
    //========================================================================
    MPO_Get_version::~MPO_Get_version()
    {
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Get_version::checkPrerequisites()
    {

        

        // currently no checks implemented    

        return 0;
    }


    // getId
    //========================================================================
    int MPO_Get_version::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_GET_VERSION_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Get_version::checkPost(void)
    {
    
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Get_version::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Get_version(my_version, my_subversion);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Get_version::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Get_version";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_GET_VERSION";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Get_version::debugout(std::ostream & output) const
    {
        output 
            << getCallname() 
            << "(*version, "
            << "*subversion";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Get_version::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_version;
        // out < my_subversion;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Get_version::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_version;
        // in > my_subversion;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/env/getversion.c.
/*@
  MPI_Get_version - Gets the version of MPI

  Output Parameters:
  + version - Major version of MPI (1 or 2)
  - subversion - Minor version of MPI.  

  Notes:
  The defined values 'MPI_VERSION' and 'MPI_SUBVERSION' contain the same 
  information.  This routine allows you to check that the library matches 
  the version specified in the 'mpi.h' and 'mpif.h' files.

  .N fortran

  int MPI_Get_version( 
  int *version, 
  int *subversion )
  @ */
