/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Graphdims_get.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Graphdims_get.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Graphdims_get::MPO_Graphdims_get()
        : MPO_Graph(MPI_COMM_NULL),
          my_nnodes(0),
          my_nedges(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Graphdims_get::MPO_Graphdims_get(MPO_Comm communicator,
                                         int *nnodes,
                                         int *nedges)
        : MPO_Graph(communicator),
          my_nnodes(nnodes),
          my_nedges(nedges)
    {
    }


    // destructor
    //========================================================================
    MPO_Graphdims_get::~MPO_Graphdims_get()
    {
    }


    // getId
    //========================================================================
    int MPO_Graphdims_get::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_GRAPHDIMS_GET_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Graphdims_get::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Graphdims_get::checkPrerequisites(void)
    {


        MPO_Graph::checkPrerequisites();

        our_comm_traces->findAndAddUsage(my_comm,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Graphdims_get::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Graphdims_get(my_comm, my_nnodes, my_nedges);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Graphdims_get::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Graphdims_get";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_GRAPHDIMS_GET";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Graphdims_get::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(comm = "
            << my_comm
            << ", *nnodes, "
            << "*nedges";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Graphdims_get::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_comm;
        // out < my_nnodes;
        // out < my_nedges;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Graphdims_get::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_comm;
        // in > my_nnodes;
        // in > my_nedges;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from
// mpich-1.2.4/src/topol/graphdimsget.c.
/*@

MPI_Graphdims_get - Retrieves graph topology information associated with a
communicator

Input Parameters:
. comm - communicator for group with graph structure (handle)

Output Parameter:
+ nnodes - number of nodes in graph (integer)
- nedges - number of edges in graph (integer)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_TOPOLOGY
.N MPI_ERR_COMM
.N MPI_ERR_ARG

int MPI_Graphdims_get (
MPI_Comm comm,
int *nnodes,
int *nedges )
@ */

