/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Group representing MPI-groups.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Group.cc 1078 2010-02-15 12:32:44Z tobias $
 */



#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "config.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Group::MPO_Group()
        : my_group(MPI_GROUP_NULL), my_name("")
    {
    }


    // proper constructor
    //========================================================================
    MPO_Group::MPO_Group(MPI_Group group)
        : my_name("")
    {
        my_group = group;
    }


    // destructor
    //========================================================================
    MPO_Group::~MPO_Group()
    {
    }


    // MPI_Group
    //========================================================================
    MPO_Group::operator  MPI_Group() const
    {
        return my_group;
    }


    // getId
    //========================================================================
    int MPO_Group::getId() const
    {
        // returns the unique type identity set by enum MPO_TYPE_ID
        return MPO_GROUP_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Group::debugout(std::ostream & output) const
    {
    	if (my_name == "")
    	{
			if (my_group == MPI_GROUP_NULL)
			{
				output << "MPI_GROUP_NULL";
			}
			else if (my_group == MPI_GROUP_EMPTY)
			{
				output << "MPI_GROUP_EMPTY";
			}
			else
			{
				output << "self-defined group";
			}
    	}
    	else
    	{
    		output << my_name;
    	}

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Group::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifndef MPI_GROUP_NON_BASIC
        out < my_group;
#else
        if (my_group == MPI_GROUP_NULL)
        	out < (int) MPO_GROUP_NULL;
        else if (my_group == MPI_GROUP_EMPTY)
   	    	out < (int) MPO_GROUP_EMPTY;
        else
        	out < (int)	MPO_LAST_GROUP_TYPE;
#endif

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Group::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifndef MPI_GROUP_NON_BASIC
        in > my_group;
#else
        int x;
        in > x;
        MPO_GROUP_TYPE t = (MPO_GROUP_TYPE) x;

        if (t == MPO_GROUP_NULL)
        	my_name = "MPI_GROUP_NULL";
        else if (t == MPO_GROUP_EMPTY)
        	my_name = "MPI_GROUP_EMPTY";
        else
        	my_name = "self-defined group";

        my_group = MPI_GROUP_NULL;
#endif

        return in;
    }

}                               // end of namespace MPO
