/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Group_intersection.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Group_intersection.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Group_intersection::MPO_Group_intersection()
        : MPO_Groupconstructor(MPI_GROUP_NULL,
                               MPI_GROUP_NULL,
                               0,
                               -1)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Group_intersection::MPO_Group_intersection(MPO_Group group,
                                                   MPO_Group group2,
                                                   MPI_Group *
                                                   newgroup)
        : MPO_Groupconstructor(group,
                               group2,
                               newgroup,
                               2)
    {
    }


    // destructor
    //========================================================================
    MPO_Group_intersection::~MPO_Group_intersection()
    {
    }


    // getId
    //========================================================================
    int MPO_Group_intersection::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_GROUP_INTERSECTION_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Group_intersection::checkPost(void)
    {

        insertGroup();

        our_group_traces->addAndSetCreation(*my_newgroup,getSourceLocation());
        our_group_traces->findAndTryAddDependendency(*my_newgroup,my_group);
        our_group_traces->findAndTryAddDependendency(*my_newgroup,my_group2);

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Group_intersection::checkPrerequisites(void)
    {


        MPO_Groupconstructor::checkPrerequisites();

        our_group_traces->findAndAddUsage(my_group,getSourceLocation());
        our_group_traces->findAndAddUsage(my_group2,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Group_intersection::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Group_intersection(my_group, my_group2, my_newgroup);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Group_intersection::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Group_intersection";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_GROUP_INTERSECTION";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Group_intersection::debugout(std::ostream
                                                    & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(group1 = "
            << my_group
            << ", group2 = "
            << my_group2
            << ", *newgroup";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Group_intersection::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_group;
        out < my_group2;
        // out < my_newgroup;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Group_intersection::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_group;
        in > my_group2;
        // in > my_newgroup;

        return in;
    }



}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from
// mpich-1.2.4/src/context/group_inter.c.
/*@

MPI_Group_intersection - Produces a group as the intersection of two
existing groups

Input Parameters:
+ group1 - first group (handle)
- group2 - second group (handle)

Output Parameter:
. newgroup - intersection group (handle)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_GROUP
.N MPI_ERR_EXHAUSTED

.seealso: MPI_Group_free

int MPI_Group_intersection (
MPI_Group group1,
MPI_Group group2,
MPI_Group *group_out )
@ */

