/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Group_rank.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Group_rank.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Group_rank::MPO_Group_rank()
        : MPO_Grouphandler(MPI_GROUP_NULL,
                           MPI_GROUP_NULL,
                           -1),
          my_rank(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Group_rank::MPO_Group_rank(MPO_Group group,
                                   int *rank)
        : MPO_Grouphandler(group,
                           MPI_GROUP_NULL,
                           1),
          my_rank(rank)
    {
    }


    // destructor
    //========================================================================
    MPO_Group_rank::~MPO_Group_rank()
    {
    }


    // getId
    //========================================================================
    int MPO_Group_rank::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_GROUP_RANK_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Group_rank::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Group_rank::checkPrerequisites(void)
    {


        MPO_Grouphandler::checkPrerequisites();

        our_group_traces->findAndAddUsage(my_group,getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Group_rank::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Group_rank(my_group, my_rank);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Group_rank::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Group_rank";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_GROUP_RANK";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Group_rank::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(group = "
            << my_group
            << ", *rank";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Group_rank::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_group;
        // out < my_rank;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Group_rank::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_group;
        // in > my_rank;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from
// mpich-1.2.4/src/context/group_rank.c.
/*@

MPI_Group_rank - Returns the rank of this process in the given group

Input Parameters:
. group - group (handle)

Output Parameter:
. rank - rank of the calling process in group, or   'MPI_UNDEFINED'  if the
process is not a member (integer)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_GROUP
.N MPI_ERR_ARG

int MPI_Group_rank (
MPI_Group group,
int *rank )
@ */

