/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Keyval_free.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Keyval_free.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Keyval_free::MPO_Keyval_free()
        : my_keyval(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Keyval_free::MPO_Keyval_free(int *keyval)
        : my_keyval(keyval)
    {
    }


    // destructor
    //========================================================================
    MPO_Keyval_free::~MPO_Keyval_free()
    {
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Keyval_free::checkPrerequisites()
    {

        

        // currently no checks implemented    

        return 0;
    }


    // getId
    //========================================================================
    int MPO_Keyval_free::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_KEYVAL_FREE_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Keyval_free::checkPost(void)
    {
    
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Keyval_free::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Keyval_free(my_keyval);
           #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Keyval_free::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Keyval_free";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_KEYVAL_FREE";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Keyval_free::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname() 
            << "(*keyval";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Keyval_free::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        // out < my_keyval;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Keyval_free::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        // in > my_keyval;

        return in;
    }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from
// mpich-1.2.4/src/context/keyval_free.c.
/*@

MPI_Keyval_free - Frees attribute key for communicator cache attribute

Input Parameter:
. keyval - Frees the integer key value (integer) 

Note:
Key values are global (they can be used with any and all communicators)

.N fortran

.N Errors
.N MPI_SUCCESS
.N MPI_ERR_ARG
.N MPI_ERR_PERM_KEY

.seealso: MPI_Keyval_create

int MPI_Keyval_free ( int *keyval )
@ */

