/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 * @file
 * This Sourcefile implements the MPO_LogToServerDDT class.
 * This object recieves MPO_MsgObject objects and sends them to the DebugServer where they
 * should be logged. Additionaly it logs to std::cerr and uses "mpo-breakpoints.cc"
 * @author Tobias Hilbrich
 * @date 25.09.2007
 * $Id$
 */

#include "mpo_LogToServerDDT.h"

namespace MPO 
{
    /**
     * Constructor. By Tobias, Date 05.11.2005
     */
    MPO_LogToServerDDT::MPO_LogToServerDDT ()
     : MPO_LogToServer()
    {
        // nothing to be done here !
        init();
    }
    
    /**
     * Init call. By Tobias, Date 05.11.2005
     */
    bool MPO_LogToServerDDT::init ()
    {
        // nothing to be done here !
        return true;
    }
   
    /**
     * Standard destructor. By Tobias, Date 05.11.2005
     */
    MPO_LogToServerDDT::~MPO_LogToServerDDT ()
    {
        // nothing to be done here !
    }
   
    /**
     * Gets a message object. By Tobias, Date 05.11.2005
     */
    bool MPO_LogToServerDDT::getMsg (MPO_MsgObject &pMsg)
    {
        //log to std::cerr
        pMsg.debugout(std::cerr);
        std::cerr << std::endl;
        std::cerr << std::endl;
        
        //use breakpoints
        if (MSG_TYPE_ERROR == pMsg.getMsgType() )
        {
            DebugServer::insertBreakpointMarmotError("MarmotError");
        }
        else if (MSG_TYPE_WARNING == pMsg.getMsgType() )
        {
            DebugServer::insertBreakpointMarmotWarning("MarmotWarning");
        }
        else if (MSG_TYPE_NOTE == pMsg.getMsgType() )
        {
            DebugServer::insertBreakpointMarmotNote("MarmotNote");
        }
        else
        {
            // do nothing
        }
        
        //log as LogToServer would do
        MPO_LogToServer::getMsg(pMsg);
      
        return (true);
    }
}//end of namespace MPO







