/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Op representing MPI-operators.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Op.cc 1078 2010-02-15 12:32:44Z tobias $
 */



#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "config.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Op::MPO_Op()
        : my_op(MPI_OP_NULL), my_name("")
    {
    }


    // proper constructor
    //========================================================================
    MPO_Op::MPO_Op(MPI_Op op)
		: my_name("")
    {
        my_op = op;
    }


    // destructor
    //========================================================================
    MPO_Op::~MPO_Op()
    {
    }


    // MPI_Op
    //========================================================================
    MPO_Op::operator MPI_Op()const
    {
        return my_op;
    }


    // getId
    //========================================================================
    int MPO_Op::getId() const
    {
        // returns the unique call identity set by enum MPO_TYPE_ID
        return MPO_OP_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Op::debugout(std::ostream & output) const
    {
    	if (my_name == "")
    	{
			if (our_op_names.find(my_op) != our_op_names.end())
			{
				output << our_op_names.find(my_op)->second;
			}
			else
			{
				output << "non-predefined operator";
			}
    	}
    	else
    	{
    		output << my_name;
    	}

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Op::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifndef MPI_OP_NON_BASIC
        out < my_op;
#else
        std::map < MPO_Op, std::string >::iterator
			j,
			i = our_op_names.find(my_op);
        if (i != our_op_names.end())
        {
        	int index = 0;
        	for (j = our_op_names.begin(); j != i; j++, index++);

        	out < index;
        }
        else
        {
        	out < (int)our_op_names.size();
        }
#endif

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Op::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifndef MPI_OP_NON_BASIC
        in > my_op;
#else
        int x;
        in > x;

        if (x == our_op_names.size())
        {
        	my_name = "non-predefined operator";
        }
        else
        {
            std::map < MPO_Op, std::string >::iterator i = our_op_names.begin();
        	while (x > 0) {x--; i++;};
        	my_name = i->second;
        }

        my_op = MPI_OP_NULL;
#endif

        return in;
    }

}                               // end of namespace MPO
