/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Request representing MPI-operators.
 *
 *  @author Tobias Hilbrich
 *
 *  $Id$
 */



#include "mpo.h"
#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "config.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Request::MPO_Request()
		: my_name ("")
    {
#ifdef MARMOT_REQUESTS_AS_VALUES
        my_request = MPI_REQUEST_NULL;
#else
        my_request = NULL;
#endif
    }


    // proper constructor
    //========================================================================
    MPO_Request::MPO_Request(MPI_Request *request)
		: my_name ("")
    {
#ifdef MARMOT_REQUESTS_AS_VALUES
        //@todo this might be dangerous !!!
        // enhancement with "try" "catch" ???
        if (request != NULL)
            my_request = *request;
        else my_request= MPI_REQUEST_NULL;
#else
        my_request = request;
#endif
    }


    // destructor
    //========================================================================
    MPO_Request::~MPO_Request()
    {
        //nothing to be done here
    }


    // ==
    //========================================================================
    bool MPO_Request::operator== (MPO_Request other) const
    {
        return (my_request == other);
    }

    // <
    //========================================================================
    bool MPO_Request::operator < (MPO_Request other) const
    {
        return (my_request < other);
    }

    // ()
    //========================================================================
#ifdef MARMOT_REQUESTS_AS_VALUES
    MPO_Request::operator MPI_Request () const
#else
    MPO_Request::operator MPI_Request* () const
#endif
    {
        return my_request;
    }

    // getId
    //========================================================================
    int MPO_Request::getId() const
    {
        // returns the unique call identity set by enum MPO_TYPE_ID
        return MPO_REQUEST_ID;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Request::debugout(std::ostream & output) const
    {
    	if(my_name == "")
    	{
#ifdef MARMOT_REQUESTS_AS_VALUES
			if (my_request == MPI_REQUEST_NULL)
			{
				output << "MPI_REQUEST_NULL";
			}
			else
			{
				output << "non-predefined request";
#ifndef MPI_REQUEST_NON_BASIC
				output << " with handle value " << my_request;
#endif
			}
#else
			/*
			 * I am not sure whether it is a good idea or not to access the memory
			 * of the Request pointer here ....
			 */
			if (my_request == NULL)
				return output;

			if (*my_request == MPI_REQUEST_NULL)
			{
				output << "MPI_REQUEST_NULL";
			}
			else
			{
				output << "non-predefined request";
#ifndef MPI_REQUEST_NON_BASIC
				output << " with handle value " << *my_request;
#endif
			}
#endif
    	}
    	else
    	{
    		output << my_name;
    	}
        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Request::save(std::ostream & out) const
    {
        // we don't save and restore pointers
#ifdef MARMOT_REQUESTS_AS_VALUES
#ifndef MPI_REQUEST_NON_BASIC
        out < my_request;
#else /*MPI_REQUEST_NON_BASIC*/
        if (my_request == MPI_REQUEST_NULL)
        	out < (int)MPO_REQUEST_NULL;
        else
        	out < (int)MPO_LAST_REQUEST_TYPE;
#endif /*MPI_REQUEST_NON_BASIC*/
#endif

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Request::restore(std::istream & in)
    {
        // we don't save and restore pointers
#ifdef MARMOT_REQUESTS_AS_VALUES
#ifndef MPI_REQUEST_NON_BASIC
        in > my_request;
#else /*MPI_REQUEST_NON_BASIC*/
        int x;
        in > x;
        MPO_REQUEST_TYPE t = (MPO_REQUEST_TYPE) x;

        if (t == MPO_REQUEST_NULL)
        	my_name = "MPI_REQUEST_NULL";
        else
        	my_name = "non-predefined request";

        my_request = MPI_REQUEST_NULL;
#endif /*MPI_REQUEST_NON_BASIC*/
#else
        my_request = NULL;
#endif
        return in;
    }

}                               // end of namespace MPO
