/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Start.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Start.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

   // default constructor
   //========================================================================
   MPO_Start::MPO_Start()
       : MPO_Starts(0,
                    0)
   {
   }


   /** Constructor taking proper arguments.
    *
    *  The  constructor performs the following tasks:
    *  1. Check whether request is in our_requests
    *  - no: print warning to stderr
    *  - yes:
    *
    *  We store the pointer and not the request itself.
    *  Because the request may change during the message transfer
    *  (it is a opaque type and thus  may contain some internal status etc.)
    */
   // proper constructor
   //========================================================================
   MPO_Start::MPO_Start(MPI_Request * request)
       : MPO_Starts(1,
                    request)
   {
   }


   // destructor
   //========================================================================
   MPO_Start::~MPO_Start()
   {
   }


   // getId
   //========================================================================
   int MPO_Start::getId() const
   {
       // returns the unique call identity set by enum MPO_CALL_ID
       return MPO_START_ID;
   }


   // checkPost
    //========================================================================
    int MPO_Start::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Start::checkPrerequisites(void)
    {


        MPO_Starts::checkPrerequisites();

        our_request_traces->findAndAddUsage(my_request_array,getSourceLocation());


        return 0;
    }


    // callNativeMPI
   //========================================================================
   int MPO_Start::callNativeMPI()
   {
        int rvalue = 1;

       #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
       rvalue = PMPI_Start(my_request_array);
       #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


   // getCallname
   //========================================================================
   string MPO_Start::getCallname() const
   {
       /*  Distinguish between C and Fortran interface, default: C interface.
        */

       string rvalue = "MPI_Start";

       if (F_INTERFACE == INTERFACE_MODE)
       {
           rvalue = "MPI_START";
       }

       return rvalue;
   }


   // debugout
   //========================================================================
   std::ostream & MPO_Start::debugout(std::ostream & output) const
   {
       /*  Distinguish between C and Fortran interface, default: C interface.
        */

       output
           << getCallname()
           << "(*request";

       if (F_INTERFACE == INTERFACE_MODE)
       {
           output << ", ierror";
       }

       output
           << ")"
           << std::endl;

       return output;
   }


   // save
   //========================================================================
   std::ostream & MPO_Start::save(std::ostream & out) const
   {
       // we don't save and restore pointers
       MPO_Callobject::save(out);
       // out < my_request_p;

       return out;
   }


   // restore
   //========================================================================
   std::istream & MPO_Start::restore(std::istream & in)
   {
       // we don't save and restore pointers
       MPO_Callobject::restore(in);
       // in > my_request_p;

       return in;
   }


}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/pt2pt/start.c.
/*@
  MPI_Start - Initiates a communication with a persistent request handle

  Input Parameter:
  . request - communication request (handle)

  .N fortran

  .N Errors
  .N MPI_SUCCESS
  .N MPI_ERR_REQUEST

  int MPI_Start(
  MPI_Request *request)
  @ */

