/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Startall.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Startall.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Startall::MPO_Startall()
        : MPO_Starts(-1,
                     0)
    {
    }


    /** Constructor taking proper arguments.
     *
     *  The  constructor performs the following tasks:
     *  1. Check whether requests are in our_requests
     *  - no: print warning to stderr
     *  - yes:
     */
    // proper constructor
    //========================================================================
    MPO_Startall::MPO_Startall(int count,
                               MPI_Request * request_array)
        : MPO_Starts(count,
                     request_array)
    {
    }


    // destructor
    //========================================================================
    MPO_Startall::~MPO_Startall()
    {
    }


    // getIde
    //========================================================================
    int MPO_Startall::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_STARTALL_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Startall::checkPost(void)
    {

        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Startall::checkPrerequisites(void)
    {


        MPO_Starts::checkPrerequisites();

        for (int i = 0; i < my_count;i++)
            our_request_traces->findAndAddUsage(&(my_request_array[i]),getSourceLocation());


        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Startall::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Startall(my_count, my_request_array);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Startall::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Startall";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_STARTALL";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Startall::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output
            << getCallname()
            << "(count = "
            << my_count
            << ", *array_of_requests";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Startall::save(std::ostream & out) const
    {
        MPO_Callobject::save(out);
        out < my_count;
        // we don't save and restore pointers
        // out < my_request_array;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Startall::restore(std::istream & in)
    {
        MPO_Callobject::restore(in);
        in > my_count;
        // we don't save and restore pointers
        // in > my_request_array;

        return in;
    }


}                               // end of namespace MPO


// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/pt2pt/startall.c.
/*@
  MPI_Startall - Starts a collection of requests

  Input Parameters:
  + count - list length (integer)
  - array_of_requests - array of requests (array of handle)

  .N fortran

  int MPI_Startall(
  int count,
  MPI_Request array_of_requests[] )
  @ */


