/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
  * \file mpo_ThreadSynchronisationDummy.cc
  * This Sourcefile implements the MPO_ThreadSynchronisationDummy class.
  * @author Tobias Hilbrich
  * @date 06.06.2007
  * $Id$
  */

#include "mpo.h"
#include "mpo_SourceLocation.h"
#include "mpo_ThreadSynchronisationDummy.h"
#include "config.h"

#include "marmot-interface.h"

extern int MPO_in_profile;

namespace MPO
{

    // enterMARMOT
    //========================================================================
    bool MPO_ThreadSynchronisationDummy::enterMARMOT (int line,const char * file)
    {
        //we cant do anything here
        //when using MPO_ThreadSynchronisationDummy we assume that there are no threads

        //// we unset MPO_in_profile here - it is an artifact and we schould discuss
        //// the remainders of the marmot interface
        ///** \todo think about the marmot interface - changes necessary here*/
        //MPO_in_profile = 0;

        return true;
    }

    // leaveMARMOT
    //========================================================================
    void MPO_ThreadSynchronisationDummy::leaveMARMOT (int line,const char * file,bool enter_succeeded)
    {
        //we cant do anything here
        //when using MPO_ThreadSynchronisationDummy we assume that there are no threads

        // we unset MPO_in_profile here - it is an artifact and we schould discuss
        // the remainders of the marmot interface
        /** \todo think about the marmot interface - changes necessary here*/
        //MPO_in_profile = 0;
    }

    // enterPMPI
    //========================================================================
    void MPO_ThreadSynchronisationDummy::enterPMPI ()
    {
        //we cant do anything here
        //when using MPO_ThreadSynchronisationDummy we assume that there are no threads
    }

    // leavePMPI
    //========================================================================
    void MPO_ThreadSynchronisationDummy::leavePMPI ()
    {
        //we cant do anything here
        //when using MPO_ThreadSynchronisationDummy we assume that there are no threads
    }

    // isInParallelRegion
    //========================================================================
    bool MPO_ThreadSynchronisationDummy::isInParallelRegion (void)
    {
        //we will return NO (false)
        //when using MPO_ThreadSynchronisationDummy we assume that there are no threads
        return false;
    }

    // getThreadRank
    //========================================================================
    int MPO_ThreadSynchronisationDummy::getThreadRank (void)
    {
        //we will 0 (main thread)
        //when using MPO_ThreadSynchronisationDummy we assume that there are no threads
        return 0;
    }

    // wrappingOn
    //========================================================================
    void MPO_ThreadSynchronisationDummy::wrappingOn (void)
    {
        MPO_in_profile = 0;
    }

    // wrappingOff
    //========================================================================
    void MPO_ThreadSynchronisationDummy::wrappingOff (void)
    {
        MPO_in_profile = 1;
    }

    // didEnterSucceed
    //========================================================================
    bool MPO_ThreadSynchronisationDummy::didEnterSucceed (void)
    {
        if (MPO_in_profile == 1)
            return true;
        return false;
    }

    // getImplementationName
    //========================================================================
    std::string MPO_ThreadSynchronisationDummy::getImplementationName (void)
    {
        return "No Threadsynchronisation";
    }

    // otherThreadsWaiting
    //========================================================================
    bool MPO_ThreadSynchronisationDummy::otherThreadsWaiting (void)
    {
        return false;
    }

    // registerRequest
    //========================================================================
    int MPO_ThreadSynchronisationDummy::registerRequest (MPI_Request *request, int sourceLine,const char *sourceName, int threadNum, int callId, std::string callName)
    {
        //we should not register anything as we couldn`t protect the resulting
        //critical section. Thus we will suppose that there are no multiple
        //threads present.
        return 0;
    }

    // registerRequests
    //========================================================================
    int MPO_ThreadSynchronisationDummy::registerRequests (MPI_Request *requests, int count, int sourceLine,const char *sourceName, int threadNum, int callId, std::string callName)
    {
        //we should not register anything as we couldn`t protect the resulting
        //critical section. Thus we will suppose that there are no multiple
        //threads present.
        return 0;
    }

    // unregisterRequests
    //========================================================================
    void MPO_ThreadSynchronisationDummy::unregisterRequests (int id)
    {
        //we should not register anything as we couldn`t protect the resulting
        //critical section. Thus we will suppose that there are no multiple
        //threads present.
    }

#ifdef MARMOT_TCHECK
    // unregisterRequestToIntEntry
    //========================================================================
    void MPO_ThreadSynchronisationDummy::unregisterRequestToIntEntry(MPI_Request *request)
    {
        //we should not register anything as we couldn`t protect the resulting
        //critical section. Thus we will suppose that there are no multiple
        //threads present.
    }

    // unregisterCommToIntEntry
    //========================================================================
    void MPO_ThreadSynchronisationDummy::unregisterCommToIntEntry(MPI_Comm comm)
    {
        //we should not register anything as we couldn`t protect the resulting
        //critical section. Thus we will suppose that there are no multiple
        //threads present.
    }
#endif

    // registerCollComm
    //========================================================================
    int MPO_ThreadSynchronisationDummy::registerCollComm (MPI_Comm comm, int sourceLine,const char *sourceName, int threadNum, int callId, std::string callName)
    {
        //we should not register anything as we couldn`t protect the resulting
        //critical section. Thus we will suppose that there are no multiple
        //threads present.
        return 0;
    }

    // unregisterCollComm
    //========================================================================
    void MPO_ThreadSynchronisationDummy::unregisterCollComm (int id)
    {
        //we should not register anything as we couldn`t protect the resulting
        //critical section. Thus we will suppose that there are no multiple
        //threads present.
    }

    // getMinThreadLvl
    //========================================================================
    int MPO_ThreadSynchronisationDummy::getMinThreadLvl (void)
    {
        return MPI_THREAD_SINGLE;
    }

    // getViolationLocationToThreadLvl
    //========================================================================
    MPO_ThreadViolationInfo MPO_ThreadSynchronisationDummy::getViolationInfoForThreadLvl (int threadLevel)
    {
        return MPO_ThreadViolationInfo();
    }

}//end of namespace MPO







