/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
  * \file mpo_ThreadSynchronisationOpenMP.cc
  * This Sourcefile implements the MPO_ThreadSynchronisationOpenMP class.
  * @author Tobias Hilbrich
  * @date 01.06.2007
  * $Id$
  */

#include "mpo_SourceLocation.h"
#include "MPO_ThreadSynchronisationSyncBase.h"
#include "mpo_ThreadSynchronisationOpenMP.h"

#ifdef MARMOT_OPENMP

#include "marmot-interface.h"
//extern int MPO_in_profile;

namespace MPO
{

    // constructor
    //========================================================================
    MPO_ThreadSynchronisationOpenMP::MPO_ThreadSynchronisationOpenMP (void)
    {
        omp_init_lock(&my_lock);
        omp_init_lock(&my_criticalLock);
        my_NumInMarmot = 0;
    }

    // destructor
    //========================================================================
    MPO_ThreadSynchronisationOpenMP::~MPO_ThreadSynchronisationOpenMP (void)
    {
        omp_destroy_lock(&my_lock);
        omp_destroy_lock(&my_criticalLock);
    }

    // enterMARMOT
    //========================================================================
    void MPO_ThreadSynchronisationOpenMP::enterMARMOTSync (int line, const char *file)
    {
        beginCritical();
        my_infoOfThreadInMARMOT.first = getThreadRank();
        my_infoOfThreadInMARMOT.second = MPO_SourceLocation(file,line);
        my_NumInMarmot++;
        endCritical();

        omp_set_lock(&my_lock);

        //// we unset MPO_in_profile here - it is an artifact and we schould discuss
        //// the remainders of the marmot interface
        ///** \todo think about the marmot interface - changes necessary here*/
        //MPO_in_profile = 0;
    }

    // leaveMARMOT
    //========================================================================
    void MPO_ThreadSynchronisationOpenMP::leaveMARMOTSync (int line, const char *file)
    {
        //// we unset MPO_in_profile here - it is an artifact and we schould discuss
        //// the remainders of the marmot interface
        ///** \todo think about the marmot interface - changes necessary here*/
        //MPO_in_profile = 0;

        //we have to synchronize here with other threads that might be not in MARMOT
        beginCritical();
        my_NumInMarmot--;
        endCritical();

        omp_unset_lock(&my_lock);
    }

    // enterPMPI
    //========================================================================
    void MPO_ThreadSynchronisationOpenMP::enterPMPI ()
    {
        omp_unset_lock(&my_lock);
    }

    // leavePMPI
    //========================================================================
    void MPO_ThreadSynchronisationOpenMP::leavePMPI ()
    {
        omp_set_lock(&my_lock);
    }

    // isInParallelRegion
    //========================================================================
    bool MPO_ThreadSynchronisationOpenMP::isInParallelRegion (void)
    {
        if (omp_in_parallel() == 0)
            return false;
        return true;
    }

    // getThreadRank
    //========================================================================
    int MPO_ThreadSynchronisationOpenMP::getThreadRank (void)
    {
        return omp_get_thread_num();
    }

    // getImplementationName
    //========================================================================
    std::string MPO_ThreadSynchronisationOpenMP::getImplementationName (void)
    {
        return "OpenMP Threadsynchronisation";
    }

    // otherThreadsWaiting
    //========================================================================
    bool MPO_ThreadSynchronisationOpenMP::otherThreadsWaiting (void)
    {
        return (my_NumInMarmot > 1);
    }

    // beginCritical
    //========================================================================
    void MPO_ThreadSynchronisationOpenMP::beginCritical ()
    {
        omp_set_lock(&my_criticalLock);
    }

    // endCritical
    //========================================================================
    void MPO_ThreadSynchronisationOpenMP::endCritical ()
    {
        omp_unset_lock(&my_criticalLock);
    }

    // otherThreadsInMARMOT
    //========================================================================
    bool MPO_ThreadSynchronisationOpenMP::otherThreadsInMARMOT (void)
    {
        return (my_NumInMarmot > 0);
    }

    // getInfoForThreadInMARMOT
    //========================================================================
    std::pair<int,MPO_SourceLocation> MPO_ThreadSynchronisationOpenMP::getInfoForThreadInMARMOT (void)
    {
        //beginCritical();
        std::pair<int,MPO_SourceLocation> ret = my_infoOfThreadInMARMOT;
        //endCritical();

        return ret;
    }

    // getNumThreads
    //========================================================================
    int MPO_ThreadSynchronisationOpenMP::getNumThreads(void)
    {
        return omp_get_num_threads();
    }

}//end of namespace MPO

#endif //#ifdef MARMOT_OPENMP




