/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**
 *  @file
 *
 *  Definition of class MPO_Type_lb.
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller, Tobias Hilbrich
 *
 *  $Id: mpo_Type_lb.cc 909 2008-12-17 09:48:19Z tobias $
 */



#include "mpo-objects.h"
#include "mpo-binaryIO.h"
#include "mpo-debugserver.h"
#include "mpo_MsgObject.h"
#include "MPO_LogFile.h"
#include "MPO_ResourceGatherer.h"


namespace MPO
{

    // default constructor
    //========================================================================
    MPO_Type_lb::MPO_Type_lb()
        : MPO_Typehandlers(MPI_DATATYPE_NULL),
          my_displacement(0)
    {
    }


    // proper constructor
    //========================================================================
    MPO_Type_lb::MPO_Type_lb(MPO_Datatype datatype,
                             MPI_Aint *
                             displacement)
        : MPO_Typehandlers(datatype),
          my_displacement(displacement)
    {
    }


    // destructor
    //========================================================================
    MPO_Type_lb::~MPO_Type_lb()
    {
    }


    // getId
    //========================================================================
    int MPO_Type_lb::getId() const
    {
        // returns the unique call identity set by enum MPO_CALL_ID
        return MPO_TYPE_LB_ID;
    }


    // checkPost
    //========================================================================
    int MPO_Type_lb::checkPost(void)
    {
            
        return 0;
    }


    // checkPrerequisites
    //========================================================================
    int MPO_Type_lb::checkPrerequisites(void)
    {

        
        our_datatype_traces->findAndAddUsage(my_datatype,getSourceLocation());

        
        return 0;
    }


    // callNativeMPI
    //========================================================================
    int MPO_Type_lb::callNativeMPI()
    {
        int rvalue = 1;

        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->enterPMPI();
        #endif
        rvalue = PMPI_Type_lb(my_datatype, my_displacement);
        #ifdef MARMOT_USE_THREADSYNC
               MPO_Space::getSyncObject()->leavePMPI();
        #endif

        return rvalue;
    }


    // getCallname
    //========================================================================
    string MPO_Type_lb::getCallname() const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        string rvalue = "MPI_Type_lb";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            rvalue = "MPI_TYPE_LB";
        }

        return rvalue;
    }


    // debugout
    //========================================================================
    std::ostream & MPO_Type_lb::debugout(std::ostream & output) const
    {
        /*  Distinguish between C and Fortran interface, default: C interface.
         */

        output 
            << getCallname()
            << "(datatype = " 
            << my_datatype 
            << ", *displacement";

        if (F_INTERFACE == INTERFACE_MODE)
        {
            output << ", ierror";
        }

        output
            << ")"  
            << std::endl;

        return output;
    }


    // save
    //========================================================================
    std::ostream & MPO_Type_lb::save(std::ostream & out) const
    {
        // we don't save and restore pointers
        MPO_Callobject::save(out);
        out < my_datatype;
        // out < my_displacement;

        return out;
    }


    // restore
    //========================================================================
    std::istream & MPO_Type_lb::restore(std::istream & in)
    {
        // we don't save and restore pointers
        MPO_Callobject::restore(in);
        in > my_datatype;
        // in > my_displacement;

        return in;
    }

}                               // end of namespace MPO



// mpich implementation
//========================================================================
// The following comment is copied from mpich-1.2.4/src/pt2pt/type_lb.c.
/*@
  MPI_Type_lb - Returns the lower-bound of a datatype

  Input Parameters:
  . datatype - datatype (handle) 

  Output Parameter:
  . displacement - displacement of lower bound from origin, 
  in bytes (integer) 

  .N fortran

  .N Errors
  .N MPI_SUCCESS
  .N MPI_ERR_TYPE
  .N MPI_ERR_ARG

  int MPI_Type_lb ( 
  MPI_Datatype datatype, 
  MPI_Aint *displacement )
  @ */

