#include "StdAfx.h"
#include "DICE_AddinComm.h"
#include "DICE_Interface.h"
#include "GUIUpdaterWin.h"
#include <fstream>

/**
 * TODO: 
 *  these two sets of definitions should be loaded from a Marmot
 *  header that is shared between Addin/DICE and Marmot core.
 *  Using mpo-enums.h won't work as that one includes mpi.h !
 */
//these defines are used to link to the parts of the message
#define MESSAGE_ID_TIMESTAMP 0
#define MESSAGE_ID_RANK 1
#define MESSAGE_ID_THREADRANK 2
#define MESSAGE_ID_MSGTYPE 3
#define MESSAGE_ID_MSGTEXT 4
#define MESSAGE_ID_LOCATIONFILE 5
#define MESSAGE_ID_LOCATIONLINE 6
#define MESSAGE_ID_CALLNAME 7
#define MESSAGE_ID_SHORTDESC 8
#define MESSAGE_ID_MPIREFERENCE 9

 enum MPO_MSG_TYPE
  {
    MSG_TYPE_INFO = 0,
    MSG_TYPE_WARNING,
    MSG_TYPE_ERROR,
    MSG_TYPE_NOTE
  };

namespace DICE_GUI
{
	CDICE_AddinComm* CDICE_AddinComm::m_theInstance = NULL;

	CDICE_AddinComm::CDICE_AddinComm(void)
	{
		m_isInitialDataSet = false;
	}

	CDICE_AddinComm::~CDICE_AddinComm(void)
	{
	}

	CDICE_AddinComm* CDICE_AddinComm::GetInstance(void)
	{
		if (m_theInstance == NULL)
			m_theInstance = new CDICE_AddinComm ();

		return m_theInstance;
	}

	void CDICE_AddinComm::popNewMarmotMsg (std::vector<std::string> messageparts)
	{
		//get the implementation of IDICE_QueryDice
		DICE_INTERFACE::IDICE_QueryDice* query = CDICE_GUIUpdaterWin::GetInstance()->getQueryDICE();

		//Set initial DICE entries if necessary !
		if (!m_isInitialDataSet)
			setInitialData();

		//calculate associated msg type
		DICE_INTERFACE::EMESSAGE_TYPE dice_type;
		std::string messages_parent_desc = "Messages";
		std::string messages_leaf_desc = messageparts[MESSAGE_ID_SHORTDESC];

		int msg_type = 0;
        if (!messageparts[MESSAGE_ID_MSGTYPE].empty())
          sscanf(messageparts[MESSAGE_ID_MSGTYPE].c_str(), "%d", &msg_type);
		switch (msg_type)
		{
			case MSG_TYPE_WARNING:
				dice_type = DICE_INTERFACE::MESSAGE_TYPE_WARNING;
				messages_parent_desc = "Warnings";
				break;
			case MSG_TYPE_ERROR:
				dice_type = DICE_INTERFACE::MESSAGE_TYPE_ERROR;
				messages_parent_desc = "Errors";
				break;
			case MSG_TYPE_NOTE:
				dice_type = DICE_INTERFACE::MESSAGE_TYPE_NOTE;
				messages_parent_desc = "Notes";
				break;
			case MSG_TYPE_INFO:
				dice_type = DICE_INTERFACE::MESSAGE_TYPE_REMARK;
				messages_parent_desc = "Remarks";
				break;
		}

		//calculate numbers
		int rank = 0;
        if (!messageparts[MESSAGE_ID_RANK].empty())
          sscanf(messageparts[MESSAGE_ID_RANK].c_str(), "%d", &rank);

		int threadID = 0;
        if (!messageparts[MESSAGE_ID_THREADRANK].empty())
          sscanf(messageparts[MESSAGE_ID_THREADRANK].c_str(), "%d", &threadID);

		int linenumber = 0;
        if (!messageparts[MESSAGE_ID_LOCATIONLINE].empty())
          sscanf(messageparts[MESSAGE_ID_LOCATIONLINE].c_str(), "%d", &linenumber);
		
		int timestamp = 0;
        if (!messageparts[MESSAGE_ID_TIMESTAMP].empty())
          sscanf(messageparts[MESSAGE_ID_TIMESTAMP].c_str(), "%d", &timestamp);

		//calculate message type dependent stuff
		std::string location_parent_desc = "Locations";
		std::string location_leaf_desc = "Locations";
		if (!messageparts[MESSAGE_ID_LOCATIONFILE].empty())
		{
			location_parent_desc = messageparts[MESSAGE_ID_LOCATIONFILE];
			location_leaf_desc = messageparts[MESSAGE_ID_CALLNAME]; //if it has source location it should also have callname
			location_leaf_desc.append("@line ");
			location_leaf_desc.append(messageparts[MESSAGE_ID_LOCATIONLINE]);
			
			//add the source file (nothing happens if it was already created)
			query->AddLocationsItem("Source Files",location_parent_desc,false);
		}
		else
		{
			location_parent_desc = "Locations";
			location_leaf_desc = "Global";
		}

		std::string system_parent_desc = "System";
		std::string system_leaf_desc = "Global";
		
		if (rank >= 0)
		{
			system_parent_desc = "Rank ";
			system_parent_desc.append (messageparts[MESSAGE_ID_RANK]);
			system_leaf_desc = "Thread ";
			system_leaf_desc.append (messageparts[MESSAGE_ID_THREADRANK]);

			//add the source file (nothing happens if it was already created)
			query->AddSystemItem("MPI Processes",system_parent_desc,false);
		}

		//remove \n from short desc and text
		std::string text = messageparts[MESSAGE_ID_MSGTEXT];

		if (!text.empty())
		{
			// replace "<br>" with "\n"
			for (std::string::size_type i=0, pos; (pos = text.find("<br>", i)) != std::string::npos; i=pos + 1)
			{
				text.replace(pos, 4, "\r\n");
			}
		}

		if (!messages_leaf_desc.empty())
		{
			// replace "<br>" with "\n"
			for (std::string::size_type i=0, pos; (pos = messages_leaf_desc.find("<br>", i)) != std::string::npos; i=pos + 1)
			{
				messages_leaf_desc.replace(pos, 4, " ");//replace with a space, can't  handle newlines in the msg list 
			}

			if (messages_leaf_desc.compare(std::string("Unknown")) == 0)
			{
				//this message is a pure text message
				messages_leaf_desc = "Text ";
				messages_leaf_desc.append(messages_parent_desc);
				messages_parent_desc = "Text Messages";
			}
		}

		//calculate the mpi reference path
		//TODO: currently only abosulute path used .... (no portability of log files)
		std::string mpi_reference = messageparts[MESSAGE_ID_MPIREFERENCE];

		//create the associated msg object
		DICE_INTERFACE::CDICE_Message msg (
			timestamp,
			rank,
			threadID,
			messageparts[MESSAGE_ID_CALLNAME],
			messages_leaf_desc,
			text,
			messageparts[MESSAGE_ID_LOCATIONFILE],
			linenumber,
			dice_type,
			mpi_reference);

		query->AddEntry(messages_parent_desc,messages_leaf_desc,
						location_parent_desc,location_leaf_desc,
						system_parent_desc,system_leaf_desc,
						1.0,
						msg);
	}

	void CDICE_AddinComm::pushOpenSourceFileLocation (std::string filename, int line)
	{
		m_pToolWinComm->popOpenSourceFileLocation (filename,line);
	}

	void CDICE_AddinComm::pushOpenHTMLFile (std::string filename)
	{
		m_pToolWinComm->popOpenHTMLFile (filename);
	}

	void CDICE_AddinComm::popClear (void)
	{
		m_isInitialDataSet = false;
		SendMessage (m_hWndDiceWindow, WM_COMMAND, MAKELONG(666,777), HandleToLong(m_hWndDiceWindow)); 
	}

	bool CDICE_AddinComm::setInitialData (void)
	{
		//get the implementation of IDICE_QueryDice
		DICE_INTERFACE::IDICE_QueryDice* query = CDICE_GUIUpdaterWin::GetInstance()->getQueryDICE();

		query->AddMessagesItem("","Messages",false);
		query->AddMessagesItem("Messages","Text Messages",false);
		query->AddMessagesItem("Messages","Errors",false);
		query->AddMessagesItem("Messages","Warnings",false);
		query->AddMessagesItem("Messages","Notes",false);
		query->AddMessagesItem("Messages","Remarks",false);
	
		query->AddLocationsItem("","Locations",false);
		query->AddLocationsItem("Locations","Global",true);
		query->AddLocationsItem("Locations","Source Files",false);		

		query->AddSystemItem("","System",false);
		query->AddSystemItem("System","Global",true);
		query->AddSystemItem("System","MPI Processes",false);	

		m_isInitialDataSet = true;

		return true;
	}

	void CDICE_AddinComm::setDiceWindow (CWnd* window, HWND hwndWindow)
	{
		m_diceWindow = window;
		m_hWndDiceWindow = hwndWindow;
	}

}//end of namespace DICE_GUI