#include "StdAfx.h"
#include "DICE_DataNode.h"
#include "DICE_DataLeaf.h"
#include <fstream>

namespace DICE_BACK
{

	CDICE_DataNode::CDICE_DataNode(void)
	{
		m_childs.clear();
	}

	CDICE_DataNode::CDICE_DataNode(std::string desc, long DICE_id)
		: CDICE_DataItem (desc,DICE_id)
	{
		m_childs.clear();
	}

	CDICE_DataNode::~CDICE_DataNode(void)
	{
		std::list<CDICE_DataItem*>::iterator i;

		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			if (*i)
				delete (*i);
		}

		m_childs.clear();
	}

	std::list<CDICE_DataItem*> CDICE_DataNode::getChilds(void)
	{
		return m_childs;
	}

	void CDICE_DataNode::addChild (CDICE_DataItem* child)
	{
		m_childs.push_back(child);
	}

	double CDICE_DataNode::calculateSeverity (void)
	{
		double ret = 0.0;

		std::list<CDICE_DataItem*>::iterator i;

		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			ret += (*i)->calculateSeverity();
		}

		return ret;
	}

	std::list<SDICE_LinkType> CDICE_DataNode::accumulateLinks (void)
	{
		std::list<SDICE_LinkType> ret;
		ret.clear();

		std::list<CDICE_DataItem*>::iterator i;

		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			std::list<SDICE_LinkType> temp = (*i)->accumulateLinks();

			for (std::list<SDICE_LinkType>::iterator j = temp.begin(); j != temp.end();j++)
			{
				//is a similar link already present ?	
				std::list<SDICE_LinkType>::iterator k;
				bool appended = false;

				for (k = ret.begin();k != ret.end();k++)
				{
					if (k->locationsID != j->locationsID)
						continue;

					if (k->systemID != j->systemID)
						continue;

					//equal, append it !
					SDICE_LinkType entry = *k;
					entry.severity += j->severity;
					ret.erase(k);
					ret.push_back(entry);

					appended = true;
					break;
				}

				//not yet present, add it !
				if (!appended)
					ret.push_back (*j);
			}
		}

		return ret;
	}

	CDICE_DataItem* CDICE_DataNode::findDICEid (long id)
	{
		if (m_DICE_id == id)
			return this;

		std::list<CDICE_DataItem*>::iterator i;

		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			CDICE_DataItem* temp = (*i)->findDICEid (id);
			if (temp != NULL)
				return temp;
		}

		return NULL;
	}

	long CDICE_DataNode::findDesc (std::string desc)
	{
		if (m_desc.compare(desc) == 0)
			return m_DICE_id;

		std::list<CDICE_DataItem*>::iterator i;

		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			long temp = (*i)->findDesc (desc);
			if (temp >= 0)
				return temp;
		}

		return -1;
	}

	bool CDICE_DataNode::isLeaf (void)
	{
		return false;
	}

	std::list<long> CDICE_DataNode::accumulateIDs (void)
	{
		std::list<long> ret;
		ret.clear();
		ret.push_back(m_DICE_id);

		std::list<CDICE_DataItem*>::iterator i;

		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			std::list<long> temp;
			temp = (*i)->accumulateIDs ();

			std::list<long>::iterator j;
			for (j = temp.begin();j != temp.end();j++)
			{
				ret.push_back(*j);
			}
		}

		return ret;
	}

	DICE_INTERFACE::CDICE_TreeItem* CDICE_DataNode::copyToInterfaceTree (void)
	{
		DICE_INTERFACE::CDICE_TreeNode* ret;

		double sev = calculateSeverity ();
		ret = new DICE_INTERFACE::CDICE_TreeNode (NULL,m_desc,sev,m_DICE_id);

		//add all the children
		std::list<CDICE_DataItem*>::iterator i;
		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			DICE_INTERFACE::CDICE_TreeItem* temp;
			temp = (*i)->copyToInterfaceTree();
			ret->addChild(temp);
			temp->setFather(ret);
		}

		return ret;
	}

	DICE_INTERFACE::CDICE_TreeItem* CDICE_DataNode::copyToInterfaceTreeWeighted (std::list<std::pair<long,double> > weights)
	{
		DICE_INTERFACE::CDICE_TreeNode* ret;

		double sev = 0.0;
		ret = new DICE_INTERFACE::CDICE_TreeNode (NULL,m_desc,0.0,m_DICE_id);

		//add all the children
		std::list<CDICE_DataItem*>::iterator i;
		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			DICE_INTERFACE::CDICE_TreeItem* temp;
			temp = (*i)->copyToInterfaceTreeWeighted(weights);
			ret->addChild(temp);
			temp->setFather(ret);
		}

		sev = ret->accumulateSeverities();
		ret->setSeverity(sev);

		return ret;
	}

	bool CDICE_DataNode::deleteTree (void)
	{
		bool ret = true;

		std::list<CDICE_DataItem*>::iterator i;
		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			ret = ret && (*i)->deleteTree ();
			
			if (*i)
				free (*i);
			(*i) = NULL;
		}

		m_childs.clear();

		return ret;
	}

	bool CDICE_DataNode::save (std::ofstream &file)
	{
		bool ret = true;

		//type
		file << "1" << std::endl;

		//data
		CDICE_DataItem::save(file);
		file << m_childs.size () << std::endl;
		
		std::list<CDICE_DataItem*>::iterator i;
		for (i = m_childs.begin(); i != m_childs.end(); i++)
		{
			ret = ret && (*i)->save (file);
		}

		return ret;
	}

	bool CDICE_DataNode::load (std::ifstream &file)
	{
		bool ret = true;

		//type already loaded by creator of this object !

		//load data
		int num_childs, i, type;
		CDICE_DataItem *subItem;

		CDICE_DataItem::load(file);

		file >> num_childs;
		
		for (i = 0; i < num_childs; i++)
		{
			file >> type;
			if (type == 0)
				subItem = new CDICE_DataLeaf ();
			else 
				subItem = new CDICE_DataNode ();

			subItem->setFather (this);

			ret = ret && subItem->load (file);
			m_childs.push_back (subItem);
		}		

		return ret;
	}

} // end of namespace DICE_BACK