/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
#pragma once

#include <string>

namespace DICE_INTERFACE
{
	enum EMESSAGE_TYPE
	{
		MESSAGE_TYPE_NOTE = 0,
		MESSAGE_TYPE_REMARK,
		MESSAGE_TYPE_WARNING,
		MESSAGE_TYPE_ERROR
	};

	class CDICE_Message
	{
	protected:
		long m_timestamp;
		int m_sourceRank;
		int m_sourceThread;
		std::string m_text;
		std::string m_shortDesc;
		std::string m_callName;
		std::string m_locationFileName;
		long m_locationLine;
		EMESSAGE_TYPE m_type;
		std::string m_mpiReference;

	public:
		CDICE_Message(void);

		/**
		 * a locationFileName of "" should be handled as NoFileName present
		 * everywhere !
		 */
		CDICE_Message(
			long timestamp,
			int sourceRank,
			int sourceThread,
			std::string callName,
			std::string shortDesc,
			std::string text,
			std::string locationFileName,
			long locationLine,
			EMESSAGE_TYPE type,
			std::string mpiReference);
		~CDICE_Message(void);
		CDICE_Message(const CDICE_Message &msg);

		long getTimestamp (void) const;
		void setTimestamp (long);

		int getSourceRank (void) const;
		void setSourceRank (int);

		int getSourceThread (void) const;
		void setSourceThread (int);

		std::string getText (void) const;
		void setText (std::string);

		std::string getLocationFileName (void) const;
		void setLocationFileName (std::string);

		long getLocationLine (void) const;
		void setLocationLine (long);

		EMESSAGE_TYPE getType (void) const;
		void setType (EMESSAGE_TYPE);

		std::string getShortDesc (void) const;
		void setShortDesc (std::string);
		 
		std::string getCallName (void) const;
		void setCallName (std::string);

		std::string getMpiReference (void) const;
		void setMpiReference (std::string);

		bool operator== (const CDICE_Message&) const;
	};

}//end of namespace DICE_INTERFACE