/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
#pragma once

#include "DICE_Message.h"
#include "GUIUpdater.h"
#include <string>

class IDICE_GUIUpdater;

namespace DICE_INTERFACE
{

	struct SDICE_ReturnIDs
	{
		long m_IDmessages;
		long m_IDlocations;
		long m_IDsystem;
	};

	class IDICE_QueryDice
	{
	protected:
		IDICE_QueryDice(void);

	public:
		~IDICE_QueryDice(void);

		/**
		 * This method has to be called before anything is done with an 
		 * implementation of this interface !
		 */
		virtual void setGUIUpdater (DICE_INTERFACE::IDICE_GUIUpdater* pUpdater) = 0;

		/**
		 * Parents with ID < 0 or name == "" will be handled as NoParentPresent!
		 */
		virtual SDICE_ReturnIDs AddEntry (
			std::string messagesParent,
			std::string messagesDesc,
			std::string locationsParent,
			std::string locationsDesc,
			std::string systemParent,
			std::string systemDesc,
			double severity,
			CDICE_Message msg) = 0;

		virtual SDICE_ReturnIDs AddEntry (
			std::string messagesParent,
			std::string messagesDesc,
			std::string locationsParent,
			std::string locationsDesc,
			std::string systemParent,
			std::string systemDesc,
			double severity) = 0;

		virtual SDICE_ReturnIDs AddEntry (
			long messagesParent,
			std::string messagesDesc,
			long locationsParent,
			std::string locationsDesc,
			long systemParent,
			std::string systemDesc,
			double severity,
			CDICE_Message msg) = 0;

		virtual SDICE_ReturnIDs AddEntry (
			long messagesParent,
			std::string messagesDesc,
			long locationsParent,
			std::string locationsDesc,
			long systemParent,
			std::string systemDesc,
			double severity) = 0;

		virtual SDICE_ReturnIDs AddEntry (
			long messagesParent,
			long messagesID,
			long locationsParent,
			long locationsID,
			long systemParent,
			long systemID,
			double severity,
			CDICE_Message msg) = 0;

		virtual SDICE_ReturnIDs AddEntry (
			long messagesParent,
			long messagesID,
			long locationsParent,
			long locationsID,
			long systemParent,
			long systemID,
			double severity) = 0;

		virtual long AddMessagesItem (std::string parentName, std::string itemName, bool bLeaf) = 0;
		virtual long AddMessagesItem (long parentID, std::string itemName, bool bLeaf) = 0;
		virtual long AddLocationsItem (std::string parentName, std::string itemName, bool bLeaf) = 0;
		virtual long AddLocationsItem (long parentID, std::string itemName, bool bLeaf) = 0;
		virtual long AddSystemItem (std::string parentName, std::string itemName, bool bLeaf) = 0;
		virtual long AddSystemItem (long parentID, std::string itemName, bool bLeaf) = 0;

		virtual bool changeSelection (SDICE_ReturnIDs selection) = 0;
		virtual bool changeMessagesSelection (long id) = 0;
		virtual bool changeLocationsSelection (long id) = 0;
		virtual bool changeSystemSelection (long id) = 0;

		virtual std::list<CDICE_Message> getReports (SDICE_ReturnIDs selection) = 0;

		virtual bool clear (void) = 0;
		virtual bool save (std::ofstream &file) = 0;
		virtual bool load (std::ifstream &file) = 0;

		virtual bool isEmpty (void) = 0;
	};

}//end of namespace DICE_INTERFACE