#include "StdAfx.h"
#include "DICE_QueryDiceImpl.h"
#include "DICE_MessagesData.h"
#include "DICE_LocationsData.h"
#include "DICE_SystemData.h"
#include "DICE_ReportCollection.h"

namespace DICE_BACK
{
	CDICE_QueryDiceImpl* CDICE_QueryDiceImpl::m_theInstance = NULL;

	CDICE_QueryDiceImpl::CDICE_QueryDiceImpl(void)
	{
		m_pUpdater = NULL;
		m_curSelection.m_IDlocations = 0;
		m_curSelection.m_IDmessages = 0;
		m_curSelection.m_IDsystem = 0;
		m_avoidGUIUpdate = 0;
		m_systemDirty = false;
		m_messagesDirty = false;
		m_locationsDirty = false;
		m_reportsDirty = false;
		m_lastSelMessages = 0;
		m_lastSelLocations = 0;
		m_lastSelSystem = 0;
	}

	CDICE_QueryDiceImpl::~CDICE_QueryDiceImpl(void)
	{
	}

	CDICE_QueryDiceImpl* CDICE_QueryDiceImpl::GetInstance(void)
	{
		if (m_theInstance == NULL)
		{
			m_theInstance = new CDICE_QueryDiceImpl ();
		}

		return m_theInstance;
	}

	DICE_INTERFACE::SDICE_ReturnIDs CDICE_QueryDiceImpl::AddEntry (
		std::string messagesParent,
		std::string messagesDesc,
		std::string locationsParent,
		std::string locationsDesc,
		std::string systemParent,
		std::string systemDesc,
		double severity,
		DICE_INTERFACE::CDICE_Message msg)
	{
		DICE_INTERFACE::SDICE_ReturnIDs ret;
		m_avoidGUIUpdate++;
		//add the entry to the trees
		ret = AddEntry (messagesParent,messagesDesc,locationsParent,locationsDesc,systemParent,systemDesc,severity);		

		//add the message to the report collection
		CDICE_ReportCollection::GetInstance()->addReport(msg,ret.m_IDmessages,ret.m_IDlocations,ret.m_IDsystem);
		m_reportsDirty = true;

		//update GUI
		m_avoidGUIUpdate--;
		UpdateGUI();
		return ret;
	}

	DICE_INTERFACE::SDICE_ReturnIDs CDICE_QueryDiceImpl::AddEntry (
		std::string messagesParent,
		std::string messagesDesc,
		std::string locationsParent,
		std::string locationsDesc,
		std::string systemParent,
		std::string systemDesc,
		double severity)
	{
		DICE_INTERFACE::SDICE_ReturnIDs ret;
		
		//1) get ParentID's (Locations,Messages,System)
		long parent_msgs = CDICE_MessagesData::GetInstance()->convertDescToID(messagesParent);
		long parent_locs = CDICE_LocationsData::GetInstance()->convertDescToID(locationsParent);
		long parent_sys = CDICE_SystemData::GetInstance()->convertDescToID(systemParent);

		if ( (parent_msgs <= 0) ||
			 (parent_locs <= 0) ||
			 (parent_sys <= 0)     )
		{
			ret.m_IDlocations = -1;
			ret.m_IDmessages = -1;
			ret.m_IDsystem = -1;
			return ret;
		}

		//add the entry to the trees
		return AddEntry (parent_msgs,messagesDesc,parent_locs,locationsDesc,parent_sys,systemDesc,severity);
	}

	DICE_INTERFACE::SDICE_ReturnIDs CDICE_QueryDiceImpl::AddEntry (
		long messagesParent,
		std::string messagesDesc,
		long locationsParent,
		std::string locationsDesc,
		long systemParent,
		std::string systemDesc,
		double severity,
		DICE_INTERFACE::CDICE_Message msg)
	{
		DICE_INTERFACE::SDICE_ReturnIDs ret;
		m_avoidGUIUpdate++;
		//add the entry to the trees
		ret = AddEntry(messagesParent,messagesDesc,locationsParent,locationsDesc,systemParent,systemDesc,severity);

		//add the message to the report collection
		CDICE_ReportCollection::GetInstance()->addReport(msg,ret.m_IDmessages,ret.m_IDlocations,ret.m_IDsystem);
		m_reportsDirty = true;
		
		//update GUI
		m_avoidGUIUpdate--;
		UpdateGUI();
		return ret;
	}

	DICE_INTERFACE::SDICE_ReturnIDs CDICE_QueryDiceImpl::AddEntry (
		long messagesParent,
		std::string messagesDesc,
		long locationsParent,
		std::string locationsDesc,
		long systemParent,
		std::string systemDesc,
		double severity)
	{
		//Create/Search the leafs for the entry
		long msgID = CDICE_MessagesData::GetInstance()->convertDescToID(messagesParent,messagesDesc);
		long locID = CDICE_LocationsData::GetInstance()->convertDescToID(locationsParent,locationsDesc);
		long sysID = CDICE_SystemData::GetInstance()->convertDescToID(systemParent,systemDesc);

		if (msgID <= 0)
		{
			//needs to be created
			msgID = CDICE_MessagesData::GetInstance()->addNewLeaf(messagesDesc,messagesParent,NULL);
		}

		if (locID <= 0)
		{
			//needs to be created
			locID = CDICE_LocationsData::GetInstance()->addNewLeaf(locationsDesc,locationsParent,NULL);
		}

		if (sysID <= 0)
		{
			//needs to be created
			sysID = CDICE_SystemData::GetInstance()->addNewLeaf(systemDesc,systemParent,NULL);
		}

		return AddEntry (messagesParent,msgID,locationsParent,locID,systemParent,sysID,severity);
	}

	DICE_INTERFACE::SDICE_ReturnIDs CDICE_QueryDiceImpl::AddEntry (
		long messagesParent,
		long messagesID,
		long locationsParent,
		long locationsID,
		long systemParent,
		long systemID,
		double severity,
		DICE_INTERFACE::CDICE_Message msg)
	{
		DICE_INTERFACE::SDICE_ReturnIDs ret;
		m_avoidGUIUpdate++;

		//add the entry to the trees
		ret = AddEntry(messagesParent,messagesID,locationsParent,locationsID,systemParent,systemID,severity);

		//add the message to the report collection
		CDICE_ReportCollection::GetInstance()->addReport(msg,ret.m_IDmessages,ret.m_IDlocations,ret.m_IDsystem);		
		m_reportsDirty = true;
		
		//update GUI
		m_avoidGUIUpdate--;
		UpdateGUI();
		return ret;
	}

	DICE_INTERFACE::SDICE_ReturnIDs CDICE_QueryDiceImpl::AddEntry (
		long messagesParent,
		long messagesID,
		long locationsParent,
		long locationsID,
		long systemParent,
		long systemID,
		double severity)
	{
		DICE_INTERFACE::SDICE_ReturnIDs ret;
		
		//create the links
		CDICE_MessagesData::GetInstance()->addLink(messagesID,locationsID,systemID,severity);

		//prepare return values
		ret.m_IDmessages = messagesID;
		ret.m_IDlocations = locationsID;
		ret.m_IDsystem = systemID;

		//set dirty flags
		m_systemDirty = true;
		m_locationsDirty = true;
		m_messagesDirty = true;
		
		//update the GUI
		UpdateGUI ();

		return ret;
	}

	long CDICE_QueryDiceImpl::AddMessagesItem (std::string parentName, std::string itemName, bool bLeaf)
	{
		long id = CDICE_MessagesData::GetInstance()->convertDescToID(parentName);
		return AddMessagesItem(id,itemName,bLeaf);
	}

	long CDICE_QueryDiceImpl::AddMessagesItem (long parentID, std::string itemName, bool bLeaf)
	{
		long id = -1;
		bool bWasAlreadyPresent;

		if (bLeaf)
			id = CDICE_MessagesData::GetInstance()->addNewLeaf(itemName,parentID,&bWasAlreadyPresent);
		else id = CDICE_MessagesData::GetInstance()->addNewNode(itemName,parentID,&bWasAlreadyPresent);
		
		//was the item already present ?
		if (bWasAlreadyPresent)
			return id;

		//set dirty flags
		m_messagesDirty = true;

		//update the GUI
		UpdateGUI ();

		return id;
	}

	long CDICE_QueryDiceImpl::AddLocationsItem (std::string parentName, std::string itemName, bool bLeaf)
	{
		long id = CDICE_LocationsData::GetInstance()->convertDescToID(parentName);
		return AddLocationsItem(id,itemName,bLeaf);
	}

	long CDICE_QueryDiceImpl::AddLocationsItem (long parentID, std::string itemName, bool bLeaf)
	{
		long id = -1;
		bool bWasAlreadyPresent;

		if (bLeaf)
			id = CDICE_LocationsData::GetInstance()->addNewLeaf(itemName,parentID,&bWasAlreadyPresent);
		else id = CDICE_LocationsData::GetInstance()->addNewNode(itemName,parentID,&bWasAlreadyPresent);
		
		//was the item already present ?
		if (bWasAlreadyPresent)
			return id;

		//set dirty flags
		m_locationsDirty = true;

		//update the GUI
		UpdateGUI ();

		return id;
	}

	long CDICE_QueryDiceImpl::AddSystemItem (std::string parentName, std::string itemName, bool bLeaf)
	{
		long id = CDICE_SystemData::GetInstance()->convertDescToID(parentName);
		return AddSystemItem(id,itemName,bLeaf);
	}

	long CDICE_QueryDiceImpl::AddSystemItem (long parentID, std::string itemName, bool bLeaf)
	{
		long id = -1;
		bool bWasAlreadyPresent;

		if (bLeaf)
			id = CDICE_SystemData::GetInstance()->addNewLeaf(itemName,parentID,&bWasAlreadyPresent);
		else id = CDICE_SystemData::GetInstance()->addNewNode(itemName,parentID,&bWasAlreadyPresent);
		
		//was the item already present ?
		if (bWasAlreadyPresent)
			return id;

		//set dirty flags
		m_systemDirty = true;

		//update the GUI
		UpdateGUI ();

		return id;
	}

	void CDICE_QueryDiceImpl::setGUIUpdater (DICE_INTERFACE::IDICE_GUIUpdater* pUpdater)
	{
		m_pUpdater = pUpdater;
	}

	DICE_INTERFACE::IDICE_GUIUpdater* CDICE_QueryDiceImpl::getGUIUpdater (void)
	{
		return m_pUpdater;
	}

	void CDICE_QueryDiceImpl::UpdateGUI (void)
	{
		if (m_avoidGUIUpdate > 0)
			return;

		//1) Create data
		DICE_INTERFACE::CDICE_TreeItem *messages = NULL;
		DICE_INTERFACE::CDICE_TreeItem *locations = NULL;
		DICE_INTERFACE::CDICE_TreeItem *system = NULL;
		
		//2) Collect links for Locations and System Trees
		std::list<SDICE_LinkType> locations_links, system_links;
		std::list<long> restriction_list;
		
		locations_links = CDICE_MessagesData::GetInstance()->collectLinksForSelection(m_curSelection.m_IDmessages);
		
		restriction_list = CDICE_LocationsData::GetInstance()->collectIDsBySelection(m_curSelection.m_IDlocations);

		system_links = CDICE_SystemData::GetInstance()->restrictLinksToLocationsSelection(locations_links,restriction_list);
		
		//3) Create Trees using the link lists
		if (m_messagesDirty)
			messages = CDICE_MessagesData::GetInstance()->createFullInterfaceTree();

		if (m_locationsDirty)
			locations = CDICE_LocationsData::GetInstance()->createRestrictedInterfaceTree(locations_links);

		if (m_systemDirty)
			system = CDICE_SystemData::GetInstance()->createRestrictedInterfaceTree(system_links);
		
		//4) Push the trees into the GUI
		//if (messages != NULL)
		if (m_messagesDirty)
			m_pUpdater->updateMessages(messages);

		//if (locations != NULL)
		if (m_locationsDirty)
			m_pUpdater->updateLocations(locations);

		//if (system != NULL)
		if (m_systemDirty)
			m_pUpdater->updateSystem(system);

		//5) Update the report list !
		if (m_reportsDirty)
		{
			DICE_INTERFACE::SDICE_ReturnIDs rep_selection;
			rep_selection.m_IDmessages = m_lastSelMessages;
			rep_selection.m_IDlocations = m_lastSelLocations;
			rep_selection.m_IDsystem = m_lastSelSystem;
			std::list<DICE_INTERFACE::CDICE_Message> reports = CDICE_QueryDiceImpl::getReports (rep_selection);
			m_pUpdater->updateReportList (reports);
		}

		m_messagesDirty = false;
		m_locationsDirty = false;
		m_systemDirty = false;
		m_reportsDirty = false;
	}

	bool CDICE_QueryDiceImpl::changeSelection (DICE_INTERFACE::SDICE_ReturnIDs selection)
	{
		if (m_curSelection.m_IDmessages != selection.m_IDmessages)
		{
			m_curSelection.m_IDmessages = selection.m_IDmessages;
			m_messagesDirty = true;
		}
		
		if (m_curSelection.m_IDlocations != selection.m_IDlocations)
		{
			m_curSelection.m_IDlocations = selection.m_IDlocations;
			m_locationsDirty = true;
		}

		if (m_curSelection.m_IDsystem != selection.m_IDsystem)
		{
			m_curSelection.m_IDsystem = selection.m_IDsystem;
			m_systemDirty = true;
		}

		UpdateGUI ();

		return true;
	}

	bool CDICE_QueryDiceImpl::changeMessagesSelection (long id)
	{
		if (m_curSelection.m_IDmessages != id)
		{
			m_curSelection.m_IDmessages = id;
			//m_messagesDirty = true;
			m_locationsDirty = true;
			m_systemDirty = true;
		}

		UpdateGUI ();

		return true;
	}

	bool CDICE_QueryDiceImpl::changeLocationsSelection (long id)
	{
		if (m_curSelection.m_IDlocations != id)
		{
			m_curSelection.m_IDlocations = id;
			//m_locationsDirty = true;
			m_systemDirty = true;
		}

		UpdateGUI ();

		return true;
	}
	
	bool CDICE_QueryDiceImpl::changeSystemSelection (long id)
	{
		if (m_curSelection.m_IDsystem != id)
		{
			m_curSelection.m_IDsystem = id;
			m_systemDirty = true;
		}

		UpdateGUI ();

		return true;
	}

	std::list<DICE_INTERFACE::CDICE_Message> CDICE_QueryDiceImpl::getReports (DICE_INTERFACE::SDICE_ReturnIDs selection)
	{
		m_lastSelMessages = selection.m_IDmessages;
		m_lastSelLocations = selection.m_IDlocations;
		m_lastSelSystem = selection.m_IDsystem;

		std::list<long> msgs = CDICE_MessagesData::GetInstance()->collectIDsBySelection(selection.m_IDmessages);
		std::list<long> locs = CDICE_LocationsData::GetInstance()->collectIDsBySelection(selection.m_IDlocations);
		std::list<long> sys = CDICE_SystemData::GetInstance()->collectIDsBySelection(selection.m_IDsystem);

		return CDICE_ReportCollection::GetInstance()->getMatchingReports(msgs,locs,sys);
	}

	bool CDICE_QueryDiceImpl::clear (void)
	 {
		 bool ret = true;
		 ret = ret && CDICE_MessagesData::GetInstance()->deleteTree ();
		 ret = ret && CDICE_LocationsData::GetInstance()->deleteTree ();
		 ret = ret && CDICE_SystemData::GetInstance()->deleteTree ();
		 ret = ret && CDICE_ReportCollection::GetInstance()->deleteReports();

    	 m_systemDirty =
			 m_messagesDirty =
				m_locationsDirty = 
					m_reportsDirty = true;

		 m_curSelection.m_IDlocations = 0;
		 m_curSelection.m_IDmessages = 0;
	     m_curSelection.m_IDsystem = 0;

		 m_lastSelMessages = 0;
		 m_lastSelLocations = 0;
		 m_lastSelSystem = 0;

		 UpdateGUI ();

		 return true;
	 }

	bool CDICE_QueryDiceImpl::save (std::ofstream &file)
	{
		bool ret = true;
		ret = ret && CDICE_MessagesData::GetInstance()->save (file);
		ret = ret && CDICE_LocationsData::GetInstance()->save (file);
		ret = ret && CDICE_SystemData::GetInstance()->save (file);
		ret = ret && CDICE_ReportCollection::GetInstance()->save(file);
		return ret;
	}

    bool CDICE_QueryDiceImpl::load (std::ifstream &file)
	{
		bool ret = true;

		clear ();

		ret = ret && CDICE_MessagesData::GetInstance()->load (file);
		ret = ret && CDICE_LocationsData::GetInstance()->load (file);
		ret = ret && CDICE_SystemData::GetInstance()->load (file);
		ret = ret && CDICE_ReportCollection::GetInstance()->load(file);
		
		m_systemDirty =
			 m_messagesDirty =
				m_locationsDirty = 
					m_reportsDirty = true;

		 m_curSelection.m_IDlocations = 0;
		 m_curSelection.m_IDmessages = 0;
	     m_curSelection.m_IDsystem = 0;

		 m_lastSelMessages = 0;
		 m_lastSelLocations = 0;
		 m_lastSelSystem = 0;

		 UpdateGUI ();
				
		 return ret;
	}

	bool CDICE_QueryDiceImpl::isEmpty (void)
	{
		//TODO Maybe a bit more detailed ...
		return CDICE_MessagesData::GetInstance()->isEmpty();
	}

}//end of namespace DICE_BACK