#include "StdAfx.h"
#include "DICE_ReportListWin.h"
#include "DICE_ReportWin.h"
#include "DICE_AddinComm.h"
#include "Resource.h"
#include <fstream>
#include <assert.h>

namespace DICE_GUI
{

	// Message map

	BEGIN_MESSAGE_MAP(CDICE_ReportListWin, CListCtrl)
		ON_WM_CREATE()
		ON_NOTIFY(HDN_ITEMCHANGEDA, 0, &CDICE_ReportListWin::OnHdnItemchanged)
		ON_NOTIFY(HDN_ITEMCHANGEDW, 0, &CDICE_ReportListWin::OnHdnItemchanged)
		ON_NOTIFY_REFLECT(LVN_ITEMACTIVATE, &CDICE_ReportListWin::OnLvnItemActivate)
		ON_NOTIFY_REFLECT(LVN_GETDISPINFO, &CDICE_ReportListWin::OnLvnGetdispinfo)
		ON_WM_SIZE()
		ON_NOTIFY(HDN_ITEMCLICKA, 0, &CDICE_ReportListWin::OnHdnItemclick)
		ON_NOTIFY(HDN_ITEMCLICKW, 0, &CDICE_ReportListWin::OnHdnItemclick)
		ON_NOTIFY_REFLECT(NM_CLICK, &CDICE_ReportListWin::OnNMClick)
		ON_NOTIFY_REFLECT(LVN_KEYDOWN, &CDICE_ReportListWin::OnLvnKeydown)
		ON_WM_KEYUP()
		ON_WM_CONTEXTMENU()
		ON_NOTIFY(HDN_ENDTRACKA, 0, &CDICE_ReportListWin::OnHdnEndtrack)
		ON_NOTIFY(HDN_ENDTRACKW, 0, &CDICE_ReportListWin::OnHdnEndtrack)
		ON_NOTIFY(HDN_TRACKA, 0, &CDICE_ReportListWin::OnHdnTrack)
		ON_NOTIFY(HDN_TRACKW, 0, &CDICE_ReportListWin::OnHdnTrack)
	END_MESSAGE_MAP()



	// Dispatch map

	BEGIN_DISPATCH_MAP(CDICE_ReportListWin, CListCtrl)
	END_DISPATCH_MAP()



	CDICE_ReportListWin* CDICE_ReportListWin::m_theInstance = NULL;
	const int CDICE_ReportListWin::m_cNumColumns = 8;
	std::list<int> CDICE_ReportListWin::m_appliedSorts;
	std::vector<bool> CDICE_ReportListWin::m_sortStates;

	CDICE_ReportListWin::CDICE_ReportListWin(void)
	{
		m_pImageList = NULL;
		m_reports.clear();
		
		m_appliedSorts.clear(); //stores what sorting was used
		m_appliedSorts.push_back(0);

		m_sortStates.clear();
		m_sortStates.resize(m_cNumColumns);
		for (int i = 0; i < m_cNumColumns;i++)
			m_sortStates[i] = false;
	}

	CDICE_ReportListWin::~CDICE_ReportListWin(void)
	{
		//delete image list
		if (m_pImageList)
			delete (m_pImageList);
		m_pImageList = NULL;

		//delete items
		std::list<DICE_INTERFACE::CDICE_Message*>::iterator j;
		for (j = m_reports.begin();j != m_reports.end();j++)
		{
			if (*j)
				delete (*j);
		}
		m_reports.clear();
	}

	CDICE_ReportListWin* CDICE_ReportListWin::GetInstance ()
	{
		if (m_theInstance == NULL)
		{
			m_theInstance = new CDICE_ReportListWin ();
		}

		return m_theInstance;
	}

	void CDICE_ReportListWin::setReports(std::list<DICE_INTERFACE::CDICE_Message> reports)
	{
		//iterate over all curent items and remove all items not 
		//present in the new report list
		int numItems = GetItemCount();
		int nItemSelected = -1;
		bool bResetedSelection = false;
		POSITION pos = GetFirstSelectedItemPosition();
		if (pos != NULL)
			nItemSelected = GetNextSelectedItem(pos);

		for (int item = numItems-1;item >= 0; item--)
		{
			DICE_INTERFACE::CDICE_Message* msg =
				(DICE_INTERFACE::CDICE_Message*)GetItemData(item);

			//is msg in the reports ?
			bool bfound = false;

			if (msg)
			{
				std::list<DICE_INTERFACE::CDICE_Message>::iterator i;
				for (i = reports.begin();i != reports.end();i++)
				{
					if ((*i) == (*msg))
					{
						//item found, remove it from the list of the reports 
						bfound = true;
						reports.erase(i);
						break;
					}
				}
			}
			
			//has the item to be removed ?
			if (!bfound)
			{
				//remove item from list
				DeleteItem(item);

				//was it the selected item ?
				if (item == nItemSelected)
					bResetedSelection = true;

				//find the item in our message list, remove it
				std::list<DICE_INTERFACE::CDICE_Message*>::iterator j;
				for (j = m_reports.begin();j != m_reports.end();j++)
				{
					if (*j == msg)
					{
						delete (*j);
						m_reports.erase(j);
						break;
					}
				}
			}
		}

		//Add all the remaining items !
		int nextID = GetItemCount();
		std::list<DICE_INTERFACE::CDICE_Message>::iterator i;
		for (i = reports.begin();i != reports.end();i++,nextID++)
		{
			//insert into report map
			DICE_INTERFACE::CDICE_Message* pMsg = new DICE_INTERFACE::CDICE_Message (*i);
			m_reports.push_back(pMsg);

			//insert item
			LV_ITEM item;
			item.mask = LVIF_IMAGE | LVIF_TEXT | LVIF_PARAM | LVIF_STATE;
			item.iItem = nextID;
			item.iSubItem = 0;
			item.state = 0;
			item.stateMask = 0;
			item.pszText = LPSTR_TEXTCALLBACK;
			item.lParam = (LPARAM) pMsg;
			item.iIndent = 0;
			item.iGroupId = 0;
			item.cColumns = 0;
			item.puColumns = 0;
			item.piColFmt = 0;
			item.cchTextMax = 1024;
			
			switch(i->getType())
			{
			case DICE_INTERFACE::MESSAGE_TYPE_ERROR:
				item.iImage = 0;	
				break;

			case DICE_INTERFACE::MESSAGE_TYPE_NOTE:
				item.iImage = 1;	
				break;

			case DICE_INTERFACE::MESSAGE_TYPE_REMARK:
				item.iImage = 2;	
				break;

			case DICE_INTERFACE::MESSAGE_TYPE_WARNING:
				item.iImage = 3;	
				break;
			}			

			InsertItem(&item);
		}

		//take care that the sorting is refreshed
		//we redo all sorting applied in reverse order
		//than it was applied
		//b)invert invert flags
		for (int j = 0; j < m_cNumColumns;j++)
			m_sortStates[j] = !m_sortStates[j];

		//a)apply
		std::list<int> temp_sort = m_appliedSorts;
		std::list<int>::iterator k;
		for (k = temp_sort.begin(); k != temp_sort.end();k++)
		{
			SortItems(SortFunc, *k);
		}

		//c)invert again
		for (int j = 0; j < m_cNumColumns;j++)
			m_sortStates[j] = !m_sortStates[j];

		//reset content of the report text window
		//do so only if selected item was removed
		if (bResetedSelection)
			CDICE_ReportWin::GetInstance()->setText("");
	}

	std::wstring CDICE_ReportListWin::string2wstring (std::string org)
	{
		std::wstring str(org.length(), L' '); // Make room for characters
		std::copy(org.begin(), org.end(), str.begin());
		return str;
	}

	int CALLBACK CDICE_ReportListWin::SortFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
	{
		DICE_INTERFACE::CDICE_Message* pData1 = (DICE_INTERFACE::CDICE_Message*)lParam1;
		DICE_INTERFACE::CDICE_Message* pData2 = (DICE_INTERFACE::CDICE_Message*)lParam2;

		//calculate the invert parameter
		//(used to switch between ascending and descending sorting)
		int invert = 1;
		if ((lParamSort >= 0) &&
			(lParamSort <= m_cNumColumns))
		{
			if (m_sortStates[lParamSort] == true)
			{
				invert = -1;
			}
		}
		
		//apply the sorting
		switch(lParamSort)
		{
		case 0://icon column
			{
				switch (pData1->getType())
				{
				case DICE_INTERFACE::MESSAGE_TYPE_ERROR:
					if (pData2->getType() == DICE_INTERFACE::MESSAGE_TYPE_ERROR)
						return 0; //equal
					return -1*invert;
				case DICE_INTERFACE::MESSAGE_TYPE_WARNING:
					if (pData2->getType() == DICE_INTERFACE::MESSAGE_TYPE_WARNING)
						return 0; //equal
					if (pData2->getType() == DICE_INTERFACE::MESSAGE_TYPE_ERROR)
						return 1*invert; //equal
					return -1*invert;
					break;
				case DICE_INTERFACE::MESSAGE_TYPE_NOTE:
					if (pData2->getType() == DICE_INTERFACE::MESSAGE_TYPE_NOTE)
						return 0; //equal
					if (pData2->getType() == DICE_INTERFACE::MESSAGE_TYPE_REMARK)
						return -1*invert; //equal
					return 1*invert;
					break;
				case DICE_INTERFACE::MESSAGE_TYPE_REMARK:
					if (pData2->getType() == DICE_INTERFACE::MESSAGE_TYPE_REMARK)
						return 0; //equal
					return 1*invert;
					break;
				}
			}	
			break;
		case 1://timestamp column
			{
				if (pData1->getTimestamp() < pData2->getTimestamp())
					return -1*invert;
				if (pData1->getTimestamp() > pData2->getTimestamp())
					return 1*invert;
				return 0;
			}
			break;
		case 2://rank column
			{
				if (pData1->getSourceRank() < pData2->getSourceRank())
					return -1*invert;
				if (pData1->getSourceRank() > pData2->getSourceRank())
					return 1*invert;
				return 0;
			}
			break;
		case 3://thread column
			{
				if (pData1->getSourceThread() < pData2->getSourceThread())
					return -1*invert;
				if (pData1->getSourceThread() > pData2->getSourceThread())
					return 1*invert;
				return 0;
			}
			break;
		case 4://line column
			{
				if (pData1->getLocationLine() < pData2->getLocationLine())
					return -1*invert;
				if (pData1->getLocationLine() > pData2->getLocationLine())
					return 1*invert;
				return 0;
			}
			break;
		case 5://file column
			{
				return pData1->getLocationFileName().compare(pData2->getLocationFileName())*invert;
			}
			break;
		case 6://callname column
			{
				return pData1->getCallName().compare(pData2->getCallName())*invert;
			}
			break;
		case 7://description column
			{
				return pData1->getShortDesc().compare(pData2->getShortDesc())*invert;
			}
			break;
			
		default:
			//TODO: Unknown column, that should be reported
			break;
		}

		return 0;
	}

} //end of namespace DICE_GUI
int DICE_GUI::CDICE_ReportListWin::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CListCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	//1) Create the image List
	m_pImageList = new CImageList();
	m_pImageList->Create(16, 16, ILC_COLOR32, 4, 4);

	m_pImageList->Add(LoadIcon(NULL, MAKEINTRESOURCE(IDI_ERROR)));
	m_pImageList->Add(LoadIcon(NULL, MAKEINTRESOURCE(IDI_INFORMATION)));
	m_pImageList->Add(LoadIcon(NULL, MAKEINTRESOURCE(IDI_INFORMATION)));
	m_pImageList->Add(LoadIcon(NULL, MAKEINTRESOURCE(IDI_WARNING)));
	
	SetImageList(m_pImageList, LVSIL_SMALL);

	//2) Set the columns
	LVCOLUMN lvc; 
    lvc.mask = LVCFMT_FIXED_WIDTH | LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM | LVCF_IMAGE;
	lvc.cx = 100;
	lvc.fmt = LVCFMT_LEFT;

	lvc.iSubItem = 0;
    lvc.pszText = _T("");
	this->InsertColumn (0,&lvc);

	lvc.mask = LVCFMT_FIXED_WIDTH | LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	lvc.iSubItem = 1;
    lvc.pszText = _T("Timestamp");
	this->InsertColumn (1,&lvc);
	lvc.iSubItem = 2;
    lvc.pszText = _T("Rank");
	this->InsertColumn (2,&lvc);
	lvc.iSubItem = 3;
    lvc.pszText = _T("Thread");	
	this->InsertColumn (3,&lvc);
	lvc.iSubItem = 4;
    lvc.pszText = _T("Line");	
	this->InsertColumn (4,&lvc);
	lvc.iSubItem = 5;
    lvc.pszText = _T("File");	
	this->InsertColumn (5,&lvc);
	lvc.iSubItem = 6;
    lvc.pszText = _T("Call");	
	this->InsertColumn (6,&lvc);
	lvc.iSubItem = 7;
    lvc.pszText = _T("Description");	
	this->InsertColumn (7,&lvc);

	//3) Set full row select if possible ...
	DWORD dwStyle = ::SendMessage(m_hWnd,LVM_GETEXTENDEDLISTVIEWSTYLE,0,0);
    dwStyle |= LVS_EX_FULLROWSELECT;
    ::SendMessage(m_hWnd,LVM_SETEXTENDEDLISTVIEWSTYLE,0,dwStyle);

	return 0;
}

void DICE_GUI::CDICE_ReportListWin::OnHdnItemchanged(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMHEADER phdr = reinterpret_cast<LPNMHEADER>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;
}

void DICE_GUI::CDICE_ReportListWin::OnLvnItemActivate(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);

	DWORD_PTR data = GetItemData(pNMIA->iItem);
	if (data != NULL)
	{
		//get the message
		DICE_INTERFACE::CDICE_Message *msg = (DICE_INTERFACE::CDICE_Message*) data;

		//do so only for valid locations
		if (msg->getLocationFileName().compare("") != 0)
		{
			//say the addin that it should open the location
			CDICE_AddinComm::GetInstance()->pushOpenSourceFileLocation(msg->getLocationFileName(),msg->getLocationLine());
		}
	}

	*pResult = 0;
}

void DICE_GUI::CDICE_ReportListWin::OnLvnGetdispinfo(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMLVDISPINFO *pDispInfo = reinterpret_cast<NMLVDISPINFO*>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;

	//Memory used for the strings .... (avoids dynamic memory mangling)
	static WCHAR ws_timestamp[128];
	static WCHAR ws_rank[128];
	static WCHAR ws_column[128];
	static WCHAR ws_line[128];
	static WCHAR ws_file[256];
	static WCHAR ws_callname[256];
	static WCHAR ws_desc[512];

	DICE_INTERFACE::CDICE_Message *msg = (DICE_INTERFACE::CDICE_Message*) pDispInfo->item.lParam;//= i->second;

	switch (pDispInfo->item.iSubItem)
	{
		case 0://icon column
			pDispInfo->item.pszText = _T("");
			break;
		case 1://timestamp column
			{
				wsprintf(ws_timestamp,_T("%d\0"),msg->getTimestamp());
				pDispInfo->item.pszText = ws_timestamp;
			}
			break;
		case 2://rank column
			{
				if (msg->getSourceRank() >= 0)
					 wsprintf(ws_rank,_T("%d\0"),msg->getSourceRank());
				else wsprintf(ws_rank,_T("Global"),msg->getSourceRank());
				pDispInfo->item.pszText = ws_rank;
			}
			break;
		case 3://thread column
			{
				if (msg->getSourceRank() >= 0)
					 wsprintf(ws_column,_T("%d\0"),msg->getSourceThread());
				else wsprintf(ws_column,_T(""));
				pDispInfo->item.pszText = ws_column;					
			}
			break;
		case 4://line column
			{
				if (msg->getLocationFileName().compare(std::string("")) != 0)
					 wsprintf(ws_line,_T("%d\0"),msg->getLocationLine());
				else wsprintf(ws_line,_T(""));
				pDispInfo->item.pszText = ws_line;					
			}
			break;
		case 5://file column
			{
				wsprintf(ws_file,_T("%s\0"),string2wstring(msg->getLocationFileName()).c_str());
				pDispInfo->item.pszText = ws_file;
			}
			break;
		case 6://callname column
			{
				wsprintf(ws_callname,_T("%s\0"),string2wstring(msg->getCallName()).c_str());
				pDispInfo->item.pszText = ws_callname;
			}
			break;
		case 7://description column
			{
				wsprintf(ws_desc,_T("%s\0"),string2wstring(msg->getShortDesc()).c_str());
				pDispInfo->item.pszText = ws_desc;
			}
			break;
		
		default:
			//TODO: Unknown column, that should be reported
			break;
	}//end switch
}

void DICE_GUI::CDICE_ReportListWin::OnSize(UINT nType, int cx, int cy)
{
	CListCtrl::OnSize(nType, cx, cy);

	// TODO: Add your message handler code here
	//set the width of the columns
	
	//severity (icon)
	int w1 = GetStringWidth(_T("RankO")); //extra space for icon ...
	SetColumnWidth(0,w1);

	//timestamp ...
	int w2 = GetStringWidth(_T("0TIMESTAMP0")); //extra space for icon ...
	SetColumnWidth(1,w2);

	//rank ...
	int w3 = GetStringWidth(_T("00RANK00")); //extra space for icon ...
	SetColumnWidth(2,w3);

	//thread ...
	int w4 = GetStringWidth(_T("0THREAD0"));
	SetColumnWidth(3,w4);

	//line ...
	int w5 = GetStringWidth(_T("00LINE00"));
	SetColumnWidth(4,w5);

	//File ...
	int wrest = cx - w1 - w2 - w3 - w4 - w5;
	int w6 = wrest*0.25;
	SetColumnWidth(5,w6);

	//Call ...
	int w7 = wrest*0.25;
	SetColumnWidth(6,w7);

	//Description ...
	SetColumnWidth(7,(wrest-w6)-w7);
}

void DICE_GUI::CDICE_ReportListWin::OnHdnItemclick(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMHEADER phdr = reinterpret_cast<LPNMHEADER>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;

	SortItems(SortFunc, phdr->iItem);

	//correct index ?
	if ((phdr->iItem >= 0) &&
		(phdr->iItem <= m_cNumColumns))
	{
		//calculate the invert parameter
		//(used to switch between ascending and descending sorting)
		m_sortStates[phdr->iItem] = !m_sortStates[phdr->iItem];

		//store that this sorting was applied
		std::list<int>::iterator i;
		for (i = m_appliedSorts.begin(); i != m_appliedSorts.end();i++)
		{
			if (*i == phdr->iItem)
			{
				m_appliedSorts.erase(i);
				break;
			}
		}
		m_appliedSorts.push_back(phdr->iItem);
	}
}//end of OnHdnItemclick

void DICE_GUI::CDICE_ReportListWin::OnNMClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMItemActivate = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;

	//Update the Text window
	if (pNMItemActivate->iItem != -1)
	{
		DICE_INTERFACE::CDICE_Message *msg = (DICE_INTERFACE::CDICE_Message*) GetItemData(pNMItemActivate->iItem);
		if (msg != NULL)
			CDICE_ReportWin::GetInstance()->setText(msg->getText());
	}
}

void DICE_GUI::CDICE_ReportListWin::OnLvnKeydown(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMLVKEYDOWN pLVKeyDow = reinterpret_cast<LPNMLVKEYDOWN>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;
}

void DICE_GUI::CDICE_ReportListWin::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	// TODO: Add your message handler code here and/or call default

	CListCtrl::OnKeyUp(nChar, nRepCnt, nFlags);

	//Update the Text window
	POSITION pos = GetFirstSelectedItemPosition();
	if (pos != NULL)
	{
		int nItemSelected = GetNextSelectedItem(pos);
		DICE_INTERFACE::CDICE_Message *msg = (DICE_INTERFACE::CDICE_Message*) GetItemData(nItemSelected);
		if (msg != NULL)
			CDICE_ReportWin::GetInstance()->setText(msg->getText());
	}
}

void DICE_GUI::CDICE_ReportListWin::OnContextMenu(CWnd* /*pWnd*/, CPoint point/*point*/)
{
	// TODO: Add your message handler code here

	//Get the selected item's data
	POSITION pos = GetFirstSelectedItemPosition();
	if (pos == NULL)
		return;
	
	int nItemSelected = GetNextSelectedItem(pos);
	DWORD_PTR data = GetItemData(nItemSelected);
	if (data == NULL)
		return;

	//get the message
	DICE_INTERFACE::CDICE_Message *msg = (DICE_INTERFACE::CDICE_Message*) data;

	//show the context menu
    CMenu Menu;
    if(Menu.LoadMenu(IDR_REPORTLIST_CONTEXTMENU))
    {
        CMenu* pPopup = Menu.GetSubMenu(0);
        int nID = pPopup->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RETURNCMD,point.x,point.y,this);

		switch (nID)
		{
		case ID_GOTO_SOURCE:
			{
				//do so only for valid locations
				if (msg->getLocationFileName().compare("") != 0)
				{
					//say the addin that it should open the location
					CDICE_AddinComm::GetInstance()->pushOpenSourceFileLocation(msg->getLocationFileName(),msg->getLocationLine());
				}
			}//case
			break;
		case ID_GOTO_MPI_STANDARD:
			{
				//do so only for valid locations
				if (msg->getMpiReference().compare("") != 0)
				{
					//say the addin that it should open the html file
					CDICE_AddinComm::GetInstance()->pushOpenHTMLFile(msg->getMpiReference());
				}
			}
			break;
		}
    }
	
}

void DICE_GUI::CDICE_ReportListWin::OnHdnEndtrack(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMHEADER phdr = reinterpret_cast<LPNMHEADER>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;

	RedrawItems(0,this->GetItemCount());
}

void DICE_GUI::CDICE_ReportListWin::OnHdnTrack(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMHEADER phdr = reinterpret_cast<LPNMHEADER>(pNMHDR);
	// TODO: Add your control notification handler code here
	*pResult = 0;

	RedrawItems(0,this->GetItemCount());
}
