#include "StdAfx.h"
#include "DICE_TreeWin.h"
#include <fstream>

namespace DICE_GUI
{

	// Message map

	BEGIN_MESSAGE_MAP(CDICE_TreeWin, CTreeCtrl)
		ON_NOTIFY_REFLECT(TVN_ITEMCHANGED, &CDICE_TreeWin::OnTvnItemChanged)
		ON_NOTIFY_REFLECT(TVN_ITEMEXPANDED, &CDICE_TreeWin::OnTvnItemexpanded)
		ON_NOTIFY_REFLECT(TVN_SELCHANGED, &CDICE_TreeWin::OnTvnSelchanged)
		ON_WM_CREATE()
	END_MESSAGE_MAP()



	// Dispatch map

	BEGIN_DISPATCH_MAP(CDICE_TreeWin, CTreeCtrl)
	END_DISPATCH_MAP()


	CDICE_TreeWin::CDICE_TreeWin(void)
	{
		m_items.clear();
		m_imageList = NULL;
		m_expandedList.clear();
		m_selectedID = 0;
	}

	CDICE_TreeWin::~CDICE_TreeWin(void)
	{
		if (m_imageList)
			delete (m_imageList);
		m_imageList = NULL;
	}

	bool CDICE_TreeWin::updateContent (DICE_INTERFACE::CDICE_TreeItem* tree)
	{
		//Delete all items that disappeared
		//(Should only happen at a reset of the DICE data)
		std::list<std::pair<HTREEITEM,long> > copyList = m_items;

		std::list<std::pair<HTREEITEM, long> >::iterator i;
		for (i = copyList.begin(); i != copyList.end();i++)
		{
			//find the item in the tree
			DICE_INTERFACE::CDICE_TreeItem* temp;
			if (tree)
				temp = tree->findDICEid(i->second);
			else
				temp = NULL;

			if (temp != NULL)
				continue;

			//remove it from the "is-expanded" list
			m_expandedList.remove(i->second);

			//Item disappeared, delete it
			DeleteItem(i->first);
			m_items.remove (*i);
		}

		//create the tree
		if (tree)
			addItemRecursive(tree,NULL,tree->getSeverity());

		//set the old selection
		Select (getTreeItemForDICEid(m_selectedID),TVGN_CARET);
		
		return true;
	}

	bool CDICE_TreeWin::addItemRecursive(DICE_INTERFACE::CDICE_TreeItem* item,HTREEITEM parent, double max_sev)
	{
		//find out whether this tree node is already present
		long DICE_id = item->getDICEid();
		HTREEITEM treeItem = NULL;
		bool bWasPresent = false;
		std::list<std::pair<HTREEITEM, long> >::iterator itemIt;
		for (itemIt = m_items.begin(); itemIt != m_items.end();itemIt++)
		{
			if (itemIt->second == DICE_id)
			{
				treeItem = itemIt->first;
				bWasPresent = true;
				break;
			}
		}

		//create the item text and image index
		//stupid string to wstring conversion ...
		char sev_str[128];
		sprintf(sev_str,"[%.1f] ",item->getSeverity());
		std::string org (sev_str);
		org.append (item->getDesc());
		std::wstring str(org.length(), L' '); // Make room for characters
		std::copy(org.begin(), org.end(), str.begin());

		//claculate the correct image index
		int index = (int)((item->getSeverity()/max_sev)*99);

		//already known or do we need to create the tree item ?
		if (treeItem == NULL)
		{
			//create the item
			treeItem = InsertItem ((LPWSTR)str.c_str(),index,index,parent,NULL);
			m_items.push_back(std::make_pair(treeItem,item->getDICEid()));
		}
		else
		{
			SetItemText(treeItem,(LPWSTR)str.c_str());
			SetItemImage(treeItem,index,index);
		}
		
		//recursion
		if (!item->isLeaf())
		{
			//the item is a node, cast it !
			DICE_INTERFACE::CDICE_TreeNode* node = (DICE_INTERFACE::CDICE_TreeNode*) item;

			//add all children
			std::list<DICE_INTERFACE::CDICE_TreeItem*> list = node->getChilds();
			std::list<DICE_INTERFACE::CDICE_TreeItem*>::iterator i;
			
			for (i = list.begin();i != list.end();i++)
			{
				addItemRecursive (*i,treeItem,max_sev);
			}			
		}

		//now that childs are created, have we ever been expanded ?
		if (!bWasPresent)
		{
			/**
			 * TODO: this should be unnecessary now ....
			 */
			std::list<long>::iterator i;
			for (i = m_expandedList.begin(); i != m_expandedList.end();i++)
			{
				if (*i == item->getDICEid())
				{
					Expand(treeItem,TVE_EXPAND);
					break;
				}
			}
		}

		return true;
	}

	long CDICE_TreeWin::getDICEidForTreeITem (HTREEITEM item)
	{
		std::list<std::pair<HTREEITEM, long> >::iterator i;
		for (i = m_items.begin();i != m_items.end();i++)
		{
			if (i->first == item)
			{
				return i->second;
			}
		}

		return -1;
	}

	HTREEITEM CDICE_TreeWin::getTreeItemForDICEid (long id)
	{
		std::list<std::pair<HTREEITEM, long> >::iterator i;
		for (i = m_items.begin();i != m_items.end();i++)
		{
			if (i->second == id)
			{
				return i->first;
			}
		}

		return NULL;
	}

	long CDICE_TreeWin::getSelection (void)
	{
		return m_selectedID;
	}

}//end of namespace DICE_GUI
void DICE_GUI::CDICE_TreeWin::OnTvnItemChanged(NMHDR *pNMHDR, LRESULT *pResult)
{
	NMTVITEMCHANGE *pNMTVItemChange = reinterpret_cast<NMTVITEMCHANGE*>(pNMHDR);
	// TODO: Add your control notification handler code here
	//std::ofstream out("MyFile.txt",std::ios_base::app);
	//out << "TvnItemChanged: id:" << pNMHDR->idFrom << "code:" << pNMHDR->code << std::endl;
	//out.close();

	*pResult = 0;
}

void DICE_GUI::CDICE_TreeWin::OnTvnItemexpanded(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
	// TODO: Add your control notification handler code here
	//std::ofstream out("MyFile.txt",std::ios_base::app);
	//out << "TvnItemExpanded: id:" << pNMHDR->idFrom << "code:" << pNMHDR->code << std::endl;
	//out.close();

	//strange Windows stuff, whats that good for ?
	*pResult = 0;

	//find DICE id for the item
	long id = getDICEidForTreeITem(pNMTreeView->itemNew.hItem);

	//is it already in the expanded list ?
	std::list<long>::iterator i;
	for (i = m_expandedList.begin(); i != m_expandedList.end();i++)
	{
		if (*i == id)
		{
			//it is in the list
			if (pNMTreeView->action == TVE_COLLAPSE)
			{
				//remove it from the expanded list 
				m_expandedList.erase(i);
				return;
			}
			if (pNMTreeView->action == TVE_EXPAND)
			{
				//already in expanded list, nothing to do ;)
				return;
			}
		}
	}

	//not yet in list
	if (pNMTreeView->action == TVE_EXPAND)
	{
		//add it to the expanded list
		m_expandedList.push_back(id);
	}
}

void DICE_GUI::CDICE_TreeWin::OnTvnSelchanged(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);
	// TODO: Add your control notification handler code here
	//std::ofstream out("MyFile.txt",std::ios_base::app);
	//out << "TvnSelchanged: id:" << pNMHDR->idFrom << "code:" << pNMHDR->code << std::endl;
	//out.close();

	std::list<std::pair<HTREEITEM, long> >::iterator i;
	long id = getDICEidForTreeITem(pNMTreeView->itemNew.hItem);
	selectionChanged(id);

	*pResult = 0;
}

int DICE_GUI::CDICE_TreeWin::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CTreeCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;

	// TODO:  Add your specialized creation code here
	//std::ofstream out("MyFile.txt",std::ios_base::app);
	//out << "Tree view create" << std::endl;
	//out.close();

	if (m_imageList != NULL)
		delete m_imageList;

	m_imageList = new CImageList();
	m_imageList->Create (16,16,ILC_COLOR32,100,100);
	DWORD myPicture[16*16];
	
	for (int i = 0; i <= 99; i++)
	{
		CBitmap bitmap;
		COLORREF refcolor = 0x00AAAAAA;
		
		//DWORD color = 0xFF-((DWORD)(((double)(0xFF)/99.0)*i));
		//DWORD halfcolor = 0x00880000 + ((DWORD)(color*0.5)<<8) + (DWORD)(color*0.5);
		//color = 0x00FF0000 + (color<<8) + color;

		DWORD color, halfcolor;
		halfcolor = 0x00888888;

		if (i == 0)
		{
			//halfcolor = 0x00888888;
			color = 0x00FFFFFF;
		}
		else
		if (99.0 / 2.0 > double(i))
		{
			//first 50 %
			DWORD red = ((DWORD)(((double)(0xFF)/(99.0/2.0))*i));
			//halfcolor = 0x0000FF00 + ((DWORD)(red*0.5)<<16);
			color = 0x0000FF00 + (red<<16);
		}
		else
		{
			//second 50%
			DWORD green = 0xFF-((DWORD)(((double)(0xFF)/(99.0/2.0))*(i-99.0/2.0)));
			//halfcolor = 0x00FF0000 + ((DWORD)(green*0.5)<<8);
			color = 0x00FF0000 + (green<<8);
		}

		for (int j = 0; j < 16*16;j++)
		{
			if ( (j < 16) || (j>=16*15) || (j%16==0) || (j%16==15) )
			{
				myPicture[j] = 0x00FFFFFF;
				continue;
			}

			if ( (j < 32) || (j>=16*14) || (j%16==1) || (j%16==14) )
			{
				myPicture[j] =halfcolor;
				continue;
			}

			myPicture[j] = color;
		}

		bitmap.CreateBitmap(16,16,32,1,myPicture);
		m_imageList->Add (&bitmap,refcolor);
	}

	SetImageList(m_imageList,TVSIL_NORMAL);

	return 0;
}