/**********************************************************************
 * @file   Connect.cpp
 * @brief  Implementation of CConnect
 * 
 * @author Blasius Czink
 * @date   12.12.2007
 **********************************************************************
 * Version History:
 * V 0.1  12.12.2007  : First Revision
 **********************************************************************/

#include "stdafx.h"
#include "AddIn.h"
#include "Connect.h"
#include <string>
#include <vector>
#include <sstream>
#include <atlbase.h>

extern CAddInModule _AtlModule;

#if _MSC_VER>=1500
#define ADDIN_NAME L"MarmotAddin"
#define ADDIN_REG_KEY "SOFTWARE\\Microsoft\\VisualStudio\\9.0\\Addins\\MarmotAddin.Connect"
#define TOOLS_REG_KEY "SOFTWARE\\Microsoft\\VisualStudio\\9.0\\ToolWindows"
#elif _MSC_VER==1400
#define ADDIN_NAME L"MarmotAddin"
#define ADDIN_REG_KEY "SOFTWARE\\Microsoft\\VisualStudio\\8.0\\Addins\\MarmotAddin.Connect"
#define TOOLS_REG_KEY "SOFTWARE\\Microsoft\\VisualStudio\\8.0\\ToolWindows"
#else
#define ADDIN_NAME L"MarmotAddin"
#define ADDIN_REG_KEY "SOFTWARE\\Microsoft\\VisualStudio\\7.1\\Addins\\MarmotAddin.Connect"
#define TOOLS_REG_KEY "SOFTWARE\\Microsoft\\VisualStudio\\7.1\\ToolWindows"
#endif

using namespace std;

//#import <VCProjectEngine.dll> raw_interfaces_only raw_native_types
//using namespace VCProjectEngineLibrary;

// CConnect
STDMETHODIMP CConnect::OnConnection(IDispatch *pApplication, ext_ConnectMode ConnectMode, IDispatch *pAddInInst, SAFEARRAY ** /*custom*/ )
{
	HRESULT hr = S_OK;
	pApplication->QueryInterface(__uuidof(DTE2), (LPVOID*)&m_pDTE);
	pAddInInst->QueryInterface(__uuidof(AddIn), (LPVOID*)&m_pAddInInstance);
  //m_pDTE->QueryInterface(__uuidof(ErrorList), (LPVOID*)&m_pErrorList);
  
  CComPtr<IDispatch> pDisp;
  CComQIPtr<Commands> pCommands;
	CComQIPtr<Commands2> pCommands2;
	CComQIPtr<_CommandBars> pCommandBars;
	CComPtr<CommandBarControl> pCommandBarControl;
	CComPtr<CommandBar> pMenuBarCommandBar;
	CComPtr<CommandBarControls> pMenuBarControls;
	CComPtr<CommandBarControl> pToolsCommandBarControl;
	CComQIPtr<CommandBar> pToolsCommandBar;
	CComQIPtr<CommandBarPopup> pToolsPopup;
  
  CComPtr<IDispatch> mytoolbar;
  CComPtr<Command> pCmd;
  CComPtr<Command> pSettingsCmd;
	CComPtr<Command> pRunCmd;
  CComPtr<Command> pCmdLine;

  CComPtr<Microsoft_VisualStudio_CommandBars::_CommandBarButton> pControl;

  // Create a tool window handle for the Output window. 
  {
    CComPtr<EnvDTE::Windows> the_windows;
    IfFailGoCheck(m_pDTE->get_Windows(&the_windows), the_windows);
    CComPtr<EnvDTE::Window> the_output;
    IfFailGoCheck(the_windows->Item(CComVariant(CComBSTR(EnvDTE::vsWindowKindOutput)), &the_output), the_output);
    CComPtr<IDispatch> spOutputWindowDisp;
    IfFailGoCheck(the_output->get_Object(&spOutputWindowDisp), spOutputWindowDisp);
    CComQIPtr<EnvDTE::OutputWindow> spOutputWindow = spOutputWindowDisp;
    if (spOutputWindow == NULL)
      goto Error;
    IfFailGoCheck(spOutputWindow->get_OutputWindowPanes(&m_pOutputWindowPanes), m_pOutputWindowPanes);
    if (SUCCEEDED(m_pOutputWindowPanes->Item(CComVariant(L"Marmot-Output"), &m_pMarmotOutput)) && m_pMarmotOutput)
    {
      //MessageBox(NULL, "output-pane already there", "Info", MB_OK);
      //DumpToOutput("MarmotAddin - Using existing output!\n");
    } else {
      //MessageBox(NULL, "output-pane not yet added", "Info", MB_OK);
      if (SUCCEEDED(m_pOutputWindowPanes->Add(CComBSTR(L"Marmot-Output"), &m_pMarmotOutput)) && m_pMarmotOutput)
      {
        //MessageBox(NULL, "Successfully added output-pane", "Info", MB_OK);
        //DumpToOutput("MarmotAddin - Output added!\n");
      }
    }
    m_pMarmotOutput->Clear();
    // m_pDTE->ExecuteCommand(A2BSTR("Build.Cancel"), A2BSTR(""));
    if (SUCCEEDED(m_pOutputWindowPanes->Item(CComVariant(L"Build"), &m_pBuildOutput)) && m_pBuildOutput)
    {
      m_pBuildOutput->Activate();
      DumpToOutput("MarmotAddin - got existing build-output-pane\n");
    } else {
      if (SUCCEEDED(m_pOutputWindowPanes->Add(CComBSTR(L"Build"), &m_pBuildOutput)) && m_pBuildOutput)
      {
        DumpToOutput("MarmotAddin - added build-output-pane\n");
      }
    }
    m_pBuildOutput->Clear();

#ifdef USE_MARMOT_DICE
	//create the DICE window
	CWin_DiceWindow::GetInstance()->create(m_pDTE,m_pAddInInstance,m_pBuildOutput);
#endif 	
  }

	IfFailGoCheck(m_pDTE->get_Commands(&pCommands), pCommands);
  pCommands2 = pCommands;
  pCommands2->Item(CComVariant(ADDIN_NAME L".Connect.Settings"), 0, &pCmd);
  if (!pCmd)
  {
    // remove old commands
		pCommands2->Item(CComVariant(ADDIN_NAME L".Connect.Run"), 0, &pCmd);
		if (pCmd) pCmd->Delete();
		pCmd=NULL;
		pCommands2->Item(CComVariant(ADDIN_NAME L".Connect.CommandLine"), 0, &pCmd);
		if (pCmd) pCmd->Delete();
		pCmd=NULL;


    // register commands
    if (FAILED(pCommands2->AddNamedCommand2(m_pAddInInstance, CComBSTR("Settings"), CComBSTR("Settings"), CComBSTR("MarmotAddin Settings"), VARIANT_TRUE, CComVariant(3049), NULL, vsCommandStatusSupported+vsCommandStatusEnabled, vsCommandStylePictAndText, vsCommandControlTypeButton, &pSettingsCmd)) && (pSettingsCmd))
      DumpToOutput("Could not AddNamedCommad2(Settings..)");
    if (FAILED(pCommands2->AddNamedCommand2(m_pAddInInstance, CComBSTR("Run"), CComBSTR("Run"), CComBSTR("MarmotAddin Run"), VARIANT_TRUE, CComVariant(6743), NULL, vsCommandStatusSupported+vsCommandStatusEnabled, vsCommandStylePictAndText, vsCommandControlTypeButton, &pRunCmd)) && (pRunCmd))
      DumpToOutput("Could not AddNamedCommad2(Run..)");
    if (FAILED(pCommands2->AddNamedCommand2(m_pAddInInstance, CComBSTR("CommandLine"), CComBSTR("CommandLine"), CComBSTR("MarmotAddin CommadLine"), VARIANT_TRUE, CComVariant(0), NULL, vsCommandStatusSupported+vsCommandStatusEnabled, EnvDTE80::vsCommandStyleText, EnvDTE80::vsCommandControlTypeMRUCombo, &pCmdLine)) && (pCmdLine))
      DumpToOutput("Could not AddNamedCommad2(CommandLine..)");

    // create our toolbar
    pCommands2->AddCommandBar(CComBSTR(ADDIN_NAME), vsCommandBarTypeToolbar, NULL, 0, &mytoolbar.p);
    // show the toolbar
		((Microsoft_VisualStudio_CommandBars::CommandBar*)mytoolbar.p)->put_Visible(VARIANT_TRUE);
    // add the buttons
    pSettingsCmd->AddControl(mytoolbar, 1, (IDispatch**)&pControl);
		pControl->put_TooltipText(CComBSTR(ADDIN_NAME L" Settings"));
		pControl->put_Style(Microsoft_VisualStudio_CommandBars::msoButtonIcon);
    pControl = NULL;
    pRunCmd->AddControl(mytoolbar, 2, (IDispatch**)&pControl);
		pControl->put_TooltipText(CComBSTR(ADDIN_NAME L" Run"));
		pControl->put_Style(Microsoft_VisualStudio_CommandBars::msoButtonIcon);
    pControl = NULL;
    pCmdLine->AddControl(mytoolbar, 3, (IDispatch**)&pControl);
		pControl->put_TooltipText(CComBSTR(ADDIN_NAME L" CommandLine"));
    pControl = NULL;

    //pControl->put_Enabled(VARIANT_TRUE);
    //pControl->put_Visible(VARIANT_TRUE);

	return S_OK;
  }

  // create m_comm
  m_comm = new CMarmotComm(m_pDTE, m_pMarmotOutput, m_pBuildOutput);
  if (m_comm && m_comm->init())
  {
    m_marmot_log_host = m_comm->getMarmotLogHost();
    m_marmot_log_port = m_comm->getMarmotLogPort();
  } else {
    DumpToOutput("Could not init MarmotComm!");
  }

Error:
	return hr;
}

STDMETHODIMP CConnect::OnDisconnection(ext_DisconnectMode /*RemoveMode*/, SAFEARRAY ** /*custom*/ )
{
  if (m_comm)
  {
    delete m_comm; m_comm = NULL;
  }
  DumpToOutput("MarmotAddin disconnected!\n");

#ifdef USE_MARMOT_DICE
	//remove the DICE window
	CWin_DiceWindow::GetInstance()->close();
#endif 	  

  m_pMarmotOutput = NULL;
  m_pOutputWindowPanes = NULL;
  m_pDTE = NULL;
	m_pAddInInstance = NULL;
	return S_OK;
}

STDMETHODIMP CConnect::OnAddInsUpdate (SAFEARRAY ** /*custom*/ )
{
	return S_OK;
}

STDMETHODIMP CConnect::OnStartupComplete (SAFEARRAY ** /*custom*/ )
{
	return S_OK;
}

STDMETHODIMP CConnect::OnBeginShutdown (SAFEARRAY ** /*custom*/ )
{
#ifdef USE_MARMOT_DICE
	//remove the DICE window
	CWin_DiceWindow::GetInstance()->close();
#endif 
	return S_OK;
}

STDMETHODIMP CConnect::QueryStatus(BSTR bstrCmdName, vsCommandStatusTextWanted NeededText, vsCommandStatus *pStatusOption, VARIANT *pvarCommandText)
{
  if(NeededText == vsCommandStatusTextWantedNone)
	{
    if ((!_wcsicmp(bstrCmdName, ADDIN_NAME L".Connect.Settings")) || (!_wcsicmp(bstrCmdName, ADDIN_NAME L".Connect.Run")) || (!_wcsicmp(bstrCmdName, ADDIN_NAME L".Connect.CommandLine")) )
	  {
		  *pStatusOption = (vsCommandStatus)(vsCommandStatusEnabled+vsCommandStatusSupported);
	  }
  }
	return S_OK;
}

STDMETHODIMP CConnect::Exec(BSTR bstrCmdName, vsCommandExecOption ExecuteOption, VARIANT *pvarVariantIn, VARIANT * /*pvarVariantOut*/, VARIANT_BOOL *pvbHandled)
{
  HRESULT hr;
	*pvbHandled = VARIANT_FALSE;
  USES_CONVERSION;

  CComPtr<Microsoft_VisualStudio_CommandBars::CommandBarControl> pControl;
  CComQIPtr<Microsoft_VisualStudio_CommandBars::_CommandBarComboBox> pComboBox;

  if(ExecuteOption == vsCommandExecOptionDoDefault)
	{
    //
    // the "settings" button on the marmot toolbar was clicked
    //
		if(!_wcsicmp(bstrCmdName, ADDIN_NAME L".Connect.Settings"))
		{
      DumpToOutput("Settings-Exec() - called!\n");
      VARIANT vdest;
      vdest.vt = VT_EMPTY;

      CComPtr<Project> pProject;
      string projname;
      if ((GetStartupProject(pProject, projname) == S_OK) && pProject)
      {
        // get the active configuration and it's properties
        CComPtr<ConfigurationManager> pConfManager;
        IfFailGoCheck(pProject->get_ConfigurationManager(&pConfManager), pConfManager);
        
        CComPtr<Configuration> pConfig;
        IfFailGoCheck(pConfManager->get_ActiveConfiguration(&pConfig), pConfig);
        CComBSTR config_name;
        pConfig->get_ConfigurationName(&config_name);
        std::string config_str = W2CA(BSTR(config_name));
        //DumpToOutput(config_str.c_str());

        CComQIPtr<Properties> pProperties;
        IfFailGoCheck(pConfig->get_Properties(&pProperties), pProperties);
        if (SUCCEEDED(GetPropertyValue(pProperties, "OutputPath", &vdest)))
        {
          // get the proj. properties
          CComQIPtr<Properties> pProjProperties;
          IfFailGoCheck(pProject->get_Properties(&pProjProperties), pProjProperties);
          VARIANT kindvar;
          kindvar.vt = VT_EMPTY;
          if (SUCCEEDED(GetPropertyValue(pProjProperties, "Kind", &kindvar)))
          {
            CComBSTR kindbstr(kindvar.bstrVal);
            if (kindbstr == "VCProject")
            {
              // ok. we have a vcproject
              string hstr, vcprojfullpath = W2CA(vdest.bstrVal);
              hstr = projname;
              hstr.append(".vcproj");
              string exefullpath;
              if (GetProjOutputFile(pProject, config_str, exefullpath) != S_OK)
              {
                DumpToOutput("Could not get linker tool output file name (using fallback)\n");
                exefullpath = vcprojfullpath;
                exefullpath.append(projname);
                exefullpath.append(".exe");
              } else {
                // replace macros $(OutDir) and $(ProjectName) if neccessary
                transform(exefullpath.begin(), exefullpath.end(), exefullpath.begin(), (int(*)(int))tolower);
                hstr = "$(outdir)";
                string::size_type pos = exefullpath.find(hstr);
                if (pos != string::npos)
                {
                  exefullpath.replace(pos, hstr.length(), vcprojfullpath);
                }
                hstr = "$(projectname)";
                pos = exefullpath.find(hstr);
                if (pos != string::npos)
                {
                  exefullpath.replace(pos, hstr.length(), projname);
                }
                // check whether outputfile ends with ".exe" and give a warning otherwise
                pos = exefullpath.rfind(".exe");
                if (pos == string::npos)
                {
                  DumpToOutput("OutputFile's extension is not '.exe' !!! Did you set the correct StartUp-Project?\n");
                }
              }
              // try to get a "short-path-name"
              char shortPath[MAX_PATH];
              if (GetShortPathName(exefullpath.c_str(), shortPath, sizeof(shortPath)) != 0)
              {
                exefullpath = shortPath;
              } else {
                // GetShortPathName() failed, so simply use the long path-name and add ""
                exefullpath.insert(0, "\"");
                exefullpath.append("\"");
              }
              hstr = std::string("OutputFileName is ") + exefullpath;
              DumpToOutput(hstr.c_str());
              DumpToOutput("\n");
              // create an example command-line with mpiexec
              char portstr[20];
              NetUtil::itoa(m_marmot_log_port, portstr, 10);
              string commandline("mpiexec -env MARMOT_LOG_HOST ");
              commandline.append(m_marmot_log_host);
              commandline.append(" -env MARMOT_LOG_PORT ");
              commandline.append(portstr);
              // also add MARMOT_HOME if set (as short pathname)
              string marmot_home;
              if (getMarmotHome(marmot_home))
              {
                commandline.append(" -env MARMOT_HOME ");
                commandline.append(marmot_home);
              }
              commandline.append(" -n 4 ");
              commandline.append(exefullpath);
              //DumpToOutput(commandline.c_str());
              //DumpToOutput("\n");

              //  place commandline in the dropdown widget
              int control_count;
              if (SUCCEEDED(GetToolbarControl(3, control_count, pControl)))
              {
                pComboBox = pControl;
                // is it a combobox?
                if (pComboBox)
                {
                  HRESULT combo_err;
                  // set item
                  CComBSTR myparam(commandline.c_str());
                  combo_err = pComboBox->put_Text(BSTR(myparam));
                  if (SUCCEEDED(combo_err))
                  {
                    // DumpToOutput("put_Text() successful!\n");
                  } else {
                    DumpToOutput("Could not add command-line to combo-box!\n");
                  }
                }
              }
              // ----
            }
          }
        }
      }

			*pvbHandled = VARIANT_TRUE;
      //DumpMarmotMessage(vsTaskPriorityHigh, "c:\\myprojects\\mpihello2\\mpihello.cc", 28, "Error-MPI_Recv: Use of MPI_ANY_SOURCE may cause race conditions!");
      //DumpMarmotMessage(vsTaskPriorityMedium, "c:\\myprojects\\mpihello2\\mpihello.cc", 28, "Warning-MPI_Recv: Use of MPI_ANY_SOURCE may cause race conditions!");
      //DumpMarmotMessage(vsTaskPriorityLow, "c:\\myprojects\\mpihello2\\mpihello.cc", 28, "Info-MPI_Recv: Use of MPI_ANY_SOURCE may cause race conditions!");
			return S_OK;
		}

    //
    // the "run" button on the marmot toolbar was clicked
    //
		if(!_wcsicmp(bstrCmdName, ADDIN_NAME L".Connect.Run"))
		{
      DumpToOutput("Run-Exec() - called!\n");
      // take command line from the combo-box
      // and try to launch it
      int control_count;
      if (SUCCEEDED(GetToolbarControl(3, control_count, pControl)))
      {
        pComboBox = pControl;
        // is it a combobox?
        if (pComboBox)
        {
          // get currently selected item
          BSTR myparam;
          if (SUCCEEDED(pComboBox->get_List(1, &myparam)))
          {
            DumpToOutput("going to launch:");
            DumpToOutput(W2CA(myparam));
            DumpToOutput("\n");
            // launch
            if (m_comm)
            {
              m_pBuildOutput->Clear();
              m_comm->setAppToLaunch(W2CA(myparam));
            }
            // Wait until child process exits.
            //WaitForSingleObject(pi.hProcess, INFINITE);
            // Close process and thread handles. 
            //CloseHandle(pi.hProcess);
            //CloseHandle( pi.hThread );
          }
        }
      }
			*pvbHandled = VARIANT_TRUE;
			return S_OK;
		}

    //
    // input in the "command-line" widget
    //
		if(!_wcsicmp(bstrCmdName, ADDIN_NAME L".Connect.CommandLine"))
		{
            

/*
      IfFailGoCheck(pSol->get_Properties(&pProperties), pProperties);
      if (FAILED(pProperties->Item(CComVariant("OutputFileName"), &pProp)))
      {
        DumpToOutput("getProperty on OutputFileName failed");
      } else {
        if (FAILED(pProp->get_Value(&myvari)))
        {
          DumpToOutput("get_Value failed");
        } else {
          string fnstr = W2CA(myvari.bstrVal);
          DumpToOutput(fnstr.c_str());
        }
      }
*/
      string tmpstr, cmdlinestr;
      if (pvarVariantIn->vt==VT_BSTR && pvarVariantIn->bstrVal)
      {
        cmdlinestr = W2CA(pvarVariantIn->bstrVal);
      }
      tmpstr = "CommandLine Exec(";
      tmpstr.append(cmdlinestr);
      tmpstr.append(") - called!\n");
      DumpToOutput(tmpstr.c_str());

			*pvbHandled = VARIANT_TRUE;
			return S_OK;
		}
  }
Error:
	return S_OK;
}

STDMETHODIMP CConnect::DumpToOutput(LPCSTR mes)
{
  if (m_pMarmotOutput)
  {
    // Write text to this new pane
    m_pMarmotOutput->OutputString(A2BSTR(mes));
    return S_OK;
  }
  return S_FALSE;
}

STDMETHODIMP CConnect::DumpMarmotMessage(EnvDTE::vsTaskPriority priority, LPCSTR filename, long line_no, LPCSTR mes)
{
    // test output to "Error List"
    //CComPtr<EnvDTE80::ErrorItems> pErrorItems;
    //IfFailGoCheck(m_pErrorList->get_ErrorItems(&pErrorItems), pErrorItems);

    // get the Build Output Pane
    if (m_pBuildOutput)
    {
      char line_str[20];
      _itoa_s(line_no, line_str, 20, 10);
      string prep_mes = filename + string("(") + string(line_str) + ")" + " : " + mes;

      //m_pBuildOutput->OutputTaskItemString(A2BSTR("c:\\myprojects\\mpihello2\\mpihello.cc(28) : MPI_Recv: Use of MPI_ANY_SOURCE may cause race conditions!\n"), vsTaskPriorityHigh, A2BSTR("MyCategory"), vsTaskIconCompile, A2BSTR("C:\\MyProjects\\mpihello2\\mpihello.cc"), 28, A2BSTR(mes), true);
      m_pBuildOutput->OutputTaskItemString(A2BSTR(prep_mes.c_str()), priority, A2BSTR(vsTaskCategoryUser), vsTaskIconCompile, A2BSTR(filename), 28, A2BSTR(mes), true);
      m_pDTE->ExecuteCommand(A2BSTR("Build.Cancel"), A2BSTR(""));
      m_pBuildOutput->OutputString(A2BSTR("\nOutput from MarmotAddin - done.\n\n"));
      m_pBuildOutput->ForceItemsToTaskList();
      return S_OK;
    }
    return S_FALSE;
}

// workaround for ULONG_PTR (namespace collision problem)
#define MY_HKEY_LOCAL_MACHINE (( HKEY ) (__int32)((LONG)0x80000002) )
bool CConnect::getMarmotHome(std::string &marmot_home)
{
   bool retval = false;
   char shortPath[MAX_PATH];
   marmot_home.clear();
   char *marmothomepath = getenv("MARMOT_HOME");
   if (marmothomepath)
   {
     // try to get short path-name
     if (GetShortPathName(marmothomepath, shortPath, sizeof(shortPath)) != 0)
     {
        marmot_home = shortPath;
     } else {
        // GetShortPathName() failed, so simply use the value of the env.var and add ""
        marmot_home = "\"";
        marmot_home.append(marmothomepath);
        marmot_home.append("\"");
     }
     return true;
   } else {
     // try registry if env.var MARMOT_HOME was not set
     HKEY hKey;
     TCHAR szMarmotInstallationPath[MAX_PATH];
     DWORD dwBufLen=MAX_PATH*sizeof(TCHAR);
     LONG lRet = RegOpenKeyEx(MY_HKEY_LOCAL_MACHINE, TEXT("SOFTWARE\\HLRS\\Marmot"), 0, KEY_QUERY_VALUE|KEY_WOW64_32KEY, &hKey);
     if (lRet != ERROR_SUCCESS) // try reflected
       LONG lRet = RegOpenKeyEx(MY_HKEY_LOCAL_MACHINE, TEXT("SOFTWARE\\HLRS\\Marmot"), 0, KEY_QUERY_VALUE|KEY_WOW64_64KEY, &hKey);
     if (lRet == ERROR_SUCCESS)
     {
       lRet = RegQueryValueEx(hKey, NULL, NULL, NULL, (LPBYTE)szMarmotInstallationPath, &dwBufLen);
       if (lRet == ERROR_SUCCESS)
       {
         // try to get short path-name
         if (GetShortPathName(szMarmotInstallationPath, shortPath, sizeof(shortPath)) != 0)
         {
           marmot_home = shortPath;
           retval = true;
         }
       }
       RegCloseKey(hKey);
     }
     return retval;
   }
   return retval;
}

void CConnect::listAllProps(CComPtr<Property> &propPtr)
{
  USES_CONVERSION;
  if (propPtr != NULL)
  {
    
    CComPtr<Properties> pProperties;
    if (FAILED(propPtr->get_Collection(&pProperties)))
      return;
    long prop_count = 0;
    if (FAILED(pProperties->get_Count(&prop_count)))
      return;
    if (prop_count <= 0)
      return;
    for (long i=1; i<=prop_count; i++)
    {
      CComVariant propIndex = i;
      CComPtr<Property> pProperty = NULL;
      if (FAILED(pProperties->Item(propIndex, &pProperty)))
        return;
      if (pProperty == NULL)
        return;
      CComBSTR propname;
      if (FAILED(pProperty->get_Name(&propname)))
        return;
      string hstr = W2CA(BSTR(propname));
      hstr.append("\n");
      DumpToOutput(hstr.c_str());
    }
  }
}

STDMETHODIMP CConnect::GetStartupProject(CComPtr<Project> &pProject, std::string &projname)
{
  USES_CONVERSION;

  HRESULT hr;
  VARIANT v, vdest;
  v.vt = VT_EMPTY;
  vdest.vt = VT_EMPTY;

  CComQIPtr<_Solution> pSol;
  CComQIPtr<Properties> pProperties;
  CComQIPtr<Projects> pProjects;
  CComPtr<Property> pProp;
  CComPtr<SolutionBuild> pSolBuild;

  hr = m_pDTE->get_Solution(&pSol);
  if (FAILED(hr) || !pSol)
  {
    DumpToOutput("pDTE->get_Solution() failed!\n");
    goto Error;
  }
  hr = pSol->get_SolutionBuild(&pSolBuild);
  if (FAILED(hr) || !pSolBuild)
  {
    DumpToOutput("pSol->get_SolutionBuild() failed!\n");
    goto Error;
  }
  hr = pSol->get_Projects(&pProjects);
  if (FAILED(hr) || !pProjects)
  {
    DumpToOutput("pSol->get_Projects() failed!\n");
    goto Error;
  }
  hr = pSolBuild->get_StartupProjects(&v);
  if (SUCCEEDED(hr))
  {
    long i = 0;
    if (SUCCEEDED(SafeArrayGetElement(v.parray, &i, &vdest)))
    {
      // strip extension & path part from startup-project name
      string hstr;
      string rel_proj_name = W2CA(vdest.bstrVal);
      string workstr = rel_proj_name;
      hstr = workstr.substr(workstr.length()-7);
      transform(hstr.begin(), hstr.end(), hstr.begin(), (int(*)(int))tolower);
      if (hstr == ".vcproj")
        workstr.erase(workstr.length()-7);
      string::size_type pos = workstr.find_last_of("/\\");
      if (pos != string::npos)
        workstr.erase(0, pos+1);
      projname = workstr;
      //DumpToOutput(workstr.c_str());
      //DumpToOutput("\n");
      // iterate over the projects and "find" the startup project
      // why is there no simpler way to get the CComPtr<Project> of the startup project ??? 
      long lCount = 0;
      if (SUCCEEDED(pProjects->get_Count(&lCount)))
      {
        for (i=1; i<=lCount; i++)
        {
          //hstr = "Comparing to: ";
          CComVariant projectIndex = i;
          CComPtr<Project> pCurrProj;
          pProjects->Item(projectIndex, &pCurrProj);
          CComBSTR name; //startup_name=workstr.c_str();
          //if (SUCCEEDED(pCurrProj->get_Name(&name)))
          if (SUCCEEDED(pCurrProj->get_FileName(&name)))
          {
            //hstr.append(W2CA(BSTR(name)));
            //hstr.append("\n");
            //DumpToOutput(hstr.c_str());
            hstr = W2CA(BSTR(name));
            string::size_type pos = hstr.find(rel_proj_name);
            if (pos != string::npos)
            //if (name == startup_name)
            {
              pProject = pCurrProj;
              return S_OK;
            }
          }
        }
      }
    }
  } else {
    DumpToOutput("pSolBuild->get_StartupProjects() failed!\n");
  }
Error:
  DumpToOutput("Could not get startup projects!\n");
  return S_FALSE;
}

STDMETHODIMP CConnect::GetPropertyValue(CComQIPtr<Properties> &pProperties, LPCSTR lookupname, VARIANT *pOutputVar)
{
  USES_CONVERSION;
  // small check
  if ((lookupname==NULL) || (pOutputVar==NULL))
    return S_FALSE;

  long prop_count = 0;
  pProperties->get_Count(&prop_count);
  for (long j=1; j<=prop_count; j++)
  {
    CComVariant propIndex = j;
    CComPtr<Property> pProperty;
    pProperties->Item(propIndex, &pProperty);
    CComBSTR propname;
    pProperty->get_Name(&propname);

    if (propname == lookupname)
    {
      pOutputVar->vt = VT_EMPTY;
      if (SUCCEEDED(pProperty->get_Value(pOutputVar)))
      {
        //string hstr = W2CA(pOutputVar->bstrVal);
        //hstr.append("\n");
        //DumpToOutput(hstr.c_str());
        return S_OK;
      } else {
        //DumpToOutput("Could not get value");
        return S_FALSE;
      }
    }
  }
  return S_FALSE;
}

STDMETHODIMP CConnect::GetToolbarControl(int idx, int &control_count, CComPtr<Microsoft_VisualStudio_CommandBars::CommandBarControl> &pControl)
{
  HRESULT hr;
  CComPtr<Microsoft_VisualStudio_CommandBars::CommandBarControl> pTmpCtrl;
  CComPtr<CommandBar> pMarmotCmdBar;
  CComPtr<Microsoft_VisualStudio_CommandBars::CommandBarControls> pControls;
  CComQIPtr<_CommandBars> pCommandBars;
  CComPtr<IDispatch> pDisp;

  IfFailGoCheck(m_pDTE->get_CommandBars(&pDisp), pDisp);
	pCommandBars = pDisp;
  IfFailGoCheck(pCommandBars->get_Item(CComVariant(L"MarmotAddin"), &pMarmotCmdBar), pMarmotCmdBar);
  pMarmotCmdBar->get_Controls(&pControls);
  control_count = 0;
  if (SUCCEEDED(pControls->get_Count(&control_count)))
  {
    if ((idx > control_count) || (idx < 1))
      return S_FALSE;
    //  CComPtr<Microsoft_VisualStudio_CommandBars::_CommandBarComboBox> pComboBox;
    if (SUCCEEDED(pControls->get_Item(CComVariant(idx), &pTmpCtrl)))
    {
      pControl = pTmpCtrl;
      return S_OK;
    }
  }
Error:
  return S_FALSE;
}


STDMETHODIMP CConnect::GetProjOutputFile(CComPtr<Project> &pProject, const std::string &configname, std::string &outputfile)
{
  USES_CONVERSION;
  HRESULT hr = S_OK;
  if (!pProject)
  {
    DumpToOutput("Error: GetProjOutputFile() - pProject == NULL\n");
    return S_FALSE;
  }

  CComPtr<IDispatch> spDispProj;
  hr = pProject->get_Object(&spDispProj);
  if (FAILED(hr) || !spDispProj)
  {
    DumpToOutput("Error: cannot get the IDispatch interface for the project.\n");
    return S_FALSE;
  }
  
  CComQIPtr<VCProject> pVCProject = spDispProj;
  if (!pVCProject)
  {
    DumpToOutput("Error: cannot query VCProject interface.\n");
    return S_FALSE;
  }
  
  //DumpToOutput("Got VCProject interface\n");

  // Access the configurations collection of the project
	CComPtr<IDispatch> spDispConfigs;
	hr = pVCProject->get_Configurations( &spDispConfigs );
	if (FAILED(hr) || !spDispConfigs)
	{
		DumpToOutput("Error: cannot access the configurations collecion.\n");
		return S_FALSE;
	}
	CComQIPtr<IVCCollection> spConfigCol = spDispConfigs;

	// Enumerate the Configurations collection by name to get the Debug configuration of the project
	CComPtr<IDispatch> spDispConfig;
	hr = spConfigCol->Item(CComVariant(configname.c_str()), &spDispConfig);
	if (FAILED(hr) || !spDispConfig)
	{
		DumpToOutput("Error: cannot get the Debug configuraion.\n");
		return S_FALSE;
	}
	CComQIPtr<VCConfiguration> spConfig = spDispConfig;

	// Access the Tools collection for this configuration
	CComPtr<IDispatch> spDispTools;
	hr = spConfig->get_Tools(&spDispTools);
	if (FAILED(hr))
	{
		DumpToOutput("Error: cannot access the tools collection.\n");
		return hr;
	}
	CComQIPtr<IVCCollection> spToolCol = spDispTools;

	// Enumerate the Tools collection by name to get the Linker tool
	CComPtr<IDispatch> spDispTool;
	hr = spToolCol->Item(CComVariant(L"VCLinkerTool"), &spDispTool);
	if (FAILED(hr))
  {
    DumpToOutput("Error: cannot get IDispatch interface of the linker tool.\n");
    return hr;
  }

	// Get the VCLinkerTool interface and directly change a property
	CComPtr<VCLinkerTool> spLinker;
	hr = spDispTool->QueryInterface( __uuidof( VCLinkerTool ), (void**)&spLinker );
	if (FAILED(hr) || !spLinker)
  {
    DumpToOutput("Error: cannot get the raw interface of the linker tool.\n");
    return S_FALSE;
  }
  
  CComBSTR outputfile_name;
  hr = spLinker->get_OutputFile(&outputfile_name);
  if (FAILED(hr))
  {
    DumpToOutput("Error: could not get outputfile-name.\n");
    return hr;
  }
  outputfile = W2CA(BSTR(outputfile_name));

  return S_OK;
}
