/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**********************************************************************
 * @file   Connect.h
 * @brief  Declaration of the CConnect class 
 * 
 * @author Blasius Czink
 * @date   12.12.2007
 **********************************************************************
 * Version History:
 * V 0.1  12.12.2007  : First Revision
 **********************************************************************/
#pragma once
#include <atlbase.h>

#include "resource.h"       // main symbols
#include "SocketLib.h"
#include "MarmotComm.h"

#ifdef USE_MARMOT_DICE
	#include "Win_DiceWindow.h"
#endif

using namespace AddInDesignerObjects;
using namespace EnvDTE;
using namespace EnvDTE80;
using namespace Microsoft_VisualStudio_CommandBars;

#import <VCProjectEngine.dll> raw_interfaces_only raw_native_types
using namespace VCProjectEngineLibrary;

/// <summary>The object for implementing an Add-in.</summary>
/// <seealso class='IDTExtensibility2' />
class ATL_NO_VTABLE CConnect : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CConnect, &CLSID_Connect>,
	public IDispatchImpl<IDTCommandTarget, &IID_IDTCommandTarget, &LIBID_EnvDTE, 7, 0>,
	public IDispatchImpl<_IDTExtensibility2, &IID__IDTExtensibility2, &LIBID_AddInDesignerObjects, 1, 0>
{
public:
	/// <summary>Implements the constructor for the Add-in object. Place your initialization code within this method.</summary>
	CConnect() : m_comm(NULL)
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_ADDIN)
DECLARE_NOT_AGGREGATABLE(CConnect)

BEGIN_COM_MAP(CConnect)
	COM_INTERFACE_ENTRY(IDTExtensibility2)
	COM_INTERFACE_ENTRY(IDTCommandTarget)
	COM_INTERFACE_ENTRY2(IDispatch, IDTExtensibility2)
END_COM_MAP()



	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
	}

public:
//IDTExtensibility2 implementation:

	/// <summary>Implements the OnConnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being loaded.</summary>
	/// <param term='application'>Root object of the host application.</param>
	/// <param term='connectMode'>Describes how the Add-in is being loaded.</param>
	/// <param term='addInInst'>Object representing this Add-in.</param>
	/// <seealso class='IDTExtensibility2' />
	STDMETHOD(OnConnection)(IDispatch * Application, ext_ConnectMode ConnectMode, IDispatch *AddInInst, SAFEARRAY **custom);

	/// <summary>Implements the OnDisconnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being unloaded.</summary>
	/// <param term='disconnectMode'>Describes how the Add-in is being unloaded.</param>
	/// <param term='custom'>Array of parameters that are host application specific.</param>
	/// <seealso class='IDTExtensibility2' />
	STDMETHOD(OnDisconnection)(ext_DisconnectMode RemoveMode, SAFEARRAY **custom );

	/// <summary>Implements the OnAddInsUpdate method of the IDTExtensibility2 interface. Receives notification when the collection of Add-ins has changed.</summary>
	/// <param term='custom'>Array of parameters that are host application specific.</param>
	/// <seealso class='IDTExtensibility2' />	
	STDMETHOD(OnAddInsUpdate)(SAFEARRAY **custom );

	/// <summary>Implements the OnStartupComplete method of the IDTExtensibility2 interface. Receives notification that the host application has completed loading.</summary>
	/// <param term='custom'>Array of parameters that are host application specific.</param>
	/// <seealso class='IDTExtensibility2' />
	STDMETHOD(OnStartupComplete)(SAFEARRAY **custom );

	/// <summary>Implements the OnBeginShutdown method of the IDTExtensibility2 interface. Receives notification that the host application is being unloaded.</summary>
	/// <param term='custom'>Array of parameters that are host application specific.</param>
	/// <seealso class='IDTExtensibility2' />
	STDMETHOD(OnBeginShutdown)(SAFEARRAY **custom );
	
//IDTCommandTarget implementation:

	/// <summary>Implements the QueryStatus method of the IDTCommandTarget interface. This is called when the command's availability is updated</summary>
	/// <param term='commandName'>The name of the command to determine state for.</param>
	/// <param term='neededText'>Text that is needed for the command.</param>
	/// <param term='status'>The state of the command in the user interface.</param>
	/// <param term='commandText'>Text requested by the neededText parameter.</param>
	/// <seealso class='Exec' />
	STDMETHOD(QueryStatus)(BSTR CmdName, vsCommandStatusTextWanted NeededText, vsCommandStatus *StatusOption, VARIANT *CommandText);

	/// <summary>Implements the Exec method of the IDTCommandTarget interface. This is called when the command is invoked.</summary>
	/// <param term='commandName'>The name of the command to execute.</param>
	/// <param term='executeOption'>Describes how the command should be run.</param>
	/// <param term='varIn'>Parameters passed from the caller to the command handler.</param>
	/// <param term='varOut'>Parameters passed from the command handler to the caller.</param>
	/// <param term='handled'>Informs the caller if the command was handled or not.</param>
	/// <seealso class='Exec' />
	STDMETHOD(Exec)(BSTR CmdName, vsCommandExecOption ExecuteOption, VARIANT *VariantIn, VARIANT *VariantOut, VARIANT_BOOL *Handled);
  
  STDMETHOD(DumpToOutput)(LPCSTR mes);
  STDMETHOD(DumpMarmotMessage)(EnvDTE::vsTaskPriority priority, LPCSTR filename, long line_no, LPCSTR mes);

private:
  bool getMarmotHome(std::string &marmot_home);
  void listAllProps(CComPtr<Property> &propPtr);
  STDMETHOD(GetStartupProject)(CComPtr<Project> &pProject, std::string &projname);
  STDMETHOD(GetPropertyValue)(CComQIPtr<Properties> &pProperties, LPCSTR lookupname, VARIANT *pOutputVar);
  STDMETHOD(GetToolbarControl)(int idx, int &control_count, CComPtr<Microsoft_VisualStudio_CommandBars::CommandBarControl> &pControl);
  STDMETHOD(GetProjOutputFile)(CComPtr<Project> &pProject, const std::string &configname, std::string &outputfile);

  CComPtr<DTE2> m_pDTE;
  CComPtr<AddIn> m_pAddInInstance;
  //CComPtr<VCProjectEngine> m_pProjEngine;
  //CComPtr<ErrorList> m_pErrorList;

  //CComPtr<Command> m_pCreatedCommand;
  CComPtr<OutputWindowPane>  m_pMarmotOutput;
  CComPtr<OutputWindowPane>  m_pBuildOutput;
  CComPtr<OutputWindowPanes> m_pOutputWindowPanes;
  
  // For communication with Marmot
  CMarmotComm *m_comm;
  std::string  m_marmot_log_host;
  int          m_marmot_log_port;
  //CComPtr<Microsoft_VisualStudio_CommandBars::_CommandBarButton> m_pComboControl;
};

OBJECT_ENTRY_AUTO(__uuidof(Connect), CConnect)
