/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
/**********************************************************************
 * @file   MarmotComm.h
 * @brief  Declaration of the CMarmotComm class 
 * 
 * @author Blasius Czink
 * @date   26.02.2008
 **********************************************************************
 * Version History:
 * V 0.1  26.02.2008  : First Revision
 **********************************************************************/

#ifndef MarmotComm_H
#define MarmotComm_H

#include "stdafx.h"
#include "AddIn.h"

#include "ThreadLib.h"
#include "SocketLib.h"

using namespace AddInDesignerObjects;
using namespace EnvDTE;
using namespace EnvDTE80;
using namespace Microsoft_VisualStudio_CommandBars;

class CMarmotComm : public CThread
{
public:
  CMarmotComm(DTE2 *pDTE, OutputWindowPane *pMarmotOutput, OutputWindowPane *pBuildOutput);
  virtual ~CMarmotComm();

  virtual bool init();
  virtual void waitForFinish();

  virtual std::string getMarmotLogHost() { return m_marmot_log_host; };
  virtual int  getMarmotLogPort()        { return m_marmot_log_port; };
  virtual void setAppToLaunch(const char *cmdline);

  STDMETHOD(OutputVS)(LPCSTR mes);
  STDMETHOD(OutputMarmotMessage)(EnvDTE::vsTaskPriority priority, LPCSTR filename, long line_no, LPCSTR mes);
  
  virtual void run();

  enum MPO_MSG_TYPE
  {
    MSG_TYPE_INFO = 0,
    MSG_TYPE_WARNING,
    MSG_TYPE_ERROR,
    MSG_TYPE_NOTE
  };
  
  static const char MARMOT_MSG_DELIM[];

private:
  CMarmotComm(const CMarmotComm& ) {}
	CMarmotComm& operator=(const CMarmotComm& ) { return *this; }

  // helper to actually launch application
  bool appLaunch();

  DTE2 *m_pDTE;
  OutputWindowPane *m_pMarmotOutput;
  OutputWindowPane *m_pBuildOutput;

  CTcpSocket *m_serversocket;
  CTcpSocket *m_channel;

  std::string m_apptolaunch;
  std::string m_marmot_log_host;
  int         m_marmot_log_port;
  unsigned int m_poll_limit;

  bool        m_finished, m_initialized;
  static	    CMutex m_output_mutex;
  static      CMutex m_applaunch_mutex;
};

#endif
