/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
#include <QtCore>
#include <QString>
#include <QFile>
#include <QDir>
#include "MarmotGUIConfiguration.h"

#if !defined(DEBUG) && !defined(_DEBUG)
#define QT_NO_DEBUG_OUTPUT
#endif

MarmotGUIConfiguration::MarmotGUIConfiguration()
{
	qDebug("DEBUG: MarmotGUIConfiguration::MarmotGUIConfiguration() called\n");
  setDefaultConf();
}

void MarmotGUIConfiguration::setConfFile(QString filename)
{
	qDebug("DEBUG: MarmotGUIConfiguration::setConfFile(%s) called", qPrintable(filename));
  if (filename.isEmpty())
  {
    setDefaultConf();
  } else {
	  m_filename = filename;
  }
}

QString MarmotGUIConfiguration::getConfFile()
{
  return m_filename;
}

bool MarmotGUIConfiguration::getConfiguration()
{
  QString temp_str;
  ParseErrCodesEnv *temp_err;
  m_ini.setPath(m_filename);
  if (!m_ini.readFile())
  {
    qDebug("DEBUG: Could not read conf-file (%s)", qPrintable(m_filename));
    return false;
  }

  m_ini.getValue("MarmotGUI", "Exe", m_exe_path);
  m_ini.getValue("MarmotGUI", "ExeArgs", m_exe_arguments);
  m_ini.getValue("MarmotGUI", "ExeNumProcs", m_exe_numprocs);
  m_ini.getValue("MarmotGUI", "DebugMode", m_env_debugmode);
  m_ini.getValue("MarmotGUI", "LogfileType", m_env_logfiletype);
  m_ini.getValue("MarmotGUI", "LogHost", m_env_loghost);
  m_ini.getValue("MarmotGUI", "LogPort", m_env_logport);
  m_ini.getValue("MarmotGUI", "MpiexecEnvParam", m_mpiexec_env_param);  // read only for the moment
  m_ini.getValue("MarmotGUI", "ErrCodes", temp_str);
  temp_err=new ParseErrCodesEnv(temp_str.toStdString(),181,1);
  temp_err->evaluate();
  temp_err->getErrCodes(m_errcodes);
  std::cerr << *(temp_err->getString());
  delete(temp_err);
  m_ini.getValue("MarmotGUI", "StatusCommErr", m_status_comm_err);
  m_ini.getValue("MarmotGUI", "StatusBufferErr", m_status_buffer_err);
  m_ini.getValue("MarmotGUI", "StatusRankErr", m_status_rank_err);
  m_ini.getValue("MarmotGUI", "StatusTypeErr", m_status_type_err);
  m_ini.getValue("MarmotGUI", "StatusRequestErr", m_status_request_err);
  m_ini.getValue("MarmotGUI", "StatusIOErr", m_status_io_err);
  m_ini.getValue("MarmotGUI", "StatusMiscErr", m_status_misc_err);
  m_ini.getValue("MarmotGUI", "StatusCommWarn", m_status_comm_warn);
  m_ini.getValue("MarmotGUI", "StatusRankWarn", m_status_rank_warn);
  m_ini.getValue("MarmotGUI", "StatusRequestWarn", m_status_request_warn);
  m_ini.getValue("MarmotGUI", "StatusTypeWarn", m_status_type_warn);
  m_ini.getValue("MarmotGUI", "StatusThreadWarn", m_status_thread_warn);
  m_ini.getValue("MarmotGUI", "StatusMiscWarn", m_status_misc_warn);
  m_ini.getValue("MarmotGUI", "StatusTypeNote", m_status_type_note);
  m_ini.getValue("MarmotGUI", "StatusMiscNote", m_status_misc_note);

	return true;
}

bool MarmotGUIConfiguration::setConfiguration()
{
  QString temp_str;
  for(int i=0;i<181;i++)
  {
	if(!m_errcodes[i])
	{
		temp_str.append("^");
	}
	temp_str.append(QString::number(i));
	if(i<181-1)
	{
		temp_str.append(",");
	}
  }
  m_ini.setPath(m_filename);
  
  m_ini.setValue("MarmotGUI", "Exe", m_exe_path);
  m_ini.setValue("MarmotGUI", "ExeArgs", m_exe_arguments);
  m_ini.setValue("MarmotGUI", "ExeNumProcs", m_exe_numprocs);
  m_ini.setValue("MarmotGUI", "DebugMode", m_env_debugmode);
  m_ini.setValue("MarmotGUI", "LogfileType", m_env_logfiletype);
  m_ini.setValue("MarmotGUI", "LogHost", m_env_loghost);
  m_ini.setValue("MarmotGUI", "LogPort", m_env_logport);
  //m_ini.setValue("MarmotGUI", "MpiexecEnvParam", m_mpiexec_env_param);
  m_ini.setValue("MarmotGUI", "ErrCodes", temp_str);
  m_ini.setValue("MarmotGUI", "StatusCommErr", m_status_comm_err);
  m_ini.setValue("MarmotGUI", "StatusBufferErr", m_status_buffer_err);
  m_ini.setValue("MarmotGUI", "StatusRankErr", m_status_rank_err);
  m_ini.setValue("MarmotGUI", "StatusTypeErr", m_status_type_err);
  m_ini.setValue("MarmotGUI", "StatusRequestErr", m_status_request_err);
  m_ini.setValue("MarmotGUI", "StatusIOErr", m_status_io_err);
  m_ini.setValue("MarmotGUI", "StatusMiscErr", m_status_misc_err);
  m_ini.setValue("MarmotGUI", "StatusCommWarn", m_status_comm_warn);
  m_ini.setValue("MarmotGUI", "StatusRankWarn", m_status_rank_warn);
  m_ini.setValue("MarmotGUI", "StatusRequestWarn", m_status_request_warn);
  m_ini.setValue("MarmotGUI", "StatusTypeWarn", m_status_type_warn);
  m_ini.setValue("MarmotGUI", "StatusThreadWarn", m_status_thread_warn);
  m_ini.setValue("MarmotGUI", "StatusMiscWarn", m_status_misc_warn);
  m_ini.setValue("MarmotGUI", "StatusTypeNote", m_status_type_note);
  m_ini.setValue("MarmotGUI", "StatusMiscNote", m_status_misc_note);

  if (!m_ini.writeFile())
  {
    qDebug("DEBUG: Could not write conf-file (%s)", qPrintable(m_filename));
    return false;
  }
  
	return true;
}
/*
void MarmotGUIConfiguration::createFile(QString filename)
{
	qDebug("DEBUG: MarmotGUIConfiguration::createFile(%s) called\n", qPrintable(filename));
	
  xmlDocPtr doc;		//document pointer
	xmlNodePtr marmot;	//node pointer
	xmlNodePtr exe;			//exe node
	xmlNodePtr exe_path;		//path node
	xmlNodePtr exe_arguments;	//arguments node
	xmlNodePtr exe_numprocs;	//numprocs node
	xmlNodePtr env;			//env node
	xmlNodePtr env_debugmode;	//debugmode node
	xmlNodePtr env_logfiletype;	//logfiletype node
	xmlNodePtr env_loghost;		//loghost node
	xmlNodePtr env_logport;		//logport node
	doc=xmlNewDoc((const xmlChar*)"1.0");		//create new XML 1.0 document
	marmot=xmlNewNode(NULL,(const xmlChar*)"marmot");	
	exe=xmlNewNode(NULL,(const xmlChar*)"exe");		
	exe_path=xmlNewNode(NULL,(const xmlChar*)"path");
	exe_arguments=xmlNewNode(NULL,(const xmlChar*)"arguments");
	exe_numprocs=xmlNewNode(NULL,(const xmlChar*)"numprocs");
	env=xmlNewNode(NULL,(const xmlChar*)"env");
	env_debugmode=xmlNewNode(NULL,(const xmlChar*)"debugmode");
	env_logfiletype=xmlNewNode(NULL,(const xmlChar*)"logfiletype");
	env_loghost=xmlNewNode(NULL,(const xmlChar*)"loghost");
	env_logport=xmlNewNode(NULL,(const xmlChar*)"logport");
	xmlDocSetRootElement(doc,marmot);		//the root node is marmot	
	xmlAddChild(marmot,exe);			//child of marmot
	xmlAddChild(exe,exe_path);			//child of exe
	xmlAddChild(exe,exe_arguments);			//child of exe
	xmlAddChild(exe,exe_numprocs);			//child of exe
	xmlAddChild(marmot,env);			//child of marmot
	xmlAddChild(env,env_debugmode);			//child of env
	xmlAddChild(env,env_logfiletype);		//child of env
	xmlAddChild(env,env_loghost);			//child of env
	xmlAddChild(env,env_logport);			//child of env
	xmlSaveFormatFile((filename.toStdString()).c_str(),doc,1);	//save the file
}
*/	

QString MarmotGUIConfiguration::getExePath()
{
	return m_exe_path;
}

void MarmotGUIConfiguration::setExePath(QString exepath)
{
	m_exe_path = exepath;
}

QString MarmotGUIConfiguration::getExeArguments()
{
	return m_exe_arguments;
}

void MarmotGUIConfiguration::setExeArguments(QString exearguments)
{
	m_exe_arguments = exearguments;
}

int MarmotGUIConfiguration::getExeNumprocs()
{
	return m_exe_numprocs;
}

void MarmotGUIConfiguration::setExeNumprocs(int exenumprocs)
{
	m_exe_numprocs = exenumprocs;
}

int MarmotGUIConfiguration::getDebugMode()
{
	return m_env_debugmode;
}

void MarmotGUIConfiguration::setEnvDebugmode(int debugmode)
{
	m_env_debugmode = debugmode;
}

int MarmotGUIConfiguration::getLogfileType()
{
	return m_env_logfiletype;
}

void MarmotGUIConfiguration::setEnvLogfiletype(int logfiletype)
{
	m_env_logfiletype = logfiletype;
}

QString MarmotGUIConfiguration::getLogHost()
{
	return m_env_loghost;
}

void MarmotGUIConfiguration::setEnvLoghost(QString loghost)
{
	m_env_loghost = loghost;
}

int MarmotGUIConfiguration::getLogPort()
{
	return m_env_logport;
}

void MarmotGUIConfiguration::setEnvLogport(int logport)
{
	m_env_logport = logport;
}

void MarmotGUIConfiguration::setDefaultConf()
{
  m_filename = QDir::toNativeSeparators(QDir::homePath());
  m_filename.append(QDir::separator());
  m_filename.append(".marmot_current.conf");
}

void MarmotGUIConfiguration::getErrCodes(bool *errcodes)
{
	for(int i=0;i<NUMERRCODES;i++)
	{
		errcodes[i]=m_errcodes[i];
		//just for testing:
		//errcodes[i]=(i%2)?false:true;
	}
	return;
}

void MarmotGUIConfiguration::getEffectiveErrCodes(bool *effective_errcodes)
{
	for(int i=0;i<NUMERRCODES;i++)
	{
		effective_errcodes[i]=m_effective_errcodes[i];
	}
}

void MarmotGUIConfiguration::setErrCodes(bool *errcodes)
{
	for(int i=0;i<NUMERRCODES;i++)
	{
		m_errcodes[i]=errcodes[i];
	}
	return;
}

void MarmotGUIConfiguration::setEffectiveErrCodes(bool *effective_errcodes)
{
	for(int i=0;i<NUMERRCODES;i++)
	{
		m_effective_errcodes[i]=effective_errcodes[i];
	}
	return;
}

QString MarmotGUIConfiguration::getMpiexecEnvParam()
{
	return m_mpiexec_env_param;
}



