/*
 * Copyright (c) 2002-2008 High Performance Computing Center Stuttgart,
 *                         University of Stuttgart.  All rights reserved.
 * Copyright (c) 2006-2008 The Center for Information Services and HPC
 *                         Technical University of Dresden.
 *                         All rights reserved.
 *
 * $COPYRIGHT$
 *
 * Additional copyrights may follow
 *
 * $HEADER$
 */
#ifndef MARMOTGUICONFIGURATION_H
#define MARMOTGUICONFIGURATION_H

#define NUMERRCODES 181

#include <QString>
#include "inifile.h"
#include "ParseErrCodesEnv.h"

/*!
 provides read/write access to the MarmotGUI configuration file
 */
class MarmotGUIConfiguration
{
	public:
		MarmotGUIConfiguration();
		/*!
		 sets the filename of the configuration file
		 \param filename the filename
		 */
		void setConfFile(QString filename);
		/*!
		 get the filename of the configuration file
		 */
    QString getConfFile();
		/*!
		 reads the configuration from the configuration file
		 */
		bool getConfiguration();
		/*!
		 writes the current configuration to the configuration file
		 */
		bool setConfiguration();
		/*!
		 @return the path to the executable
		 */
		QString getExePath();
		/*!
		 sets the path to the executable
		 \param exepath the path to the executable
		 */
		void setExePath(QString exepath);
		/*!
		 @return the arguments for the executable
		 */
		QString getExeArguments();
		/*!
		 sets the arhuments for the executable
		 \param arguments the arguments for the executable
		 */
		void setExeArguments(QString exearguments);
		/*!
		 @return the number of processes
		 */
		int getExeNumprocs();
		/*!
		 sets the number of processes
		 \param exenumprocs the number of processes
		 */
		void setExeNumprocs(int exenumprocs);
		/*!
		 @return the debug mode
		 */
		int getDebugMode();
		/*!
		 sets the debug mode
		 \param debugmode the debug mode
		 */
		void setEnvDebugmode(int debugmode);
		/*!
		 @return the logfile type
		 */
		int getLogfileType();
		/*!
		 sets the logfile type
		 \param logfiletype the logfile type
		 */
		void setEnvLogfiletype(int logfiletype);
		/*!
		 @return the host
		 */
		QString getLogHost();
		/*!
		 sets the host
		 \param loghost the host
		 */
		void setEnvLoghost(QString loghost);
		/*!
		 @return the port
		 */
		int getLogPort();
		/*!
		 sets the port
		 \param logport the port
		 */
		void setEnvLogport(int logport);
		/*!
		 gets the err codes
		 \param errcodes array to be filled
		 */
		void getErrCodes(bool *errcodes); 
		/*!
		 gets the effective err codes, i.e. the err codes resulting from the err codes array and the tristate switches (enable,disable,selection) for error groups (e.g. comm_err)
		 \param effective_errcodes effective errcodes array to be filled
		 */
		void getEffectiveErrCodes(bool *effective_errcodes);
		/*!
		 sets the err codes
		 \param errcodes array from which the err codes are taken
		 */
		void setErrCodes(bool *errcodes);
		/*!
		 sets the effective err codes
		 \param effective_errcodes array from which the effective err codes are taken
		 */
		void setEffectiveErrCodes(bool *effective_errcodes);
		/*!
		 gets the comm_err status
		 @return the status
		 */
		inline int getStatusCommErr() { return m_status_comm_err; }
		/*!
		 sets the comm_err status
		 \param status_comm_err the status
		 */
		inline void setStatusCommErr(int status_comm_err) { m_status_comm_err=status_comm_err; return; }
		/*!
		 gets the buffer_err status
		 @return the status
		 */
		inline int getStatusBufferErr() { return m_status_buffer_err; }
		/*!
		 sets the buffer_err status
		 \param status_buffer_err the status
		 */
		inline void setStatusBufferErr(int status_buffer_err) { m_status_buffer_err=status_buffer_err; return; }
		/*!
		 gets the rank_err status
		 @return the status
		 */
		inline int getStatusRankErr() { return m_status_rank_err; }
		/*!
		 sets the rank_err status
		 \param status_rank_err the status
		 */
		inline void setStatusRankErr(int status_rank_err) { m_status_rank_err=status_rank_err; return; }
		/*!
		 gets the type_err status
		 @return the status
		 */
		inline int getStatusTypeErr() { return m_status_type_err; }
		/*!
		 sets the type_err status
		 \param status_type_err the status
		 */
		inline void setStatusTypeErr(int status_type_err) { m_status_type_err=status_type_err; return; }
		/*!
		 gets the request_err status
		 @return the status
		 */
		inline int getStatusRequestErr() { return m_status_request_err; }
		/*!
		 sets the request_err status
		 \param status_request_err the status
		 */
		inline void setStatusRequestErr(int status_request_err) { m_status_request_err=status_request_err; return; }
		/*!
		 gets the io_err status
		 @return the status
		 */
		inline int getStatusIOErr() { return m_status_io_err; }
		/*!
		 sets the io_err status
		 \param status_io_err the status
		 */
		inline void setStatusIOErr(int status_io_err) { m_status_io_err=status_io_err; return; }
		/*!
		 gets the misc_err status
		 @return the status
		 */
		inline int getStatusMiscErr() { return m_status_misc_err; }
		/*!
		 sets the misc_err status
		 \param status_misc_err the status
		 */
		inline void setStatusMiscErr(int status_misc_err) { m_status_misc_err=status_misc_err; return; }
		/*!
		 gets the comm_warn status
		 @return the status
		 */
		inline int getStatusCommWarn() { return m_status_comm_warn; }
		/*!
		 sets the comm_warn status
		 \param status_comm_warn the status
		 */
		inline void setStatusCommWarn(int status_comm_warn) { m_status_comm_warn=status_comm_warn; return; }
		/*!
		 gets the rank_warn status
		 @return the status
		 */
		inline int getStatusRankWarn() { return m_status_rank_warn; }
		/*!
		 sets the rank_warn status
		 \param status_rank_warn the status
		 */
		inline void setStatusRankWarn(int status_rank_warn) { m_status_rank_warn=status_rank_warn; return; }
		/*!
		 gets the request_warn status
		 @return the status
		 */
		inline int getStatusRequestWarn() { return m_status_request_warn; }
		/*!
		 sets the request_warn status
		 \param status_request_warn the status
		 */
		inline void setStatusRequestWarn(int status_request_warn) { m_status_request_warn=status_request_warn; return; }
		/*!
		 gets the type_warn status
		 @return the status
		 */
		inline int getStatusTypeWarn() { return m_status_type_warn; }
		/*!
		 sets the type_warn status
		 \param status_type_warn the status
		 */
		inline void setStatusTypeWarn(int status_type_warn) { m_status_type_warn=status_type_warn; return; }
		/*!
		 gets the thread_warn status
		 @return the status
		 */
		inline int getStatusThreadWarn() { return m_status_thread_warn; }
		/*!
		 sets the thread_warn status
		 \param status_thread_warn the status
		 */
		inline void setStatusThreadWarn(int status_thread_warn) { m_status_thread_warn=status_thread_warn; return; }
		/*!
		 gets the misc_warn status
		 @return the status
		 */
		inline int getStatusMiscWarn() { return m_status_misc_warn; }
		/*!
		 sets the misc_warn status
		 \param status_misc_warn the status
		 */
		inline void setStatusMiscWarn(int status_misc_warn) { m_status_misc_warn=status_misc_warn; return; }
		/*!
		 gets the type_note status
		 @return the status
		 */
		inline int getStatusTypeNote() { return m_status_type_note; }
		/*!
		 sets the type_note status
		 \param status_type_note the status
		 */
		inline void setStatusTypeNote(int status_type_note) { m_status_type_note=status_type_note; return; }
		/*!
		 gets the misc_note status
		 @return the status
		 */
		inline int getStatusMiscNote() { return m_status_misc_note; }
		/*!
		 sets the misc_note status
		 \param status_misc_note the status
		 */
		inline void setStatusMiscNote(int status_misc_note) { m_status_misc_note=status_misc_note; return; }
    QString getMpiexecEnvParam();


	private:
    void setDefaultConf();

		/*!
		 the filename of the configuration file
		 */
		QString m_filename;
		/*!
		 the path to the executable
		 */
		QString m_exe_path;
		/*!
		 arguments for the executable
		 */
		QString m_exe_arguments;
		/*!
		 number of processes
		 */
		int m_exe_numprocs;
		/*!
		 the debug mode
		 */
		int m_env_debugmode;
		/*!
		 the logfile type
		 */
		int m_env_logfiletype;
		/*!
		 the host
		 */
		QString m_env_loghost;
		/*!
		 the port
		 */
		int m_env_logport;
		/*!
		 the err codes array
		 */
		bool m_errcodes[NUMERRCODES];
		bool m_effective_errcodes[NUMERRCODES];

		int m_status_comm_err;
		int m_status_buffer_err;
		int m_status_rank_err;
		int m_status_type_err;
		int m_status_request_err;
		int m_status_io_err;
		int m_status_misc_err;

		int m_status_comm_warn;
		int m_status_rank_warn;
		int m_status_request_warn;
		int m_status_type_warn;
		int m_status_thread_warn;
		int m_status_misc_warn;

		int m_status_type_note;
		int m_status_misc_note;

        QString m_mpiexec_env_param;
	
    CIniFile m_ini;
};


#endif


