package de.hlrs.marmoclipse.log;

/**
 * models a MARMOT log entry, which contains 11 different fields. 
 * Object of this class is constructed from a received line of log 
 * which contains all the fields truncated with a special character 
 * (Unicode: 0xfffd) as separator.
 * 
 * @author peng
 *
 */

public class MarmotLog {
	public static final int TYPE_INFO = 0;
	public static final int TYPE_WARNING = 1;
	public static final int TYPE_ERROR = 2;
	public static final int TYPE_NOTE = 3;
	
	public static MarmotLogFormat[] Formats = {
		new MarmotLogFormat("Text file", "*.txt", "ASCII"),
		new MarmotLogFormat("HTML File", "*.html", "HTML"),
		new MarmotLogFormat("CUBE File", "*.cube", "CUBE")
	};
	private static final String LOG_SEPARATOR = "\ufffd";
	
	private int time = 0;
	private int rank;
	private int thread;
	private String type;
	private String message;
	private String file;
	private String line;
	private String call;
	private String location;
	private String reference;
	private String field11;

	/**
	 * construct a log from a line of truncated parts
	 * 
	 * @param line
	 */
	public MarmotLog(String line) {
		if (line == null)
			throw new IllegalArgumentException("The provided log line is NULL.");

		String[] parts = line.split(LOG_SEPARATOR, 11);

		setTime(Integer.parseInt(parts[0]));
		setRank(Integer.parseInt(parts[1]));
		setThread(Integer.parseInt(parts[2]));
		setType(parts[3]);
		setDetail(parts[4]);
		setFile(parts[5]);
		setLine(parts[6]);
		setCall(parts[7]);
		setLocation(parts[8]);
		setReference(parts[9]);
		setField11((parts.length < 11) ? "" : parts[10]);
	}

	public void setTime(int time) {
		this.time = time;
	}

	public int getTime() {
		return time;
	}

	public void setRank(int rank) {
		this.rank = rank;
	}

	public int getRank() {
		return rank;
	}

	public void setThread(int thread) {
		this.thread = thread;
	}

	public int getThread() {
		return thread;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getType() {
		return type;
	}

	public void setDetail(String detail) {
		this.message = detail;
	}

	public String getDetail() {
		return message;
	}

	public void setFile(String file) {
		this.file = file;
	}

	public String getFile() {
		return file;
	}

	public void setLine(String line) {
		this.line = line;
	}

	public String getLine() {
		return line;
	}

	public void setLocation(String location) {
		this.location = location;
	}

	public String getLocation() {
		return location;
	}

	public void setReference(String reference) {
		this.reference = reference;
	}

	public String getReference() {
		return reference;
	}

	public void setCall(String call) {
		this.call = call;
	}

	public String getCall() {
		return call;
	}

	public void setField11(String field11) {
		this.field11 = field11;
	}

	public String getField11() {
		return field11;
	}

}
