package de.hlrs.marmoclipse.log;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import de.hlrs.marmoclipse.log.ui.ILogListViewer;

/**
 * models a list of {@link MarmotLog} objects and provides several helper methods
 * for adding, removing log entry to the list and clearing the list. 
 * This class also contains a simple implementation of change listeners 
 * which will be used by LogContentProvider class.
 * 
 * @author peng
 *
 */
public class MarmotLogList {
	private List<MarmotLog> logs;
	private Set<ILogListViewer> changeListeners = new HashSet<ILogListViewer>();

	public MarmotLogList() {
		super();
		logs = new ArrayList<MarmotLog>();
//		this.initData();
	}

//	private void initData() {
//		// logs.add(new
//		// MarmotLog("0°-1°0°0°The following entries describ°°°°Unknown°°"));
//		// logs.add(new
//		// MarmotLog("0°-1°0°0°MARMOT_DEBUG_MODE = 2<br>adfd°°°°Unknown°°"));
//		// logs.add(new
//		// MarmotLog("0°-1°0°0°MARMOT_LOGFILE_TYPE = 0°°°°Unknown°°"));
//	}

	public List<MarmotLog> getLogs() {
		return logs;
	}

	public void addLog(String line) {
		MarmotLog log;
		try {
			log = new MarmotLog(line);
		} catch (IllegalArgumentException e) {
			System.out.println("An error occurred during adding a log line: "
					+ e.getMessage());
			return;
		}
//		logs.add(logs.size(), log);
		logs.add(log);
		Iterator<ILogListViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			iterator.next().addLog(log);
	}

	public void removeLog(MarmotLog log) {
		logs.remove(log);
		Iterator<ILogListViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			iterator.next().removeLog(log);
	}

	public void logChanged(MarmotLog log) {
		Iterator<ILogListViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			iterator.next().updateLog(log);
	}
	
	public void clearLogs() {
		logs.clear();
		Iterator<ILogListViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			iterator.next().clearLogs();
	}

	public void removeChangeListener(ILogListViewer viewer) {
		changeListeners.remove(viewer);
	}

	public void addChangeListener(ILogListViewer viewer) {
		changeListeners.add(viewer);
	}
}
