package de.hlrs.marmoclipse.log.ui;

import org.eclipse.jface.viewers.ITableColorProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.swt.widgets.Display;

import de.hlrs.marmoclipse.log.MarmotLog;

/**
 * defines how the data is displayed in the table viewer cell
 * 
 * @author peng
 *
 */
public class LogLabelProvider extends LabelProvider implements
	ITableLabelProvider, ITableColorProvider {
	
	private Color fgColorWarning;
	private Color fgColorError;
	private Color fgColorNote;
	private Color fgColorNormal;

	public LogLabelProvider() {
		super();
		fgColorWarning = new Color(Display.getDefault(), 193, 125, 17);
		fgColorError = new Color(Display.getDefault(), 204, 0, 0);
		fgColorNote = new Color(Display.getDefault(), 52, 101, 164);
		fgColorNormal = new Color(Display.getDefault(), 46, 52, 54);
	}

	public Image getColumnImage(Object element, int columnIndex) {
		Image img = null;
		MarmotLog log = (MarmotLog) element;
		if (columnIndex == 0) {
			switch (Integer.valueOf(log.getType())) {
			case MarmotLog.TYPE_INFO:
				img = PlatformUI.getWorkbench().getSharedImages()
					.getImage(ISharedImages.IMG_OBJS_INFO_TSK);
				break;
			case MarmotLog.TYPE_WARNING:
				img = PlatformUI.getWorkbench().getSharedImages()
					.getImage(ISharedImages.IMG_OBJS_WARN_TSK);
				break;
			case MarmotLog.TYPE_ERROR:
				img = PlatformUI.getWorkbench().getSharedImages()
					.getImage(ISharedImages.IMG_OBJS_ERROR_TSK);
				break;
			case MarmotLog.TYPE_NOTE:
				img = PlatformUI.getWorkbench().getSharedImages()
					.getImage(ISharedImages.IMG_OBJ_FILE);
			default:
				break;
			}
		}
		return img;
	}

	public String getColumnText(Object element, int columnIndex) {
		String result = null;
		MarmotLog log = (MarmotLog) element;
		switch (columnIndex) {
		// case 0:
		// result = log.getType();
		// break;
		case 1:
			result = log.getTime() + "";
			break;
		case 2:
			result = (log.getRank() == -1) ? "Global" : (log.getRank() + "");
			break;
		case 3:
			result = log.getThread() + "";
			break;
		case 4:
			String str = log.getDetail();
			result = str.replaceAll("<br>[^$]", "\n").replaceAll("<br>$", "");
			break;
		case 5:
			result = log.getFile();
			break;
		case 6:
			result = log.getLine();
			break;
		case 7:
			result = log.getCall();
			break;
		case 8:
			result = log.getLocation();
			break;
		case 9:
			result = log.getReference();
			break;
		case 10:
			result = log.getField11();
			break;
		default:
			break;
		}
		return result;
	}

	public Color getBackground(Object element, int columnIndex) {
		return null;
	}

	public Color getForeground(Object element, int columnIndex) {
		Color c = null;
		MarmotLog log = (MarmotLog) element;
		switch (Integer.valueOf(log.getType())) {
		case MarmotLog.TYPE_WARNING:
			c = fgColorWarning;
			break;
		case MarmotLog.TYPE_ERROR:
			c = fgColorError;
			break;
		case MarmotLog.TYPE_NOTE:
			c = fgColorNote;
			break;
		default:
			c = fgColorNormal;
		}
		return c;
	}

	public void dispose() {
		fgColorError.dispose();
		fgColorWarning.dispose();
		fgColorNote.dispose();
		fgColorNormal.dispose();
	}
}
