/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *  
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Send
 *  - MPI_Recv
 *  - MPI_Probe
 *  - MPI_Get_count
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: get-count.c 319 2004-08-16 11:25:02Z rusbetti $  
 */


#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int MSG_TAG = 17;

    int size = -1;
    int rank = -1;
    int value = -1;
    int count = -1;

    MPI::Status status;
    MPI::Status status2;

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (rank == 0)
    {
	MPI::COMM_WORLD.Probe(1, MSG_TAG, status2);
        printf("Message can be received \n");
        count=status.Get_count(MPI::INT);
        /* going to receive message */
	MPI::COMM_WORLD.Recv(&value, count, MPI::INT, 1, MSG_TAG, status);
        assert(value == 19);
    }

    if (rank == 1)
    {
        /* going to send message */
        value = 19;
        count = 1;
	MPI::COMM_WORLD.Send(&value, count, MPI::INT, 0, MSG_TAG);
    }

    MPI::Finalize();

    return 0;
}
