/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Cart_create
 *  - MPI_Comm_group
 *  - MPI_Comm_free
 *  - MPI_Group_translate_ranks
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: group-translate-ranks2.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    int size = -1;
    int notfound = 0;
    int i = 0;
    int rank = -1;
    int dims[2] = { 2, 3 };
    bool periods[2] = { 0, 0 };
    int *ranks1 = 0;
    int *ranks2 = 0;

    MPI::Cartcomm comm_cart;

    MPI::Group worldgroup;
    MPI::Group comm_group;

    printf("Here we use a comm/group we already freed. \n");
    printf("dim1=%d, dim2=%d \n", dims[0], dims[1]);
    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();

    if (size != (dims[0] * dims[1]))
    {
        if (rank == 0)
        {
            printf("Number of nodes not matching new topology! \n");
        }
    }
    else
    {
        printf(" I am rank %d of %d PEs\n", rank, size);

        ranks1 = (int *) malloc(sizeof(MPI::INT) * size);
        ranks2 = (int *) malloc(sizeof(MPI::INT) * size);

        comm_cart=MPI::COMM_WORLD.Create_cart(2, dims, periods, 1);
        worldgroup=MPI::COMM_WORLD.Get_group();
        comm_cart.Free();
        comm_group=comm_cart.Get_group();
        comm_group.Free();
	MPI::Group::Translate_ranks(worldgroup, size, ranks1, comm_group,
                                  ranks2);
        for (i = 0; i <= size; i++)
        {
            if (ranks2[i] == MPI::UNDEFINED)
            {
                notfound++;
            }
        }
        if (notfound > 0)
        {
            printf("notfound>0: %d \n", notfound);
        }
    }                           /* end else */

    free(ranks1);
    free(ranks2);

    MPI::Finalize();

    return 1;
}
