/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Issend
 *  - MPI_Wait
 *  - MPI_Irecv
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: issend-rr.c 319 2004-08-16 11:25:02Z rusbetti $  
 */


/* 
** Here we re-use a request we didn't free before
*/

#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int COUNT = 1;
    const int MSG_TAG = 17;

    int size = -1;
    int rank = -1;
    int value = -1;
    int value2 = -1;

    MPI::Status send_status;
    MPI::Status status;

    MPI::Request send_request;

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (rank == 0)
    {
        /* going to receive the message and reuse a non-freed request */
	    MPI::COMM_WORLD.Recv(&value, COUNT, MPI::INT, 1, MSG_TAG, status);
        assert(value == 19);
    }

    if (rank == 1)
    {
        value2 = 19;
        /* this is just to use the request */
        /*MPI_Isend(&value,COUNT,MPI_INT,0,18,MPI_COMM_WORLD,&send_request); */

        /* going to send the message */
        send_request=MPI::COMM_WORLD.Issend(&value2, COUNT, MPI::INT, 0, MSG_TAG);
        send_request.Wait(send_status);
    }

    MPI::Finalize();

    return 0;
}
