/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Isend
 *  - MPI_Wait
 *  - MPI_Irecv
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: request-reuse1.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

/* 
** Here we re-use a request we didn't free before
*/

#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int COUNT = 1;
    const int MSG_TAG_1 = 16;
    const int MSG_TAG_2 = 17;
    const int MSG_TAG_3 = 18;

    int size = -1;
    int rank = -1;
    int value = -1;
    int value2 = -1;

    MPI::Status send_status, recv_status;

    MPI::Request send_request, recv_request;

    printf("We call Irecv and Isend with non-freed requests.\n");
    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (rank == 0)
    {
        /* This is just to get the request used */
        recv_request=MPI::COMM_WORLD.Isend(&value, COUNT, MPI::INT, 1, MSG_TAG_1);

        /* going to receive the message and reuse a non-freed request */
        recv_request=MPI::COMM_WORLD.Irecv(&value, COUNT, MPI::INT, 1, MSG_TAG_2);
        recv_request.Wait(recv_status);
        assert(value == 19);
    }

    if (rank == 1)
    {
        value2 = 19;
        /* this is just to use the request */
        send_request=MPI::COMM_WORLD.Isend(&value, COUNT, MPI::INT, 0, MSG_TAG_3);

        /* going to send the message */
        send_request=MPI::COMM_WORLD.Isend(&value2, COUNT, MPI::INT, 1, MSG_TAG_2);
        send_request.Wait(send_status);
    }

    MPI::Finalize();

    return 0;
}
