/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Rsend
 *  - MPI_Recv
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: rsend.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int MSG_TAG_1 = 17;
    const int MSG_TAG_2 = 18;
    const int COUNT = 1;

    int size = -1;
    int rank = -1;
    int value = -1;
    int value2 = -1;

    MPI::Status status;
    MPI::Request request;

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (rank == 0)
    {
        /* going to receive message */
        value2 = 37;
        request=MPI::COMM_WORLD.Irecv(&value, COUNT, MPI::INT, 1, MSG_TAG_1);
	MPI::COMM_WORLD.Send(&value2, COUNT, MPI::INT, 1, MSG_TAG_2);
        request.Wait(status);
        assert(value == 19);
    }

    if (rank == 1)
    {
        /* going to send message */
        value2 = 19;
	MPI::COMM_WORLD.Recv(&value, COUNT, MPI::INT, 0, MSG_TAG_2,status);
	MPI::COMM_WORLD.Rsend(&value2, COUNT, MPI::INT, 0, MSG_TAG_1);
    }

    MPI::Finalize();

    return 0;
}
