/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Isend
 *  - MPI_Wait
 *  - MPI_Irecv
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: test_Ipt2pt.c 319 2004-08-16 11:25:02Z rusbetti $  
 */

#include <stdio.h>
#include <assert.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int COUNT = 1;
    const int MSG_TAG = 17;

    int size = -1;
    int rank = -1;
    int value = -1;
    int value2 = -1;

    MPI::Status send_status, recv_status;
    MPI::Request send_request, recv_request;

    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (rank == 0)
    {
        /* going to receive message */
        recv_request=MPI::COMM_WORLD.Irecv(&value, COUNT, MPI::INT, 1, MSG_TAG);
        recv_request.Wait(recv_status);
        assert(value == 19);
    }

    if (rank == 1)
    {
        /* going to send message */
        value2 = 19;
        send_request=MPI::COMM_WORLD.Isend(&value2, COUNT, MPI::INT, 0, MSG_TAG);
        send_request.Wait(send_status);
    }

    MPI::Finalize();

    return 0;
}
