/**
 *  @file
 *
 *  This program is a very simple MPI program to verify that the 
 *  library works correctly.
 *
 *  It just calls 
 *  - MPI_Init
 *  - MPI_Finalize 
 *  - MPI_Comm_rank
 *  - MPI_Comm_size
 *  - MPI_Isend
 *  - MPI_Testany
 *  - MPI_Recv
 *
 *  @author Bettina Krammer, Katrin Bidmon, Matthias Mueller
 *
 *  $Id: testany-nvr.c 319 2004-08-16 11:25:02Z rusbetti $  
 */


 /*
  ** We have 3 requests in the array - one is reused (but freed before),
  ** one usual and one which is non-valid
  */

#include <stdio.h>
#include <assert.h>
/* #include <time.h>
 * use instead
 */
#include <unistd.h>
#include "mpi.h"

int main(int argc, char **argv)
{
    const int COUNT = 1;
    const int MSG_TAG_1 = 17;
    const int MSG_TAG_2 = 18;

    int size = -1;
    int rank = -1;
    int value = -1;
    int value2 = -1;
    int index = -1;
    int flag = 0;

    MPI::Status status;
    MPI::Status status_arr[3];

    MPI::Request request_arr[3];

    printf("We call Testany with 3 requests: MPI_REQUEST_NULL, \
valid, non-valid.\n");
    MPI::Init(argc, argv);
    size=MPI::COMM_WORLD.Get_size();
    rank=MPI::COMM_WORLD.Get_rank();
    printf(" I am rank %d of %d PEs\n", rank, size);

    if (rank == 0)
    {
        /* going to receive message */
	    MPI::COMM_WORLD.Recv(&value2, COUNT, MPI::INT, 1, MSG_TAG_1, status);
        assert(value2 == 19);
	MPI::COMM_WORLD.Recv(&value2, COUNT, MPI::INT, 1, MSG_TAG_2, status);
    }

    if (rank == 1)
    {
        /* going to send message */
        value = 19;
        request_arr[0]=MPI::COMM_WORLD.Isend(&value, COUNT, MPI::INT, 0, MSG_TAG_1);
        request_arr[0].Wait(status);
        request_arr[0]=MPI::COMM_WORLD.Isend(&value, COUNT, MPI::INT, 0, MSG_TAG_2);
        sleep(1);
        flag=MPI::Request::Testany(3, request_arr, index, status_arr[0]);
        printf(" index(of now freed request)= %d \n", index);
    }

    MPI::Finalize();

    return 1;
}
